# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './mw4/gui/widgets/hemisphere.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_HemisphereDialog(object):
    def setupUi(self, HemisphereDialog):
        HemisphereDialog.setObjectName("HemisphereDialog")
        HemisphereDialog.resize(800, 600)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(HemisphereDialog.sizePolicy().hasHeightForWidth())
        HemisphereDialog.setSizePolicy(sizePolicy)
        HemisphereDialog.setMinimumSize(QtCore.QSize(800, 600))
        HemisphereDialog.setMaximumSize(QtCore.QSize(1600, 1200))
        HemisphereDialog.setSizeIncrement(QtCore.QSize(10, 10))
        HemisphereDialog.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        HemisphereDialog.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(HemisphereDialog)
        self.verticalLayout.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout.setSpacing(4)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(4, 4, 4, 4)
        self.horizontalLayout.setSpacing(4)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.groupBox_3 = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout.setContentsMargins(10, 10, 10, 10)
        self.gridLayout.setSpacing(10)
        self.gridLayout.setObjectName("gridLayout")
        self.checkShowMeridian = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowMeridian.setFont(font)
        self.checkShowMeridian.setObjectName("checkShowMeridian")
        self.gridLayout.addWidget(self.checkShowMeridian, 1, 0, 1, 1)
        self.checkShowCelestial = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowCelestial.setFont(font)
        self.checkShowCelestial.setObjectName("checkShowCelestial")
        self.gridLayout.addWidget(self.checkShowCelestial, 2, 0, 1, 1)
        self.checkUseHorizon = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkUseHorizon.setFont(font)
        self.checkUseHorizon.setChecked(True)
        self.checkUseHorizon.setObjectName("checkUseHorizon")
        self.gridLayout.addWidget(self.checkUseHorizon, 0, 1, 1, 1)
        self.checkShowAlignStar = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowAlignStar.setFont(font)
        self.checkShowAlignStar.setObjectName("checkShowAlignStar")
        self.gridLayout.addWidget(self.checkShowAlignStar, 0, 0, 1, 1)
        self.checkShowSlewPath = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowSlewPath.setFont(font)
        self.checkShowSlewPath.setObjectName("checkShowSlewPath")
        self.gridLayout.addWidget(self.checkShowSlewPath, 3, 0, 1, 1)
        self.horizontalLayout.addWidget(self.groupBox_3)
        self.line_3 = QtWidgets.QFrame(HemisphereDialog)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_3.setMidLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_3.setObjectName("line_3")
        self.horizontalLayout.addWidget(self.line_3)
        self.groupBox_2 = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_2.setContentsMargins(10, 10, 10, 10)
        self.gridLayout_2.setSpacing(10)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.checkPolarAlignment = QtWidgets.QRadioButton(self.groupBox_2)
        self.checkPolarAlignment.setEnabled(True)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkPolarAlignment.setFont(font)
        self.checkPolarAlignment.setObjectName("checkPolarAlignment")
        self.gridLayout_2.addWidget(self.checkPolarAlignment, 3, 0, 1, 1)
        self.checkEditHorizonMask = QtWidgets.QRadioButton(self.groupBox_2)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkEditHorizonMask.setFont(font)
        self.checkEditHorizonMask.setObjectName("checkEditHorizonMask")
        self.gridLayout_2.addWidget(self.checkEditHorizonMask, 2, 0, 1, 1)
        self.checkEditBuildPoints = QtWidgets.QRadioButton(self.groupBox_2)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkEditBuildPoints.setFont(font)
        self.checkEditBuildPoints.setObjectName("checkEditBuildPoints")
        self.gridLayout_2.addWidget(self.checkEditBuildPoints, 1, 0, 1, 1)
        self.checkEditNone = QtWidgets.QRadioButton(self.groupBox_2)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkEditNone.setFont(font)
        self.checkEditNone.setChecked(True)
        self.checkEditNone.setObjectName("checkEditNone")
        self.gridLayout_2.addWidget(self.checkEditNone, 0, 0, 1, 1)
        self.addPositionToHorizon = QtWidgets.QPushButton(self.groupBox_2)
        self.addPositionToHorizon.setEnabled(False)
        self.addPositionToHorizon.setMinimumSize(QtCore.QSize(90, 0))
        self.addPositionToHorizon.setObjectName("addPositionToHorizon")
        self.gridLayout_2.addWidget(self.addPositionToHorizon, 2, 1, 1, 1)
        self.horizontalLayout.addWidget(self.groupBox_2)
        self.line_4 = QtWidgets.QFrame(HemisphereDialog)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_4.setMidLineWidth(1)
        self.line_4.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_4.setObjectName("line_4")
        self.horizontalLayout.addWidget(self.line_4)
        self.groupBox = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox.setEnabled(True)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_2.setSpacing(4)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.showPolar = QtWidgets.QCheckBox(self.groupBox)
        self.showPolar.setEnabled(True)
        self.showPolar.setObjectName("showPolar")
        self.verticalLayout_2.addWidget(self.showPolar)
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setContentsMargins(-1, 4, -1, 4)
        self.gridLayout_3.setSpacing(4)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setMinimumSize(QtCore.QSize(60, 0))
        self.label.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.gridLayout_3.addWidget(self.label, 0, 0, 1, 1)
        self.altitude = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.altitude.sizePolicy().hasHeightForWidth())
        self.altitude.setSizePolicy(sizePolicy)
        self.altitude.setMaximumSize(QtCore.QSize(60, 16777215))
        self.altitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitude.setObjectName("altitude")
        self.gridLayout_3.addWidget(self.altitude, 0, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setMinimumSize(QtCore.QSize(60, 0))
        self.label_2.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.gridLayout_3.addWidget(self.label_2, 1, 0, 1, 1)
        self.azimuth = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.azimuth.sizePolicy().hasHeightForWidth())
        self.azimuth.setSizePolicy(sizePolicy)
        self.azimuth.setMaximumSize(QtCore.QSize(60, 16777215))
        self.azimuth.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuth.setObjectName("azimuth")
        self.gridLayout_3.addWidget(self.azimuth, 1, 1, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout_3)
        self.horizontalLayout.addWidget(self.groupBox)
        spacerItem = QtWidgets.QSpacerItem(80, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.horizontalLayout.setStretch(5, 1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.line = QtWidgets.QFrame(HemisphereDialog)
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setLineWidth(2)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.graphics = QtWidgets.QHBoxLayout()
        self.graphics.setObjectName("graphics")
        self.hemisphere = QtWidgets.QWidget(HemisphereDialog)
        self.hemisphere.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hemisphere.sizePolicy().hasHeightForWidth())
        self.hemisphere.setSizePolicy(sizePolicy)
        self.hemisphere.setMinimumSize(QtCore.QSize(0, 270))
        self.hemisphere.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.hemisphere.setSizeIncrement(QtCore.QSize(10, 10))
        self.hemisphere.setBaseSize(QtCore.QSize(10, 10))
        self.hemisphere.setAutoFillBackground(True)
        self.hemisphere.setStyleSheet("")
        self.hemisphere.setObjectName("hemisphere")
        self.hemisphereMove = QtWidgets.QWidget(self.hemisphere)
        self.hemisphereMove.setEnabled(True)
        self.hemisphereMove.setGeometry(QtCore.QRect(1, 0, 391, 423))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hemisphereMove.sizePolicy().hasHeightForWidth())
        self.hemisphereMove.setSizePolicy(sizePolicy)
        self.hemisphereMove.setMinimumSize(QtCore.QSize(0, 270))
        self.hemisphereMove.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.hemisphereMove.setSizeIncrement(QtCore.QSize(10, 10))
        self.hemisphereMove.setBaseSize(QtCore.QSize(10, 10))
        self.hemisphereMove.setAutoFillBackground(True)
        self.hemisphereMove.setStyleSheet("")
        self.hemisphereMove.setObjectName("hemisphereMove")
        self.graphics.addWidget(self.hemisphere)
        self.polar = QtWidgets.QWidget(HemisphereDialog)
        self.polar.setObjectName("polar")
        self.polarMove = QtWidgets.QWidget(self.polar)
        self.polarMove.setEnabled(False)
        self.polarMove.setGeometry(QtCore.QRect(0, 0, 391, 423))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.polarMove.sizePolicy().hasHeightForWidth())
        self.polarMove.setSizePolicy(sizePolicy)
        self.polarMove.setMinimumSize(QtCore.QSize(0, 270))
        self.polarMove.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.polarMove.setSizeIncrement(QtCore.QSize(10, 10))
        self.polarMove.setBaseSize(QtCore.QSize(10, 10))
        self.polarMove.setAutoFillBackground(True)
        self.polarMove.setStyleSheet("")
        self.polarMove.setObjectName("polarMove")
        self.graphics.addWidget(self.polar)
        self.graphics.setStretch(0, 1)
        self.graphics.setStretch(1, 1)
        self.verticalLayout.addLayout(self.graphics)

        self.retranslateUi(HemisphereDialog)
        QtCore.QMetaObject.connectSlotsByName(HemisphereDialog)

    def retranslateUi(self, HemisphereDialog):
        _translate = QtCore.QCoreApplication.translate
        HemisphereDialog.setWindowTitle(_translate("HemisphereDialog", "Hemisphere"))
        self.groupBox_3.setToolTip(_translate("HemisphereDialog", "Showing different elements on the hemisphere diagram."))
        self.groupBox_3.setTitle(_translate("HemisphereDialog", "Show Elements"))
        self.checkShowMeridian.setText(_translate("HemisphereDialog", "Meridian limits "))
        self.checkShowCelestial.setText(_translate("HemisphereDialog", "Celestial paths "))
        self.checkUseHorizon.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p>Use horizon mask file in hemisphere window.</p></body></html>"))
        self.checkUseHorizon.setText(_translate("HemisphereDialog", "Horizon mask "))
        self.checkShowAlignStar.setText(_translate("HemisphereDialog", "Align stars "))
        self.checkShowSlewPath.setText(_translate("HemisphereDialog", "Slew path "))
        self.groupBox_2.setTitle(_translate("HemisphereDialog", "Operation Mode"))
        self.checkPolarAlignment.setToolTip(_translate("HemisphereDialog", "Enabling the polar or ortho align procedure. For starting the procedure, enable align stars and choose a polar align star with left mouse click and a ortho align star with right mouse click."))
        self.checkPolarAlignment.setText(_translate("HemisphereDialog", "Align Polar/Ortho "))
        self.checkEditHorizonMask.setToolTip(_translate("HemisphereDialog", "Enabling the edit of the horizon points."))
        self.checkEditHorizonMask.setText(_translate("HemisphereDialog", "Edit Horizon Mask "))
        self.checkEditBuildPoints.setToolTip(_translate("HemisphereDialog", "Enabling the edit mode for model build points."))
        self.checkEditBuildPoints.setText(_translate("HemisphereDialog", "Edit Build Points"))
        self.checkEditNone.setToolTip(_translate("HemisphereDialog", "Switching the hemisphere window to normal working mode."))
        self.checkEditNone.setText(_translate("HemisphereDialog", "Normal"))
        self.addPositionToHorizon.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p>Add actual telescope position (Alt/Az) as point to the horizon.</p></body></html>"))
        self.addPositionToHorizon.setText(_translate("HemisphereDialog", "Add actual pos"))
        self.groupBox.setTitle(_translate("HemisphereDialog", "Views / Coords"))
        self.showPolar.setText(_translate("HemisphereDialog", "Show Polar Diagram"))
        self.label.setText(_translate("HemisphereDialog", "Altitude:"))
        self.label_2.setText(_translate("HemisphereDialog", "Azimuth:"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    HemisphereDialog = QtWidgets.QWidget()
    ui = Ui_HemisphereDialog()
    ui.setupUi(HemisphereDialog)
    HemisphereDialog.show()
    sys.exit(app.exec_())
