# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './mw4/gui/widgets/devicePopup.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DevicePopup(object):
    def setupUi(self, DevicePopup):
        DevicePopup.setObjectName("DevicePopup")
        DevicePopup.resize(460, 300)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(DevicePopup.sizePolicy().hasHeightForWidth())
        DevicePopup.setSizePolicy(sizePolicy)
        DevicePopup.setMinimumSize(QtCore.QSize(460, 300))
        DevicePopup.setMaximumSize(QtCore.QSize(460, 300))
        DevicePopup.setSizeIncrement(QtCore.QSize(10, 10))
        DevicePopup.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        DevicePopup.setFont(font)
        self.tab = QtWidgets.QTabWidget(DevicePopup)
        self.tab.setGeometry(QtCore.QRect(80, 5, 371, 256))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.tab.setFont(font)
        self.tab.setObjectName("tab")
        self.ALPACA = QtWidgets.QWidget()
        self.ALPACA.setObjectName("ALPACA")
        self.label_mountIP_7 = QtWidgets.QLabel(self.ALPACA)
        self.label_mountIP_7.setGeometry(QtCore.QRect(10, 100, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_7.setFont(font)
        self.label_mountIP_7.setObjectName("label_mountIP_7")
        self.alpacaPort = QtWidgets.QLineEdit(self.ALPACA)
        self.alpacaPort.setGeometry(QtCore.QRect(100, 70, 50, 21))
        self.alpacaPort.setMinimumSize(QtCore.QSize(0, 0))
        self.alpacaPort.setMaximumSize(QtCore.QSize(50, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.alpacaPort.setFont(font)
        self.alpacaPort.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.alpacaPort.setObjectName("alpacaPort")
        self.label_mountIP_8 = QtWidgets.QLabel(self.ALPACA)
        self.label_mountIP_8.setGeometry(QtCore.QRect(10, 40, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_8.setFont(font)
        self.label_mountIP_8.setObjectName("label_mountIP_8")
        self.label_mountIP_10 = QtWidgets.QLabel(self.ALPACA)
        self.label_mountIP_10.setGeometry(QtCore.QRect(10, 70, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_10.setFont(font)
        self.label_mountIP_10.setObjectName("label_mountIP_10")
        self.alpacaHostAddress = QtWidgets.QLineEdit(self.ALPACA)
        self.alpacaHostAddress.setGeometry(QtCore.QRect(100, 40, 201, 21))
        self.alpacaHostAddress.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.alpacaHostAddress.setFont(font)
        self.alpacaHostAddress.setMouseTracking(False)
        self.alpacaHostAddress.setAcceptDrops(False)
        self.alpacaHostAddress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.alpacaHostAddress.setMaxLength(50)
        self.alpacaHostAddress.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.alpacaHostAddress.setDragEnabled(True)
        self.alpacaHostAddress.setReadOnly(False)
        self.alpacaHostAddress.setObjectName("alpacaHostAddress")
        self.label_mountIP_11 = QtWidgets.QLabel(self.ALPACA)
        self.label_mountIP_11.setGeometry(QtCore.QRect(10, 130, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_11.setFont(font)
        self.label_mountIP_11.setObjectName("label_mountIP_11")
        self.label_mountIP_12 = QtWidgets.QLabel(self.ALPACA)
        self.label_mountIP_12.setGeometry(QtCore.QRect(10, 160, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_12.setFont(font)
        self.label_mountIP_12.setObjectName("label_mountIP_12")
        self.alpacaUser = QtWidgets.QLineEdit(self.ALPACA)
        self.alpacaUser.setGeometry(QtCore.QRect(100, 130, 201, 21))
        self.alpacaUser.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.alpacaUser.setFont(font)
        self.alpacaUser.setMouseTracking(False)
        self.alpacaUser.setAcceptDrops(False)
        self.alpacaUser.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.alpacaUser.setText("")
        self.alpacaUser.setMaxLength(50)
        self.alpacaUser.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.alpacaUser.setDragEnabled(True)
        self.alpacaUser.setReadOnly(False)
        self.alpacaUser.setObjectName("alpacaUser")
        self.alpacaPassword = QtWidgets.QLineEdit(self.ALPACA)
        self.alpacaPassword.setGeometry(QtCore.QRect(100, 160, 201, 21))
        self.alpacaPassword.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.alpacaPassword.setFont(font)
        self.alpacaPassword.setMouseTracking(False)
        self.alpacaPassword.setAcceptDrops(False)
        self.alpacaPassword.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.alpacaPassword.setText("")
        self.alpacaPassword.setMaxLength(50)
        self.alpacaPassword.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.alpacaPassword.setDragEnabled(True)
        self.alpacaPassword.setReadOnly(False)
        self.alpacaPassword.setObjectName("alpacaPassword")
        self.alpacaDeviceList = QtWidgets.QComboBox(self.ALPACA)
        self.alpacaDeviceList.setGeometry(QtCore.QRect(100, 100, 201, 21))
        self.alpacaDeviceList.setObjectName("alpacaDeviceList")
        self.alpacaCopyConfig = QtWidgets.QCheckBox(self.ALPACA)
        self.alpacaCopyConfig.setEnabled(True)
        self.alpacaCopyConfig.setGeometry(QtCore.QRect(100, 190, 201, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.alpacaCopyConfig.setFont(font)
        self.alpacaCopyConfig.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.alpacaCopyConfig.setChecked(False)
        self.alpacaCopyConfig.setTristate(False)
        self.alpacaCopyConfig.setObjectName("alpacaCopyConfig")
        self.alpacaDiscover = QtWidgets.QPushButton(self.ALPACA)
        self.alpacaDiscover.setGeometry(QtCore.QRect(305, 100, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.alpacaDiscover.setFont(font)
        self.alpacaDiscover.setObjectName("alpacaDiscover")
        self.tab.addTab(self.ALPACA, "")
        self.ASCOM = QtWidgets.QWidget()
        self.ASCOM.setObjectName("ASCOM")
        self.ascomSelector = QtWidgets.QPushButton(self.ASCOM)
        self.ascomSelector.setGeometry(QtCore.QRect(100, 190, 201, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.ascomSelector.setFont(font)
        self.ascomSelector.setObjectName("ascomSelector")
        self.label_mountIP_15 = QtWidgets.QLabel(self.ASCOM)
        self.label_mountIP_15.setGeometry(QtCore.QRect(10, 100, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_15.setFont(font)
        self.label_mountIP_15.setObjectName("label_mountIP_15")
        self.ascomDevice = QtWidgets.QLineEdit(self.ASCOM)
        self.ascomDevice.setGeometry(QtCore.QRect(100, 100, 196, 21))
        self.ascomDevice.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.ascomDevice.setFont(font)
        self.ascomDevice.setMouseTracking(False)
        self.ascomDevice.setAcceptDrops(False)
        self.ascomDevice.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.ascomDevice.setText("")
        self.ascomDevice.setMaxLength(50)
        self.ascomDevice.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.ascomDevice.setDragEnabled(True)
        self.ascomDevice.setReadOnly(False)
        self.ascomDevice.setObjectName("ascomDevice")
        self.tab.addTab(self.ASCOM, "")
        self.INDI = QtWidgets.QWidget()
        self.INDI.setObjectName("INDI")
        self.label_mountIP_3 = QtWidgets.QLabel(self.INDI)
        self.label_mountIP_3.setGeometry(QtCore.QRect(10, 40, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_3.setFont(font)
        self.label_mountIP_3.setObjectName("label_mountIP_3")
        self.indiHostAddress = QtWidgets.QLineEdit(self.INDI)
        self.indiHostAddress.setGeometry(QtCore.QRect(100, 40, 201, 21))
        self.indiHostAddress.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.indiHostAddress.setFont(font)
        self.indiHostAddress.setMouseTracking(False)
        self.indiHostAddress.setAcceptDrops(False)
        self.indiHostAddress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.indiHostAddress.setMaxLength(50)
        self.indiHostAddress.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.indiHostAddress.setDragEnabled(True)
        self.indiHostAddress.setReadOnly(False)
        self.indiHostAddress.setObjectName("indiHostAddress")
        self.label_mountIP_4 = QtWidgets.QLabel(self.INDI)
        self.label_mountIP_4.setGeometry(QtCore.QRect(10, 70, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_4.setFont(font)
        self.label_mountIP_4.setObjectName("label_mountIP_4")
        self.indiDiscover = QtWidgets.QPushButton(self.INDI)
        self.indiDiscover.setGeometry(QtCore.QRect(305, 100, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.indiDiscover.setFont(font)
        self.indiDiscover.setObjectName("indiDiscover")
        self.indiDeviceList = QtWidgets.QComboBox(self.INDI)
        self.indiDeviceList.setGeometry(QtCore.QRect(100, 100, 201, 21))
        self.indiDeviceList.setObjectName("indiDeviceList")
        self.label_mountIP_5 = QtWidgets.QLabel(self.INDI)
        self.label_mountIP_5.setGeometry(QtCore.QRect(10, 100, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_5.setFont(font)
        self.label_mountIP_5.setObjectName("label_mountIP_5")
        self.indiMessages = QtWidgets.QCheckBox(self.INDI)
        self.indiMessages.setGeometry(QtCore.QRect(100, 135, 201, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.indiMessages.setFont(font)
        self.indiMessages.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.indiMessages.setChecked(False)
        self.indiMessages.setObjectName("indiMessages")
        self.indiPort = QtWidgets.QLineEdit(self.INDI)
        self.indiPort.setGeometry(QtCore.QRect(100, 70, 50, 21))
        self.indiPort.setMinimumSize(QtCore.QSize(0, 0))
        self.indiPort.setMaximumSize(QtCore.QSize(50, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.indiPort.setFont(font)
        self.indiPort.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.indiPort.setObjectName("indiPort")
        self.indiLoadConfig = QtWidgets.QCheckBox(self.INDI)
        self.indiLoadConfig.setEnabled(True)
        self.indiLoadConfig.setGeometry(QtCore.QRect(100, 160, 201, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.indiLoadConfig.setFont(font)
        self.indiLoadConfig.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.indiLoadConfig.setChecked(False)
        self.indiLoadConfig.setTristate(False)
        self.indiLoadConfig.setObjectName("indiLoadConfig")
        self.indiCopyConfig = QtWidgets.QCheckBox(self.INDI)
        self.indiCopyConfig.setEnabled(True)
        self.indiCopyConfig.setGeometry(QtCore.QRect(100, 185, 201, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.indiCopyConfig.setFont(font)
        self.indiCopyConfig.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.indiCopyConfig.setChecked(False)
        self.indiCopyConfig.setTristate(False)
        self.indiCopyConfig.setObjectName("indiCopyConfig")
        self.tab.addTab(self.INDI, "")
        self.ASTROMETRY = QtWidgets.QWidget()
        self.ASTROMETRY.setObjectName("ASTROMETRY")
        self.label_mountIP_14 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_mountIP_14.setGeometry(QtCore.QRect(10, 65, 146, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_14.setFont(font)
        self.label_mountIP_14.setObjectName("label_mountIP_14")
        self.astrometryIndexPath = QtWidgets.QLineEdit(self.ASTROMETRY)
        self.astrometryIndexPath.setEnabled(True)
        self.astrometryIndexPath.setGeometry(QtCore.QRect(10, 90, 346, 21))
        self.astrometryIndexPath.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astrometryIndexPath.setFont(font)
        self.astrometryIndexPath.setMouseTracking(False)
        self.astrometryIndexPath.setAcceptDrops(False)
        self.astrometryIndexPath.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.astrometryIndexPath.setMaxLength(100)
        self.astrometryIndexPath.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.astrometryIndexPath.setDragEnabled(True)
        self.astrometryIndexPath.setReadOnly(False)
        self.astrometryIndexPath.setObjectName("astrometryIndexPath")
        self.selectAstrometryIndexPath = QtWidgets.QPushButton(self.ASTROMETRY)
        self.selectAstrometryIndexPath.setGeometry(QtCore.QRect(310, 65, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.selectAstrometryIndexPath.setFont(font)
        self.selectAstrometryIndexPath.setObjectName("selectAstrometryIndexPath")
        self.astrometryDeviceList = QtWidgets.QComboBox(self.ASTROMETRY)
        self.astrometryDeviceList.setGeometry(QtCore.QRect(100, 120, 201, 21))
        self.astrometryDeviceList.setObjectName("astrometryDeviceList")
        self.label_mountIP_6 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_mountIP_6.setGeometry(QtCore.QRect(10, 120, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_6.setFont(font)
        self.label_mountIP_6.setObjectName("label_mountIP_6")
        self.label_316 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_316.setGeometry(QtCore.QRect(10, 180, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_316.setFont(font)
        self.label_316.setObjectName("label_316")
        self.astrometrySearchRadius = QtWidgets.QDoubleSpinBox(self.ASTROMETRY)
        self.astrometrySearchRadius.setGeometry(QtCore.QRect(100, 150, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astrometrySearchRadius.setFont(font)
        self.astrometrySearchRadius.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.astrometrySearchRadius.setDecimals(0)
        self.astrometrySearchRadius.setMinimum(1.0)
        self.astrometrySearchRadius.setMaximum(180.0)
        self.astrometrySearchRadius.setSingleStep(5.0)
        self.astrometrySearchRadius.setProperty("value", 25.0)
        self.astrometrySearchRadius.setObjectName("astrometrySearchRadius")
        self.astrometryTimeout = QtWidgets.QDoubleSpinBox(self.ASTROMETRY)
        self.astrometryTimeout.setGeometry(QtCore.QRect(100, 180, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astrometryTimeout.setFont(font)
        self.astrometryTimeout.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.astrometryTimeout.setDecimals(0)
        self.astrometryTimeout.setMinimum(2.0)
        self.astrometryTimeout.setMaximum(300.0)
        self.astrometryTimeout.setSingleStep(5.0)
        self.astrometryTimeout.setProperty("value", 30.0)
        self.astrometryTimeout.setObjectName("astrometryTimeout")
        self.label_315 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_315.setGeometry(QtCore.QRect(10, 150, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_315.setFont(font)
        self.label_315.setObjectName("label_315")
        self.label_317 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_317.setGeometry(QtCore.QRect(160, 180, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_317.setFont(font)
        self.label_317.setObjectName("label_317")
        self.label_318 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_318.setGeometry(QtCore.QRect(160, 150, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_318.setFont(font)
        self.label_318.setObjectName("label_318")
        self.label_mountIP_17 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_mountIP_17.setGeometry(QtCore.QRect(10, 10, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_17.setFont(font)
        self.label_mountIP_17.setObjectName("label_mountIP_17")
        self.astrometryAppPath = QtWidgets.QLineEdit(self.ASTROMETRY)
        self.astrometryAppPath.setEnabled(True)
        self.astrometryAppPath.setGeometry(QtCore.QRect(10, 35, 346, 21))
        self.astrometryAppPath.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astrometryAppPath.setFont(font)
        self.astrometryAppPath.setMouseTracking(False)
        self.astrometryAppPath.setAcceptDrops(False)
        self.astrometryAppPath.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.astrometryAppPath.setMaxLength(100)
        self.astrometryAppPath.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.astrometryAppPath.setDragEnabled(True)
        self.astrometryAppPath.setReadOnly(False)
        self.astrometryAppPath.setObjectName("astrometryAppPath")
        self.selectAstrometryAppPath = QtWidgets.QPushButton(self.ASTROMETRY)
        self.selectAstrometryAppPath.setEnabled(True)
        self.selectAstrometryAppPath.setGeometry(QtCore.QRect(310, 10, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.selectAstrometryAppPath.setFont(font)
        self.selectAstrometryAppPath.setObjectName("selectAstrometryAppPath")
        self.tab.addTab(self.ASTROMETRY, "")
        self.ASTAP = QtWidgets.QWidget()
        self.ASTAP.setObjectName("ASTAP")
        self.label_mountIP_16 = QtWidgets.QLabel(self.ASTAP)
        self.label_mountIP_16.setGeometry(QtCore.QRect(10, 120, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_16.setFont(font)
        self.label_mountIP_16.setObjectName("label_mountIP_16")
        self.astapTimeout = QtWidgets.QDoubleSpinBox(self.ASTAP)
        self.astapTimeout.setGeometry(QtCore.QRect(100, 180, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astapTimeout.setFont(font)
        self.astapTimeout.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.astapTimeout.setDecimals(0)
        self.astapTimeout.setMinimum(2.0)
        self.astapTimeout.setMaximum(300.0)
        self.astapTimeout.setSingleStep(5.0)
        self.astapTimeout.setProperty("value", 30.0)
        self.astapTimeout.setObjectName("astapTimeout")
        self.astapSearchRadius = QtWidgets.QDoubleSpinBox(self.ASTAP)
        self.astapSearchRadius.setGeometry(QtCore.QRect(100, 150, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astapSearchRadius.setFont(font)
        self.astapSearchRadius.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.astapSearchRadius.setDecimals(0)
        self.astapSearchRadius.setMinimum(1.0)
        self.astapSearchRadius.setMaximum(180.0)
        self.astapSearchRadius.setSingleStep(5.0)
        self.astapSearchRadius.setProperty("value", 25.0)
        self.astapSearchRadius.setObjectName("astapSearchRadius")
        self.label_319 = QtWidgets.QLabel(self.ASTAP)
        self.label_319.setGeometry(QtCore.QRect(10, 180, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_319.setFont(font)
        self.label_319.setObjectName("label_319")
        self.label_320 = QtWidgets.QLabel(self.ASTAP)
        self.label_320.setGeometry(QtCore.QRect(10, 150, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_320.setFont(font)
        self.label_320.setObjectName("label_320")
        self.label_321 = QtWidgets.QLabel(self.ASTAP)
        self.label_321.setGeometry(QtCore.QRect(160, 180, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_321.setFont(font)
        self.label_321.setObjectName("label_321")
        self.label_322 = QtWidgets.QLabel(self.ASTAP)
        self.label_322.setGeometry(QtCore.QRect(160, 150, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_322.setFont(font)
        self.label_322.setObjectName("label_322")
        self.astapIndexPath = QtWidgets.QLineEdit(self.ASTAP)
        self.astapIndexPath.setEnabled(True)
        self.astapIndexPath.setGeometry(QtCore.QRect(10, 90, 346, 21))
        self.astapIndexPath.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astapIndexPath.setFont(font)
        self.astapIndexPath.setMouseTracking(False)
        self.astapIndexPath.setAcceptDrops(False)
        self.astapIndexPath.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.astapIndexPath.setMaxLength(100)
        self.astapIndexPath.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.astapIndexPath.setDragEnabled(True)
        self.astapIndexPath.setReadOnly(False)
        self.astapIndexPath.setObjectName("astapIndexPath")
        self.astapAppPath = QtWidgets.QLineEdit(self.ASTAP)
        self.astapAppPath.setEnabled(True)
        self.astapAppPath.setGeometry(QtCore.QRect(10, 35, 346, 21))
        self.astapAppPath.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astapAppPath.setFont(font)
        self.astapAppPath.setMouseTracking(False)
        self.astapAppPath.setAcceptDrops(False)
        self.astapAppPath.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.astapAppPath.setMaxLength(100)
        self.astapAppPath.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.astapAppPath.setDragEnabled(True)
        self.astapAppPath.setReadOnly(False)
        self.astapAppPath.setObjectName("astapAppPath")
        self.selectAstapIndexPath = QtWidgets.QPushButton(self.ASTAP)
        self.selectAstapIndexPath.setEnabled(True)
        self.selectAstapIndexPath.setGeometry(QtCore.QRect(310, 65, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.selectAstapIndexPath.setFont(font)
        self.selectAstapIndexPath.setObjectName("selectAstapIndexPath")
        self.selectAstapAppPath = QtWidgets.QPushButton(self.ASTAP)
        self.selectAstapAppPath.setEnabled(True)
        self.selectAstapAppPath.setGeometry(QtCore.QRect(310, 10, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.selectAstapAppPath.setFont(font)
        self.selectAstapAppPath.setObjectName("selectAstapAppPath")
        self.astapDeviceList = QtWidgets.QComboBox(self.ASTAP)
        self.astapDeviceList.setGeometry(QtCore.QRect(100, 120, 201, 21))
        self.astapDeviceList.setObjectName("astapDeviceList")
        self.label_mountIP_18 = QtWidgets.QLabel(self.ASTAP)
        self.label_mountIP_18.setGeometry(QtCore.QRect(10, 65, 146, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_18.setFont(font)
        self.label_mountIP_18.setObjectName("label_mountIP_18")
        self.label_mountIP_19 = QtWidgets.QLabel(self.ASTAP)
        self.label_mountIP_19.setGeometry(QtCore.QRect(10, 10, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_19.setFont(font)
        self.label_mountIP_19.setObjectName("label_mountIP_19")
        self.tab.addTab(self.ASTAP, "")
        self.weather = QtWidgets.QWidget()
        self.weather.setObjectName("weather")
        self.label_mountIP_33 = QtWidgets.QLabel(self.weather)
        self.label_mountIP_33.setGeometry(QtCore.QRect(10, 40, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_33.setFont(font)
        self.label_mountIP_33.setObjectName("label_mountIP_33")
        self.onlineWeatherHostAddress = QtWidgets.QLineEdit(self.weather)
        self.onlineWeatherHostAddress.setEnabled(True)
        self.onlineWeatherHostAddress.setGeometry(QtCore.QRect(100, 40, 241, 21))
        self.onlineWeatherHostAddress.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherHostAddress.setFont(font)
        self.onlineWeatherHostAddress.setMouseTracking(False)
        self.onlineWeatherHostAddress.setAcceptDrops(False)
        self.onlineWeatherHostAddress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherHostAddress.setMaxLength(50)
        self.onlineWeatherHostAddress.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.onlineWeatherHostAddress.setDragEnabled(True)
        self.onlineWeatherHostAddress.setReadOnly(False)
        self.onlineWeatherHostAddress.setObjectName("onlineWeatherHostAddress")
        self.onlineWeatherApiKey = QtWidgets.QLineEdit(self.weather)
        self.onlineWeatherApiKey.setGeometry(QtCore.QRect(100, 70, 241, 21))
        self.onlineWeatherApiKey.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherApiKey.setFont(font)
        self.onlineWeatherApiKey.setMouseTracking(False)
        self.onlineWeatherApiKey.setAcceptDrops(False)
        self.onlineWeatherApiKey.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherApiKey.setText("")
        self.onlineWeatherApiKey.setMaxLength(50)
        self.onlineWeatherApiKey.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.onlineWeatherApiKey.setDragEnabled(True)
        self.onlineWeatherApiKey.setReadOnly(False)
        self.onlineWeatherApiKey.setObjectName("onlineWeatherApiKey")
        self.label_mountIP_34 = QtWidgets.QLabel(self.weather)
        self.label_mountIP_34.setGeometry(QtCore.QRect(10, 70, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_34.setFont(font)
        self.label_mountIP_34.setObjectName("label_mountIP_34")
        self.tab.addTab(self.weather, "")
        self.relay = QtWidgets.QWidget()
        self.relay.setObjectName("relay")
        self.label_mountIP_35 = QtWidgets.QLabel(self.relay)
        self.label_mountIP_35.setGeometry(QtCore.QRect(10, 40, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_35.setFont(font)
        self.label_mountIP_35.setObjectName("label_mountIP_35")
        self.relayHostAddress = QtWidgets.QLineEdit(self.relay)
        self.relayHostAddress.setEnabled(True)
        self.relayHostAddress.setGeometry(QtCore.QRect(100, 40, 241, 21))
        self.relayHostAddress.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayHostAddress.setFont(font)
        self.relayHostAddress.setMouseTracking(False)
        self.relayHostAddress.setAcceptDrops(False)
        self.relayHostAddress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayHostAddress.setText("")
        self.relayHostAddress.setMaxLength(50)
        self.relayHostAddress.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayHostAddress.setDragEnabled(True)
        self.relayHostAddress.setReadOnly(False)
        self.relayHostAddress.setObjectName("relayHostAddress")
        self.relayPassword = QtWidgets.QLineEdit(self.relay)
        self.relayPassword.setGeometry(QtCore.QRect(100, 100, 241, 21))
        self.relayPassword.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayPassword.setFont(font)
        self.relayPassword.setMouseTracking(False)
        self.relayPassword.setAcceptDrops(False)
        self.relayPassword.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayPassword.setText("")
        self.relayPassword.setMaxLength(50)
        self.relayPassword.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayPassword.setDragEnabled(True)
        self.relayPassword.setReadOnly(False)
        self.relayPassword.setObjectName("relayPassword")
        self.relayUser = QtWidgets.QLineEdit(self.relay)
        self.relayUser.setGeometry(QtCore.QRect(100, 70, 241, 21))
        self.relayUser.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayUser.setFont(font)
        self.relayUser.setMouseTracking(False)
        self.relayUser.setAcceptDrops(False)
        self.relayUser.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayUser.setText("")
        self.relayUser.setMaxLength(50)
        self.relayUser.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayUser.setDragEnabled(True)
        self.relayUser.setReadOnly(False)
        self.relayUser.setObjectName("relayUser")
        self.label_mountIP_36 = QtWidgets.QLabel(self.relay)
        self.label_mountIP_36.setGeometry(QtCore.QRect(10, 70, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_36.setFont(font)
        self.label_mountIP_36.setObjectName("label_mountIP_36")
        self.label_mountIP_37 = QtWidgets.QLabel(self.relay)
        self.label_mountIP_37.setGeometry(QtCore.QRect(10, 100, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_37.setFont(font)
        self.label_mountIP_37.setObjectName("label_mountIP_37")
        self.tab.addTab(self.relay, "")
        self.ok = QtWidgets.QPushButton(DevicePopup)
        self.ok.setGeometry(QtCore.QRect(370, 270, 75, 21))
        self.ok.setObjectName("ok")
        self.cancel = QtWidgets.QPushButton(DevicePopup)
        self.cancel.setGeometry(QtCore.QRect(270, 270, 75, 21))
        self.cancel.setObjectName("cancel")
        self.iconPixmap = QtWidgets.QLabel(DevicePopup)
        self.iconPixmap.setGeometry(QtCore.QRect(10, 110, 64, 64))
        self.iconPixmap.setObjectName("iconPixmap")

        self.retranslateUi(DevicePopup)
        self.tab.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(DevicePopup)
        DevicePopup.setTabOrder(self.alpacaHostAddress, self.alpacaPort)
        DevicePopup.setTabOrder(self.alpacaPort, self.alpacaDeviceList)
        DevicePopup.setTabOrder(self.alpacaDeviceList, self.alpacaDiscover)
        DevicePopup.setTabOrder(self.alpacaDiscover, self.alpacaUser)
        DevicePopup.setTabOrder(self.alpacaUser, self.alpacaPassword)
        DevicePopup.setTabOrder(self.alpacaPassword, self.alpacaCopyConfig)
        DevicePopup.setTabOrder(self.alpacaCopyConfig, self.ascomDevice)
        DevicePopup.setTabOrder(self.ascomDevice, self.ascomSelector)
        DevicePopup.setTabOrder(self.ascomSelector, self.indiHostAddress)
        DevicePopup.setTabOrder(self.indiHostAddress, self.indiPort)
        DevicePopup.setTabOrder(self.indiPort, self.indiDeviceList)
        DevicePopup.setTabOrder(self.indiDeviceList, self.indiDiscover)
        DevicePopup.setTabOrder(self.indiDiscover, self.indiMessages)
        DevicePopup.setTabOrder(self.indiMessages, self.indiLoadConfig)
        DevicePopup.setTabOrder(self.indiLoadConfig, self.indiCopyConfig)
        DevicePopup.setTabOrder(self.indiCopyConfig, self.selectAstrometryAppPath)
        DevicePopup.setTabOrder(self.selectAstrometryAppPath, self.astrometryAppPath)
        DevicePopup.setTabOrder(self.astrometryAppPath, self.selectAstrometryIndexPath)
        DevicePopup.setTabOrder(self.selectAstrometryIndexPath, self.astrometryIndexPath)
        DevicePopup.setTabOrder(self.astrometryIndexPath, self.astrometryDeviceList)
        DevicePopup.setTabOrder(self.astrometryDeviceList, self.astrometrySearchRadius)
        DevicePopup.setTabOrder(self.astrometrySearchRadius, self.astrometryTimeout)
        DevicePopup.setTabOrder(self.astrometryTimeout, self.selectAstapAppPath)
        DevicePopup.setTabOrder(self.selectAstapAppPath, self.astapAppPath)
        DevicePopup.setTabOrder(self.astapAppPath, self.selectAstapIndexPath)
        DevicePopup.setTabOrder(self.selectAstapIndexPath, self.astapIndexPath)
        DevicePopup.setTabOrder(self.astapIndexPath, self.astapDeviceList)
        DevicePopup.setTabOrder(self.astapDeviceList, self.astapSearchRadius)
        DevicePopup.setTabOrder(self.astapSearchRadius, self.astapTimeout)
        DevicePopup.setTabOrder(self.astapTimeout, self.onlineWeatherHostAddress)
        DevicePopup.setTabOrder(self.onlineWeatherHostAddress, self.onlineWeatherApiKey)
        DevicePopup.setTabOrder(self.onlineWeatherApiKey, self.relayHostAddress)
        DevicePopup.setTabOrder(self.relayHostAddress, self.relayUser)
        DevicePopup.setTabOrder(self.relayUser, self.relayPassword)
        DevicePopup.setTabOrder(self.relayPassword, self.cancel)
        DevicePopup.setTabOrder(self.cancel, self.ok)
        DevicePopup.setTabOrder(self.ok, self.tab)

    def retranslateUi(self, DevicePopup):
        _translate = QtCore.QCoreApplication.translate
        DevicePopup.setWindowTitle(_translate("DevicePopup", "Device Setup"))
        self.label_mountIP_7.setText(_translate("DevicePopup", "Device Name"))
        self.alpacaPort.setToolTip(_translate("DevicePopup", "Port of ALPACA server"))
        self.alpacaPort.setText(_translate("DevicePopup", "11111"))
        self.label_mountIP_8.setText(_translate("DevicePopup", "Host Address"))
        self.label_mountIP_10.setText(_translate("DevicePopup", "Host Port"))
        self.alpacaHostAddress.setToolTip(_translate("DevicePopup", "<html><head/><body><p>IP address of INDI server, ip address and host names!</p></body></html>"))
        self.alpacaHostAddress.setText(_translate("DevicePopup", "localhost"))
        self.label_mountIP_11.setText(_translate("DevicePopup", "User"))
        self.label_mountIP_12.setText(_translate("DevicePopup", "Password"))
        self.alpacaUser.setToolTip(_translate("DevicePopup", "User for alpaca device"))
        self.alpacaPassword.setToolTip(_translate("DevicePopup", "Passwor for alpaca device"))
        self.alpacaCopyConfig.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Checked if you would like to enable blind solver function in SGPro</p></body></html>"))
        self.alpacaCopyConfig.setText(_translate("DevicePopup", "Copy config to all devices"))
        self.alpacaDiscover.setToolTip(_translate("DevicePopup", "Search does list name of INDI devices of this certain type"))
        self.alpacaDiscover.setText(_translate("DevicePopup", "Search"))
        self.tab.setTabText(self.tab.indexOf(self.ALPACA), _translate("DevicePopup", "ALPACA"))
        self.ascomSelector.setToolTip(_translate("DevicePopup", "Starting the ASCOM device chooser."))
        self.ascomSelector.setText(_translate("DevicePopup", "ASCOM device setup"))
        self.label_mountIP_15.setText(_translate("DevicePopup", "Device Name"))
        self.ascomDevice.setToolTip(_translate("DevicePopup", "Drvice name of ASCOM driver."))
        self.tab.setTabText(self.tab.indexOf(self.ASCOM), _translate("DevicePopup", "ASCOM"))
        self.label_mountIP_3.setText(_translate("DevicePopup", "Host Address"))
        self.indiHostAddress.setToolTip(_translate("DevicePopup", "IP address of INDI server, ip address and host names!"))
        self.indiHostAddress.setText(_translate("DevicePopup", "localhost"))
        self.label_mountIP_4.setText(_translate("DevicePopup", "Host Port"))
        self.indiDiscover.setToolTip(_translate("DevicePopup", "Search does list name of INDI devices of this certain type"))
        self.indiDiscover.setText(_translate("DevicePopup", "Search"))
        self.label_mountIP_5.setText(_translate("DevicePopup", "Device Name"))
        self.indiMessages.setToolTip(_translate("DevicePopup", "When checked, the indi messages received will be shown inthe central message window"))
        self.indiMessages.setText(_translate("DevicePopup", "Show Messages"))
        self.indiPort.setToolTip(_translate("DevicePopup", "Port of INDI server"))
        self.indiPort.setText(_translate("DevicePopup", "7624"))
        self.indiLoadConfig.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Checked if you would like to enable blind solver function in SGPro</p></body></html>"))
        self.indiLoadConfig.setText(_translate("DevicePopup", "Load configuration on start"))
        self.indiCopyConfig.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Checked if you would like to enable blind solver function in SGPro</p></body></html>"))
        self.indiCopyConfig.setText(_translate("DevicePopup", "Copy config to all devices"))
        self.tab.setTabText(self.tab.indexOf(self.INDI), _translate("DevicePopup", "INDI / INDIGO"))
        self.label_mountIP_14.setText(_translate("DevicePopup", "Folder astrometry index"))
        self.astrometryIndexPath.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Path to astrometry.net index files. Where native installation or e.g. KStars put the files.</p></body></html>"))
        self.astrometryIndexPath.setText(_translate("DevicePopup", "/usr/"))
        self.selectAstrometryIndexPath.setToolTip(_translate("DevicePopup", "Select the directory of the astrometry.net index files"))
        self.selectAstrometryIndexPath.setText(_translate("DevicePopup", "Select"))
        self.astrometryDeviceList.setToolTip(_translate("DevicePopup", "Select the device."))
        self.label_mountIP_6.setText(_translate("DevicePopup", "Device Name"))
        self.label_316.setText(_translate("DevicePopup", "Timeout"))
        self.astrometrySearchRadius.setToolTip(_translate("DevicePopup", "Set the search radius for the plate solver. "))
        self.astrometryTimeout.setToolTip(_translate("DevicePopup", "Set the timeout for the plate solver."))
        self.label_315.setText(_translate("DevicePopup", "Search radius"))
        self.label_317.setText(_translate("DevicePopup", "s"))
        self.label_318.setText(_translate("DevicePopup", "deg FOV"))
        self.label_mountIP_17.setText(_translate("DevicePopup", "Folder application"))
        self.astrometryAppPath.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Path to astrometry.net binary files. Where native installation or e.g. KStars put the files.</p></body></html>"))
        self.astrometryAppPath.setText(_translate("DevicePopup", "/usr/"))
        self.selectAstrometryAppPath.setToolTip(_translate("DevicePopup", "Select the directory of the astrometry.net index files"))
        self.selectAstrometryAppPath.setText(_translate("DevicePopup", "Select"))
        self.tab.setTabText(self.tab.indexOf(self.ASTROMETRY), _translate("DevicePopup", "ASTROMETRY"))
        self.label_mountIP_16.setText(_translate("DevicePopup", "Device Name"))
        self.astapTimeout.setToolTip(_translate("DevicePopup", "Set the timeout for the plate solver."))
        self.astapSearchRadius.setToolTip(_translate("DevicePopup", "Set the search radius for the plate solver. "))
        self.label_319.setText(_translate("DevicePopup", "Timeout"))
        self.label_320.setText(_translate("DevicePopup", "Search radius"))
        self.label_321.setText(_translate("DevicePopup", "s"))
        self.label_322.setText(_translate("DevicePopup", "deg FOV"))
        self.astapIndexPath.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Path to astap star database.</p></body></html>"))
        self.astapIndexPath.setText(_translate("DevicePopup", "/usr/"))
        self.astapAppPath.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Path to astap application.</p></body></html>"))
        self.astapAppPath.setText(_translate("DevicePopup", "/usr/"))
        self.selectAstapIndexPath.setToolTip(_translate("DevicePopup", "Select the directory of the astrometry.net index files"))
        self.selectAstapIndexPath.setText(_translate("DevicePopup", "Select"))
        self.selectAstapAppPath.setToolTip(_translate("DevicePopup", "Select the directory of the astrometry.net index files"))
        self.selectAstapAppPath.setText(_translate("DevicePopup", "Select"))
        self.astapDeviceList.setToolTip(_translate("DevicePopup", "Select the device."))
        self.label_mountIP_18.setText(_translate("DevicePopup", "Folder astrometry index"))
        self.label_mountIP_19.setText(_translate("DevicePopup", "Folder application"))
        self.tab.setTabText(self.tab.indexOf(self.ASTAP), _translate("DevicePopup", "ASTAP"))
        self.label_mountIP_33.setText(_translate("DevicePopup", "Host Address"))
        self.onlineWeatherHostAddress.setToolTip(_translate("DevicePopup", "<html><head/><body><p>IP address for open weather service</p></body></html>"))
        self.onlineWeatherHostAddress.setText(_translate("DevicePopup", "api.openweathermap.org"))
        self.onlineWeatherApiKey.setToolTip(_translate("DevicePopup", "<html><head/><body><p>api key for open weather service</p></body></html>"))
        self.label_mountIP_34.setText(_translate("DevicePopup", "API Key"))
        self.tab.setTabText(self.tab.indexOf(self.weather), _translate("DevicePopup", "Online Weather"))
        self.label_mountIP_35.setText(_translate("DevicePopup", "Host Address"))
        self.relayHostAddress.setToolTip(_translate("DevicePopup", "<html><head/><body><p>IP address for relay device</p></body></html>"))
        self.relayPassword.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Passwor for relay device</p></body></html>"))
        self.relayUser.setToolTip(_translate("DevicePopup", "<html><head/><body><p>User for relay device</p></body></html>"))
        self.label_mountIP_36.setText(_translate("DevicePopup", "User"))
        self.label_mountIP_37.setText(_translate("DevicePopup", "Password"))
        self.tab.setTabText(self.tab.indexOf(self.relay), _translate("DevicePopup", "Relay"))
        self.ok.setText(_translate("DevicePopup", "OK"))
        self.cancel.setText(_translate("DevicePopup", "Cancel"))
        self.iconPixmap.setText(_translate("DevicePopup", "TextLabel"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    DevicePopup = QtWidgets.QWidget()
    ui = Ui_DevicePopup()
    ui.setupUi(DevicePopup)
    DevicePopup.show()
    sys.exit(app.exec_())
