# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './mw4/gui/widgets/analyse.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AnalyseDialog(object):
    def setupUi(self, AnalyseDialog):
        AnalyseDialog.setObjectName("AnalyseDialog")
        AnalyseDialog.resize(1200, 800)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(AnalyseDialog.sizePolicy().hasHeightForWidth())
        AnalyseDialog.setSizePolicy(sizePolicy)
        AnalyseDialog.setMinimumSize(QtCore.QSize(1200, 800))
        AnalyseDialog.setMaximumSize(QtCore.QSize(2000, 1400))
        AnalyseDialog.setSizeIncrement(QtCore.QSize(10, 10))
        AnalyseDialog.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        AnalyseDialog.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(AnalyseDialog)
        self.verticalLayout.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame = QtWidgets.QFrame(AnalyseDialog)
        self.frame.setMinimumSize(QtCore.QSize(0, 90))
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.version = QtWidgets.QLineEdit(self.frame)
        self.version.setEnabled(False)
        self.version.setGeometry(QtCore.QRect(760, 65, 66, 25))
        self.version.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.version.setFont(font)
        self.version.setMouseTracking(False)
        self.version.setAcceptDrops(False)
        self.version.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.version.setText("")
        self.version.setReadOnly(True)
        self.version.setObjectName("version")
        self.filename = QtWidgets.QLineEdit(self.frame)
        self.filename.setEnabled(False)
        self.filename.setGeometry(QtCore.QRect(91, 5, 246, 25))
        self.filename.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.filename.setFont(font)
        self.filename.setMouseTracking(False)
        self.filename.setAcceptDrops(False)
        self.filename.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.filename.setText("")
        self.filename.setReadOnly(True)
        self.filename.setObjectName("filename")
        self.subframe = QtWidgets.QLineEdit(self.frame)
        self.subframe.setEnabled(False)
        self.subframe.setGeometry(QtCore.QRect(600, 35, 36, 25))
        self.subframe.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.subframe.setFont(font)
        self.subframe.setMouseTracking(False)
        self.subframe.setAcceptDrops(False)
        self.subframe.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.subframe.setText("")
        self.subframe.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.subframe.setReadOnly(True)
        self.subframe.setObjectName("subframe")
        self.label_20 = QtWidgets.QLabel(self.frame)
        self.label_20.setGeometry(QtCore.QRect(640, 65, 24, 25))
        self.label_20.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_20.setFont(font)
        self.label_20.setObjectName("label_20")
        self.label_21 = QtWidgets.QLabel(self.frame)
        self.label_21.setGeometry(QtCore.QRect(830, 65, 16, 25))
        self.label_21.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_21.setFont(font)
        self.label_21.setObjectName("label_21")
        self.label_13 = QtWidgets.QLabel(self.frame)
        self.label_13.setGeometry(QtCore.QRect(340, 65, 41, 25))
        self.label_13.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_13.setFont(font)
        self.label_13.setObjectName("label_13")
        self.modelOrthoError = QtWidgets.QLineEdit(self.frame)
        self.modelOrthoError.setEnabled(False)
        self.modelOrthoError.setGeometry(QtCore.QRect(270, 65, 66, 25))
        self.modelOrthoError.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.modelOrthoError.setFont(font)
        self.modelOrthoError.setMouseTracking(False)
        self.modelOrthoError.setAcceptDrops(False)
        self.modelOrthoError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.modelOrthoError.setText("")
        self.modelOrthoError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.modelOrthoError.setReadOnly(True)
        self.modelOrthoError.setObjectName("modelOrthoError")
        self.label_18 = QtWidgets.QLabel(self.frame)
        self.label_18.setGeometry(QtCore.QRect(405, 5, 42, 25))
        self.label_18.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_18.setFont(font)
        self.label_18.setObjectName("label_18")
        self.pointsWest = QtWidgets.QLabel(self.frame)
        self.pointsWest.setGeometry(QtCore.QRect(1045, 5, 91, 25))
        self.pointsWest.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.pointsWest.setFont(font)
        self.pointsWest.setObjectName("pointsWest")
        self.pointsEast = QtWidgets.QLabel(self.frame)
        self.pointsEast.setGeometry(QtCore.QRect(1045, 35, 91, 25))
        self.pointsEast.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.pointsEast.setFont(font)
        self.pointsEast.setObjectName("pointsEast")
        self.label_5 = QtWidgets.QLabel(self.frame)
        self.label_5.setGeometry(QtCore.QRect(540, 35, 56, 25))
        self.label_5.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.label_3 = QtWidgets.QLabel(self.frame)
        self.label_3.setGeometry(QtCore.QRect(405, 65, 49, 25))
        self.label_3.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.firmware = QtWidgets.QLineEdit(self.frame)
        self.firmware.setEnabled(False)
        self.firmware.setGeometry(QtCore.QRect(840, 65, 66, 25))
        self.firmware.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.firmware.setFont(font)
        self.firmware.setMouseTracking(False)
        self.firmware.setAcceptDrops(False)
        self.firmware.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.firmware.setText("")
        self.firmware.setReadOnly(True)
        self.firmware.setObjectName("firmware")
        self.label_10 = QtWidgets.QLabel(self.frame)
        self.label_10.setGeometry(QtCore.QRect(200, 65, 70, 25))
        self.label_10.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_10.setFont(font)
        self.label_10.setObjectName("label_10")
        self.time = QtWidgets.QLineEdit(self.frame)
        self.time.setEnabled(False)
        self.time.setGeometry(QtCore.QRect(760, 5, 146, 25))
        self.time.setMinimumSize(QtCore.QSize(25, 0))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.time.setFont(font)
        self.time.setMouseTracking(False)
        self.time.setAcceptDrops(False)
        self.time.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.time.setText("")
        self.time.setReadOnly(True)
        self.time.setObjectName("time")
        self.label_4 = QtWidgets.QLabel(self.frame)
        self.label_4.setGeometry(QtCore.QRect(685, 5, 70, 25))
        self.label_4.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.label_9 = QtWidgets.QLabel(self.frame)
        self.label_9.setGeometry(QtCore.QRect(200, 35, 69, 25))
        self.label_9.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.label_7 = QtWidgets.QLabel(self.frame)
        self.label_7.setGeometry(QtCore.QRect(910, 35, 54, 25))
        self.label_7.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.label_6 = QtWidgets.QLabel(self.frame)
        self.label_6.setGeometry(QtCore.QRect(685, 65, 66, 25))
        self.label_6.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.load = QtWidgets.QPushButton(self.frame)
        self.load.setGeometry(QtCore.QRect(5, 5, 80, 25))
        self.load.setMinimumSize(QtCore.QSize(80, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.load.setFont(font)
        self.load.setStyleSheet("")
        self.load.setObjectName("load")
        self.eposure = QtWidgets.QLineEdit(self.frame)
        self.eposure.setEnabled(False)
        self.eposure.setGeometry(QtCore.QRect(465, 35, 46, 25))
        self.eposure.setMinimumSize(QtCore.QSize(25, 0))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.eposure.setFont(font)
        self.eposure.setMouseTracking(False)
        self.eposure.setAcceptDrops(False)
        self.eposure.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.eposure.setText("")
        self.eposure.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.eposure.setReadOnly(True)
        self.eposure.setObjectName("eposure")
        self.label_19 = QtWidgets.QLabel(self.frame)
        self.label_19.setGeometry(QtCore.QRect(520, 65, 81, 25))
        self.label_19.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_19.setFont(font)
        self.label_19.setObjectName("label_19")
        self.label_2 = QtWidgets.QLabel(self.frame)
        self.label_2.setGeometry(QtCore.QRect(685, 35, 41, 25))
        self.label_2.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.label_16 = QtWidgets.QLabel(self.frame)
        self.label_16.setGeometry(QtCore.QRect(515, 35, 16, 25))
        self.label_16.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_16.setFont(font)
        self.label_16.setObjectName("label_16")
        self.winsorizedLimit = QtWidgets.QCheckBox(self.frame)
        self.winsorizedLimit.setGeometry(QtCore.QRect(1045, 65, 86, 25))
        self.winsorizedLimit.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.winsorizedLimit.setFont(font)
        self.winsorizedLimit.setObjectName("winsorizedLimit")
        self.modelTerms = QtWidgets.QLineEdit(self.frame)
        self.modelTerms.setEnabled(False)
        self.modelTerms.setGeometry(QtCore.QRect(90, 65, 46, 25))
        self.modelTerms.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.modelTerms.setFont(font)
        self.modelTerms.setMouseTracking(False)
        self.modelTerms.setAcceptDrops(False)
        self.modelTerms.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.modelTerms.setText("")
        self.modelTerms.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.modelTerms.setReadOnly(True)
        self.modelTerms.setObjectName("modelTerms")
        self.binning = QtWidgets.QLineEdit(self.frame)
        self.binning.setEnabled(False)
        self.binning.setGeometry(QtCore.QRect(465, 65, 46, 25))
        self.binning.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.binning.setFont(font)
        self.binning.setMouseTracking(False)
        self.binning.setAcceptDrops(False)
        self.binning.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.binning.setText("")
        self.binning.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.binning.setReadOnly(True)
        self.binning.setObjectName("binning")
        self.label_12 = QtWidgets.QLabel(self.frame)
        self.label_12.setGeometry(QtCore.QRect(340, 35, 41, 25))
        self.label_12.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_12.setFont(font)
        self.label_12.setObjectName("label_12")
        self.focalLength = QtWidgets.QLineEdit(self.frame)
        self.focalLength.setEnabled(False)
        self.focalLength.setGeometry(QtCore.QRect(600, 65, 36, 25))
        self.focalLength.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.focalLength.setFont(font)
        self.focalLength.setMouseTracking(False)
        self.focalLength.setAcceptDrops(False)
        self.focalLength.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.focalLength.setText("")
        self.focalLength.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.focalLength.setReadOnly(True)
        self.focalLength.setObjectName("focalLength")
        self.label_8 = QtWidgets.QLabel(self.frame)
        self.label_8.setGeometry(QtCore.QRect(5, 65, 41, 25))
        self.label_8.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.mirrored = QtWidgets.QLineEdit(self.frame)
        self.mirrored.setEnabled(False)
        self.mirrored.setGeometry(QtCore.QRect(965, 35, 61, 25))
        self.mirrored.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mirrored.setFont(font)
        self.mirrored.setMouseTracking(False)
        self.mirrored.setAcceptDrops(False)
        self.mirrored.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mirrored.setText("")
        self.mirrored.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mirrored.setReadOnly(True)
        self.mirrored.setObjectName("mirrored")
        self.label = QtWidgets.QLabel(self.frame)
        self.label.setGeometry(QtCore.QRect(405, 35, 59, 25))
        self.label.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.label_14 = QtWidgets.QLabel(self.frame)
        self.label_14.setGeometry(QtCore.QRect(140, 35, 41, 25))
        self.label_14.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_14.setFont(font)
        self.label_14.setObjectName("label_14")
        self.label_15 = QtWidgets.QLabel(self.frame)
        self.label_15.setGeometry(QtCore.QRect(640, 40, 16, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_15.setFont(font)
        self.label_15.setObjectName("label_15")
        self.modelPolarError = QtWidgets.QLineEdit(self.frame)
        self.modelPolarError.setEnabled(False)
        self.modelPolarError.setGeometry(QtCore.QRect(270, 35, 66, 25))
        self.modelPolarError.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.modelPolarError.setFont(font)
        self.modelPolarError.setMouseTracking(False)
        self.modelPolarError.setAcceptDrops(False)
        self.modelPolarError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.modelPolarError.setText("")
        self.modelPolarError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.modelPolarError.setReadOnly(True)
        self.modelPolarError.setObjectName("modelPolarError")
        self.label_11 = QtWidgets.QLabel(self.frame)
        self.label_11.setGeometry(QtCore.QRect(5, 35, 72, 25))
        self.label_11.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_11.setFont(font)
        self.label_11.setObjectName("label_11")
        self.solver = QtWidgets.QLineEdit(self.frame)
        self.solver.setEnabled(False)
        self.solver.setGeometry(QtCore.QRect(760, 35, 146, 25))
        self.solver.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.solver.setFont(font)
        self.solver.setMouseTracking(False)
        self.solver.setAcceptDrops(False)
        self.solver.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.solver.setText("")
        self.solver.setReadOnly(True)
        self.solver.setObjectName("solver")
        self.modelErrorRMS = QtWidgets.QLineEdit(self.frame)
        self.modelErrorRMS.setEnabled(False)
        self.modelErrorRMS.setGeometry(QtCore.QRect(90, 35, 46, 25))
        self.modelErrorRMS.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.modelErrorRMS.setFont(font)
        self.modelErrorRMS.setMouseTracking(False)
        self.modelErrorRMS.setAcceptDrops(False)
        self.modelErrorRMS.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.modelErrorRMS.setText("")
        self.modelErrorRMS.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.modelErrorRMS.setReadOnly(True)
        self.modelErrorRMS.setObjectName("modelErrorRMS")
        self.profile = QtWidgets.QLineEdit(self.frame)
        self.profile.setEnabled(False)
        self.profile.setGeometry(QtCore.QRect(465, 5, 171, 25))
        self.profile.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.profile.setFont(font)
        self.profile.setMouseTracking(False)
        self.profile.setAcceptDrops(False)
        self.profile.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.profile.setText("")
        self.profile.setReadOnly(True)
        self.profile.setObjectName("profile")
        self.label_17 = QtWidgets.QLabel(self.frame)
        self.label_17.setGeometry(QtCore.QRect(910, 5, 54, 25))
        self.label_17.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_17.setFont(font)
        self.label_17.setObjectName("label_17")
        self.goodPoints = QtWidgets.QLineEdit(self.frame)
        self.goodPoints.setEnabled(False)
        self.goodPoints.setGeometry(QtCore.QRect(965, 5, 26, 25))
        self.goodPoints.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.goodPoints.setFont(font)
        self.goodPoints.setMouseTracking(False)
        self.goodPoints.setAcceptDrops(False)
        self.goodPoints.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.goodPoints.setText("")
        self.goodPoints.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.goodPoints.setReadOnly(True)
        self.goodPoints.setObjectName("goodPoints")
        self.totalPoints = QtWidgets.QLineEdit(self.frame)
        self.totalPoints.setEnabled(False)
        self.totalPoints.setGeometry(QtCore.QRect(1000, 5, 26, 25))
        self.totalPoints.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.totalPoints.setFont(font)
        self.totalPoints.setMouseTracking(False)
        self.totalPoints.setAcceptDrops(False)
        self.totalPoints.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.totalPoints.setText("")
        self.totalPoints.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.totalPoints.setReadOnly(True)
        self.totalPoints.setObjectName("totalPoints")
        self.label_22 = QtWidgets.QLabel(self.frame)
        self.label_22.setGeometry(QtCore.QRect(995, 5, 16, 25))
        self.label_22.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_22.setFont(font)
        self.label_22.setObjectName("label_22")
        self.verticalLayout.addWidget(self.frame)
        self.line = QtWidgets.QFrame(AnalyseDialog)
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setLineWidth(2)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox_7 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_7.sizePolicy().hasHeightForWidth())
        self.groupBox_7.setSizePolicy(sizePolicy)
        self.groupBox_7.setProperty("large", True)
        self.groupBox_7.setObjectName("groupBox_7")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.groupBox_7)
        self.gridLayout_8.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_8.setSpacing(0)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.modelPositions = QtWidgets.QWidget(self.groupBox_7)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.modelPositions.sizePolicy().hasHeightForWidth())
        self.modelPositions.setSizePolicy(sizePolicy)
        self.modelPositions.setToolTip("")
        self.modelPositions.setObjectName("modelPositions")
        self.gridLayout_8.addWidget(self.modelPositions, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_7, 0, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setProperty("large", True)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_3.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_3.setSpacing(0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.scaleImage = QtWidgets.QWidget(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scaleImage.sizePolicy().hasHeightForWidth())
        self.scaleImage.setSizePolicy(sizePolicy)
        self.scaleImage.setToolTip("")
        self.scaleImage.setObjectName("scaleImage")
        self.gridLayout_3.addWidget(self.scaleImage, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_2, 0, 3, 1, 1)
        self.groupBox_8 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_8.setProperty("large", True)
        self.groupBox_8.setObjectName("groupBox_8")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.groupBox_8)
        self.gridLayout_9.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_9.setSpacing(0)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.errorAscending = QtWidgets.QWidget(self.groupBox_8)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.errorAscending.sizePolicy().hasHeightForWidth())
        self.errorAscending.setSizePolicy(sizePolicy)
        self.errorAscending.setToolTip("")
        self.errorAscending.setObjectName("errorAscending")
        self.gridLayout_9.addWidget(self.errorAscending, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_8, 0, 2, 1, 1)
        self.groupBox_5 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_5.sizePolicy().hasHeightForWidth())
        self.groupBox_5.setSizePolicy(sizePolicy)
        self.groupBox_5.setProperty("large", True)
        self.groupBox_5.setObjectName("groupBox_5")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_5)
        self.gridLayout_6.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_6.setSpacing(0)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.decPointErrorsRaw = QtWidgets.QWidget(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.decPointErrorsRaw.sizePolicy().hasHeightForWidth())
        self.decPointErrorsRaw.setSizePolicy(sizePolicy)
        self.decPointErrorsRaw.setToolTip("")
        self.decPointErrorsRaw.setObjectName("decPointErrorsRaw")
        self.gridLayout_6.addWidget(self.decPointErrorsRaw, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_5, 2, 0, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
        self.groupBox_3.setSizePolicy(sizePolicy)
        self.groupBox_3.setProperty("large", True)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_5.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_5.setSpacing(0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.raPointErrorsRaw = QtWidgets.QWidget(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.raPointErrorsRaw.sizePolicy().hasHeightForWidth())
        self.raPointErrorsRaw.setSizePolicy(sizePolicy)
        self.raPointErrorsRaw.setToolTip("")
        self.raPointErrorsRaw.setObjectName("raPointErrorsRaw")
        self.gridLayout_5.addWidget(self.raPointErrorsRaw, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_3, 1, 0, 1, 1)
        self.groupBox_9 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_9.setProperty("large", True)
        self.groupBox_9.setObjectName("groupBox_9")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.groupBox_9)
        self.gridLayout_10.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_10.setSpacing(0)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.errorDistribution = QtWidgets.QWidget(self.groupBox_9)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.errorDistribution.sizePolicy().hasHeightForWidth())
        self.errorDistribution.setSizePolicy(sizePolicy)
        self.errorDistribution.setToolTip("")
        self.errorDistribution.setObjectName("errorDistribution")
        self.gridLayout_10.addWidget(self.errorDistribution, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_9, 0, 1, 1, 1)
        self.groupBox_4 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_4.sizePolicy().hasHeightForWidth())
        self.groupBox_4.setSizePolicy(sizePolicy)
        self.groupBox_4.setProperty("large", True)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_4.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_4.setSpacing(0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.raModelErrors = QtWidgets.QWidget(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.raModelErrors.sizePolicy().hasHeightForWidth())
        self.raModelErrors.setSizePolicy(sizePolicy)
        self.raModelErrors.setToolTip("")
        self.raModelErrors.setObjectName("raModelErrors")
        self.gridLayout_4.addWidget(self.raModelErrors, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_4, 1, 1, 1, 1)
        self.groupBox_10 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_10.setProperty("large", True)
        self.groupBox_10.setObjectName("groupBox_10")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.groupBox_10)
        self.gridLayout_11.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_11.setSpacing(0)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.raModelErrorsRef = QtWidgets.QWidget(self.groupBox_10)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.raModelErrorsRef.sizePolicy().hasHeightForWidth())
        self.raModelErrorsRef.setSizePolicy(sizePolicy)
        self.raModelErrorsRef.setToolTip("")
        self.raModelErrorsRef.setObjectName("raModelErrorsRef")
        self.gridLayout_11.addWidget(self.raModelErrorsRef, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_10, 1, 2, 1, 1)
        self.groupBox_12 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_12.setProperty("large", True)
        self.groupBox_12.setObjectName("groupBox_12")
        self.gridLayout_13 = QtWidgets.QGridLayout(self.groupBox_12)
        self.gridLayout_13.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_13.setSpacing(0)
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.raPointErrorsRawRef = QtWidgets.QWidget(self.groupBox_12)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.raPointErrorsRawRef.sizePolicy().hasHeightForWidth())
        self.raPointErrorsRawRef.setSizePolicy(sizePolicy)
        self.raPointErrorsRawRef.setToolTip("")
        self.raPointErrorsRawRef.setObjectName("raPointErrorsRawRef")
        self.gridLayout_13.addWidget(self.raPointErrorsRawRef, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_12, 1, 3, 1, 1)
        self.groupBox_6 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_6.sizePolicy().hasHeightForWidth())
        self.groupBox_6.setSizePolicy(sizePolicy)
        self.groupBox_6.setProperty("large", True)
        self.groupBox_6.setObjectName("groupBox_6")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBox_6)
        self.gridLayout_7.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_7.setSpacing(0)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.decModelErrors = QtWidgets.QWidget(self.groupBox_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.decModelErrors.sizePolicy().hasHeightForWidth())
        self.decModelErrors.setSizePolicy(sizePolicy)
        self.decModelErrors.setToolTip("")
        self.decModelErrors.setObjectName("decModelErrors")
        self.gridLayout_7.addWidget(self.decModelErrors, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_6, 2, 1, 1, 1)
        self.groupBox_13 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_13.setProperty("large", True)
        self.groupBox_13.setObjectName("groupBox_13")
        self.gridLayout_14 = QtWidgets.QGridLayout(self.groupBox_13)
        self.gridLayout_14.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_14.setSpacing(0)
        self.gridLayout_14.setObjectName("gridLayout_14")
        self.decPointErrorsRawRef = QtWidgets.QWidget(self.groupBox_13)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.decPointErrorsRawRef.sizePolicy().hasHeightForWidth())
        self.decPointErrorsRawRef.setSizePolicy(sizePolicy)
        self.decPointErrorsRawRef.setToolTip("")
        self.decPointErrorsRawRef.setObjectName("decPointErrorsRawRef")
        self.gridLayout_14.addWidget(self.decPointErrorsRawRef, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_13, 2, 2, 1, 1)
        self.groupBox_11 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_11.setProperty("large", True)
        self.groupBox_11.setObjectName("groupBox_11")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.groupBox_11)
        self.gridLayout_12.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_12.setSpacing(0)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.decModelErrorsRef = QtWidgets.QWidget(self.groupBox_11)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.decModelErrorsRef.sizePolicy().hasHeightForWidth())
        self.decModelErrorsRef.setSizePolicy(sizePolicy)
        self.decModelErrorsRef.setToolTip("")
        self.decModelErrorsRef.setObjectName("decModelErrorsRef")
        self.gridLayout_12.addWidget(self.decModelErrorsRef, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_11, 2, 3, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.verticalLayout.setStretch(2, 1)

        self.retranslateUi(AnalyseDialog)
        QtCore.QMetaObject.connectSlotsByName(AnalyseDialog)

    def retranslateUi(self, AnalyseDialog):
        _translate = QtCore.QCoreApplication.translate
        AnalyseDialog.setWindowTitle(_translate("AnalyseDialog", "Model Build File Analysis"))
        self.version.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.filename.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.subframe.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_20.setText(_translate("AnalyseDialog", "mm"))
        self.label_21.setText(_translate("AnalyseDialog", "/"))
        self.label_13.setText(_translate("AnalyseDialog", "arcsec"))
        self.modelOrthoError.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_18.setText(_translate("AnalyseDialog", "Profile"))
        self.pointsWest.setText(_translate("AnalyseDialog", "Pierside West"))
        self.pointsWest.setProperty("color", _translate("AnalyseDialog", "green"))
        self.pointsEast.setText(_translate("AnalyseDialog", "Pierside East"))
        self.pointsEast.setProperty("color", _translate("AnalyseDialog", "yellow"))
        self.label_5.setText(_translate("AnalyseDialog", "Subframe"))
        self.label_3.setText(_translate("AnalyseDialog", "Binning"))
        self.firmware.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_10.setText(_translate("AnalyseDialog", "Ortho Error"))
        self.time.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_4.setText(_translate("AnalyseDialog", "Time [UTC]"))
        self.label_9.setText(_translate("AnalyseDialog", "Polar Error"))
        self.label_7.setText(_translate("AnalyseDialog", "Mirrored"))
        self.label_6.setText(_translate("AnalyseDialog", "Build / FW"))
        self.load.setToolTip(_translate("AnalyseDialog", "Loads the model points from file"))
        self.load.setText(_translate("AnalyseDialog", "Load"))
        self.eposure.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_19.setText(_translate("AnalyseDialog", "Focal Length"))
        self.label_2.setText(_translate("AnalyseDialog", "Solver"))
        self.label_16.setText(_translate("AnalyseDialog", "s"))
        self.winsorizedLimit.setText(_translate("AnalyseDialog", "Winzorized"))
        self.modelTerms.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.binning.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_12.setText(_translate("AnalyseDialog", "arcsec"))
        self.focalLength.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_8.setText(_translate("AnalyseDialog", "Terms"))
        self.mirrored.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label.setText(_translate("AnalyseDialog", "Exposure"))
        self.label_14.setText(_translate("AnalyseDialog", "arcsec"))
        self.label_15.setText(_translate("AnalyseDialog", "%"))
        self.modelPolarError.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_11.setText(_translate("AnalyseDialog", "Model RMS"))
        self.solver.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.modelErrorRMS.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.profile.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_17.setText(_translate("AnalyseDialog", "Points"))
        self.goodPoints.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.totalPoints.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_22.setText(_translate("AnalyseDialog", "/"))
        self.groupBox_7.setTitle(_translate("AnalyseDialog", "Model errors overview in hemisphere"))
        self.groupBox_2.setTitle(_translate("AnalyseDialog", "Image pixel resolution as scale "))
        self.groupBox_8.setTitle(_translate("AnalyseDialog", "Model point errors in ascending order"))
        self.groupBox_5.setTitle(_translate("AnalyseDialog", "Declination - Raw Error (PlateSolve)"))
        self.groupBox_3.setTitle(_translate("AnalyseDialog", "Right Ascension - Raw Error (PlateSolve)"))
        self.groupBox_9.setTitle(_translate("AnalyseDialog", "Model point errors over polar"))
        self.groupBox_4.setTitle(_translate("AnalyseDialog", "Right Ascension - Model Error (Mount)"))
        self.groupBox_10.setTitle(_translate("AnalyseDialog", "Right Ascension - Model Error over Encoder value"))
        self.groupBox_12.setTitle(_translate("AnalyseDialog", "Right Ascension - Raw Error over Encoder value"))
        self.groupBox_6.setTitle(_translate("AnalyseDialog", "Declination - Model Error (Mount)"))
        self.groupBox_13.setTitle(_translate("AnalyseDialog", "Declination - Raw Error over Encoder value"))
        self.groupBox_11.setTitle(_translate("AnalyseDialog", "Declination - Model Error over Encoder value"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    AnalyseDialog = QtWidgets.QWidget()
    ui = Ui_AnalyseDialog()
    ui.setupUi(AnalyseDialog)
    AnalyseDialog.show()
    sys.exit(app.exec_())
