# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './mw4/gui/widgets/satellite.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SatelliteDialog(object):
    def setupUi(self, SatelliteDialog):
        SatelliteDialog.setObjectName("SatelliteDialog")
        SatelliteDialog.resize(800, 600)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(SatelliteDialog.sizePolicy().hasHeightForWidth())
        SatelliteDialog.setSizePolicy(sizePolicy)
        SatelliteDialog.setMinimumSize(QtCore.QSize(800, 600))
        SatelliteDialog.setMaximumSize(QtCore.QSize(1600, 1200))
        SatelliteDialog.setSizeIncrement(QtCore.QSize(10, 10))
        SatelliteDialog.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        SatelliteDialog.setFont(font)
        self.gridLayout = QtWidgets.QGridLayout(SatelliteDialog)
        self.gridLayout.setContentsMargins(4, 4, 4, 4)
        self.gridLayout.setSpacing(4)
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox_3 = QtWidgets.QGroupBox(SatelliteDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
        self.groupBox_3.setSizePolicy(sizePolicy)
        self.groupBox_3.setProperty("large", True)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_5.setContentsMargins(0, 8, 0, 0)
        self.gridLayout_5.setSpacing(0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.satEarth = QtWidgets.QWidget(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.satEarth.sizePolicy().hasHeightForWidth())
        self.satEarth.setSizePolicy(sizePolicy)
        self.satEarth.setObjectName("satEarth")
        self.gridLayout_5.addWidget(self.satEarth, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_3, 0, 1, 1, 1)
        self.groupBox_4 = QtWidgets.QGroupBox(SatelliteDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_4.sizePolicy().hasHeightForWidth())
        self.groupBox_4.setSizePolicy(sizePolicy)
        self.groupBox_4.setProperty("large", True)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_4.setContentsMargins(0, 8, 0, 0)
        self.gridLayout_4.setSpacing(0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.satHorizon = QtWidgets.QWidget(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.satHorizon.sizePolicy().hasHeightForWidth())
        self.satHorizon.setSizePolicy(sizePolicy)
        self.satHorizon.setObjectName("satHorizon")
        self.gridLayout_4.addWidget(self.satHorizon, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_4, 1, 1, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(SatelliteDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setProperty("large", True)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_3.setContentsMargins(0, 8, 0, 0)
        self.gridLayout_3.setSpacing(0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.satSphere1 = QtWidgets.QWidget(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.satSphere1.sizePolicy().hasHeightForWidth())
        self.satSphere1.setSizePolicy(sizePolicy)
        self.satSphere1.setObjectName("satSphere1")
        self.gridLayout_3.addWidget(self.satSphere1, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_2, 1, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(SatelliteDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setProperty("large", True)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_2.setContentsMargins(0, 8, 0, 0)
        self.gridLayout_2.setSpacing(0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.satSphere2 = QtWidgets.QWidget(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.satSphere2.sizePolicy().hasHeightForWidth())
        self.satSphere2.setSizePolicy(sizePolicy)
        self.satSphere2.setMinimumSize(QtCore.QSize(0, 0))
        self.satSphere2.setObjectName("satSphere2")
        self.gridLayout_2.addWidget(self.satSphere2, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox, 0, 0, 1, 1)

        self.retranslateUi(SatelliteDialog)
        QtCore.QMetaObject.connectSlotsByName(SatelliteDialog)

    def retranslateUi(self, SatelliteDialog):
        _translate = QtCore.QCoreApplication.translate
        SatelliteDialog.setWindowTitle(_translate("SatelliteDialog", "Satellite"))
        self.groupBox_3.setTitle(_translate("SatelliteDialog", "Satellite path over earth surface"))
        self.satEarth.setToolTip(_translate("SatelliteDialog", "Shows the satellite path vertical over ground."))
        self.groupBox_4.setTitle(_translate("SatelliteDialog", "Satellite path over horizon from observer location"))
        self.satHorizon.setToolTip(_translate("SatelliteDialog", "Shows the visible track of the satellite when being observed from the position of the mount. "))
        self.groupBox_2.setTitle(_translate("SatelliteDialog", "Satellite orbit in relation to turning earth"))
        self.satSphere1.setToolTip(_translate("SatelliteDialog", "Show the satellite path in orbit. The observer is moving with the earth turning. We get normally a classical path for the satellite."))
        self.groupBox.setTitle(_translate("SatelliteDialog", "Satellit orbit in relation to fixed observer"))
        self.satSphere2.setToolTip(_translate("SatelliteDialog", "Show the satellite path in orbit. The observer is a fixed place with the coordinates of the mount. The earth is not turning. therefore we get normally a spiral path for the satellite."))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    SatelliteDialog = QtWidgets.QWidget()
    ui = Ui_SatelliteDialog()
    ui.setupUi(SatelliteDialog)
    SatelliteDialog.show()
    sys.exit(app.exec_())
