# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './mw4/gui/widgets/main.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.setWindowModality(QtCore.Qt.NonModal)
        MainWindow.setEnabled(True)
        MainWindow.resize(800, 600)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QtCore.QSize(800, 600))
        MainWindow.setMaximumSize(QtCore.QSize(800, 1200))
        MainWindow.setSizeIncrement(QtCore.QSize(9, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        MainWindow.setFont(font)
        MainWindow.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        MainWindow.setFocusPolicy(QtCore.Qt.NoFocus)
        MainWindow.setContextMenuPolicy(QtCore.Qt.PreventContextMenu)
        MainWindow.setToolTipDuration(6)
        MainWindow.setAutoFillBackground(True)
        self.mainTabWidget = QtWidgets.QTabWidget(MainWindow)
        self.mainTabWidget.setEnabled(True)
        self.mainTabWidget.setGeometry(QtCore.QRect(4, 128, 791, 466))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mainTabWidget.sizePolicy().hasHeightForWidth())
        self.mainTabWidget.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mainTabWidget.setFont(font)
        self.mainTabWidget.setToolTip("")
        self.mainTabWidget.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.mainTabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.mainTabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.mainTabWidget.setElideMode(QtCore.Qt.ElideNone)
        self.mainTabWidget.setTabsClosable(False)
        self.mainTabWidget.setMovable(False)
        self.mainTabWidget.setTabBarAutoHide(False)
        self.mainTabWidget.setObjectName("mainTabWidget")
        self.Mount = QtWidgets.QWidget()
        self.Mount.setObjectName("Mount")
        self.groupBox_47 = QtWidgets.QGroupBox(self.Mount)
        self.groupBox_47.setGeometry(QtCore.QRect(505, 15, 271, 101))
        self.groupBox_47.setProperty("large", True)
        self.groupBox_47.setObjectName("groupBox_47")
        self.setSolarTracking = QtWidgets.QPushButton(self.groupBox_47)
        self.setSolarTracking.setGeometry(QtCore.QRect(95, 60, 81, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.setSolarTracking.setFont(font)
        self.setSolarTracking.setObjectName("setSolarTracking")
        self.setSiderealTracking = QtWidgets.QPushButton(self.groupBox_47)
        self.setSiderealTracking.setGeometry(QtCore.QRect(180, 60, 86, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.setSiderealTracking.setFont(font)
        self.setSiderealTracking.setObjectName("setSiderealTracking")
        self.setLunarTracking = QtWidgets.QPushButton(self.groupBox_47)
        self.setLunarTracking.setGeometry(QtCore.QRect(5, 60, 86, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.setLunarTracking.setFont(font)
        self.setLunarTracking.setObjectName("setLunarTracking")
        self.flipMount = QtWidgets.QPushButton(self.groupBox_47)
        self.flipMount.setGeometry(QtCore.QRect(180, 20, 86, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.flipMount.setFont(font)
        self.flipMount.setObjectName("flipMount")
        self.tracking = QtWidgets.QPushButton(self.groupBox_47)
        self.tracking.setGeometry(QtCore.QRect(5, 20, 86, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.tracking.setFont(font)
        self.tracking.setObjectName("tracking")
        self.followSat = QtWidgets.QPushButton(self.groupBox_47)
        self.followSat.setGeometry(QtCore.QRect(95, 20, 81, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.followSat.setFont(font)
        self.followSat.setObjectName("followSat")
        self.groupBox_48 = QtWidgets.QGroupBox(self.Mount)
        self.groupBox_48.setGeometry(QtCore.QRect(505, 125, 271, 61))
        self.groupBox_48.setProperty("large", True)
        self.groupBox_48.setObjectName("groupBox_48")
        self.stop = QtWidgets.QPushButton(self.groupBox_48)
        self.stop.setGeometry(QtCore.QRect(145, 20, 116, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.stop.setFont(font)
        self.stop.setObjectName("stop")
        self.park = QtWidgets.QPushButton(self.groupBox_48)
        self.park.setGeometry(QtCore.QRect(10, 20, 121, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.park.setFont(font)
        self.park.setObjectName("park")
        self.groupBox_49 = QtWidgets.QGroupBox(self.Mount)
        self.groupBox_49.setGeometry(QtCore.QRect(10, 15, 481, 171))
        self.groupBox_49.setProperty("large", True)
        self.groupBox_49.setObjectName("groupBox_49")
        self.groupBoxCoordinateSystems = QtWidgets.QGroupBox(self.groupBox_49)
        self.groupBoxCoordinateSystems.setGeometry(QtCore.QRect(320, 10, 146, 41))
        self.groupBoxCoordinateSystems.setObjectName("groupBoxCoordinateSystems")
        self.checkJNow = QtWidgets.QRadioButton(self.groupBoxCoordinateSystems)
        self.checkJNow.setGeometry(QtCore.QRect(80, 15, 61, 20))
        self.checkJNow.setSizeIncrement(QtCore.QSize(0, 18))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkJNow.setFont(font)
        self.checkJNow.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkJNow.setChecked(False)
        self.checkJNow.setObjectName("checkJNow")
        self.checkJ2000 = QtWidgets.QRadioButton(self.groupBoxCoordinateSystems)
        self.checkJ2000.setGeometry(QtCore.QRect(10, 15, 61, 20))
        self.checkJ2000.setSizeIncrement(QtCore.QSize(0, 18))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkJ2000.setFont(font)
        self.checkJ2000.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkJ2000.setChecked(True)
        self.checkJ2000.setObjectName("checkJ2000")
        self.label_77 = QtWidgets.QLabel(self.groupBox_49)
        self.label_77.setGeometry(QtCore.QRect(10, 85, 66, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label_77.setFont(font)
        self.label_77.setProperty("large", True)
        self.label_77.setObjectName("label_77")
        self.RAfloat = QtWidgets.QLineEdit(self.groupBox_49)
        self.RAfloat.setEnabled(False)
        self.RAfloat.setGeometry(QtCore.QRect(75, 105, 141, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.RAfloat.sizePolicy().hasHeightForWidth())
        self.RAfloat.setSizePolicy(sizePolicy)
        self.RAfloat.setMinimumSize(QtCore.QSize(0, 15))
        self.RAfloat.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.RAfloat.setFont(font)
        self.RAfloat.setMouseTracking(False)
        self.RAfloat.setFocusPolicy(QtCore.Qt.NoFocus)
        self.RAfloat.setAcceptDrops(False)
        self.RAfloat.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.RAfloat.setReadOnly(True)
        self.RAfloat.setObjectName("RAfloat")
        self.ALT = QtWidgets.QLineEdit(self.groupBox_49)
        self.ALT.setEnabled(False)
        self.ALT.setGeometry(QtCore.QRect(75, 130, 141, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.ALT.setFont(font)
        self.ALT.setMouseTracking(False)
        self.ALT.setFocusPolicy(QtCore.Qt.NoFocus)
        self.ALT.setAcceptDrops(False)
        self.ALT.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.ALT.setStyleSheet("")
        self.ALT.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.ALT.setReadOnly(True)
        self.ALT.setProperty("large", True)
        self.ALT.setObjectName("ALT")
        self.DECfloat = QtWidgets.QLineEdit(self.groupBox_49)
        self.DECfloat.setEnabled(False)
        self.DECfloat.setGeometry(QtCore.QRect(320, 105, 146, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.DECfloat.sizePolicy().hasHeightForWidth())
        self.DECfloat.setSizePolicy(sizePolicy)
        self.DECfloat.setMinimumSize(QtCore.QSize(0, 15))
        self.DECfloat.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.DECfloat.setFont(font)
        self.DECfloat.setMouseTracking(False)
        self.DECfloat.setFocusPolicy(QtCore.Qt.NoFocus)
        self.DECfloat.setAcceptDrops(False)
        self.DECfloat.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.DECfloat.setReadOnly(True)
        self.DECfloat.setObjectName("DECfloat")
        self.label_79 = QtWidgets.QLabel(self.groupBox_49)
        self.label_79.setGeometry(QtCore.QRect(250, 130, 66, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label_79.setFont(font)
        self.label_79.setProperty("large", True)
        self.label_79.setObjectName("label_79")
        self.HAfloat = QtWidgets.QLineEdit(self.groupBox_49)
        self.HAfloat.setEnabled(False)
        self.HAfloat.setGeometry(QtCore.QRect(75, 55, 141, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.HAfloat.sizePolicy().hasHeightForWidth())
        self.HAfloat.setSizePolicy(sizePolicy)
        self.HAfloat.setMinimumSize(QtCore.QSize(0, 15))
        self.HAfloat.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.HAfloat.setFont(font)
        self.HAfloat.setMouseTracking(False)
        self.HAfloat.setFocusPolicy(QtCore.Qt.NoFocus)
        self.HAfloat.setAcceptDrops(False)
        self.HAfloat.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.HAfloat.setReadOnly(True)
        self.HAfloat.setObjectName("HAfloat")
        self.label_102 = QtWidgets.QLabel(self.groupBox_49)
        self.label_102.setGeometry(QtCore.QRect(250, 85, 66, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label_102.setFont(font)
        self.label_102.setProperty("large", True)
        self.label_102.setObjectName("label_102")
        self.label_58 = QtWidgets.QLabel(self.groupBox_49)
        self.label_58.setGeometry(QtCore.QRect(10, 35, 66, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label_58.setFont(font)
        self.label_58.setProperty("large", True)
        self.label_58.setObjectName("label_58")
        self.RA = QtWidgets.QLineEdit(self.groupBox_49)
        self.RA.setEnabled(False)
        self.RA.setGeometry(QtCore.QRect(75, 75, 141, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.RA.setFont(font)
        self.RA.setMouseTracking(False)
        self.RA.setFocusPolicy(QtCore.Qt.NoFocus)
        self.RA.setAcceptDrops(False)
        self.RA.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.RA.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.RA.setReadOnly(True)
        self.RA.setProperty("large", True)
        self.RA.setObjectName("RA")
        self.HA = QtWidgets.QLineEdit(self.groupBox_49)
        self.HA.setEnabled(False)
        self.HA.setGeometry(QtCore.QRect(75, 25, 141, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.HA.setFont(font)
        self.HA.setMouseTracking(False)
        self.HA.setFocusPolicy(QtCore.Qt.NoFocus)
        self.HA.setAcceptDrops(False)
        self.HA.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.HA.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.HA.setReadOnly(True)
        self.HA.setProperty("large", True)
        self.HA.setObjectName("HA")
        self.AZ = QtWidgets.QLineEdit(self.groupBox_49)
        self.AZ.setEnabled(False)
        self.AZ.setGeometry(QtCore.QRect(320, 130, 146, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.AZ.setFont(font)
        self.AZ.setMouseTracking(False)
        self.AZ.setFocusPolicy(QtCore.Qt.NoFocus)
        self.AZ.setAcceptDrops(False)
        self.AZ.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.AZ.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.AZ.setReadOnly(True)
        self.AZ.setProperty("large", True)
        self.AZ.setObjectName("AZ")
        self.DEC = QtWidgets.QLineEdit(self.groupBox_49)
        self.DEC.setEnabled(False)
        self.DEC.setGeometry(QtCore.QRect(320, 75, 146, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.DEC.setFont(font)
        self.DEC.setMouseTracking(False)
        self.DEC.setFocusPolicy(QtCore.Qt.NoFocus)
        self.DEC.setAcceptDrops(False)
        self.DEC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.DEC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.DEC.setReadOnly(True)
        self.DEC.setProperty("large", True)
        self.DEC.setObjectName("DEC")
        self.label_78 = QtWidgets.QLabel(self.groupBox_49)
        self.label_78.setGeometry(QtCore.QRect(10, 130, 66, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label_78.setFont(font)
        self.label_78.setProperty("large", True)
        self.label_78.setObjectName("label_78")
        self.groupBox_50 = QtWidgets.QGroupBox(self.Mount)
        self.groupBox_50.setGeometry(QtCore.QRect(10, 195, 766, 231))
        self.groupBox_50.setProperty("large", True)
        self.groupBox_50.setObjectName("groupBox_50")
        self.label_148 = QtWidgets.QLabel(self.groupBox_50)
        self.label_148.setGeometry(QtCore.QRect(275, 175, 116, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_148.setFont(font)
        self.label_148.setObjectName("label_148")
        self.label_12 = QtWidgets.QLabel(self.groupBox_50)
        self.label_12.setGeometry(QtCore.QRect(520, 205, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_12.setFont(font)
        self.label_12.setObjectName("label_12")
        self.label_34 = QtWidgets.QLabel(self.groupBox_50)
        self.label_34.setGeometry(QtCore.QRect(520, 85, 131, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_34.setFont(font)
        self.label_34.setObjectName("label_34")
        self.refractionPress = QtWidgets.QLineEdit(self.groupBox_50)
        self.refractionPress.setEnabled(False)
        self.refractionPress.setGeometry(QtCore.QRect(155, 145, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.refractionPress.setFont(font)
        self.refractionPress.setMouseTracking(False)
        self.refractionPress.setFocusPolicy(QtCore.Qt.NoFocus)
        self.refractionPress.setAcceptDrops(False)
        self.refractionPress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.refractionPress.setMaxLength(8)
        self.refractionPress.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.refractionPress.setReadOnly(True)
        self.refractionPress.setObjectName("refractionPress")
        self.label_152 = QtWidgets.QLabel(self.groupBox_50)
        self.label_152.setGeometry(QtCore.QRect(15, 55, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_152.setFont(font)
        self.label_152.setObjectName("label_152")
        self.statusRefraction = QtWidgets.QLineEdit(self.groupBox_50)
        self.statusRefraction.setEnabled(False)
        self.statusRefraction.setGeometry(QtCore.QRect(155, 175, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.statusRefraction.setFont(font)
        self.statusRefraction.setMouseTracking(False)
        self.statusRefraction.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusRefraction.setAcceptDrops(False)
        self.statusRefraction.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.statusRefraction.setInputMask("")
        self.statusRefraction.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.statusRefraction.setReadOnly(True)
        self.statusRefraction.setProperty("input", True)
        self.statusRefraction.setObjectName("statusRefraction")
        self.label_74 = QtWidgets.QLabel(self.groupBox_50)
        self.label_74.setGeometry(QtCore.QRect(275, 115, 116, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_74.setFont(font)
        self.label_74.setObjectName("label_74")
        self.label_65 = QtWidgets.QLabel(self.groupBox_50)
        self.label_65.setGeometry(QtCore.QRect(520, 115, 131, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_65.setFont(font)
        self.label_65.setObjectName("label_65")
        self.slewRate = QtWidgets.QLineEdit(self.groupBox_50)
        self.slewRate.setEnabled(False)
        self.slewRate.setGeometry(QtCore.QRect(155, 205, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.slewRate.setFont(font)
        self.slewRate.setMouseTracking(False)
        self.slewRate.setFocusPolicy(QtCore.Qt.NoFocus)
        self.slewRate.setAcceptDrops(False)
        self.slewRate.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.slewRate.setInputMask("")
        self.slewRate.setCursorPosition(1)
        self.slewRate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.slewRate.setReadOnly(True)
        self.slewRate.setProperty("input", True)
        self.slewRate.setObjectName("slewRate")
        self.pierside = QtWidgets.QLineEdit(self.groupBox_50)
        self.pierside.setEnabled(False)
        self.pierside.setGeometry(QtCore.QRect(155, 85, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.pierside.setFont(font)
        self.pierside.setMouseTracking(False)
        self.pierside.setFocusPolicy(QtCore.Qt.NoFocus)
        self.pierside.setAcceptDrops(False)
        self.pierside.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.pierside.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.pierside.setReadOnly(True)
        self.pierside.setObjectName("pierside")
        self.refractionTemp = QtWidgets.QLineEdit(self.groupBox_50)
        self.refractionTemp.setEnabled(False)
        self.refractionTemp.setGeometry(QtCore.QRect(155, 115, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.refractionTemp.setFont(font)
        self.refractionTemp.setMouseTracking(False)
        self.refractionTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.refractionTemp.setAcceptDrops(False)
        self.refractionTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.refractionTemp.setMaxLength(8)
        self.refractionTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.refractionTemp.setReadOnly(True)
        self.refractionTemp.setObjectName("refractionTemp")
        self.siteLongitude = QtWidgets.QLineEdit(self.groupBox_50)
        self.siteLongitude.setEnabled(False)
        self.siteLongitude.setGeometry(QtCore.QRect(635, 175, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.siteLongitude.setFont(font)
        self.siteLongitude.setMouseTracking(False)
        self.siteLongitude.setFocusPolicy(QtCore.Qt.NoFocus)
        self.siteLongitude.setAcceptDrops(False)
        self.siteLongitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.siteLongitude.setMaxLength(32634)
        self.siteLongitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.siteLongitude.setReadOnly(True)
        self.siteLongitude.setCursorMoveStyle(QtCore.Qt.VisualMoveStyle)
        self.siteLongitude.setProperty("input", True)
        self.siteLongitude.setObjectName("siteLongitude")
        self.label_55 = QtWidgets.QLabel(self.groupBox_50)
        self.label_55.setGeometry(QtCore.QRect(275, 85, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_55.setFont(font)
        self.label_55.setObjectName("label_55")
        self.label_61 = QtWidgets.QLabel(self.groupBox_50)
        self.label_61.setGeometry(QtCore.QRect(445, 25, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_61.setFont(font)
        self.label_61.setAlignment(QtCore.Qt.AlignCenter)
        self.label_61.setWordWrap(False)
        self.label_61.setObjectName("label_61")
        self.label_86 = QtWidgets.QLabel(self.groupBox_50)
        self.label_86.setGeometry(QtCore.QRect(15, 175, 126, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_86.setFont(font)
        self.label_86.setObjectName("label_86")
        self.UTCExpire = QtWidgets.QLineEdit(self.groupBox_50)
        self.UTCExpire.setEnabled(False)
        self.UTCExpire.setGeometry(QtCore.QRect(125, 55, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.UTCExpire.setFont(font)
        self.UTCExpire.setMouseTracking(False)
        self.UTCExpire.setFocusPolicy(QtCore.Qt.NoFocus)
        self.UTCExpire.setAcceptDrops(False)
        self.UTCExpire.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.UTCExpire.setInputMask("")
        self.UTCExpire.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.UTCExpire.setReadOnly(True)
        self.UTCExpire.setObjectName("UTCExpire")
        self.siteLatitude = QtWidgets.QLineEdit(self.groupBox_50)
        self.siteLatitude.setEnabled(False)
        self.siteLatitude.setGeometry(QtCore.QRect(635, 145, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.siteLatitude.setFont(font)
        self.siteLatitude.setMouseTracking(False)
        self.siteLatitude.setFocusPolicy(QtCore.Qt.NoFocus)
        self.siteLatitude.setAcceptDrops(False)
        self.siteLatitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.siteLatitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.siteLatitude.setReadOnly(True)
        self.siteLatitude.setProperty("input", True)
        self.siteLatitude.setObjectName("siteLatitude")
        self.timeToFlip = QtWidgets.QLineEdit(self.groupBox_50)
        self.timeToFlip.setEnabled(False)
        self.timeToFlip.setGeometry(QtCore.QRect(395, 55, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeToFlip.setFont(font)
        self.timeToFlip.setMouseTracking(False)
        self.timeToFlip.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeToFlip.setAcceptDrops(False)
        self.timeToFlip.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeToFlip.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeToFlip.setReadOnly(True)
        self.timeToFlip.setObjectName("timeToFlip")
        self.label_4 = QtWidgets.QLabel(self.groupBox_50)
        self.label_4.setGeometry(QtCore.QRect(520, 55, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.statusUnattendedFlip = QtWidgets.QLineEdit(self.groupBox_50)
        self.statusUnattendedFlip.setEnabled(False)
        self.statusUnattendedFlip.setGeometry(QtCore.QRect(395, 85, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.statusUnattendedFlip.setFont(font)
        self.statusUnattendedFlip.setMouseTracking(False)
        self.statusUnattendedFlip.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusUnattendedFlip.setAcceptDrops(False)
        self.statusUnattendedFlip.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.statusUnattendedFlip.setInputMask("")
        self.statusUnattendedFlip.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.statusUnattendedFlip.setReadOnly(True)
        self.statusUnattendedFlip.setProperty("input", True)
        self.statusUnattendedFlip.setObjectName("statusUnattendedFlip")
        self.label_82 = QtWidgets.QLabel(self.groupBox_50)
        self.label_82.setGeometry(QtCore.QRect(715, 205, 21, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_82.setFont(font)
        self.label_82.setAlignment(QtCore.Qt.AlignCenter)
        self.label_82.setWordWrap(False)
        self.label_82.setObjectName("label_82")
        self.label_64 = QtWidgets.QLabel(self.groupBox_50)
        self.label_64.setGeometry(QtCore.QRect(210, 115, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_64.setFont(font)
        self.label_64.setObjectName("label_64")
        self.label_109 = QtWidgets.QLabel(self.groupBox_50)
        self.label_109.setGeometry(QtCore.QRect(715, 55, 16, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_109.setFont(font)
        self.label_109.setAlignment(QtCore.Qt.AlignCenter)
        self.label_109.setWordWrap(False)
        self.label_109.setObjectName("label_109")
        self.timeSidereal = QtWidgets.QLineEdit(self.groupBox_50)
        self.timeSidereal.setEnabled(False)
        self.timeSidereal.setGeometry(QtCore.QRect(135, 25, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeSidereal.setFont(font)
        self.timeSidereal.setMouseTracking(False)
        self.timeSidereal.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeSidereal.setAcceptDrops(False)
        self.timeSidereal.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeSidereal.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeSidereal.setReadOnly(True)
        self.timeSidereal.setObjectName("timeSidereal")
        self.label_130 = QtWidgets.QLabel(self.groupBox_50)
        self.label_130.setGeometry(QtCore.QRect(210, 25, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_130.setFont(font)
        self.label_130.setObjectName("label_130")
        self.label_232 = QtWidgets.QLabel(self.groupBox_50)
        self.label_232.setGeometry(QtCore.QRect(15, 115, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_232.setFont(font)
        self.label_232.setObjectName("label_232")
        self.horizonLimitHigh = QtWidgets.QLineEdit(self.groupBox_50)
        self.horizonLimitHigh.setEnabled(False)
        self.horizonLimitHigh.setGeometry(QtCore.QRect(665, 25, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.horizonLimitHigh.setFont(font)
        self.horizonLimitHigh.setMouseTracking(False)
        self.horizonLimitHigh.setFocusPolicy(QtCore.Qt.NoFocus)
        self.horizonLimitHigh.setAcceptDrops(False)
        self.horizonLimitHigh.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.horizonLimitHigh.setInputMask("")
        self.horizonLimitHigh.setCursorPosition(1)
        self.horizonLimitHigh.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.horizonLimitHigh.setReadOnly(True)
        self.horizonLimitHigh.setProperty("input", True)
        self.horizonLimitHigh.setObjectName("horizonLimitHigh")
        self.siteElevation = QtWidgets.QLineEdit(self.groupBox_50)
        self.siteElevation.setEnabled(False)
        self.siteElevation.setGeometry(QtCore.QRect(635, 205, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.siteElevation.setFont(font)
        self.siteElevation.setMouseTracking(False)
        self.siteElevation.setFocusPolicy(QtCore.Qt.NoFocus)
        self.siteElevation.setAcceptDrops(False)
        self.siteElevation.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.siteElevation.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.siteElevation.setReadOnly(True)
        self.siteElevation.setProperty("input", True)
        self.siteElevation.setObjectName("siteElevation")
        self.label_91 = QtWidgets.QLabel(self.groupBox_50)
        self.label_91.setGeometry(QtCore.QRect(445, 55, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_91.setFont(font)
        self.label_91.setAlignment(QtCore.Qt.AlignCenter)
        self.label_91.setWordWrap(False)
        self.label_91.setObjectName("label_91")
        self.label_6 = QtWidgets.QLabel(self.groupBox_50)
        self.label_6.setGeometry(QtCore.QRect(520, 145, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.meridianLimitSlew = QtWidgets.QLineEdit(self.groupBox_50)
        self.meridianLimitSlew.setEnabled(False)
        self.meridianLimitSlew.setGeometry(QtCore.QRect(665, 115, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.meridianLimitSlew.setFont(font)
        self.meridianLimitSlew.setMouseTracking(False)
        self.meridianLimitSlew.setFocusPolicy(QtCore.Qt.NoFocus)
        self.meridianLimitSlew.setAcceptDrops(False)
        self.meridianLimitSlew.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.meridianLimitSlew.setInputMask("")
        self.meridianLimitSlew.setCursorPosition(1)
        self.meridianLimitSlew.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.meridianLimitSlew.setReadOnly(True)
        self.meridianLimitSlew.setProperty("input", True)
        self.meridianLimitSlew.setObjectName("meridianLimitSlew")
        self.label_242 = QtWidgets.QLabel(self.groupBox_50)
        self.label_242.setGeometry(QtCore.QRect(15, 145, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_242.setFont(font)
        self.label_242.setObjectName("label_242")
        self.statusDualAxisTracking = QtWidgets.QLineEdit(self.groupBox_50)
        self.statusDualAxisTracking.setEnabled(False)
        self.statusDualAxisTracking.setGeometry(QtCore.QRect(395, 115, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.statusDualAxisTracking.setFont(font)
        self.statusDualAxisTracking.setMouseTracking(False)
        self.statusDualAxisTracking.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusDualAxisTracking.setAcceptDrops(False)
        self.statusDualAxisTracking.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.statusDualAxisTracking.setInputMask("")
        self.statusDualAxisTracking.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.statusDualAxisTracking.setReadOnly(True)
        self.statusDualAxisTracking.setProperty("input", True)
        self.statusDualAxisTracking.setObjectName("statusDualAxisTracking")
        self.label_52 = QtWidgets.QLabel(self.groupBox_50)
        self.label_52.setGeometry(QtCore.QRect(275, 55, 91, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_52.setFont(font)
        self.label_52.setWordWrap(False)
        self.label_52.setObjectName("label_52")
        self.statusGPSSynced = QtWidgets.QLineEdit(self.groupBox_50)
        self.statusGPSSynced.setEnabled(False)
        self.statusGPSSynced.setGeometry(QtCore.QRect(395, 175, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.statusGPSSynced.setFont(font)
        self.statusGPSSynced.setMouseTracking(False)
        self.statusGPSSynced.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusGPSSynced.setAcceptDrops(False)
        self.statusGPSSynced.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.statusGPSSynced.setInputMask("")
        self.statusGPSSynced.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.statusGPSSynced.setReadOnly(True)
        self.statusGPSSynced.setObjectName("statusGPSSynced")
        self.label_151 = QtWidgets.QLabel(self.groupBox_50)
        self.label_151.setGeometry(QtCore.QRect(275, 145, 116, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_151.setFont(font)
        self.label_151.setObjectName("label_151")
        self.label_105 = QtWidgets.QLabel(self.groupBox_50)
        self.label_105.setGeometry(QtCore.QRect(720, 175, 31, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_105.setFont(font)
        self.label_105.setAlignment(QtCore.Qt.AlignCenter)
        self.label_105.setWordWrap(False)
        self.label_105.setObjectName("label_105")
        self.label_133 = QtWidgets.QLabel(self.groupBox_50)
        self.label_133.setGeometry(QtCore.QRect(210, 205, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_133.setFont(font)
        self.label_133.setObjectName("label_133")
        self.meridianLimitTrack = QtWidgets.QLineEdit(self.groupBox_50)
        self.meridianLimitTrack.setEnabled(False)
        self.meridianLimitTrack.setGeometry(QtCore.QRect(665, 85, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.meridianLimitTrack.setFont(font)
        self.meridianLimitTrack.setMouseTracking(False)
        self.meridianLimitTrack.setFocusPolicy(QtCore.Qt.NoFocus)
        self.meridianLimitTrack.setAcceptDrops(False)
        self.meridianLimitTrack.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.meridianLimitTrack.setInputMask("")
        self.meridianLimitTrack.setCursorPosition(1)
        self.meridianLimitTrack.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.meridianLimitTrack.setReadOnly(True)
        self.meridianLimitTrack.setProperty("input", True)
        self.meridianLimitTrack.setObjectName("meridianLimitTrack")
        self.label_20 = QtWidgets.QLabel(self.groupBox_50)
        self.label_20.setGeometry(QtCore.QRect(520, 25, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_20.setFont(font)
        self.label_20.setObjectName("label_20")
        self.label_18 = QtWidgets.QLabel(self.groupBox_50)
        self.label_18.setGeometry(QtCore.QRect(15, 85, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_18.setFont(font)
        self.label_18.setObjectName("label_18")
        self.label_88 = QtWidgets.QLabel(self.groupBox_50)
        self.label_88.setGeometry(QtCore.QRect(715, 85, 16, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_88.setFont(font)
        self.label_88.setAlignment(QtCore.Qt.AlignCenter)
        self.label_88.setWordWrap(False)
        self.label_88.setObjectName("label_88")
        self.label_28 = QtWidgets.QLabel(self.groupBox_50)
        self.label_28.setGeometry(QtCore.QRect(15, 205, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_28.setFont(font)
        self.label_28.setObjectName("label_28")
        self.label_81 = QtWidgets.QLabel(self.groupBox_50)
        self.label_81.setGeometry(QtCore.QRect(210, 145, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_81.setFont(font)
        self.label_81.setObjectName("label_81")
        self.label_7 = QtWidgets.QLabel(self.groupBox_50)
        self.label_7.setGeometry(QtCore.QRect(15, 25, 116, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.label_104 = QtWidgets.QLabel(self.groupBox_50)
        self.label_104.setGeometry(QtCore.QRect(715, 25, 16, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_104.setFont(font)
        self.label_104.setAlignment(QtCore.Qt.AlignCenter)
        self.label_104.setWordWrap(False)
        self.label_104.setObjectName("label_104")
        self.label_106 = QtWidgets.QLabel(self.groupBox_50)
        self.label_106.setGeometry(QtCore.QRect(720, 145, 31, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_106.setFont(font)
        self.label_106.setAlignment(QtCore.Qt.AlignCenter)
        self.label_106.setWordWrap(False)
        self.label_106.setObjectName("label_106")
        self.label_5 = QtWidgets.QLabel(self.groupBox_50)
        self.label_5.setGeometry(QtCore.QRect(520, 175, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.label_222 = QtWidgets.QLabel(self.groupBox_50)
        self.label_222.setGeometry(QtCore.QRect(210, 55, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_222.setFont(font)
        self.label_222.setObjectName("label_222")
        self.label_90 = QtWidgets.QLabel(self.groupBox_50)
        self.label_90.setGeometry(QtCore.QRect(275, 25, 101, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_90.setFont(font)
        self.label_90.setWordWrap(False)
        self.label_90.setObjectName("label_90")
        self.label_89 = QtWidgets.QLabel(self.groupBox_50)
        self.label_89.setGeometry(QtCore.QRect(715, 115, 16, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_89.setFont(font)
        self.label_89.setAlignment(QtCore.Qt.AlignCenter)
        self.label_89.setWordWrap(False)
        self.label_89.setObjectName("label_89")
        self.timeToMeridian = QtWidgets.QLineEdit(self.groupBox_50)
        self.timeToMeridian.setEnabled(False)
        self.timeToMeridian.setGeometry(QtCore.QRect(395, 25, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeToMeridian.setFont(font)
        self.timeToMeridian.setMouseTracking(False)
        self.timeToMeridian.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeToMeridian.setAcceptDrops(False)
        self.timeToMeridian.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeToMeridian.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeToMeridian.setReadOnly(True)
        self.timeToMeridian.setObjectName("timeToMeridian")
        self.horizonLimitLow = QtWidgets.QLineEdit(self.groupBox_50)
        self.horizonLimitLow.setEnabled(False)
        self.horizonLimitLow.setGeometry(QtCore.QRect(665, 55, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.horizonLimitLow.setFont(font)
        self.horizonLimitLow.setMouseTracking(False)
        self.horizonLimitLow.setFocusPolicy(QtCore.Qt.NoFocus)
        self.horizonLimitLow.setAcceptDrops(False)
        self.horizonLimitLow.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.horizonLimitLow.setInputMask("")
        self.horizonLimitLow.setMaxLength(32767)
        self.horizonLimitLow.setFrame(True)
        self.horizonLimitLow.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.horizonLimitLow.setReadOnly(True)
        self.horizonLimitLow.setClearButtonEnabled(False)
        self.horizonLimitLow.setProperty("input", True)
        self.horizonLimitLow.setObjectName("horizonLimitLow")
        self.statusWOL = QtWidgets.QLineEdit(self.groupBox_50)
        self.statusWOL.setEnabled(False)
        self.statusWOL.setGeometry(QtCore.QRect(395, 145, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.statusWOL.setFont(font)
        self.statusWOL.setMouseTracking(False)
        self.statusWOL.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusWOL.setAcceptDrops(False)
        self.statusWOL.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.statusWOL.setInputMask("")
        self.statusWOL.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.statusWOL.setReadOnly(True)
        self.statusWOL.setProperty("input", True)
        self.statusWOL.setObjectName("statusWOL")
        self.label_mountMAC_2 = QtWidgets.QLabel(self.groupBox_50)
        self.label_mountMAC_2.setGeometry(QtCore.QRect(275, 205, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_mountMAC_2.setFont(font)
        self.label_mountMAC_2.setObjectName("label_mountMAC_2")
        self.mountTypeConnection = QtWidgets.QLineEdit(self.groupBox_50)
        self.mountTypeConnection.setEnabled(False)
        self.mountTypeConnection.setGeometry(QtCore.QRect(395, 205, 51, 21))
        self.mountTypeConnection.setMinimumSize(QtCore.QSize(25, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mountTypeConnection.setFont(font)
        self.mountTypeConnection.setMouseTracking(False)
        self.mountTypeConnection.setAcceptDrops(False)
        self.mountTypeConnection.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mountTypeConnection.setMaxLength(18)
        self.mountTypeConnection.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mountTypeConnection.setDragEnabled(True)
        self.mountTypeConnection.setReadOnly(True)
        self.mountTypeConnection.setObjectName("mountTypeConnection")
        self.mainTabWidget.addTab(self.Mount, "")
        self.Environ = QtWidgets.QWidget()
        self.Environ.setObjectName("Environ")
        self.groupBox_51 = QtWidgets.QGroupBox(self.Environ)
        self.groupBox_51.setGeometry(QtCore.QRect(10, 15, 766, 271))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_51.sizePolicy().hasHeightForWidth())
        self.groupBox_51.setSizePolicy(sizePolicy)
        self.groupBox_51.setProperty("large", True)
        self.groupBox_51.setObjectName("groupBox_51")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_51)
        self.gridLayout.setContentsMargins(10, 15, 5, 5)
        self.gridLayout.setHorizontalSpacing(10)
        self.gridLayout.setVerticalSpacing(5)
        self.gridLayout.setObjectName("gridLayout")
        self.label_25 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_25.setFont(font)
        self.label_25.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_25.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_25.setObjectName("label_25")
        self.gridLayout.addWidget(self.label_25, 3, 0, 1, 1)
        self.label_27 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_27.setFont(font)
        self.label_27.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_27.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_27.setObjectName("label_27")
        self.gridLayout.addWidget(self.label_27, 2, 0, 1, 1)
        self.label_73 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_73.setFont(font)
        self.label_73.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_73.setWordWrap(False)
        self.label_73.setObjectName("label_73")
        self.gridLayout.addWidget(self.label_73, 2, 1, 1, 1)
        self.label_142 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_142.setFont(font)
        self.label_142.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_142.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_142.setObjectName("label_142")
        self.gridLayout.addWidget(self.label_142, 9, 0, 1, 1)
        self.mountGroup = QtWidgets.QGroupBox(self.groupBox_51)
        self.mountGroup.setMinimumSize(QtCore.QSize(75, 0))
        self.mountGroup.setObjectName("mountGroup")
        self.refractionTemp1 = QtWidgets.QLineEdit(self.mountGroup)
        self.refractionTemp1.setEnabled(False)
        self.refractionTemp1.setGeometry(QtCore.QRect(10, 15, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.refractionTemp1.setFont(font)
        self.refractionTemp1.setMouseTracking(False)
        self.refractionTemp1.setFocusPolicy(QtCore.Qt.NoFocus)
        self.refractionTemp1.setAcceptDrops(False)
        self.refractionTemp1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.refractionTemp1.setMaxLength(8)
        self.refractionTemp1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.refractionTemp1.setReadOnly(True)
        self.refractionTemp1.setObjectName("refractionTemp1")
        self.refractionPress1 = QtWidgets.QLineEdit(self.mountGroup)
        self.refractionPress1.setEnabled(False)
        self.refractionPress1.setGeometry(QtCore.QRect(10, 40, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.refractionPress1.setFont(font)
        self.refractionPress1.setMouseTracking(False)
        self.refractionPress1.setFocusPolicy(QtCore.Qt.NoFocus)
        self.refractionPress1.setAcceptDrops(False)
        self.refractionPress1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.refractionPress1.setMaxLength(8)
        self.refractionPress1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.refractionPress1.setReadOnly(True)
        self.refractionPress1.setObjectName("refractionPress1")
        self.gridLayout.addWidget(self.mountGroup, 0, 9, 10, 1)
        self.label_26 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_26.setFont(font)
        self.label_26.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_26.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_26.setObjectName("label_26")
        self.gridLayout.addWidget(self.label_26, 4, 0, 1, 1)
        self.powerGroup = QtWidgets.QGroupBox(self.groupBox_51)
        self.powerGroup.setMinimumSize(QtCore.QSize(75, 0))
        self.powerGroup.setObjectName("powerGroup")
        self.powerTemp = QtWidgets.QLineEdit(self.powerGroup)
        self.powerTemp.setEnabled(False)
        self.powerTemp.setGeometry(QtCore.QRect(10, 15, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerTemp.setFont(font)
        self.powerTemp.setMouseTracking(False)
        self.powerTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerTemp.setAcceptDrops(False)
        self.powerTemp.setToolTip("")
        self.powerTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerTemp.setMaxLength(8)
        self.powerTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerTemp.setReadOnly(True)
        self.powerTemp.setObjectName("powerTemp")
        self.powerDewPoint = QtWidgets.QLineEdit(self.powerGroup)
        self.powerDewPoint.setEnabled(False)
        self.powerDewPoint.setGeometry(QtCore.QRect(10, 65, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerDewPoint.setFont(font)
        self.powerDewPoint.setMouseTracking(False)
        self.powerDewPoint.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerDewPoint.setAcceptDrops(False)
        self.powerDewPoint.setToolTip("")
        self.powerDewPoint.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerDewPoint.setMaxLength(8)
        self.powerDewPoint.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerDewPoint.setReadOnly(True)
        self.powerDewPoint.setObjectName("powerDewPoint")
        self.powerHumidity = QtWidgets.QLineEdit(self.powerGroup)
        self.powerHumidity.setEnabled(False)
        self.powerHumidity.setGeometry(QtCore.QRect(10, 90, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerHumidity.setFont(font)
        self.powerHumidity.setMouseTracking(False)
        self.powerHumidity.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerHumidity.setAcceptDrops(False)
        self.powerHumidity.setToolTip("")
        self.powerHumidity.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerHumidity.setMaxLength(8)
        self.powerHumidity.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerHumidity.setReadOnly(True)
        self.powerHumidity.setObjectName("powerHumidity")
        self.powerPress = QtWidgets.QLineEdit(self.powerGroup)
        self.powerPress.setEnabled(False)
        self.powerPress.setGeometry(QtCore.QRect(10, 40, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerPress.setFont(font)
        self.powerPress.setMouseTracking(False)
        self.powerPress.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerPress.setAcceptDrops(False)
        self.powerPress.setToolTip("")
        self.powerPress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerPress.setMaxLength(8)
        self.powerPress.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerPress.setReadOnly(True)
        self.powerPress.setObjectName("powerPress")
        self.powerRainVol = QtWidgets.QLineEdit(self.powerGroup)
        self.powerRainVol.setEnabled(False)
        self.powerRainVol.setGeometry(QtCore.QRect(10, 140, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerRainVol.setFont(font)
        self.powerRainVol.setMouseTracking(False)
        self.powerRainVol.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerRainVol.setAcceptDrops(False)
        self.powerRainVol.setToolTip("")
        self.powerRainVol.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerRainVol.setMaxLength(8)
        self.powerRainVol.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerRainVol.setReadOnly(True)
        self.powerRainVol.setObjectName("powerRainVol")
        self.powerWindDir = QtWidgets.QLineEdit(self.powerGroup)
        self.powerWindDir.setEnabled(False)
        self.powerWindDir.setGeometry(QtCore.QRect(10, 190, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerWindDir.setFont(font)
        self.powerWindDir.setMouseTracking(False)
        self.powerWindDir.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerWindDir.setAcceptDrops(False)
        self.powerWindDir.setToolTip("")
        self.powerWindDir.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerWindDir.setMaxLength(8)
        self.powerWindDir.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerWindDir.setReadOnly(True)
        self.powerWindDir.setObjectName("powerWindDir")
        self.powerWindSpeed = QtWidgets.QLineEdit(self.powerGroup)
        self.powerWindSpeed.setEnabled(False)
        self.powerWindSpeed.setGeometry(QtCore.QRect(10, 165, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerWindSpeed.setFont(font)
        self.powerWindSpeed.setMouseTracking(False)
        self.powerWindSpeed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerWindSpeed.setAcceptDrops(False)
        self.powerWindSpeed.setToolTip("")
        self.powerWindSpeed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerWindSpeed.setMaxLength(8)
        self.powerWindSpeed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerWindSpeed.setReadOnly(True)
        self.powerWindSpeed.setObjectName("powerWindSpeed")
        self.powerCloudCover = QtWidgets.QLineEdit(self.powerGroup)
        self.powerCloudCover.setEnabled(False)
        self.powerCloudCover.setGeometry(QtCore.QRect(10, 115, 51, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerCloudCover.setFont(font)
        self.powerCloudCover.setMouseTracking(False)
        self.powerCloudCover.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerCloudCover.setAcceptDrops(False)
        self.powerCloudCover.setToolTip("")
        self.powerCloudCover.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerCloudCover.setMaxLength(8)
        self.powerCloudCover.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerCloudCover.setReadOnly(True)
        self.powerCloudCover.setObjectName("powerCloudCover")
        self.powerSQR = QtWidgets.QLineEdit(self.powerGroup)
        self.powerSQR.setEnabled(False)
        self.powerSQR.setGeometry(QtCore.QRect(10, 215, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerSQR.setFont(font)
        self.powerSQR.setMouseTracking(False)
        self.powerSQR.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerSQR.setAcceptDrops(False)
        self.powerSQR.setToolTip("")
        self.powerSQR.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerSQR.setMaxLength(8)
        self.powerSQR.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerSQR.setReadOnly(True)
        self.powerSQR.setObjectName("powerSQR")
        self.gridLayout.addWidget(self.powerGroup, 0, 4, 10, 1)
        self.label_87 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_87.setFont(font)
        self.label_87.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_87.setWordWrap(False)
        self.label_87.setObjectName("label_87")
        self.gridLayout.addWidget(self.label_87, 9, 1, 1, 1)
        self.label_69 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_69.setFont(font)
        self.label_69.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_69.setWordWrap(False)
        self.label_69.setObjectName("label_69")
        self.gridLayout.addWidget(self.label_69, 8, 1, 1, 1)
        self.label_30 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_30.setFont(font)
        self.label_30.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_30.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_30.setObjectName("label_30")
        self.gridLayout.addWidget(self.label_30, 5, 0, 1, 1)
        self.label_67 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_67.setFont(font)
        self.label_67.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_67.setWordWrap(False)
        self.label_67.setObjectName("label_67")
        self.gridLayout.addWidget(self.label_67, 7, 1, 1, 1)
        self.label_31 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_31.setFont(font)
        self.label_31.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_31.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_31.setObjectName("label_31")
        self.gridLayout.addWidget(self.label_31, 6, 0, 1, 1)
        self.label_24 = QtWidgets.QLabel(self.groupBox_51)
        self.label_24.setMaximumSize(QtCore.QSize(60, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_24.setFont(font)
        self.label_24.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_24.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_24.setObjectName("label_24")
        self.gridLayout.addWidget(self.label_24, 1, 0, 1, 1)
        self.label_71 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_71.setFont(font)
        self.label_71.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_71.setWordWrap(False)
        self.label_71.setObjectName("label_71")
        self.gridLayout.addWidget(self.label_71, 3, 1, 1, 1)
        self.label_72 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_72.setFont(font)
        self.label_72.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_72.setWordWrap(False)
        self.label_72.setObjectName("label_72")
        self.gridLayout.addWidget(self.label_72, 4, 1, 1, 1)
        self.label_70 = QtWidgets.QLabel(self.groupBox_51)
        self.label_70.setMaximumSize(QtCore.QSize(30, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_70.setFont(font)
        self.label_70.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_70.setWordWrap(False)
        self.label_70.setObjectName("label_70")
        self.gridLayout.addWidget(self.label_70, 1, 1, 1, 1)
        self.refractionGroup = QtWidgets.QGroupBox(self.groupBox_51)
        self.refractionGroup.setMinimumSize(QtCore.QSize(110, 0))
        self.refractionGroup.setObjectName("refractionGroup")
        self.checkRefracCont = QtWidgets.QRadioButton(self.refractionGroup)
        self.checkRefracCont.setGeometry(QtCore.QRect(15, 40, 136, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkRefracCont.setFont(font)
        self.checkRefracCont.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkRefracCont.setChecked(False)
        self.checkRefracCont.setObjectName("checkRefracCont")
        self.checkRefracNoTrack = QtWidgets.QRadioButton(self.refractionGroup)
        self.checkRefracNoTrack.setGeometry(QtCore.QRect(15, 60, 151, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkRefracNoTrack.setFont(font)
        self.checkRefracNoTrack.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkRefracNoTrack.setChecked(False)
        self.checkRefracNoTrack.setObjectName("checkRefracNoTrack")
        self.checkRefracNone = QtWidgets.QRadioButton(self.refractionGroup)
        self.checkRefracNone.setGeometry(QtCore.QRect(15, 20, 111, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkRefracNone.setFont(font)
        self.checkRefracNone.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkRefracNone.setChecked(True)
        self.checkRefracNone.setObjectName("checkRefracNone")
        self.gridLayout.addWidget(self.refractionGroup, 0, 11, 4, 1)
        self.label_68 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_68.setFont(font)
        self.label_68.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_68.setWordWrap(False)
        self.label_68.setObjectName("label_68")
        self.gridLayout.addWidget(self.label_68, 6, 1, 1, 1)
        self.sensorWeatherGroup = QtWidgets.QGroupBox(self.groupBox_51)
        self.sensorWeatherGroup.setEnabled(True)
        self.sensorWeatherGroup.setMinimumSize(QtCore.QSize(75, 0))
        self.sensorWeatherGroup.setCheckable(True)
        self.sensorWeatherGroup.setChecked(False)
        self.sensorWeatherGroup.setObjectName("sensorWeatherGroup")
        self.sensorWeatherTemp = QtWidgets.QLineEdit(self.sensorWeatherGroup)
        self.sensorWeatherTemp.setEnabled(False)
        self.sensorWeatherTemp.setGeometry(QtCore.QRect(10, 15, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.sensorWeatherTemp.setFont(font)
        self.sensorWeatherTemp.setMouseTracking(False)
        self.sensorWeatherTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.sensorWeatherTemp.setAcceptDrops(False)
        self.sensorWeatherTemp.setToolTip("")
        self.sensorWeatherTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.sensorWeatherTemp.setMaxLength(8)
        self.sensorWeatherTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorWeatherTemp.setReadOnly(True)
        self.sensorWeatherTemp.setObjectName("sensorWeatherTemp")
        self.sensorWeatherPress = QtWidgets.QLineEdit(self.sensorWeatherGroup)
        self.sensorWeatherPress.setEnabled(False)
        self.sensorWeatherPress.setGeometry(QtCore.QRect(10, 40, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.sensorWeatherPress.setFont(font)
        self.sensorWeatherPress.setMouseTracking(False)
        self.sensorWeatherPress.setFocusPolicy(QtCore.Qt.NoFocus)
        self.sensorWeatherPress.setAcceptDrops(False)
        self.sensorWeatherPress.setToolTip("")
        self.sensorWeatherPress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.sensorWeatherPress.setMaxLength(8)
        self.sensorWeatherPress.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorWeatherPress.setReadOnly(True)
        self.sensorWeatherPress.setObjectName("sensorWeatherPress")
        self.sensorWeatherDewPoint = QtWidgets.QLineEdit(self.sensorWeatherGroup)
        self.sensorWeatherDewPoint.setEnabled(False)
        self.sensorWeatherDewPoint.setGeometry(QtCore.QRect(10, 65, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.sensorWeatherDewPoint.setFont(font)
        self.sensorWeatherDewPoint.setMouseTracking(False)
        self.sensorWeatherDewPoint.setFocusPolicy(QtCore.Qt.NoFocus)
        self.sensorWeatherDewPoint.setAcceptDrops(False)
        self.sensorWeatherDewPoint.setToolTip("")
        self.sensorWeatherDewPoint.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.sensorWeatherDewPoint.setMaxLength(8)
        self.sensorWeatherDewPoint.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorWeatherDewPoint.setReadOnly(True)
        self.sensorWeatherDewPoint.setObjectName("sensorWeatherDewPoint")
        self.sensorWeatherHumidity = QtWidgets.QLineEdit(self.sensorWeatherGroup)
        self.sensorWeatherHumidity.setEnabled(False)
        self.sensorWeatherHumidity.setGeometry(QtCore.QRect(10, 90, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.sensorWeatherHumidity.setFont(font)
        self.sensorWeatherHumidity.setMouseTracking(False)
        self.sensorWeatherHumidity.setFocusPolicy(QtCore.Qt.NoFocus)
        self.sensorWeatherHumidity.setAcceptDrops(False)
        self.sensorWeatherHumidity.setToolTip("")
        self.sensorWeatherHumidity.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.sensorWeatherHumidity.setMaxLength(8)
        self.sensorWeatherHumidity.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorWeatherHumidity.setReadOnly(True)
        self.sensorWeatherHumidity.setObjectName("sensorWeatherHumidity")
        self.sensorWeatherRainVol = QtWidgets.QLineEdit(self.sensorWeatherGroup)
        self.sensorWeatherRainVol.setEnabled(False)
        self.sensorWeatherRainVol.setGeometry(QtCore.QRect(10, 140, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.sensorWeatherRainVol.setFont(font)
        self.sensorWeatherRainVol.setMouseTracking(False)
        self.sensorWeatherRainVol.setFocusPolicy(QtCore.Qt.NoFocus)
        self.sensorWeatherRainVol.setAcceptDrops(False)
        self.sensorWeatherRainVol.setToolTip("")
        self.sensorWeatherRainVol.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.sensorWeatherRainVol.setMaxLength(8)
        self.sensorWeatherRainVol.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorWeatherRainVol.setReadOnly(True)
        self.sensorWeatherRainVol.setObjectName("sensorWeatherRainVol")
        self.sensorWeatherWindDir = QtWidgets.QLineEdit(self.sensorWeatherGroup)
        self.sensorWeatherWindDir.setEnabled(False)
        self.sensorWeatherWindDir.setGeometry(QtCore.QRect(10, 190, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.sensorWeatherWindDir.setFont(font)
        self.sensorWeatherWindDir.setMouseTracking(False)
        self.sensorWeatherWindDir.setFocusPolicy(QtCore.Qt.NoFocus)
        self.sensorWeatherWindDir.setAcceptDrops(False)
        self.sensorWeatherWindDir.setToolTip("")
        self.sensorWeatherWindDir.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.sensorWeatherWindDir.setMaxLength(8)
        self.sensorWeatherWindDir.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorWeatherWindDir.setReadOnly(True)
        self.sensorWeatherWindDir.setObjectName("sensorWeatherWindDir")
        self.sensorWeatherWindSpeed = QtWidgets.QLineEdit(self.sensorWeatherGroup)
        self.sensorWeatherWindSpeed.setEnabled(False)
        self.sensorWeatherWindSpeed.setGeometry(QtCore.QRect(10, 165, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.sensorWeatherWindSpeed.setFont(font)
        self.sensorWeatherWindSpeed.setMouseTracking(False)
        self.sensorWeatherWindSpeed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.sensorWeatherWindSpeed.setAcceptDrops(False)
        self.sensorWeatherWindSpeed.setToolTip("")
        self.sensorWeatherWindSpeed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.sensorWeatherWindSpeed.setMaxLength(8)
        self.sensorWeatherWindSpeed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorWeatherWindSpeed.setReadOnly(True)
        self.sensorWeatherWindSpeed.setObjectName("sensorWeatherWindSpeed")
        self.sensorWeatherCloudCover = QtWidgets.QLineEdit(self.sensorWeatherGroup)
        self.sensorWeatherCloudCover.setEnabled(False)
        self.sensorWeatherCloudCover.setGeometry(QtCore.QRect(10, 115, 51, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.sensorWeatherCloudCover.setFont(font)
        self.sensorWeatherCloudCover.setMouseTracking(False)
        self.sensorWeatherCloudCover.setFocusPolicy(QtCore.Qt.NoFocus)
        self.sensorWeatherCloudCover.setAcceptDrops(False)
        self.sensorWeatherCloudCover.setToolTip("")
        self.sensorWeatherCloudCover.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.sensorWeatherCloudCover.setMaxLength(8)
        self.sensorWeatherCloudCover.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorWeatherCloudCover.setReadOnly(True)
        self.sensorWeatherCloudCover.setObjectName("sensorWeatherCloudCover")
        self.sensorWeatherSQR = QtWidgets.QLineEdit(self.sensorWeatherGroup)
        self.sensorWeatherSQR.setEnabled(False)
        self.sensorWeatherSQR.setGeometry(QtCore.QRect(10, 215, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.sensorWeatherSQR.setFont(font)
        self.sensorWeatherSQR.setMouseTracking(False)
        self.sensorWeatherSQR.setFocusPolicy(QtCore.Qt.NoFocus)
        self.sensorWeatherSQR.setAcceptDrops(False)
        self.sensorWeatherSQR.setToolTip("")
        self.sensorWeatherSQR.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.sensorWeatherSQR.setMaxLength(8)
        self.sensorWeatherSQR.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorWeatherSQR.setReadOnly(True)
        self.sensorWeatherSQR.setObjectName("sensorWeatherSQR")
        self.gridLayout.addWidget(self.sensorWeatherGroup, 0, 6, 10, 1)
        self.onlineWeatherGroup = QtWidgets.QGroupBox(self.groupBox_51)
        self.onlineWeatherGroup.setMinimumSize(QtCore.QSize(75, 0))
        self.onlineWeatherGroup.setCheckable(True)
        self.onlineWeatherGroup.setChecked(False)
        self.onlineWeatherGroup.setObjectName("onlineWeatherGroup")
        self.onlineWeatherRainVol = QtWidgets.QLineEdit(self.onlineWeatherGroup)
        self.onlineWeatherRainVol.setEnabled(False)
        self.onlineWeatherRainVol.setGeometry(QtCore.QRect(10, 140, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherRainVol.setFont(font)
        self.onlineWeatherRainVol.setMouseTracking(False)
        self.onlineWeatherRainVol.setFocusPolicy(QtCore.Qt.NoFocus)
        self.onlineWeatherRainVol.setAcceptDrops(False)
        self.onlineWeatherRainVol.setToolTip("")
        self.onlineWeatherRainVol.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherRainVol.setMaxLength(8)
        self.onlineWeatherRainVol.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.onlineWeatherRainVol.setReadOnly(True)
        self.onlineWeatherRainVol.setObjectName("onlineWeatherRainVol")
        self.onlineWeatherWindSpeed = QtWidgets.QLineEdit(self.onlineWeatherGroup)
        self.onlineWeatherWindSpeed.setEnabled(False)
        self.onlineWeatherWindSpeed.setGeometry(QtCore.QRect(10, 165, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherWindSpeed.setFont(font)
        self.onlineWeatherWindSpeed.setMouseTracking(False)
        self.onlineWeatherWindSpeed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.onlineWeatherWindSpeed.setAcceptDrops(False)
        self.onlineWeatherWindSpeed.setToolTip("")
        self.onlineWeatherWindSpeed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherWindSpeed.setMaxLength(8)
        self.onlineWeatherWindSpeed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.onlineWeatherWindSpeed.setReadOnly(True)
        self.onlineWeatherWindSpeed.setObjectName("onlineWeatherWindSpeed")
        self.onlineWeatherCloudCover = QtWidgets.QLineEdit(self.onlineWeatherGroup)
        self.onlineWeatherCloudCover.setEnabled(False)
        self.onlineWeatherCloudCover.setGeometry(QtCore.QRect(10, 115, 51, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherCloudCover.setFont(font)
        self.onlineWeatherCloudCover.setMouseTracking(False)
        self.onlineWeatherCloudCover.setFocusPolicy(QtCore.Qt.NoFocus)
        self.onlineWeatherCloudCover.setAcceptDrops(False)
        self.onlineWeatherCloudCover.setToolTip("")
        self.onlineWeatherCloudCover.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherCloudCover.setMaxLength(8)
        self.onlineWeatherCloudCover.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.onlineWeatherCloudCover.setReadOnly(True)
        self.onlineWeatherCloudCover.setObjectName("onlineWeatherCloudCover")
        self.onlineWeatherTemp = QtWidgets.QLineEdit(self.onlineWeatherGroup)
        self.onlineWeatherTemp.setEnabled(False)
        self.onlineWeatherTemp.setGeometry(QtCore.QRect(10, 15, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherTemp.setFont(font)
        self.onlineWeatherTemp.setMouseTracking(False)
        self.onlineWeatherTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.onlineWeatherTemp.setAcceptDrops(False)
        self.onlineWeatherTemp.setToolTip("")
        self.onlineWeatherTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherTemp.setMaxLength(8)
        self.onlineWeatherTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.onlineWeatherTemp.setReadOnly(True)
        self.onlineWeatherTemp.setObjectName("onlineWeatherTemp")
        self.onlineWeatherPress = QtWidgets.QLineEdit(self.onlineWeatherGroup)
        self.onlineWeatherPress.setEnabled(False)
        self.onlineWeatherPress.setGeometry(QtCore.QRect(10, 40, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherPress.setFont(font)
        self.onlineWeatherPress.setMouseTracking(False)
        self.onlineWeatherPress.setFocusPolicy(QtCore.Qt.NoFocus)
        self.onlineWeatherPress.setAcceptDrops(False)
        self.onlineWeatherPress.setToolTip("")
        self.onlineWeatherPress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherPress.setMaxLength(8)
        self.onlineWeatherPress.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.onlineWeatherPress.setReadOnly(True)
        self.onlineWeatherPress.setObjectName("onlineWeatherPress")
        self.onlineWeatherHumidity = QtWidgets.QLineEdit(self.onlineWeatherGroup)
        self.onlineWeatherHumidity.setEnabled(False)
        self.onlineWeatherHumidity.setGeometry(QtCore.QRect(10, 90, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherHumidity.setFont(font)
        self.onlineWeatherHumidity.setMouseTracking(False)
        self.onlineWeatherHumidity.setFocusPolicy(QtCore.Qt.NoFocus)
        self.onlineWeatherHumidity.setAcceptDrops(False)
        self.onlineWeatherHumidity.setToolTip("")
        self.onlineWeatherHumidity.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherHumidity.setMaxLength(8)
        self.onlineWeatherHumidity.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.onlineWeatherHumidity.setReadOnly(True)
        self.onlineWeatherHumidity.setObjectName("onlineWeatherHumidity")
        self.onlineWeatherWindDir = QtWidgets.QLineEdit(self.onlineWeatherGroup)
        self.onlineWeatherWindDir.setEnabled(False)
        self.onlineWeatherWindDir.setGeometry(QtCore.QRect(10, 190, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherWindDir.setFont(font)
        self.onlineWeatherWindDir.setMouseTracking(False)
        self.onlineWeatherWindDir.setFocusPolicy(QtCore.Qt.NoFocus)
        self.onlineWeatherWindDir.setAcceptDrops(False)
        self.onlineWeatherWindDir.setToolTip("")
        self.onlineWeatherWindDir.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherWindDir.setMaxLength(8)
        self.onlineWeatherWindDir.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.onlineWeatherWindDir.setReadOnly(True)
        self.onlineWeatherWindDir.setObjectName("onlineWeatherWindDir")
        self.onlineWeatherDewPoint = QtWidgets.QLineEdit(self.onlineWeatherGroup)
        self.onlineWeatherDewPoint.setEnabled(False)
        self.onlineWeatherDewPoint.setGeometry(QtCore.QRect(10, 65, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherDewPoint.setFont(font)
        self.onlineWeatherDewPoint.setMouseTracking(False)
        self.onlineWeatherDewPoint.setFocusPolicy(QtCore.Qt.NoFocus)
        self.onlineWeatherDewPoint.setAcceptDrops(False)
        self.onlineWeatherDewPoint.setToolTip("")
        self.onlineWeatherDewPoint.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherDewPoint.setMaxLength(8)
        self.onlineWeatherDewPoint.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.onlineWeatherDewPoint.setReadOnly(True)
        self.onlineWeatherDewPoint.setObjectName("onlineWeatherDewPoint")
        self.onlineWeatherSQR = QtWidgets.QLineEdit(self.onlineWeatherGroup)
        self.onlineWeatherSQR.setEnabled(False)
        self.onlineWeatherSQR.setGeometry(QtCore.QRect(10, 215, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherSQR.setFont(font)
        self.onlineWeatherSQR.setMouseTracking(False)
        self.onlineWeatherSQR.setFocusPolicy(QtCore.Qt.NoFocus)
        self.onlineWeatherSQR.setAcceptDrops(False)
        self.onlineWeatherSQR.setToolTip("")
        self.onlineWeatherSQR.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherSQR.setMaxLength(8)
        self.onlineWeatherSQR.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.onlineWeatherSQR.setReadOnly(True)
        self.onlineWeatherSQR.setObjectName("onlineWeatherSQR")
        self.gridLayout.addWidget(self.onlineWeatherGroup, 0, 5, 10, 1)
        self.label_54 = QtWidgets.QLabel(self.groupBox_51)
        self.label_54.setMinimumSize(QtCore.QSize(60, 0))
        self.label_54.setMaximumSize(QtCore.QSize(16777215, 4))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_54.setFont(font)
        self.label_54.setText("")
        self.label_54.setObjectName("label_54")
        self.gridLayout.addWidget(self.label_54, 0, 0, 1, 2)
        self.label_32 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_32.setFont(font)
        self.label_32.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_32.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_32.setObjectName("label_32")
        self.gridLayout.addWidget(self.label_32, 7, 0, 1, 1)
        self.label_33 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_33.setFont(font)
        self.label_33.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_33.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_33.setObjectName("label_33")
        self.gridLayout.addWidget(self.label_33, 8, 0, 1, 1)
        self.label_143 = QtWidgets.QLabel(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_143.setFont(font)
        self.label_143.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_143.setWordWrap(False)
        self.label_143.setObjectName("label_143")
        self.gridLayout.addWidget(self.label_143, 5, 1, 1, 1)
        self.skymeterGroup = QtWidgets.QGroupBox(self.groupBox_51)
        self.skymeterGroup.setMinimumSize(QtCore.QSize(75, 0))
        self.skymeterGroup.setObjectName("skymeterGroup")
        self.skymeterSQR = QtWidgets.QLineEdit(self.skymeterGroup)
        self.skymeterSQR.setEnabled(False)
        self.skymeterSQR.setGeometry(QtCore.QRect(10, 215, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.skymeterSQR.setFont(font)
        self.skymeterSQR.setMouseTracking(False)
        self.skymeterSQR.setFocusPolicy(QtCore.Qt.NoFocus)
        self.skymeterSQR.setAcceptDrops(False)
        self.skymeterSQR.setToolTip("")
        self.skymeterSQR.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.skymeterSQR.setMaxLength(8)
        self.skymeterSQR.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterSQR.setReadOnly(True)
        self.skymeterSQR.setObjectName("skymeterSQR")
        self.skymeterTemp = QtWidgets.QLineEdit(self.skymeterGroup)
        self.skymeterTemp.setEnabled(False)
        self.skymeterTemp.setGeometry(QtCore.QRect(10, 15, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.skymeterTemp.setFont(font)
        self.skymeterTemp.setMouseTracking(False)
        self.skymeterTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.skymeterTemp.setAcceptDrops(False)
        self.skymeterTemp.setToolTip("")
        self.skymeterTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.skymeterTemp.setMaxLength(8)
        self.skymeterTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterTemp.setReadOnly(True)
        self.skymeterTemp.setObjectName("skymeterTemp")
        self.skymeterPress = QtWidgets.QLineEdit(self.skymeterGroup)
        self.skymeterPress.setEnabled(False)
        self.skymeterPress.setGeometry(QtCore.QRect(10, 40, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.skymeterPress.setFont(font)
        self.skymeterPress.setMouseTracking(False)
        self.skymeterPress.setFocusPolicy(QtCore.Qt.NoFocus)
        self.skymeterPress.setAcceptDrops(False)
        self.skymeterPress.setToolTip("")
        self.skymeterPress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.skymeterPress.setMaxLength(8)
        self.skymeterPress.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterPress.setReadOnly(True)
        self.skymeterPress.setObjectName("skymeterPress")
        self.skymeterRainVol = QtWidgets.QLineEdit(self.skymeterGroup)
        self.skymeterRainVol.setEnabled(False)
        self.skymeterRainVol.setGeometry(QtCore.QRect(10, 140, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.skymeterRainVol.setFont(font)
        self.skymeterRainVol.setMouseTracking(False)
        self.skymeterRainVol.setFocusPolicy(QtCore.Qt.NoFocus)
        self.skymeterRainVol.setAcceptDrops(False)
        self.skymeterRainVol.setToolTip("")
        self.skymeterRainVol.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.skymeterRainVol.setMaxLength(8)
        self.skymeterRainVol.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterRainVol.setReadOnly(True)
        self.skymeterRainVol.setObjectName("skymeterRainVol")
        self.skymeterWindDir = QtWidgets.QLineEdit(self.skymeterGroup)
        self.skymeterWindDir.setEnabled(False)
        self.skymeterWindDir.setGeometry(QtCore.QRect(10, 190, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.skymeterWindDir.setFont(font)
        self.skymeterWindDir.setMouseTracking(False)
        self.skymeterWindDir.setFocusPolicy(QtCore.Qt.NoFocus)
        self.skymeterWindDir.setAcceptDrops(False)
        self.skymeterWindDir.setToolTip("")
        self.skymeterWindDir.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.skymeterWindDir.setMaxLength(8)
        self.skymeterWindDir.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterWindDir.setReadOnly(True)
        self.skymeterWindDir.setObjectName("skymeterWindDir")
        self.skymeterDewPoint = QtWidgets.QLineEdit(self.skymeterGroup)
        self.skymeterDewPoint.setEnabled(False)
        self.skymeterDewPoint.setGeometry(QtCore.QRect(10, 65, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.skymeterDewPoint.setFont(font)
        self.skymeterDewPoint.setMouseTracking(False)
        self.skymeterDewPoint.setFocusPolicy(QtCore.Qt.NoFocus)
        self.skymeterDewPoint.setAcceptDrops(False)
        self.skymeterDewPoint.setToolTip("")
        self.skymeterDewPoint.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.skymeterDewPoint.setMaxLength(8)
        self.skymeterDewPoint.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterDewPoint.setReadOnly(True)
        self.skymeterDewPoint.setObjectName("skymeterDewPoint")
        self.skymeterWindSpeed = QtWidgets.QLineEdit(self.skymeterGroup)
        self.skymeterWindSpeed.setEnabled(False)
        self.skymeterWindSpeed.setGeometry(QtCore.QRect(10, 165, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.skymeterWindSpeed.setFont(font)
        self.skymeterWindSpeed.setMouseTracking(False)
        self.skymeterWindSpeed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.skymeterWindSpeed.setAcceptDrops(False)
        self.skymeterWindSpeed.setToolTip("")
        self.skymeterWindSpeed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.skymeterWindSpeed.setMaxLength(8)
        self.skymeterWindSpeed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterWindSpeed.setReadOnly(True)
        self.skymeterWindSpeed.setObjectName("skymeterWindSpeed")
        self.skymeterHumidity = QtWidgets.QLineEdit(self.skymeterGroup)
        self.skymeterHumidity.setEnabled(False)
        self.skymeterHumidity.setGeometry(QtCore.QRect(10, 90, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.skymeterHumidity.setFont(font)
        self.skymeterHumidity.setMouseTracking(False)
        self.skymeterHumidity.setFocusPolicy(QtCore.Qt.NoFocus)
        self.skymeterHumidity.setAcceptDrops(False)
        self.skymeterHumidity.setToolTip("")
        self.skymeterHumidity.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.skymeterHumidity.setMaxLength(8)
        self.skymeterHumidity.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterHumidity.setReadOnly(True)
        self.skymeterHumidity.setObjectName("skymeterHumidity")
        self.skymeterCloudCover = QtWidgets.QLineEdit(self.skymeterGroup)
        self.skymeterCloudCover.setEnabled(False)
        self.skymeterCloudCover.setGeometry(QtCore.QRect(10, 115, 51, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.skymeterCloudCover.setFont(font)
        self.skymeterCloudCover.setMouseTracking(False)
        self.skymeterCloudCover.setFocusPolicy(QtCore.Qt.NoFocus)
        self.skymeterCloudCover.setAcceptDrops(False)
        self.skymeterCloudCover.setToolTip("")
        self.skymeterCloudCover.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.skymeterCloudCover.setMaxLength(8)
        self.skymeterCloudCover.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterCloudCover.setReadOnly(True)
        self.skymeterCloudCover.setObjectName("skymeterCloudCover")
        self.gridLayout.addWidget(self.skymeterGroup, 0, 3, 10, 1)
        spacerItem = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 8, 1, 1)
        self.setRefractionManual = QtWidgets.QPushButton(self.groupBox_51)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.setRefractionManual.setFont(font)
        self.setRefractionManual.setFlat(False)
        self.setRefractionManual.setObjectName("setRefractionManual")
        self.gridLayout.addWidget(self.setRefractionManual, 4, 11, 2, 1)
        self.directWeatherGroup = QtWidgets.QGroupBox(self.groupBox_51)
        self.directWeatherGroup.setEnabled(True)
        self.directWeatherGroup.setMinimumSize(QtCore.QSize(75, 0))
        self.directWeatherGroup.setCheckable(True)
        self.directWeatherGroup.setChecked(False)
        self.directWeatherGroup.setObjectName("directWeatherGroup")
        self.directWeatherTemp = QtWidgets.QLineEdit(self.directWeatherGroup)
        self.directWeatherTemp.setEnabled(False)
        self.directWeatherTemp.setGeometry(QtCore.QRect(10, 15, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.directWeatherTemp.setFont(font)
        self.directWeatherTemp.setMouseTracking(False)
        self.directWeatherTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.directWeatherTemp.setAcceptDrops(False)
        self.directWeatherTemp.setToolTip("")
        self.directWeatherTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.directWeatherTemp.setMaxLength(8)
        self.directWeatherTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.directWeatherTemp.setReadOnly(True)
        self.directWeatherTemp.setObjectName("directWeatherTemp")
        self.directWeatherPress = QtWidgets.QLineEdit(self.directWeatherGroup)
        self.directWeatherPress.setEnabled(False)
        self.directWeatherPress.setGeometry(QtCore.QRect(10, 40, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.directWeatherPress.setFont(font)
        self.directWeatherPress.setMouseTracking(False)
        self.directWeatherPress.setFocusPolicy(QtCore.Qt.NoFocus)
        self.directWeatherPress.setAcceptDrops(False)
        self.directWeatherPress.setToolTip("")
        self.directWeatherPress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.directWeatherPress.setMaxLength(8)
        self.directWeatherPress.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.directWeatherPress.setReadOnly(True)
        self.directWeatherPress.setObjectName("directWeatherPress")
        self.directWeatherDewPoint = QtWidgets.QLineEdit(self.directWeatherGroup)
        self.directWeatherDewPoint.setEnabled(False)
        self.directWeatherDewPoint.setGeometry(QtCore.QRect(10, 65, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.directWeatherDewPoint.setFont(font)
        self.directWeatherDewPoint.setMouseTracking(False)
        self.directWeatherDewPoint.setFocusPolicy(QtCore.Qt.NoFocus)
        self.directWeatherDewPoint.setAcceptDrops(False)
        self.directWeatherDewPoint.setToolTip("")
        self.directWeatherDewPoint.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.directWeatherDewPoint.setMaxLength(8)
        self.directWeatherDewPoint.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.directWeatherDewPoint.setReadOnly(True)
        self.directWeatherDewPoint.setObjectName("directWeatherDewPoint")
        self.directWeatherHumidity = QtWidgets.QLineEdit(self.directWeatherGroup)
        self.directWeatherHumidity.setEnabled(False)
        self.directWeatherHumidity.setGeometry(QtCore.QRect(10, 90, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.directWeatherHumidity.setFont(font)
        self.directWeatherHumidity.setMouseTracking(False)
        self.directWeatherHumidity.setFocusPolicy(QtCore.Qt.NoFocus)
        self.directWeatherHumidity.setAcceptDrops(False)
        self.directWeatherHumidity.setToolTip("")
        self.directWeatherHumidity.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.directWeatherHumidity.setMaxLength(8)
        self.directWeatherHumidity.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.directWeatherHumidity.setReadOnly(True)
        self.directWeatherHumidity.setObjectName("directWeatherHumidity")
        self.gridLayout.addWidget(self.directWeatherGroup, 0, 7, 10, 1)
        self.refractionGroup_2 = QtWidgets.QGroupBox(self.Environ)
        self.refractionGroup_2.setGeometry(QtCore.QRect(10, 290, 766, 131))
        self.refractionGroup_2.setMinimumSize(QtCore.QSize(110, 0))
        self.refractionGroup_2.setToolTip("")
        self.refractionGroup_2.setProperty("large", True)
        self.refractionGroup_2.setObjectName("refractionGroup_2")
        self.picClearOutside = QtWidgets.QLabel(self.refractionGroup_2)
        self.picClearOutside.setGeometry(QtCore.QRect(5, 15, 626, 111))
        self.picClearOutside.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.picClearOutside.setFrameShadow(QtWidgets.QFrame.Plain)
        self.picClearOutside.setLineWidth(0)
        self.picClearOutside.setMidLineWidth(0)
        self.picClearOutside.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.picClearOutside.setObjectName("picClearOutside")
        self.mainTabWidget.addTab(self.Environ, "")
        self.Almanac = QtWidgets.QWidget()
        self.Almanac.setObjectName("Almanac")
        self.refractionGroup_4 = QtWidgets.QGroupBox(self.Almanac)
        self.refractionGroup_4.setGeometry(QtCore.QRect(5, 10, 496, 421))
        self.refractionGroup_4.setMinimumSize(QtCore.QSize(110, 0))
        self.refractionGroup_4.setToolTip("")
        self.refractionGroup_4.setProperty("large", True)
        self.refractionGroup_4.setObjectName("refractionGroup_4")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.refractionGroup_4)
        self.gridLayout_2.setContentsMargins(4, 8, 4, 4)
        self.gridLayout_2.setSpacing(0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.twilight = QtWidgets.QWidget(self.refractionGroup_4)
        self.twilight.setEnabled(True)
        self.twilight.setAutoFillBackground(True)
        self.twilight.setObjectName("twilight")
        self.gridLayout_2.addWidget(self.twilight, 0, 0, 1, 1)
        self.refractionGroup_5 = QtWidgets.QGroupBox(self.Almanac)
        self.refractionGroup_5.setGeometry(QtCore.QRect(510, 10, 271, 156))
        self.refractionGroup_5.setMinimumSize(QtCore.QSize(110, 0))
        self.refractionGroup_5.setToolTip("")
        self.refractionGroup_5.setProperty("large", True)
        self.refractionGroup_5.setObjectName("refractionGroup_5")
        self.twilightEvents = QtWidgets.QTextBrowser(self.refractionGroup_5)
        self.twilightEvents.setGeometry(QtCore.QRect(5, 25, 261, 121))
        self.twilightEvents.setObjectName("twilightEvents")
        self.refractionGroup_3 = QtWidgets.QGroupBox(self.Almanac)
        self.refractionGroup_3.setGeometry(QtCore.QRect(510, 230, 271, 201))
        self.refractionGroup_3.setMinimumSize(QtCore.QSize(110, 0))
        self.refractionGroup_3.setToolTip("")
        self.refractionGroup_3.setProperty("large", True)
        self.refractionGroup_3.setObjectName("refractionGroup_3")
        self.moonPhaseIllumination = QtWidgets.QLineEdit(self.refractionGroup_3)
        self.moonPhaseIllumination.setEnabled(False)
        self.moonPhaseIllumination.setGeometry(QtCore.QRect(120, 175, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moonPhaseIllumination.setFont(font)
        self.moonPhaseIllumination.setMouseTracking(False)
        self.moonPhaseIllumination.setFocusPolicy(QtCore.Qt.NoFocus)
        self.moonPhaseIllumination.setAcceptDrops(False)
        self.moonPhaseIllumination.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.moonPhaseIllumination.setMaxLength(8)
        self.moonPhaseIllumination.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.moonPhaseIllumination.setReadOnly(True)
        self.moonPhaseIllumination.setObjectName("moonPhaseIllumination")
        self.moonPhaseText = QtWidgets.QLineEdit(self.refractionGroup_3)
        self.moonPhaseText.setEnabled(False)
        self.moonPhaseText.setGeometry(QtCore.QRect(145, 15, 116, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moonPhaseText.setFont(font)
        self.moonPhaseText.setMouseTracking(False)
        self.moonPhaseText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.moonPhaseText.setAcceptDrops(False)
        self.moonPhaseText.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.moonPhaseText.setMaxLength(20)
        self.moonPhaseText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.moonPhaseText.setReadOnly(True)
        self.moonPhaseText.setObjectName("moonPhaseText")
        self.label_146 = QtWidgets.QLabel(self.refractionGroup_3)
        self.label_146.setGeometry(QtCore.QRect(175, 175, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_146.setFont(font)
        self.label_146.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_146.setWordWrap(False)
        self.label_146.setObjectName("label_146")
        self.moonPic = QtWidgets.QLabel(self.refractionGroup_3)
        self.moonPic.setGeometry(QtCore.QRect(60, 40, 130, 130))
        self.moonPic.setObjectName("moonPic")
        self.moonPhaseDegree = QtWidgets.QLineEdit(self.refractionGroup_3)
        self.moonPhaseDegree.setEnabled(False)
        self.moonPhaseDegree.setGeometry(QtCore.QRect(10, 15, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moonPhaseDegree.setFont(font)
        self.moonPhaseDegree.setMouseTracking(False)
        self.moonPhaseDegree.setFocusPolicy(QtCore.Qt.NoFocus)
        self.moonPhaseDegree.setAcceptDrops(False)
        self.moonPhaseDegree.setToolTip("")
        self.moonPhaseDegree.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.moonPhaseDegree.setMaxLength(8)
        self.moonPhaseDegree.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.moonPhaseDegree.setReadOnly(True)
        self.moonPhaseDegree.setObjectName("moonPhaseDegree")
        self.label_155 = QtWidgets.QLabel(self.refractionGroup_3)
        self.label_155.setGeometry(QtCore.QRect(50, 15, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_155.setFont(font)
        self.label_155.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_155.setWordWrap(False)
        self.label_155.setObjectName("label_155")
        self.moonPhasePercent = QtWidgets.QLineEdit(self.refractionGroup_3)
        self.moonPhasePercent.setEnabled(False)
        self.moonPhasePercent.setGeometry(QtCore.QRect(10, 175, 26, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moonPhasePercent.setFont(font)
        self.moonPhasePercent.setMouseTracking(False)
        self.moonPhasePercent.setFocusPolicy(QtCore.Qt.NoFocus)
        self.moonPhasePercent.setAcceptDrops(False)
        self.moonPhasePercent.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.moonPhasePercent.setMaxLength(8)
        self.moonPhasePercent.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.moonPhasePercent.setReadOnly(True)
        self.moonPhasePercent.setObjectName("moonPhasePercent")
        self.label_161 = QtWidgets.QLabel(self.refractionGroup_3)
        self.label_161.setGeometry(QtCore.QRect(40, 175, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_161.setFont(font)
        self.label_161.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_161.setWordWrap(False)
        self.label_161.setObjectName("label_161")
        self.refractionGroup_6 = QtWidgets.QGroupBox(self.Almanac)
        self.refractionGroup_6.setGeometry(QtCore.QRect(510, 175, 271, 46))
        self.refractionGroup_6.setMinimumSize(QtCore.QSize(110, 0))
        self.refractionGroup_6.setToolTip("")
        self.refractionGroup_6.setProperty("large", True)
        self.refractionGroup_6.setObjectName("refractionGroup_6")
        self.lunarNodes = QtWidgets.QLineEdit(self.refractionGroup_6)
        self.lunarNodes.setEnabled(False)
        self.lunarNodes.setGeometry(QtCore.QRect(10, 20, 251, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.lunarNodes.setFont(font)
        self.lunarNodes.setMouseTracking(False)
        self.lunarNodes.setFocusPolicy(QtCore.Qt.NoFocus)
        self.lunarNodes.setAcceptDrops(False)
        self.lunarNodes.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.lunarNodes.setMaxLength(20)
        self.lunarNodes.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.lunarNodes.setReadOnly(True)
        self.lunarNodes.setObjectName("lunarNodes")
        self.mainTabWidget.addTab(self.Almanac, "")
        self.Modelpoints = QtWidgets.QWidget()
        self.Modelpoints.setObjectName("Modelpoints")
        self.line_40 = QtWidgets.QFrame(self.Modelpoints)
        self.line_40.setGeometry(QtCore.QRect(390, 10, 11, 416))
        self.line_40.setStyleSheet("")
        self.line_40.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_40.setLineWidth(2)
        self.line_40.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_40.setObjectName("line_40")
        self.groupBox_2 = QtWidgets.QGroupBox(self.Modelpoints)
        self.groupBox_2.setGeometry(QtCore.QRect(15, 25, 366, 76))
        self.groupBox_2.setProperty("large", True)
        self.groupBox_2.setObjectName("groupBox_2")
        self.buildPFileName = QtWidgets.QLineEdit(self.groupBox_2)
        self.buildPFileName.setEnabled(False)
        self.buildPFileName.setGeometry(QtCore.QRect(5, 20, 356, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.buildPFileName.setFont(font)
        self.buildPFileName.setMouseTracking(False)
        self.buildPFileName.setAcceptDrops(False)
        self.buildPFileName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.buildPFileName.setText("")
        self.buildPFileName.setReadOnly(True)
        self.buildPFileName.setObjectName("buildPFileName")
        self.saveBuildPointsAs = QtWidgets.QPushButton(self.groupBox_2)
        self.saveBuildPointsAs.setEnabled(True)
        self.saveBuildPointsAs.setGeometry(QtCore.QRect(280, 45, 81, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.saveBuildPointsAs.setFont(font)
        self.saveBuildPointsAs.setStyleSheet("")
        self.saveBuildPointsAs.setObjectName("saveBuildPointsAs")
        self.saveBuildPoints = QtWidgets.QPushButton(self.groupBox_2)
        self.saveBuildPoints.setEnabled(True)
        self.saveBuildPoints.setGeometry(QtCore.QRect(195, 45, 81, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.saveBuildPoints.setFont(font)
        self.saveBuildPoints.setStyleSheet("")
        self.saveBuildPoints.setObjectName("saveBuildPoints")
        self.genBuildFile = QtWidgets.QPushButton(self.groupBox_2)
        self.genBuildFile.setGeometry(QtCore.QRect(5, 45, 101, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.genBuildFile.setFont(font)
        self.genBuildFile.setStyleSheet("")
        self.genBuildFile.setObjectName("genBuildFile")
        self.loadBuildPoints = QtWidgets.QPushButton(self.groupBox_2)
        self.loadBuildPoints.setGeometry(QtCore.QRect(110, 45, 81, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.loadBuildPoints.setFont(font)
        self.loadBuildPoints.setStyleSheet("")
        self.loadBuildPoints.setObjectName("loadBuildPoints")
        self.groupBox_3 = QtWidgets.QGroupBox(self.Modelpoints)
        self.groupBox_3.setGeometry(QtCore.QRect(15, 125, 366, 81))
        self.groupBox_3.setProperty("large", True)
        self.groupBox_3.setObjectName("groupBox_3")
        self.label_178 = QtWidgets.QLabel(self.groupBox_3)
        self.label_178.setGeometry(QtCore.QRect(5, 25, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_178.setFont(font)
        self.label_178.setObjectName("label_178")
        self.numberGridPointsRow = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.numberGridPointsRow.setGeometry(QtCore.QRect(40, 25, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberGridPointsRow.setFont(font)
        self.numberGridPointsRow.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberGridPointsRow.setDecimals(0)
        self.numberGridPointsRow.setMinimum(2.0)
        self.numberGridPointsRow.setMaximum(8.0)
        self.numberGridPointsRow.setSingleStep(1.0)
        self.numberGridPointsRow.setProperty("value", 5.0)
        self.numberGridPointsRow.setObjectName("numberGridPointsRow")
        self.label_179 = QtWidgets.QLabel(self.groupBox_3)
        self.label_179.setGeometry(QtCore.QRect(90, 25, 26, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_179.setFont(font)
        self.label_179.setObjectName("label_179")
        self.label_190 = QtWidgets.QLabel(self.groupBox_3)
        self.label_190.setGeometry(QtCore.QRect(170, 25, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_190.setFont(font)
        self.label_190.setObjectName("label_190")
        self.label_203 = QtWidgets.QLabel(self.groupBox_3)
        self.label_203.setGeometry(QtCore.QRect(250, 25, 16, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_203.setFont(font)
        self.label_203.setAlignment(QtCore.Qt.AlignCenter)
        self.label_203.setWordWrap(False)
        self.label_203.setObjectName("label_203")
        self.altitudeMax = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.altitudeMax.setGeometry(QtCore.QRect(310, 25, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeMax.setFont(font)
        self.altitudeMax.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeMax.setDecimals(0)
        self.altitudeMax.setMinimum(5.0)
        self.altitudeMax.setMaximum(85.0)
        self.altitudeMax.setSingleStep(5.0)
        self.altitudeMax.setProperty("value", 75.0)
        self.altitudeMax.setObjectName("altitudeMax")
        self.label_144 = QtWidgets.QLabel(self.groupBox_3)
        self.label_144.setGeometry(QtCore.QRect(350, 25, 16, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_144.setFont(font)
        self.label_144.setAlignment(QtCore.Qt.AlignCenter)
        self.label_144.setWordWrap(False)
        self.label_144.setObjectName("label_144")
        self.numberGridPointsCol = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.numberGridPointsCol.setGeometry(QtCore.QRect(120, 25, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberGridPointsCol.setFont(font)
        self.numberGridPointsCol.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberGridPointsCol.setDecimals(0)
        self.numberGridPointsCol.setMinimum(2.0)
        self.numberGridPointsCol.setMaximum(15.0)
        self.numberGridPointsCol.setSingleStep(2.0)
        self.numberGridPointsCol.setProperty("value", 6.0)
        self.numberGridPointsCol.setObjectName("numberGridPointsCol")
        self.label_180 = QtWidgets.QLabel(self.groupBox_3)
        self.label_180.setGeometry(QtCore.QRect(265, 25, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_180.setFont(font)
        self.label_180.setObjectName("label_180")
        self.genBuildGrid = QtWidgets.QPushButton(self.groupBox_3)
        self.genBuildGrid.setGeometry(QtCore.QRect(5, 50, 356, 26))
        self.genBuildGrid.setObjectName("genBuildGrid")
        self.altitudeMin = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.altitudeMin.setGeometry(QtCore.QRect(210, 25, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeMin.setFont(font)
        self.altitudeMin.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeMin.setDecimals(0)
        self.altitudeMin.setMinimum(5.0)
        self.altitudeMin.setMaximum(85.0)
        self.altitudeMin.setSingleStep(5.0)
        self.altitudeMin.setProperty("value", 30.0)
        self.altitudeMin.setObjectName("altitudeMin")
        self.groupBox_4 = QtWidgets.QGroupBox(self.Modelpoints)
        self.groupBox_4.setGeometry(QtCore.QRect(15, 305, 366, 51))
        self.groupBox_4.setProperty("large", True)
        self.groupBox_4.setObjectName("groupBox_4")
        self.genBuildMax = QtWidgets.QPushButton(self.groupBox_4)
        self.genBuildMax.setGeometry(QtCore.QRect(5, 20, 86, 26))
        self.genBuildMax.setObjectName("genBuildMax")
        self.genBuildMed = QtWidgets.QPushButton(self.groupBox_4)
        self.genBuildMed.setGeometry(QtCore.QRect(95, 20, 86, 26))
        self.genBuildMed.setObjectName("genBuildMed")
        self.genBuildMin = QtWidgets.QPushButton(self.groupBox_4)
        self.genBuildMin.setGeometry(QtCore.QRect(275, 20, 86, 26))
        self.genBuildMin.setObjectName("genBuildMin")
        self.genBuildNorm = QtWidgets.QPushButton(self.groupBox_4)
        self.genBuildNorm.setGeometry(QtCore.QRect(185, 20, 86, 26))
        self.genBuildNorm.setObjectName("genBuildNorm")
        self.groupBox_5 = QtWidgets.QGroupBox(self.Modelpoints)
        self.groupBox_5.setGeometry(QtCore.QRect(410, 100, 366, 76))
        self.groupBox_5.setProperty("large", True)
        self.groupBox_5.setObjectName("groupBox_5")
        self.label_188 = QtWidgets.QLabel(self.groupBox_5)
        self.label_188.setGeometry(QtCore.QRect(5, 20, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_188.setFont(font)
        self.label_188.setObjectName("label_188")
        self.numberDSOPoints = QtWidgets.QDoubleSpinBox(self.groupBox_5)
        self.numberDSOPoints.setGeometry(QtCore.QRect(45, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberDSOPoints.setFont(font)
        self.numberDSOPoints.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberDSOPoints.setDecimals(0)
        self.numberDSOPoints.setMinimum(5.0)
        self.numberDSOPoints.setMaximum(50.0)
        self.numberDSOPoints.setSingleStep(1.0)
        self.numberDSOPoints.setProperty("value", 15.0)
        self.numberDSOPoints.setObjectName("numberDSOPoints")
        self.durationDSO = QtWidgets.QDoubleSpinBox(self.groupBox_5)
        self.durationDSO.setGeometry(QtCore.QRect(165, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.durationDSO.setFont(font)
        self.durationDSO.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.durationDSO.setDecimals(0)
        self.durationDSO.setMinimum(1.0)
        self.durationDSO.setMaximum(15.0)
        self.durationDSO.setSingleStep(1.0)
        self.durationDSO.setProperty("value", 7.0)
        self.durationDSO.setObjectName("durationDSO")
        self.timeShiftDSO = QtWidgets.QDoubleSpinBox(self.groupBox_5)
        self.timeShiftDSO.setGeometry(QtCore.QRect(305, 20, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeShiftDSO.setFont(font)
        self.timeShiftDSO.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeShiftDSO.setDecimals(0)
        self.timeShiftDSO.setMinimum(-12.0)
        self.timeShiftDSO.setMaximum(12.0)
        self.timeShiftDSO.setSingleStep(1.0)
        self.timeShiftDSO.setProperty("value", 0.0)
        self.timeShiftDSO.setObjectName("timeShiftDSO")
        self.label_204 = QtWidgets.QLabel(self.groupBox_5)
        self.label_204.setGeometry(QtCore.QRect(350, 20, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_204.setFont(font)
        self.label_204.setObjectName("label_204")
        self.label_189 = QtWidgets.QLabel(self.groupBox_5)
        self.label_189.setGeometry(QtCore.QRect(245, 20, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_189.setFont(font)
        self.label_189.setObjectName("label_189")
        self.label_303 = QtWidgets.QLabel(self.groupBox_5)
        self.label_303.setGeometry(QtCore.QRect(220, 20, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_303.setFont(font)
        self.label_303.setObjectName("label_303")
        self.label_181 = QtWidgets.QLabel(self.groupBox_5)
        self.label_181.setGeometry(QtCore.QRect(110, 20, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_181.setFont(font)
        self.label_181.setObjectName("label_181")
        self.genBuildDSO = QtWidgets.QPushButton(self.groupBox_5)
        self.genBuildDSO.setGeometry(QtCore.QRect(5, 45, 356, 26))
        self.genBuildDSO.setObjectName("genBuildDSO")
        self.groupBox_6 = QtWidgets.QGroupBox(self.Modelpoints)
        self.groupBox_6.setGeometry(QtCore.QRect(410, 25, 366, 51))
        self.groupBox_6.setProperty("large", True)
        self.groupBox_6.setObjectName("groupBox_6")
        self.genBuildSpiralMin = QtWidgets.QPushButton(self.groupBox_6)
        self.genBuildSpiralMin.setGeometry(QtCore.QRect(275, 20, 86, 26))
        self.genBuildSpiralMin.setObjectName("genBuildSpiralMin")
        self.genBuildSpiralNorm = QtWidgets.QPushButton(self.groupBox_6)
        self.genBuildSpiralNorm.setGeometry(QtCore.QRect(185, 20, 86, 26))
        self.genBuildSpiralNorm.setObjectName("genBuildSpiralNorm")
        self.genBuildSpiralMed = QtWidgets.QPushButton(self.groupBox_6)
        self.genBuildSpiralMed.setGeometry(QtCore.QRect(95, 20, 86, 26))
        self.genBuildSpiralMed.setObjectName("genBuildSpiralMed")
        self.genBuildSpiralMax = QtWidgets.QPushButton(self.groupBox_6)
        self.genBuildSpiralMax.setGeometry(QtCore.QRect(5, 20, 86, 26))
        self.genBuildSpiralMax.setObjectName("genBuildSpiralMax")
        self.groupBoxCoordinateSystems_3 = QtWidgets.QGroupBox(self.Modelpoints)
        self.groupBoxCoordinateSystems_3.setGeometry(QtCore.QRect(410, 190, 361, 71))
        self.groupBoxCoordinateSystems_3.setObjectName("groupBoxCoordinateSystems_3")
        self.checkSortHL = QtWidgets.QRadioButton(self.groupBoxCoordinateSystems_3)
        self.checkSortHL.setGeometry(QtCore.QRect(245, 20, 111, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkSortHL.setFont(font)
        self.checkSortHL.setObjectName("checkSortHL")
        self.checkSortEW = QtWidgets.QRadioButton(self.groupBoxCoordinateSystems_3)
        self.checkSortEW.setGeometry(QtCore.QRect(130, 20, 106, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkSortEW.setFont(font)
        self.checkSortEW.setObjectName("checkSortEW")
        self.checkSortNothing = QtWidgets.QRadioButton(self.groupBoxCoordinateSystems_3)
        self.checkSortNothing.setGeometry(QtCore.QRect(15, 20, 106, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkSortNothing.setFont(font)
        self.checkSortNothing.setChecked(True)
        self.checkSortNothing.setObjectName("checkSortNothing")
        self.checkAvoidFlip = QtWidgets.QCheckBox(self.groupBoxCoordinateSystems_3)
        self.checkAvoidFlip.setGeometry(QtCore.QRect(15, 45, 336, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkAvoidFlip.setFont(font)
        self.checkAvoidFlip.setObjectName("checkAvoidFlip")
        self.groupBox_10 = QtWidgets.QGroupBox(self.Modelpoints)
        self.groupBox_10.setGeometry(QtCore.QRect(410, 280, 366, 146))
        self.groupBox_10.setProperty("large", True)
        self.groupBox_10.setObjectName("groupBox_10")
        self.saveHorizonMask = QtWidgets.QPushButton(self.groupBox_10)
        self.saveHorizonMask.setEnabled(True)
        self.saveHorizonMask.setGeometry(QtCore.QRect(100, 45, 81, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.saveHorizonMask.setFont(font)
        self.saveHorizonMask.setStyleSheet("")
        self.saveHorizonMask.setObjectName("saveHorizonMask")
        self.saveHorizonMaskAs = QtWidgets.QPushButton(self.groupBox_10)
        self.saveHorizonMaskAs.setEnabled(True)
        self.saveHorizonMaskAs.setGeometry(QtCore.QRect(190, 45, 81, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.saveHorizonMaskAs.setFont(font)
        self.saveHorizonMaskAs.setStyleSheet("")
        self.saveHorizonMaskAs.setObjectName("saveHorizonMaskAs")
        self.loadHorizonMask = QtWidgets.QPushButton(self.groupBox_10)
        self.loadHorizonMask.setGeometry(QtCore.QRect(10, 45, 81, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.loadHorizonMask.setFont(font)
        self.loadHorizonMask.setStyleSheet("")
        self.loadHorizonMask.setObjectName("loadHorizonMask")
        self.horizonFileName = QtWidgets.QLineEdit(self.groupBox_10)
        self.horizonFileName.setEnabled(False)
        self.horizonFileName.setGeometry(QtCore.QRect(10, 20, 351, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.horizonFileName.setFont(font)
        self.horizonFileName.setMouseTracking(False)
        self.horizonFileName.setFocusPolicy(QtCore.Qt.NoFocus)
        self.horizonFileName.setAcceptDrops(False)
        self.horizonFileName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.horizonFileName.setText("")
        self.horizonFileName.setReadOnly(True)
        self.horizonFileName.setObjectName("horizonFileName")
        self.checkAutoDeleteHorizon = QtWidgets.QCheckBox(self.groupBox_10)
        self.checkAutoDeleteHorizon.setGeometry(QtCore.QRect(10, 80, 256, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkAutoDeleteHorizon.setFont(font)
        self.checkAutoDeleteHorizon.setChecked(True)
        self.checkAutoDeleteHorizon.setObjectName("checkAutoDeleteHorizon")
        self.checkAutoDeleteMeridian = QtWidgets.QCheckBox(self.groupBox_10)
        self.checkAutoDeleteMeridian.setGeometry(QtCore.QRect(10, 120, 256, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkAutoDeleteMeridian.setFont(font)
        self.checkAutoDeleteMeridian.setObjectName("checkAutoDeleteMeridian")
        self.clearHorizonMask = QtWidgets.QPushButton(self.groupBox_10)
        self.clearHorizonMask.setEnabled(True)
        self.clearHorizonMask.setGeometry(QtCore.QRect(280, 45, 81, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.clearHorizonMask.setFont(font)
        self.clearHorizonMask.setStyleSheet("")
        self.clearHorizonMask.setObjectName("clearHorizonMask")
        self.safetyMarginHorizon = QtWidgets.QDoubleSpinBox(self.groupBox_10)
        self.safetyMarginHorizon.setGeometry(QtCore.QRect(290, 100, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.safetyMarginHorizon.setFont(font)
        self.safetyMarginHorizon.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.safetyMarginHorizon.setDecimals(0)
        self.safetyMarginHorizon.setMinimum(0.0)
        self.safetyMarginHorizon.setMaximum(10.0)
        self.safetyMarginHorizon.setSingleStep(1.0)
        self.safetyMarginHorizon.setProperty("value", 10.0)
        self.safetyMarginHorizon.setObjectName("safetyMarginHorizon")
        self.label_162 = QtWidgets.QLabel(self.groupBox_10)
        self.label_162.setGeometry(QtCore.QRect(335, 100, 16, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_162.setFont(font)
        self.label_162.setAlignment(QtCore.Qt.AlignCenter)
        self.label_162.setWordWrap(False)
        self.label_162.setObjectName("label_162")
        self.checkSafetyMarginHorizon = QtWidgets.QCheckBox(self.groupBox_10)
        self.checkSafetyMarginHorizon.setGeometry(QtCore.QRect(10, 100, 256, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkSafetyMarginHorizon.setFont(font)
        self.checkSafetyMarginHorizon.setObjectName("checkSafetyMarginHorizon")
        self.clearBuildP = QtWidgets.QPushButton(self.Modelpoints)
        self.clearBuildP.setGeometry(QtCore.QRect(20, 400, 361, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.clearBuildP.sizePolicy().hasHeightForWidth())
        self.clearBuildP.setSizePolicy(sizePolicy)
        self.clearBuildP.setMinimumSize(QtCore.QSize(100, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.clearBuildP.setFont(font)
        self.clearBuildP.setObjectName("clearBuildP")
        self.groupBox_54 = QtWidgets.QGroupBox(self.Modelpoints)
        self.groupBox_54.setGeometry(QtCore.QRect(15, 230, 366, 51))
        self.groupBox_54.setProperty("large", True)
        self.groupBox_54.setObjectName("groupBox_54")
        self.genBuildAlign3 = QtWidgets.QPushButton(self.groupBox_54)
        self.genBuildAlign3.setGeometry(QtCore.QRect(5, 20, 101, 26))
        self.genBuildAlign3.setObjectName("genBuildAlign3")
        self.genBuildAlign6 = QtWidgets.QPushButton(self.groupBox_54)
        self.genBuildAlign6.setGeometry(QtCore.QRect(135, 20, 101, 26))
        self.genBuildAlign6.setObjectName("genBuildAlign6")
        self.genBuildAlign9 = QtWidgets.QPushButton(self.groupBox_54)
        self.genBuildAlign9.setGeometry(QtCore.QRect(260, 20, 101, 26))
        self.genBuildAlign9.setObjectName("genBuildAlign9")
        self.keepGeneratedPoints = QtWidgets.QCheckBox(self.Modelpoints)
        self.keepGeneratedPoints.setGeometry(QtCore.QRect(25, 375, 336, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.keepGeneratedPoints.setFont(font)
        self.keepGeneratedPoints.setObjectName("keepGeneratedPoints")
        self.mainTabWidget.addTab(self.Modelpoints, "")
        self.Modelbuild = QtWidgets.QWidget()
        self.Modelbuild.setObjectName("Modelbuild")
        self.line_64 = QtWidgets.QFrame(self.Modelbuild)
        self.line_64.setGeometry(QtCore.QRect(390, 10, 11, 416))
        self.line_64.setStyleSheet("")
        self.line_64.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_64.setLineWidth(2)
        self.line_64.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_64.setObjectName("line_64")
        self.groupBox_11 = QtWidgets.QGroupBox(self.Modelbuild)
        self.groupBox_11.setGeometry(QtCore.QRect(15, 25, 366, 146))
        self.groupBox_11.setProperty("large", True)
        self.groupBox_11.setObjectName("groupBox_11")
        self.label_169 = QtWidgets.QLabel(self.groupBox_11)
        self.label_169.setGeometry(QtCore.QRect(310, 85, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_169.setFont(font)
        self.label_169.setAlignment(QtCore.Qt.AlignCenter)
        self.label_169.setWordWrap(False)
        self.label_169.setObjectName("label_169")
        self.label_218 = QtWidgets.QLabel(self.groupBox_11)
        self.label_218.setGeometry(QtCore.QRect(120, 115, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_218.setFont(font)
        self.label_218.setAlignment(QtCore.Qt.AlignCenter)
        self.label_218.setWordWrap(False)
        self.label_218.setObjectName("label_218")
        self.label_154 = QtWidgets.QLabel(self.groupBox_11)
        self.label_154.setGeometry(QtCore.QRect(5, 85, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_154.setFont(font)
        self.label_154.setObjectName("label_154")
        self.label_14 = QtWidgets.QLabel(self.groupBox_11)
        self.label_14.setGeometry(QtCore.QRect(5, 55, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_14.setFont(font)
        self.label_14.setObjectName("label_14")
        self.positionAngle = QtWidgets.QLineEdit(self.groupBox_11)
        self.positionAngle.setEnabled(False)
        self.positionAngle.setGeometry(QtCore.QRect(75, 115, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.positionAngle.setFont(font)
        self.positionAngle.setMouseTracking(False)
        self.positionAngle.setFocusPolicy(QtCore.Qt.NoFocus)
        self.positionAngle.setAcceptDrops(False)
        self.positionAngle.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.positionAngle.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.positionAngle.setReadOnly(True)
        self.positionAngle.setObjectName("positionAngle")
        self.label_158 = QtWidgets.QLabel(self.groupBox_11)
        self.label_158.setGeometry(QtCore.QRect(150, 55, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_158.setFont(font)
        self.label_158.setObjectName("label_158")
        self.orthoError = QtWidgets.QLineEdit(self.groupBox_11)
        self.orthoError.setEnabled(False)
        self.orthoError.setGeometry(QtCore.QRect(255, 85, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.orthoError.setFont(font)
        self.orthoError.setMouseTracking(False)
        self.orthoError.setFocusPolicy(QtCore.Qt.NoFocus)
        self.orthoError.setAcceptDrops(False)
        self.orthoError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.orthoError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.orthoError.setReadOnly(True)
        self.orthoError.setObjectName("orthoError")
        self.numberStars = QtWidgets.QLineEdit(self.groupBox_11)
        self.numberStars.setEnabled(False)
        self.numberStars.setGeometry(QtCore.QRect(75, 25, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberStars.setFont(font)
        self.numberStars.setMouseTracking(False)
        self.numberStars.setFocusPolicy(QtCore.Qt.NoFocus)
        self.numberStars.setAcceptDrops(False)
        self.numberStars.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.numberStars.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberStars.setReadOnly(True)
        self.numberStars.setObjectName("numberStars")
        self.label_15 = QtWidgets.QLabel(self.groupBox_11)
        self.label_15.setGeometry(QtCore.QRect(150, 115, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_15.setFont(font)
        self.label_15.setObjectName("label_15")
        self.label_167 = QtWidgets.QLabel(self.groupBox_11)
        self.label_167.setGeometry(QtCore.QRect(310, 25, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_167.setFont(font)
        self.label_167.setAlignment(QtCore.Qt.AlignCenter)
        self.label_167.setWordWrap(False)
        self.label_167.setObjectName("label_167")
        self.azimuthError = QtWidgets.QLineEdit(self.groupBox_11)
        self.azimuthError.setEnabled(False)
        self.azimuthError.setGeometry(QtCore.QRect(255, 55, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthError.setFont(font)
        self.azimuthError.setMouseTracking(False)
        self.azimuthError.setFocusPolicy(QtCore.Qt.NoFocus)
        self.azimuthError.setAcceptDrops(False)
        self.azimuthError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.azimuthError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthError.setReadOnly(True)
        self.azimuthError.setObjectName("azimuthError")
        self.label = QtWidgets.QLabel(self.groupBox_11)
        self.label.setGeometry(QtCore.QRect(5, 25, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.label_153 = QtWidgets.QLabel(self.groupBox_11)
        self.label_153.setGeometry(QtCore.QRect(150, 85, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_153.setFont(font)
        self.label_153.setObjectName("label_153")
        self.label_159 = QtWidgets.QLabel(self.groupBox_11)
        self.label_159.setGeometry(QtCore.QRect(150, 25, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_159.setFont(font)
        self.label_159.setObjectName("label_159")
        self.label_160 = QtWidgets.QLabel(self.groupBox_11)
        self.label_160.setGeometry(QtCore.QRect(5, 115, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_160.setFont(font)
        self.label_160.setObjectName("label_160")
        self.altitudeError = QtWidgets.QLineEdit(self.groupBox_11)
        self.altitudeError.setEnabled(False)
        self.altitudeError.setGeometry(QtCore.QRect(255, 25, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeError.setFont(font)
        self.altitudeError.setMouseTracking(False)
        self.altitudeError.setFocusPolicy(QtCore.Qt.NoFocus)
        self.altitudeError.setAcceptDrops(False)
        self.altitudeError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.altitudeError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeError.setReadOnly(True)
        self.altitudeError.setObjectName("altitudeError")
        self.errorRMS = QtWidgets.QLineEdit(self.groupBox_11)
        self.errorRMS.setEnabled(False)
        self.errorRMS.setGeometry(QtCore.QRect(75, 55, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.errorRMS.setFont(font)
        self.errorRMS.setMouseTracking(False)
        self.errorRMS.setFocusPolicy(QtCore.Qt.NoFocus)
        self.errorRMS.setAcceptDrops(False)
        self.errorRMS.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.errorRMS.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.errorRMS.setReadOnly(True)
        self.errorRMS.setObjectName("errorRMS")
        self.label_201 = QtWidgets.QLabel(self.groupBox_11)
        self.label_201.setGeometry(QtCore.QRect(310, 115, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_201.setFont(font)
        self.label_201.setAlignment(QtCore.Qt.AlignCenter)
        self.label_201.setWordWrap(False)
        self.label_201.setObjectName("label_201")
        self.terms = QtWidgets.QLineEdit(self.groupBox_11)
        self.terms.setEnabled(False)
        self.terms.setGeometry(QtCore.QRect(75, 85, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.terms.setFont(font)
        self.terms.setMouseTracking(False)
        self.terms.setFocusPolicy(QtCore.Qt.NoFocus)
        self.terms.setAcceptDrops(False)
        self.terms.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.terms.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.terms.setReadOnly(True)
        self.terms.setObjectName("terms")
        self.polarError = QtWidgets.QLineEdit(self.groupBox_11)
        self.polarError.setEnabled(False)
        self.polarError.setGeometry(QtCore.QRect(255, 115, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.polarError.setFont(font)
        self.polarError.setMouseTracking(False)
        self.polarError.setFocusPolicy(QtCore.Qt.NoFocus)
        self.polarError.setAcceptDrops(False)
        self.polarError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.polarError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.polarError.setReadOnly(True)
        self.polarError.setObjectName("polarError")
        self.label_168 = QtWidgets.QLabel(self.groupBox_11)
        self.label_168.setGeometry(QtCore.QRect(310, 55, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_168.setFont(font)
        self.label_168.setAlignment(QtCore.Qt.AlignCenter)
        self.label_168.setWordWrap(False)
        self.label_168.setObjectName("label_168")
        self.groupBox_9 = QtWidgets.QGroupBox(self.Modelbuild)
        self.groupBox_9.setGeometry(QtCore.QRect(410, 310, 366, 51))
        self.groupBox_9.setProperty("large", True)
        self.groupBox_9.setObjectName("groupBox_9")
        self.plateSolveSync = QtWidgets.QPushButton(self.groupBox_9)
        self.plateSolveSync.setEnabled(True)
        self.plateSolveSync.setGeometry(QtCore.QRect(5, 20, 356, 26))
        self.plateSolveSync.setObjectName("plateSolveSync")
        self.groupBox_7 = QtWidgets.QGroupBox(self.Modelbuild)
        self.groupBox_7.setGeometry(QtCore.QRect(410, 375, 366, 51))
        self.groupBox_7.setProperty("large", True)
        self.groupBox_7.setObjectName("groupBox_7")
        self.batchModel = QtWidgets.QPushButton(self.groupBox_7)
        self.batchModel.setEnabled(True)
        self.batchModel.setGeometry(QtCore.QRect(5, 20, 356, 26))
        self.batchModel.setObjectName("batchModel")
        self.groupBox_8 = QtWidgets.QGroupBox(self.Modelbuild)
        self.groupBox_8.setGeometry(QtCore.QRect(410, 25, 366, 271))
        self.groupBox_8.setProperty("large", True)
        self.groupBox_8.setObjectName("groupBox_8")
        self.cancelModel = QtWidgets.QPushButton(self.groupBox_8)
        self.cancelModel.setEnabled(False)
        self.cancelModel.setGeometry(QtCore.QRect(275, 20, 86, 26))
        self.cancelModel.setStyleSheet("")
        self.cancelModel.setObjectName("cancelModel")
        self.mSlew = QtWidgets.QLineEdit(self.groupBox_8)
        self.mSlew.setEnabled(False)
        self.mSlew.setGeometry(QtCore.QRect(145, 105, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mSlew.setFont(font)
        self.mSlew.setMouseTracking(False)
        self.mSlew.setFocusPolicy(QtCore.Qt.NoFocus)
        self.mSlew.setAcceptDrops(False)
        self.mSlew.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mSlew.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mSlew.setReadOnly(True)
        self.mSlew.setObjectName("mSlew")
        self.label_35 = QtWidgets.QLabel(self.groupBox_8)
        self.label_35.setGeometry(QtCore.QRect(195, 55, 91, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_35.setFont(font)
        self.label_35.setObjectName("label_35")
        self.runModel = QtWidgets.QPushButton(self.groupBox_8)
        self.runModel.setGeometry(QtCore.QRect(5, 20, 86, 26))
        self.runModel.setObjectName("runModel")
        self.mPoints = QtWidgets.QLineEdit(self.groupBox_8)
        self.mPoints.setEnabled(False)
        self.mPoints.setGeometry(QtCore.QRect(50, 105, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mPoints.setFont(font)
        self.mPoints.setMouseTracking(False)
        self.mPoints.setFocusPolicy(QtCore.Qt.NoFocus)
        self.mPoints.setAcceptDrops(False)
        self.mPoints.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mPoints.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mPoints.setReadOnly(True)
        self.mPoints.setObjectName("mPoints")
        self.label_29 = QtWidgets.QLabel(self.groupBox_8)
        self.label_29.setGeometry(QtCore.QRect(195, 80, 81, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_29.setFont(font)
        self.label_29.setObjectName("label_29")
        self.mSolve = QtWidgets.QLineEdit(self.groupBox_8)
        self.mSolve.setEnabled(False)
        self.mSolve.setGeometry(QtCore.QRect(330, 105, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mSolve.setFont(font)
        self.mSolve.setMouseTracking(False)
        self.mSolve.setFocusPolicy(QtCore.Qt.NoFocus)
        self.mSolve.setAcceptDrops(False)
        self.mSolve.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mSolve.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mSolve.setReadOnly(True)
        self.mSolve.setObjectName("mSolve")
        self.label_310 = QtWidgets.QLabel(self.groupBox_8)
        self.label_310.setGeometry(QtCore.QRect(100, 105, 51, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_310.setFont(font)
        self.label_310.setObjectName("label_310")
        self.label_311 = QtWidgets.QLabel(self.groupBox_8)
        self.label_311.setGeometry(QtCore.QRect(290, 105, 46, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_311.setFont(font)
        self.label_311.setObjectName("label_311")
        self.mImage = QtWidgets.QLineEdit(self.groupBox_8)
        self.mImage.setEnabled(False)
        self.mImage.setGeometry(QtCore.QRect(245, 105, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mImage.setFont(font)
        self.mImage.setMouseTracking(False)
        self.mImage.setFocusPolicy(QtCore.Qt.NoFocus)
        self.mImage.setAcceptDrops(False)
        self.mImage.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mImage.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mImage.setReadOnly(True)
        self.mImage.setObjectName("mImage")
        self.timeElapsed = QtWidgets.QLineEdit(self.groupBox_8)
        self.timeElapsed.setEnabled(False)
        self.timeElapsed.setGeometry(QtCore.QRect(300, 80, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeElapsed.setFont(font)
        self.timeElapsed.setMouseTracking(False)
        self.timeElapsed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeElapsed.setAcceptDrops(False)
        self.timeElapsed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeElapsed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeElapsed.setReadOnly(True)
        self.timeElapsed.setObjectName("timeElapsed")
        self.label_216 = QtWidgets.QLabel(self.groupBox_8)
        self.label_216.setGeometry(QtCore.QRect(5, 105, 51, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_216.setFont(font)
        self.label_216.setObjectName("label_216")
        self.timeFinished = QtWidgets.QLineEdit(self.groupBox_8)
        self.timeFinished.setEnabled(False)
        self.timeFinished.setGeometry(QtCore.QRect(115, 55, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeFinished.setFont(font)
        self.timeFinished.setMouseTracking(False)
        self.timeFinished.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeFinished.setAcceptDrops(False)
        self.timeFinished.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeFinished.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeFinished.setReadOnly(True)
        self.timeFinished.setObjectName("timeFinished")
        self.modelProgress = QtWidgets.QProgressBar(self.groupBox_8)
        self.modelProgress.setGeometry(QtCore.QRect(5, 135, 356, 26))
        self.modelProgress.setMaximum(100)
        self.modelProgress.setProperty("value", 0)
        self.modelProgress.setAlignment(QtCore.Qt.AlignCenter)
        self.modelProgress.setTextVisible(True)
        self.modelProgress.setOrientation(QtCore.Qt.Horizontal)
        self.modelProgress.setInvertedAppearance(False)
        self.modelProgress.setTextDirection(QtWidgets.QProgressBar.TopToBottom)
        self.modelProgress.setObjectName("modelProgress")
        self.label_312 = QtWidgets.QLabel(self.groupBox_8)
        self.label_312.setGeometry(QtCore.QRect(195, 105, 51, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_312.setFont(font)
        self.label_312.setObjectName("label_312")
        self.timeEstimated = QtWidgets.QLineEdit(self.groupBox_8)
        self.timeEstimated.setEnabled(False)
        self.timeEstimated.setGeometry(QtCore.QRect(300, 55, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeEstimated.setFont(font)
        self.timeEstimated.setMouseTracking(False)
        self.timeEstimated.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeEstimated.setAcceptDrops(False)
        self.timeEstimated.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeEstimated.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeEstimated.setReadOnly(True)
        self.timeEstimated.setObjectName("timeEstimated")
        self.label_42 = QtWidgets.QLabel(self.groupBox_8)
        self.label_42.setGeometry(QtCore.QRect(5, 55, 91, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_42.setFont(font)
        self.label_42.setObjectName("label_42")
        self.pauseModel = QtWidgets.QPushButton(self.groupBox_8)
        self.pauseModel.setEnabled(False)
        self.pauseModel.setGeometry(QtCore.QRect(95, 20, 86, 26))
        self.pauseModel.setObjectName("pauseModel")
        self.checkDisableDAT = QtWidgets.QCheckBox(self.groupBox_8)
        self.checkDisableDAT.setGeometry(QtCore.QRect(5, 170, 356, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkDisableDAT.setFont(font)
        self.checkDisableDAT.setChecked(True)
        self.checkDisableDAT.setObjectName("checkDisableDAT")
        self.parkMountAfterModel = QtWidgets.QCheckBox(self.groupBox_8)
        self.parkMountAfterModel.setGeometry(QtCore.QRect(5, 195, 266, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.parkMountAfterModel.setFont(font)
        self.parkMountAfterModel.setChecked(False)
        self.parkMountAfterModel.setObjectName("parkMountAfterModel")
        self.excludeDonePoints = QtWidgets.QCheckBox(self.groupBox_8)
        self.excludeDonePoints.setEnabled(True)
        self.excludeDonePoints.setGeometry(QtCore.QRect(5, 220, 336, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.excludeDonePoints.setFont(font)
        self.excludeDonePoints.setChecked(False)
        self.excludeDonePoints.setObjectName("excludeDonePoints")
        self.endModel = QtWidgets.QPushButton(self.groupBox_8)
        self.endModel.setEnabled(False)
        self.endModel.setGeometry(QtCore.QRect(185, 20, 86, 26))
        self.endModel.setObjectName("endModel")
        self.numberBuildRetries = QtWidgets.QDoubleSpinBox(self.groupBox_8)
        self.numberBuildRetries.setEnabled(True)
        self.numberBuildRetries.setGeometry(QtCore.QRect(145, 245, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberBuildRetries.setFont(font)
        self.numberBuildRetries.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberBuildRetries.setDecimals(0)
        self.numberBuildRetries.setMinimum(0.0)
        self.numberBuildRetries.setMaximum(5.0)
        self.numberBuildRetries.setSingleStep(1.0)
        self.numberBuildRetries.setProperty("value", 0.0)
        self.numberBuildRetries.setObjectName("numberBuildRetries")
        self.label_36 = QtWidgets.QLabel(self.groupBox_8)
        self.label_36.setGeometry(QtCore.QRect(195, 245, 141, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_36.setFont(font)
        self.label_36.setObjectName("label_36")
        self.label_59 = QtWidgets.QLabel(self.groupBox_8)
        self.label_59.setGeometry(QtCore.QRect(5, 245, 141, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_59.setFont(font)
        self.label_59.setObjectName("label_59")
        self.groupBox_28 = QtWidgets.QGroupBox(self.Modelbuild)
        self.groupBox_28.setGeometry(QtCore.QRect(15, 200, 366, 226))
        self.groupBox_28.setProperty("large", True)
        self.groupBox_28.setObjectName("groupBox_28")
        self.label_157 = QtWidgets.QLabel(self.groupBox_28)
        self.label_157.setGeometry(QtCore.QRect(20, 25, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_157.setFont(font)
        self.label_157.setObjectName("label_157")
        self.picALT = QtWidgets.QLabel(self.groupBox_28)
        self.picALT.setGeometry(QtCore.QRect(15, 70, 144, 144))
        self.picALT.setFrameShape(QtWidgets.QFrame.Box)
        self.picALT.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picALT.setMidLineWidth(0)
        self.picALT.setProperty("iconpicture", True)
        self.picALT.setObjectName("picALT")
        self.altitudeTurns = QtWidgets.QLineEdit(self.groupBox_28)
        self.altitudeTurns.setEnabled(False)
        self.altitudeTurns.setGeometry(QtCore.QRect(15, 45, 146, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeTurns.setFont(font)
        self.altitudeTurns.setMouseTracking(False)
        self.altitudeTurns.setFocusPolicy(QtCore.Qt.NoFocus)
        self.altitudeTurns.setAcceptDrops(False)
        self.altitudeTurns.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.altitudeTurns.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeTurns.setReadOnly(True)
        self.altitudeTurns.setObjectName("altitudeTurns")
        self.label_156 = QtWidgets.QLabel(self.groupBox_28)
        self.label_156.setGeometry(QtCore.QRect(205, 25, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_156.setFont(font)
        self.label_156.setObjectName("label_156")
        self.azimuthTurns = QtWidgets.QLineEdit(self.groupBox_28)
        self.azimuthTurns.setEnabled(False)
        self.azimuthTurns.setGeometry(QtCore.QRect(205, 45, 146, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthTurns.setFont(font)
        self.azimuthTurns.setMouseTracking(False)
        self.azimuthTurns.setFocusPolicy(QtCore.Qt.NoFocus)
        self.azimuthTurns.setAcceptDrops(False)
        self.azimuthTurns.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.azimuthTurns.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthTurns.setReadOnly(True)
        self.azimuthTurns.setObjectName("azimuthTurns")
        self.picAZ = QtWidgets.QLabel(self.groupBox_28)
        self.picAZ.setGeometry(QtCore.QRect(205, 70, 144, 144))
        self.picAZ.setFrameShape(QtWidgets.QFrame.Box)
        self.picAZ.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picAZ.setMidLineWidth(0)
        self.picAZ.setProperty("iconpicture", True)
        self.picAZ.setObjectName("picAZ")
        self.mainTabWidget.addTab(self.Modelbuild, "")
        self.ManageModel = QtWidgets.QWidget()
        self.ManageModel.setObjectName("ManageModel")
        self.groupBox_55 = QtWidgets.QGroupBox(self.ManageModel)
        self.groupBox_55.setGeometry(QtCore.QRect(500, 5, 281, 156))
        self.groupBox_55.setProperty("large", True)
        self.groupBox_55.setObjectName("groupBox_55")
        self.clearModel = QtWidgets.QPushButton(self.groupBox_55)
        self.clearModel.setGeometry(QtCore.QRect(100, 20, 91, 26))
        self.clearModel.setObjectName("clearModel")
        self.label_75 = QtWidgets.QLabel(self.groupBox_55)
        self.label_75.setGeometry(QtCore.QRect(105, 50, 71, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_75.setFont(font)
        self.label_75.setObjectName("label_75")
        self.cancelOptimize = QtWidgets.QPushButton(self.groupBox_55)
        self.cancelOptimize.setGeometry(QtCore.QRect(10, 80, 86, 26))
        self.cancelOptimize.setObjectName("cancelOptimize")
        self.runOptimize = QtWidgets.QPushButton(self.groupBox_55)
        self.runOptimize.setGeometry(QtCore.QRect(10, 50, 86, 26))
        self.runOptimize.setObjectName("runOptimize")
        self.refreshModel = QtWidgets.QPushButton(self.groupBox_55)
        self.refreshModel.setGeometry(QtCore.QRect(195, 20, 81, 26))
        self.refreshModel.setObjectName("refreshModel")
        self.targetRMS = QtWidgets.QDoubleSpinBox(self.groupBox_55)
        self.targetRMS.setGeometry(QtCore.QRect(195, 80, 81, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.targetRMS.setFont(font)
        self.targetRMS.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.targetRMS.setDecimals(1)
        self.targetRMS.setMaximum(100.0)
        self.targetRMS.setSingleStep(0.5)
        self.targetRMS.setProperty("value", 10.0)
        self.targetRMS.setObjectName("targetRMS")
        self.errorRMS1 = QtWidgets.QLineEdit(self.groupBox_55)
        self.errorRMS1.setEnabled(False)
        self.errorRMS1.setGeometry(QtCore.QRect(195, 50, 61, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.errorRMS1.setFont(font)
        self.errorRMS1.setMouseTracking(False)
        self.errorRMS1.setFocusPolicy(QtCore.Qt.NoFocus)
        self.errorRMS1.setAcceptDrops(False)
        self.errorRMS1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.errorRMS1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.errorRMS1.setReadOnly(True)
        self.errorRMS1.setObjectName("errorRMS1")
        self.label_37 = QtWidgets.QLabel(self.groupBox_55)
        self.label_37.setGeometry(QtCore.QRect(105, 80, 86, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_37.setFont(font)
        self.label_37.setObjectName("label_37")
        self.deleteWorstPoint = QtWidgets.QPushButton(self.groupBox_55)
        self.deleteWorstPoint.setGeometry(QtCore.QRect(10, 20, 86, 26))
        self.deleteWorstPoint.setObjectName("deleteWorstPoint")
        self.optimizeOverall = QtWidgets.QRadioButton(self.groupBox_55)
        self.optimizeOverall.setGeometry(QtCore.QRect(10, 110, 191, 18))
        self.optimizeOverall.setChecked(True)
        self.optimizeOverall.setObjectName("optimizeOverall")
        self.optimizeSingle = QtWidgets.QRadioButton(self.groupBox_55)
        self.optimizeSingle.setGeometry(QtCore.QRect(10, 130, 196, 18))
        self.optimizeSingle.setObjectName("optimizeSingle")
        self.groupBox_56 = QtWidgets.QGroupBox(self.ManageModel)
        self.groupBox_56.setGeometry(QtCore.QRect(500, 250, 281, 176))
        self.groupBox_56.setProperty("large", True)
        self.groupBox_56.setObjectName("groupBox_56")
        self.loadName = QtWidgets.QPushButton(self.groupBox_56)
        self.loadName.setGeometry(QtCore.QRect(200, 20, 76, 26))
        self.loadName.setObjectName("loadName")
        self.nameList = QtWidgets.QListWidget(self.groupBox_56)
        self.nameList.setGeometry(QtCore.QRect(5, 20, 186, 146))
        self.nameList.setObjectName("nameList")
        self.refreshName = QtWidgets.QPushButton(self.groupBox_56)
        self.refreshName.setGeometry(QtCore.QRect(200, 110, 76, 26))
        self.refreshName.setObjectName("refreshName")
        self.deleteName = QtWidgets.QPushButton(self.groupBox_56)
        self.deleteName.setGeometry(QtCore.QRect(200, 80, 76, 26))
        self.deleteName.setObjectName("deleteName")
        self.saveName = QtWidgets.QPushButton(self.groupBox_56)
        self.saveName.setGeometry(QtCore.QRect(200, 50, 76, 26))
        self.saveName.setObjectName("saveName")
        self.tabWidget = QtWidgets.QTabWidget(self.ManageModel)
        self.tabWidget.setGeometry(QtCore.QRect(5, 5, 486, 421))
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.modelPosition = QtWidgets.QWidget(self.tab)
        self.modelPosition.setEnabled(True)
        self.modelPosition.setGeometry(QtCore.QRect(5, 5, 481, 361))
        self.modelPosition.setAutoFillBackground(True)
        self.modelPosition.setObjectName("modelPosition")
        self.showErrorValues = QtWidgets.QRadioButton(self.tab)
        self.showErrorValues.setGeometry(QtCore.QRect(250, 365, 91, 20))
        self.showErrorValues.setSizeIncrement(QtCore.QSize(0, 18))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.showErrorValues.setFont(font)
        self.showErrorValues.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.showErrorValues.setChecked(False)
        self.showErrorValues.setObjectName("showErrorValues")
        self.showNumbers = QtWidgets.QRadioButton(self.tab)
        self.showNumbers.setGeometry(QtCore.QRect(155, 365, 81, 20))
        self.showNumbers.setSizeIncrement(QtCore.QSize(0, 18))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.showNumbers.setFont(font)
        self.showNumbers.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.showNumbers.setChecked(False)
        self.showNumbers.setObjectName("showNumbers")
        self.showNoAnnotation = QtWidgets.QRadioButton(self.tab)
        self.showNoAnnotation.setGeometry(QtCore.QRect(35, 365, 101, 20))
        self.showNoAnnotation.setSizeIncrement(QtCore.QSize(0, 18))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.showNoAnnotation.setFont(font)
        self.showNoAnnotation.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.showNoAnnotation.setChecked(True)
        self.showNoAnnotation.setObjectName("showNoAnnotation")
        self.numberStars1 = QtWidgets.QLineEdit(self.tab)
        self.numberStars1.setEnabled(False)
        self.numberStars1.setGeometry(QtCore.QRect(440, 365, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberStars1.setFont(font)
        self.numberStars1.setMouseTracking(False)
        self.numberStars1.setFocusPolicy(QtCore.Qt.NoFocus)
        self.numberStars1.setAcceptDrops(False)
        self.numberStars1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.numberStars1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberStars1.setReadOnly(True)
        self.numberStars1.setObjectName("numberStars1")
        self.label_76 = QtWidgets.QLabel(self.tab)
        self.label_76.setGeometry(QtCore.QRect(380, 365, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_76.setFont(font)
        self.label_76.setObjectName("label_76")
        self.tabWidget.addTab(self.tab, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.errorDistribution = QtWidgets.QWidget(self.tab_3)
        self.errorDistribution.setEnabled(True)
        self.errorDistribution.setGeometry(QtCore.QRect(5, 5, 481, 381))
        self.errorDistribution.setAutoFillBackground(True)
        self.errorDistribution.setObjectName("errorDistribution")
        self.tabWidget.addTab(self.tab_3, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.errorAscending = QtWidgets.QWidget(self.tab_2)
        self.errorAscending.setEnabled(True)
        self.errorAscending.setGeometry(QtCore.QRect(5, 5, 471, 381))
        self.errorAscending.setAutoFillBackground(True)
        self.errorAscending.setObjectName("errorAscending")
        self.tabWidget.addTab(self.tab_2, "")
        self.groupBox_22 = QtWidgets.QGroupBox(self.ManageModel)
        self.groupBox_22.setGeometry(QtCore.QRect(500, 165, 281, 76))
        self.groupBox_22.setProperty("large", True)
        self.groupBox_22.setObjectName("groupBox_22")
        self.originalModel = QtWidgets.QLineEdit(self.groupBox_22)
        self.originalModel.setEnabled(False)
        self.originalModel.setGeometry(QtCore.QRect(5, 20, 191, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.originalModel.setFont(font)
        self.originalModel.setMouseTracking(False)
        self.originalModel.setFocusPolicy(QtCore.Qt.NoFocus)
        self.originalModel.setAcceptDrops(False)
        self.originalModel.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.originalModel.setMaxLength(50)
        self.originalModel.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.originalModel.setReadOnly(True)
        self.originalModel.setObjectName("originalModel")
        self.openAnalyseW = QtWidgets.QPushButton(self.groupBox_22)
        self.openAnalyseW.setEnabled(True)
        self.openAnalyseW.setGeometry(QtCore.QRect(5, 45, 106, 26))
        self.openAnalyseW.setObjectName("openAnalyseW")
        self.showOriginalModelAnalyse = QtWidgets.QPushButton(self.groupBox_22)
        self.showOriginalModelAnalyse.setGeometry(QtCore.QRect(115, 45, 81, 26))
        self.showOriginalModelAnalyse.setObjectName("showOriginalModelAnalyse")
        self.autoUpdateActualAnalyse = QtWidgets.QRadioButton(self.groupBox_22)
        self.autoUpdateActualAnalyse.setGeometry(QtCore.QRect(200, 22, 76, 16))
        self.autoUpdateActualAnalyse.setObjectName("autoUpdateActualAnalyse")
        self.showActualModelAnalyse = QtWidgets.QPushButton(self.groupBox_22)
        self.showActualModelAnalyse.setGeometry(QtCore.QRect(200, 45, 76, 26))
        self.showActualModelAnalyse.setObjectName("showActualModelAnalyse")
        self.mainTabWidget.addTab(self.ManageModel, "")
        self.Satellite = QtWidgets.QWidget()
        self.Satellite.setObjectName("Satellite")
        self.line_68 = QtWidgets.QFrame(self.Satellite)
        self.line_68.setGeometry(QtCore.QRect(410, 10, 11, 416))
        self.line_68.setStyleSheet("")
        self.line_68.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_68.setLineWidth(2)
        self.line_68.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_68.setObjectName("line_68")
        self.groupBox_12 = QtWidgets.QGroupBox(self.Satellite)
        self.groupBox_12.setGeometry(QtCore.QRect(430, 15, 351, 271))
        self.groupBox_12.setProperty("large", True)
        self.groupBox_12.setObjectName("groupBox_12")
        self.label_227 = QtWidgets.QLabel(self.groupBox_12)
        self.label_227.setGeometry(QtCore.QRect(215, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_227.setFont(font)
        self.label_227.setObjectName("label_227")
        self.label_234 = QtWidgets.QLabel(self.groupBox_12)
        self.label_234.setGeometry(QtCore.QRect(10, 75, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_234.setFont(font)
        self.label_234.setObjectName("label_234")
        self.satLongitude = QtWidgets.QLineEdit(self.groupBox_12)
        self.satLongitude.setEnabled(False)
        self.satLongitude.setGeometry(QtCore.QRect(255, 125, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satLongitude.setFont(font)
        self.satLongitude.setMouseTracking(False)
        self.satLongitude.setAcceptDrops(False)
        self.satLongitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satLongitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satLongitude.setReadOnly(False)
        self.satLongitude.setObjectName("satLongitude")
        self.satDec = QtWidgets.QLineEdit(self.groupBox_12)
        self.satDec.setEnabled(False)
        self.satDec.setGeometry(QtCore.QRect(255, 100, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satDec.setFont(font)
        self.satDec.setMouseTracking(False)
        self.satDec.setAcceptDrops(False)
        self.satDec.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satDec.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satDec.setReadOnly(False)
        self.satDec.setObjectName("satDec")
        self.label_225 = QtWidgets.QLabel(self.groupBox_12)
        self.label_225.setGeometry(QtCore.QRect(10, 20, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_225.setFont(font)
        self.label_225.setObjectName("label_225")
        self.label_236 = QtWidgets.QLabel(self.groupBox_12)
        self.label_236.setGeometry(QtCore.QRect(215, 75, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_236.setFont(font)
        self.label_236.setObjectName("label_236")
        self.satRa = QtWidgets.QLineEdit(self.groupBox_12)
        self.satRa.setEnabled(False)
        self.satRa.setGeometry(QtCore.QRect(255, 75, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satRa.setFont(font)
        self.satRa.setMouseTracking(False)
        self.satRa.setAcceptDrops(False)
        self.satRa.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satRa.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satRa.setReadOnly(False)
        self.satRa.setObjectName("satRa")
        self.label_239 = QtWidgets.QLabel(self.groupBox_12)
        self.label_239.setGeometry(QtCore.QRect(10, 125, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_239.setFont(font)
        self.label_239.setObjectName("label_239")
        self.label_235 = QtWidgets.QLabel(self.groupBox_12)
        self.label_235.setGeometry(QtCore.QRect(10, 100, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_235.setFont(font)
        self.label_235.setObjectName("label_235")
        self.satAzimuth = QtWidgets.QLineEdit(self.groupBox_12)
        self.satAzimuth.setEnabled(False)
        self.satAzimuth.setGeometry(QtCore.QRect(75, 75, 66, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satAzimuth.setFont(font)
        self.satAzimuth.setMouseTracking(False)
        self.satAzimuth.setAcceptDrops(False)
        self.satAzimuth.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satAzimuth.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satAzimuth.setReadOnly(False)
        self.satAzimuth.setObjectName("satAzimuth")
        self.label_233 = QtWidgets.QLabel(self.groupBox_12)
        self.label_233.setGeometry(QtCore.QRect(310, 45, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_233.setFont(font)
        self.label_233.setObjectName("label_233")
        self.label_207 = QtWidgets.QLabel(self.groupBox_12)
        self.label_207.setGeometry(QtCore.QRect(140, 100, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_207.setFont(font)
        self.label_207.setAlignment(QtCore.Qt.AlignCenter)
        self.label_207.setWordWrap(False)
        self.label_207.setObjectName("label_207")
        self.satelliteEpoch = QtWidgets.QLineEdit(self.groupBox_12)
        self.satelliteEpoch.setEnabled(False)
        self.satelliteEpoch.setGeometry(QtCore.QRect(75, 45, 126, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satelliteEpoch.setFont(font)
        self.satelliteEpoch.setMouseTracking(False)
        self.satelliteEpoch.setAcceptDrops(False)
        self.satelliteEpoch.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satelliteEpoch.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satelliteEpoch.setReadOnly(False)
        self.satelliteEpoch.setObjectName("satelliteEpoch")
        self.label_221 = QtWidgets.QLabel(self.groupBox_12)
        self.label_221.setGeometry(QtCore.QRect(310, 75, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_221.setFont(font)
        self.label_221.setAlignment(QtCore.Qt.AlignCenter)
        self.label_221.setWordWrap(False)
        self.label_221.setObjectName("label_221")
        self.label_231 = QtWidgets.QLabel(self.groupBox_12)
        self.label_231.setGeometry(QtCore.QRect(215, 45, 26, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_231.setFont(font)
        self.label_231.setObjectName("label_231")
        self.label_237 = QtWidgets.QLabel(self.groupBox_12)
        self.label_237.setGeometry(QtCore.QRect(215, 100, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_237.setFont(font)
        self.label_237.setObjectName("label_237")
        self.label_230 = QtWidgets.QLabel(self.groupBox_12)
        self.label_230.setGeometry(QtCore.QRect(10, 45, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_230.setFont(font)
        self.label_230.setObjectName("label_230")
        self.satelliteDataAge = QtWidgets.QLineEdit(self.groupBox_12)
        self.satelliteDataAge.setEnabled(False)
        self.satelliteDataAge.setGeometry(QtCore.QRect(255, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satelliteDataAge.setFont(font)
        self.satelliteDataAge.setMouseTracking(False)
        self.satelliteDataAge.setAcceptDrops(False)
        self.satelliteDataAge.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satelliteDataAge.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satelliteDataAge.setReadOnly(False)
        self.satelliteDataAge.setObjectName("satelliteDataAge")
        self.label_252 = QtWidgets.QLabel(self.groupBox_12)
        self.label_252.setGeometry(QtCore.QRect(305, 125, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_252.setFont(font)
        self.label_252.setAlignment(QtCore.Qt.AlignCenter)
        self.label_252.setWordWrap(False)
        self.label_252.setObjectName("label_252")
        self.label_202 = QtWidgets.QLabel(self.groupBox_12)
        self.label_202.setGeometry(QtCore.QRect(140, 75, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_202.setFont(font)
        self.label_202.setAlignment(QtCore.Qt.AlignCenter)
        self.label_202.setWordWrap(False)
        self.label_202.setObjectName("label_202")
        self.satAltitude = QtWidgets.QLineEdit(self.groupBox_12)
        self.satAltitude.setEnabled(False)
        self.satAltitude.setGeometry(QtCore.QRect(75, 100, 66, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satAltitude.setFont(font)
        self.satAltitude.setMouseTracking(False)
        self.satAltitude.setAcceptDrops(False)
        self.satAltitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satAltitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satAltitude.setReadOnly(False)
        self.satAltitude.setObjectName("satAltitude")
        self.satelliteNumber = QtWidgets.QLineEdit(self.groupBox_12)
        self.satelliteNumber.setEnabled(False)
        self.satelliteNumber.setGeometry(QtCore.QRect(255, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satelliteNumber.setFont(font)
        self.satelliteNumber.setMouseTracking(False)
        self.satelliteNumber.setAcceptDrops(False)
        self.satelliteNumber.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satelliteNumber.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satelliteNumber.setReadOnly(False)
        self.satelliteNumber.setObjectName("satelliteNumber")
        self.satLatitude = QtWidgets.QLineEdit(self.groupBox_12)
        self.satLatitude.setEnabled(False)
        self.satLatitude.setGeometry(QtCore.QRect(75, 125, 66, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satLatitude.setFont(font)
        self.satLatitude.setMouseTracking(False)
        self.satLatitude.setAcceptDrops(False)
        self.satLatitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satLatitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satLatitude.setReadOnly(False)
        self.satLatitude.setObjectName("satLatitude")
        self.label_253 = QtWidgets.QLabel(self.groupBox_12)
        self.label_253.setGeometry(QtCore.QRect(140, 125, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_253.setFont(font)
        self.label_253.setAlignment(QtCore.Qt.AlignCenter)
        self.label_253.setWordWrap(False)
        self.label_253.setObjectName("label_253")
        self.satelliteName = QtWidgets.QLineEdit(self.groupBox_12)
        self.satelliteName.setEnabled(False)
        self.satelliteName.setGeometry(QtCore.QRect(75, 20, 126, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satelliteName.setFont(font)
        self.satelliteName.setMouseTracking(False)
        self.satelliteName.setAcceptDrops(False)
        self.satelliteName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satelliteName.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satelliteName.setReadOnly(False)
        self.satelliteName.setObjectName("satelliteName")
        self.label_226 = QtWidgets.QLabel(self.groupBox_12)
        self.label_226.setGeometry(QtCore.QRect(305, 100, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_226.setFont(font)
        self.label_226.setAlignment(QtCore.Qt.AlignCenter)
        self.label_226.setWordWrap(False)
        self.label_226.setObjectName("label_226")
        self.label_240 = QtWidgets.QLabel(self.groupBox_12)
        self.label_240.setGeometry(QtCore.QRect(215, 125, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_240.setFont(font)
        self.label_240.setObjectName("label_240")
        self.satTransitStartUTC_1 = QtWidgets.QLineEdit(self.groupBox_12)
        self.satTransitStartUTC_1.setEnabled(False)
        self.satTransitStartUTC_1.setGeometry(QtCore.QRect(75, 190, 126, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satTransitStartUTC_1.setFont(font)
        self.satTransitStartUTC_1.setMouseTracking(False)
        self.satTransitStartUTC_1.setAcceptDrops(False)
        self.satTransitStartUTC_1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satTransitStartUTC_1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satTransitStartUTC_1.setReadOnly(False)
        self.satTransitStartUTC_1.setObjectName("satTransitStartUTC_1")
        self.satTransitEndUTC_1 = QtWidgets.QLineEdit(self.groupBox_12)
        self.satTransitEndUTC_1.setEnabled(False)
        self.satTransitEndUTC_1.setGeometry(QtCore.QRect(215, 190, 126, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satTransitEndUTC_1.setFont(font)
        self.satTransitEndUTC_1.setMouseTracking(False)
        self.satTransitEndUTC_1.setAcceptDrops(False)
        self.satTransitEndUTC_1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satTransitEndUTC_1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satTransitEndUTC_1.setReadOnly(False)
        self.satTransitEndUTC_1.setObjectName("satTransitEndUTC_1")
        self.satTransitEndUTC_2 = QtWidgets.QLineEdit(self.groupBox_12)
        self.satTransitEndUTC_2.setEnabled(False)
        self.satTransitEndUTC_2.setGeometry(QtCore.QRect(215, 215, 126, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satTransitEndUTC_2.setFont(font)
        self.satTransitEndUTC_2.setMouseTracking(False)
        self.satTransitEndUTC_2.setAcceptDrops(False)
        self.satTransitEndUTC_2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satTransitEndUTC_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satTransitEndUTC_2.setReadOnly(False)
        self.satTransitEndUTC_2.setObjectName("satTransitEndUTC_2")
        self.satTransitStartUTC_2 = QtWidgets.QLineEdit(self.groupBox_12)
        self.satTransitStartUTC_2.setEnabled(False)
        self.satTransitStartUTC_2.setGeometry(QtCore.QRect(75, 215, 126, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satTransitStartUTC_2.setFont(font)
        self.satTransitStartUTC_2.setMouseTracking(False)
        self.satTransitStartUTC_2.setAcceptDrops(False)
        self.satTransitStartUTC_2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satTransitStartUTC_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satTransitStartUTC_2.setReadOnly(False)
        self.satTransitStartUTC_2.setObjectName("satTransitStartUTC_2")
        self.satTransitEndUTC_3 = QtWidgets.QLineEdit(self.groupBox_12)
        self.satTransitEndUTC_3.setEnabled(False)
        self.satTransitEndUTC_3.setGeometry(QtCore.QRect(215, 240, 126, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satTransitEndUTC_3.setFont(font)
        self.satTransitEndUTC_3.setMouseTracking(False)
        self.satTransitEndUTC_3.setAcceptDrops(False)
        self.satTransitEndUTC_3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satTransitEndUTC_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satTransitEndUTC_3.setReadOnly(False)
        self.satTransitEndUTC_3.setObjectName("satTransitEndUTC_3")
        self.satTransitStartUTC_3 = QtWidgets.QLineEdit(self.groupBox_12)
        self.satTransitStartUTC_3.setEnabled(False)
        self.satTransitStartUTC_3.setGeometry(QtCore.QRect(75, 240, 126, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satTransitStartUTC_3.setFont(font)
        self.satTransitStartUTC_3.setMouseTracking(False)
        self.satTransitStartUTC_3.setAcceptDrops(False)
        self.satTransitStartUTC_3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satTransitStartUTC_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satTransitStartUTC_3.setReadOnly(False)
        self.satTransitStartUTC_3.setObjectName("satTransitStartUTC_3")
        self.label_298 = QtWidgets.QLabel(self.groupBox_12)
        self.label_298.setGeometry(QtCore.QRect(75, 170, 126, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_298.setFont(font)
        self.label_298.setObjectName("label_298")
        self.label_309 = QtWidgets.QLabel(self.groupBox_12)
        self.label_309.setGeometry(QtCore.QRect(215, 170, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_309.setFont(font)
        self.label_309.setObjectName("label_309")
        self.label_241 = QtWidgets.QLabel(self.groupBox_12)
        self.label_241.setGeometry(QtCore.QRect(10, 190, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_241.setFont(font)
        self.label_241.setObjectName("label_241")
        self.label_254 = QtWidgets.QLabel(self.groupBox_12)
        self.label_254.setGeometry(QtCore.QRect(10, 215, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_254.setFont(font)
        self.label_254.setObjectName("label_254")
        self.label_260 = QtWidgets.QLabel(self.groupBox_12)
        self.label_260.setGeometry(QtCore.QRect(10, 240, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_260.setFont(font)
        self.label_260.setObjectName("label_260")
        self.groupBox_19 = QtWidgets.QGroupBox(self.Satellite)
        self.groupBox_19.setGeometry(QtCore.QRect(430, 305, 351, 116))
        self.groupBox_19.setProperty("large", True)
        self.groupBox_19.setObjectName("groupBox_19")
        self.stopSatelliteTracking = QtWidgets.QPushButton(self.groupBox_19)
        self.stopSatelliteTracking.setEnabled(False)
        self.stopSatelliteTracking.setGeometry(QtCore.QRect(180, 80, 166, 26))
        self.stopSatelliteTracking.setStyleSheet("")
        self.stopSatelliteTracking.setObjectName("stopSatelliteTracking")
        self.startSatelliteTracking = QtWidgets.QPushButton(self.groupBox_19)
        self.startSatelliteTracking.setEnabled(False)
        self.startSatelliteTracking.setGeometry(QtCore.QRect(5, 80, 161, 26))
        self.startSatelliteTracking.setStyleSheet("")
        self.startSatelliteTracking.setObjectName("startSatelliteTracking")
        self.label_271 = QtWidgets.QLabel(self.groupBox_19)
        self.label_271.setGeometry(QtCore.QRect(290, 25, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_271.setFont(font)
        self.label_271.setObjectName("label_271")
        self.satTransitStartUTC = QtWidgets.QLineEdit(self.groupBox_19)
        self.satTransitStartUTC.setEnabled(False)
        self.satTransitStartUTC.setGeometry(QtCore.QRect(5, 45, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satTransitStartUTC.setFont(font)
        self.satTransitStartUTC.setMouseTracking(False)
        self.satTransitStartUTC.setAcceptDrops(False)
        self.satTransitStartUTC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satTransitStartUTC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satTransitStartUTC.setReadOnly(False)
        self.satTransitStartUTC.setObjectName("satTransitStartUTC")
        self.satTransitEndUTC = QtWidgets.QLineEdit(self.groupBox_19)
        self.satTransitEndUTC.setEnabled(False)
        self.satTransitEndUTC.setGeometry(QtCore.QRect(150, 45, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satTransitEndUTC.setFont(font)
        self.satTransitEndUTC.setMouseTracking(False)
        self.satTransitEndUTC.setAcceptDrops(False)
        self.satTransitEndUTC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satTransitEndUTC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satTransitEndUTC.setReadOnly(False)
        self.satTransitEndUTC.setObjectName("satTransitEndUTC")
        self.satNeedFlip = QtWidgets.QLineEdit(self.groupBox_19)
        self.satNeedFlip.setEnabled(False)
        self.satNeedFlip.setGeometry(QtCore.QRect(290, 45, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satNeedFlip.setFont(font)
        self.satNeedFlip.setMouseTracking(False)
        self.satNeedFlip.setAcceptDrops(False)
        self.satNeedFlip.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satNeedFlip.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satNeedFlip.setReadOnly(False)
        self.satNeedFlip.setObjectName("satNeedFlip")
        self.label_300 = QtWidgets.QLabel(self.groupBox_19)
        self.label_300.setGeometry(QtCore.QRect(5, 25, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_300.setFont(font)
        self.label_300.setObjectName("label_300")
        self.label_301 = QtWidgets.QLabel(self.groupBox_19)
        self.label_301.setGeometry(QtCore.QRect(150, 25, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_301.setFont(font)
        self.label_301.setObjectName("label_301")
        self.groupBox_27 = QtWidgets.QGroupBox(self.Satellite)
        self.groupBox_27.setGeometry(QtCore.QRect(10, 15, 391, 56))
        self.groupBox_27.setProperty("large", True)
        self.groupBox_27.setObjectName("groupBox_27")
        self.satelliteSource = QtWidgets.QComboBox(self.groupBox_27)
        self.satelliteSource.setGeometry(QtCore.QRect(10, 25, 271, 26))
        self.satelliteSource.setMinimumSize(QtCore.QSize(0, 25))
        self.satelliteSource.setObjectName("satelliteSource")
        self.filterSatellite = QtWidgets.QLineEdit(self.groupBox_27)
        self.filterSatellite.setGeometry(QtCore.QRect(290, 30, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.filterSatellite.setFont(font)
        self.filterSatellite.setMouseTracking(False)
        self.filterSatellite.setAcceptDrops(False)
        self.filterSatellite.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.filterSatellite.setText("")
        self.filterSatellite.setReadOnly(False)
        self.filterSatellite.setObjectName("filterSatellite")
        self.label_269 = QtWidgets.QLabel(self.groupBox_27)
        self.label_269.setGeometry(QtCore.QRect(290, 10, 81, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_269.setFont(font)
        self.label_269.setObjectName("label_269")
        self.groupBox_52 = QtWidgets.QGroupBox(self.Satellite)
        self.groupBox_52.setGeometry(QtCore.QRect(10, 80, 391, 306))
        self.groupBox_52.setProperty("large", True)
        self.groupBox_52.setObjectName("groupBox_52")
        self.listSatelliteNames = QtWidgets.QListWidget(self.groupBox_52)
        self.listSatelliteNames.setGeometry(QtCore.QRect(5, 25, 381, 276))
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.listSatelliteNames.setFont(font)
        self.listSatelliteNames.setObjectName("listSatelliteNames")
        self.progSatellitesFiltered = QtWidgets.QPushButton(self.Satellite)
        self.progSatellitesFiltered.setGeometry(QtCore.QRect(210, 395, 191, 26))
        self.progSatellitesFiltered.setObjectName("progSatellitesFiltered")
        self.progSatellitesFull = QtWidgets.QPushButton(self.Satellite)
        self.progSatellitesFull.setGeometry(QtCore.QRect(10, 395, 191, 26))
        self.progSatellitesFull.setObjectName("progSatellitesFull")
        self.mainTabWidget.addTab(self.Satellite, "")
        self.MPC = QtWidgets.QWidget()
        self.MPC.setObjectName("MPC")
        self.groupBox_30 = QtWidgets.QGroupBox(self.MPC)
        self.groupBox_30.setGeometry(QtCore.QRect(10, 15, 391, 56))
        self.groupBox_30.setProperty("large", True)
        self.groupBox_30.setObjectName("groupBox_30")
        self.minorPlanetSource = QtWidgets.QComboBox(self.groupBox_30)
        self.minorPlanetSource.setGeometry(QtCore.QRect(10, 25, 271, 26))
        self.minorPlanetSource.setMinimumSize(QtCore.QSize(0, 25))
        self.minorPlanetSource.setObjectName("minorPlanetSource")
        self.filterMinorPlanet = QtWidgets.QLineEdit(self.groupBox_30)
        self.filterMinorPlanet.setGeometry(QtCore.QRect(290, 30, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.filterMinorPlanet.setFont(font)
        self.filterMinorPlanet.setMouseTracking(False)
        self.filterMinorPlanet.setAcceptDrops(False)
        self.filterMinorPlanet.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.filterMinorPlanet.setText("")
        self.filterMinorPlanet.setReadOnly(False)
        self.filterMinorPlanet.setObjectName("filterMinorPlanet")
        self.label_270 = QtWidgets.QLabel(self.groupBox_30)
        self.label_270.setGeometry(QtCore.QRect(290, 10, 81, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_270.setFont(font)
        self.label_270.setObjectName("label_270")
        self.groupBox_58 = QtWidgets.QGroupBox(self.MPC)
        self.groupBox_58.setGeometry(QtCore.QRect(10, 80, 391, 306))
        self.groupBox_58.setProperty("large", True)
        self.groupBox_58.setObjectName("groupBox_58")
        self.listMinorPlanetNames = QtWidgets.QListWidget(self.groupBox_58)
        self.listMinorPlanetNames.setGeometry(QtCore.QRect(5, 25, 381, 276))
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.listMinorPlanetNames.setFont(font)
        self.listMinorPlanetNames.setObjectName("listMinorPlanetNames")
        self.line_69 = QtWidgets.QFrame(self.MPC)
        self.line_69.setGeometry(QtCore.QRect(410, 10, 11, 416))
        self.line_69.setStyleSheet("")
        self.line_69.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_69.setLineWidth(2)
        self.line_69.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_69.setObjectName("line_69")
        self.progMinorPlanetsFull = QtWidgets.QPushButton(self.MPC)
        self.progMinorPlanetsFull.setGeometry(QtCore.QRect(10, 395, 191, 26))
        self.progMinorPlanetsFull.setObjectName("progMinorPlanetsFull")
        self.progMinorPlanetsFiltered = QtWidgets.QPushButton(self.MPC)
        self.progMinorPlanetsFiltered.setGeometry(QtCore.QRect(210, 395, 191, 26))
        self.progMinorPlanetsFiltered.setObjectName("progMinorPlanetsFiltered")
        self.groupBox_36 = QtWidgets.QGroupBox(self.MPC)
        self.groupBox_36.setGeometry(QtCore.QRect(430, 15, 351, 96))
        self.groupBox_36.setProperty("large", True)
        self.groupBox_36.setObjectName("groupBox_36")
        self.downloadIERS = QtWidgets.QPushButton(self.groupBox_36)
        self.downloadIERS.setGeometry(QtCore.QRect(10, 25, 331, 26))
        self.downloadIERS.setObjectName("downloadIERS")
        self.progEarthRotationData = QtWidgets.QPushButton(self.groupBox_36)
        self.progEarthRotationData.setGeometry(QtCore.QRect(10, 60, 331, 26))
        self.progEarthRotationData.setObjectName("progEarthRotationData")
        self.mainTabWidget.addTab(self.MPC, "")
        self.Power = QtWidgets.QWidget()
        self.Power.setObjectName("Power")
        self.groupBox_13 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_13.setGeometry(QtCore.QRect(410, 245, 366, 71))
        self.groupBox_13.setProperty("large", True)
        self.groupBox_13.setObjectName("groupBox_13")
        self.label_283 = QtWidgets.QLabel(self.groupBox_13)
        self.label_283.setGeometry(QtCore.QRect(119, 45, 16, 21))
        self.label_283.setObjectName("label_283")
        self.label_292 = QtWidgets.QLabel(self.groupBox_13)
        self.label_292.setGeometry(QtCore.QRect(11, 45, 46, 21))
        self.label_292.setObjectName("label_292")
        self.powerCurrent4 = QtWidgets.QLineEdit(self.groupBox_13)
        self.powerCurrent4.setEnabled(False)
        self.powerCurrent4.setGeometry(QtCore.QRect(59, 45, 56, 21))
        self.powerCurrent4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerCurrent4.setObjectName("powerCurrent4")
        self.powerBootPort4 = QtWidgets.QCheckBox(self.groupBox_13)
        self.powerBootPort4.setGeometry(QtCore.QRect(220, 45, 106, 21))
        self.powerBootPort4.setMinimumSize(QtCore.QSize(0, 20))
        self.powerBootPort4.setObjectName("powerBootPort4")
        self.powerLabel4 = QtWidgets.QLabel(self.groupBox_13)
        self.powerLabel4.setGeometry(QtCore.QRect(10, 20, 91, 21))
        self.powerLabel4.setObjectName("powerLabel4")
        self.powerPort4 = QtWidgets.QPushButton(self.groupBox_13)
        self.powerPort4.setGeometry(QtCore.QRect(220, 20, 101, 21))
        self.powerPort4.setObjectName("powerPort4")
        self.groupBox_17 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_17.setGeometry(QtCore.QRect(410, 170, 366, 71))
        self.groupBox_17.setProperty("large", True)
        self.groupBox_17.setObjectName("groupBox_17")
        self.label_317 = QtWidgets.QLabel(self.groupBox_17)
        self.label_317.setGeometry(QtCore.QRect(119, 45, 16, 21))
        self.label_317.setObjectName("label_317")
        self.label_318 = QtWidgets.QLabel(self.groupBox_17)
        self.label_318.setGeometry(QtCore.QRect(11, 45, 46, 21))
        self.label_318.setObjectName("label_318")
        self.powerLabel3 = QtWidgets.QLabel(self.groupBox_17)
        self.powerLabel3.setGeometry(QtCore.QRect(10, 20, 91, 21))
        self.powerLabel3.setObjectName("powerLabel3")
        self.powerCurrent3 = QtWidgets.QLineEdit(self.groupBox_17)
        self.powerCurrent3.setEnabled(False)
        self.powerCurrent3.setGeometry(QtCore.QRect(60, 45, 56, 21))
        self.powerCurrent3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerCurrent3.setObjectName("powerCurrent3")
        self.powerBootPort3 = QtWidgets.QCheckBox(self.groupBox_17)
        self.powerBootPort3.setGeometry(QtCore.QRect(220, 45, 106, 21))
        self.powerBootPort3.setMinimumSize(QtCore.QSize(0, 20))
        self.powerBootPort3.setObjectName("powerBootPort3")
        self.powerPort3 = QtWidgets.QPushButton(self.groupBox_17)
        self.powerPort3.setGeometry(QtCore.QRect(220, 20, 101, 21))
        self.powerPort3.setObjectName("powerPort3")
        self.groupBox_18 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_18.setGeometry(QtCore.QRect(410, 95, 366, 71))
        self.groupBox_18.setProperty("large", True)
        self.groupBox_18.setObjectName("groupBox_18")
        self.label_319 = QtWidgets.QLabel(self.groupBox_18)
        self.label_319.setGeometry(QtCore.QRect(119, 45, 16, 21))
        self.label_319.setObjectName("label_319")
        self.label_320 = QtWidgets.QLabel(self.groupBox_18)
        self.label_320.setGeometry(QtCore.QRect(11, 45, 46, 21))
        self.label_320.setObjectName("label_320")
        self.powerCurrent2 = QtWidgets.QLineEdit(self.groupBox_18)
        self.powerCurrent2.setEnabled(False)
        self.powerCurrent2.setGeometry(QtCore.QRect(60, 45, 56, 21))
        self.powerCurrent2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerCurrent2.setObjectName("powerCurrent2")
        self.powerLabel2 = QtWidgets.QLabel(self.groupBox_18)
        self.powerLabel2.setGeometry(QtCore.QRect(10, 20, 91, 21))
        self.powerLabel2.setObjectName("powerLabel2")
        self.powerBootPort2 = QtWidgets.QCheckBox(self.groupBox_18)
        self.powerBootPort2.setGeometry(QtCore.QRect(220, 45, 106, 21))
        self.powerBootPort2.setMinimumSize(QtCore.QSize(0, 20))
        self.powerBootPort2.setObjectName("powerBootPort2")
        self.powerPort2 = QtWidgets.QPushButton(self.groupBox_18)
        self.powerPort2.setGeometry(QtCore.QRect(220, 20, 101, 21))
        self.powerPort2.setObjectName("powerPort2")
        self.box = QtWidgets.QGroupBox(self.Power)
        self.box.setGeometry(QtCore.QRect(410, 20, 366, 71))
        self.box.setProperty("large", True)
        self.box.setObjectName("box")
        self.label_321 = QtWidgets.QLabel(self.box)
        self.label_321.setGeometry(QtCore.QRect(119, 45, 16, 21))
        self.label_321.setObjectName("label_321")
        self.label_322 = QtWidgets.QLabel(self.box)
        self.label_322.setGeometry(QtCore.QRect(11, 45, 46, 21))
        self.label_322.setObjectName("label_322")
        self.powerCurrent1 = QtWidgets.QLineEdit(self.box)
        self.powerCurrent1.setEnabled(False)
        self.powerCurrent1.setGeometry(QtCore.QRect(60, 45, 56, 21))
        self.powerCurrent1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerCurrent1.setObjectName("powerCurrent1")
        self.powerLabel1 = QtWidgets.QLabel(self.box)
        self.powerLabel1.setGeometry(QtCore.QRect(10, 20, 91, 21))
        self.powerLabel1.setObjectName("powerLabel1")
        self.powerBootPort1 = QtWidgets.QCheckBox(self.box)
        self.powerBootPort1.setGeometry(QtCore.QRect(220, 45, 106, 21))
        self.powerBootPort1.setMinimumSize(QtCore.QSize(0, 20))
        self.powerBootPort1.setObjectName("powerBootPort1")
        self.powerPort1 = QtWidgets.QPushButton(self.box)
        self.powerPort1.setGeometry(QtCore.QRect(220, 20, 101, 21))
        self.powerPort1.setObjectName("powerPort1")
        self.line_41 = QtWidgets.QFrame(self.Power)
        self.line_41.setGeometry(QtCore.QRect(390, 10, 11, 416))
        self.line_41.setStyleSheet("")
        self.line_41.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_41.setLineWidth(2)
        self.line_41.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_41.setObjectName("line_41")
        self.groupHubUSB = QtWidgets.QGroupBox(self.Power)
        self.groupHubUSB.setGeometry(QtCore.QRect(410, 320, 366, 41))
        self.groupHubUSB.setProperty("large", True)
        self.groupHubUSB.setObjectName("groupHubUSB")
        self.hubUSB = QtWidgets.QPushButton(self.groupHubUSB)
        self.hubUSB.setGeometry(QtCore.QRect(220, 15, 101, 21))
        self.hubUSB.setObjectName("hubUSB")
        self.groupBox_21 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_21.setGeometry(QtCore.QRect(15, 25, 366, 146))
        self.groupBox_21.setProperty("large", True)
        self.groupBox_21.setObjectName("groupBox_21")
        self.label_192 = QtWidgets.QLabel(self.groupBox_21)
        self.label_192.setGeometry(QtCore.QRect(330, 45, 16, 21))
        self.label_192.setObjectName("label_192")
        self.label_284 = QtWidgets.QLabel(self.groupBox_21)
        self.label_284.setGeometry(QtCore.QRect(330, 70, 20, 21))
        self.label_284.setObjectName("label_284")
        self.label_193 = QtWidgets.QLabel(self.groupBox_21)
        self.label_193.setGeometry(QtCore.QRect(140, 70, 16, 21))
        self.label_193.setObjectName("label_193")
        self.label_191 = QtWidgets.QLabel(self.groupBox_21)
        self.label_191.setGeometry(QtCore.QRect(140, 45, 16, 21))
        self.label_191.setObjectName("label_191")
        self.label_286 = QtWidgets.QLabel(self.groupBox_21)
        self.label_286.setGeometry(QtCore.QRect(10, 70, 66, 21))
        self.label_286.setObjectName("label_286")
        self.sensorPower = QtWidgets.QLineEdit(self.groupBox_21)
        self.sensorPower.setEnabled(False)
        self.sensorPower.setGeometry(QtCore.QRect(80, 45, 56, 21))
        self.sensorPower.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorPower.setObjectName("sensorPower")
        self.label_287 = QtWidgets.QLabel(self.groupBox_21)
        self.label_287.setGeometry(QtCore.QRect(185, 45, 81, 21))
        self.label_287.setObjectName("label_287")
        self.label_165 = QtWidgets.QLabel(self.groupBox_21)
        self.label_165.setGeometry(QtCore.QRect(10, 21, 66, 21))
        self.label_165.setObjectName("label_165")
        self.label_285 = QtWidgets.QLabel(self.groupBox_21)
        self.label_285.setGeometry(QtCore.QRect(200, 70, 66, 21))
        self.label_285.setObjectName("label_285")
        self.label_289 = QtWidgets.QLabel(self.groupBox_21)
        self.label_289.setGeometry(QtCore.QRect(215, 20, 51, 21))
        self.label_289.setObjectName("label_289")
        self.sensorVoltage = QtWidgets.QLineEdit(self.groupBox_21)
        self.sensorVoltage.setEnabled(False)
        self.sensorVoltage.setGeometry(QtCore.QRect(80, 21, 56, 21))
        self.sensorVoltage.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorVoltage.setObjectName("sensorVoltage")
        self.label_288 = QtWidgets.QLabel(self.groupBox_21)
        self.label_288.setGeometry(QtCore.QRect(10, 45, 66, 21))
        self.label_288.setObjectName("label_288")
        self.consumptionWattHours = QtWidgets.QLineEdit(self.groupBox_21)
        self.consumptionWattHours.setEnabled(False)
        self.consumptionWattHours.setGeometry(QtCore.QRect(270, 70, 56, 21))
        self.consumptionWattHours.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.consumptionWattHours.setObjectName("consumptionWattHours")
        self.sensorCurrent = QtWidgets.QLineEdit(self.groupBox_21)
        self.sensorCurrent.setEnabled(False)
        self.sensorCurrent.setGeometry(QtCore.QRect(270, 20, 56, 21))
        self.sensorCurrent.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorCurrent.setObjectName("sensorCurrent")
        self.consumptionAvgAmps = QtWidgets.QLineEdit(self.groupBox_21)
        self.consumptionAvgAmps.setEnabled(False)
        self.consumptionAvgAmps.setGeometry(QtCore.QRect(270, 45, 56, 21))
        self.consumptionAvgAmps.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.consumptionAvgAmps.setObjectName("consumptionAvgAmps")
        self.label_173 = QtWidgets.QLabel(self.groupBox_21)
        self.label_173.setGeometry(QtCore.QRect(140, 21, 16, 21))
        self.label_173.setObjectName("label_173")
        self.consumptionAmpHours = QtWidgets.QLineEdit(self.groupBox_21)
        self.consumptionAmpHours.setEnabled(False)
        self.consumptionAmpHours.setGeometry(QtCore.QRect(80, 70, 56, 21))
        self.consumptionAmpHours.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.consumptionAmpHours.setObjectName("consumptionAmpHours")
        self.label_172 = QtWidgets.QLabel(self.groupBox_21)
        self.label_172.setGeometry(QtCore.QRect(330, 20, 16, 21))
        self.label_172.setObjectName("label_172")
        self.rebootUPB = QtWidgets.QPushButton(self.groupBox_21)
        self.rebootUPB.setGeometry(QtCore.QRect(10, 115, 101, 21))
        self.rebootUPB.setObjectName("rebootUPB")
        self.groupDewA = QtWidgets.QGroupBox(self.Power)
        self.groupDewA.setGeometry(QtCore.QRect(15, 255, 366, 51))
        self.groupDewA.setProperty("large", True)
        self.groupDewA.setObjectName("groupDewA")
        self.dewCurrentA = QtWidgets.QLineEdit(self.groupDewA)
        self.dewCurrentA.setEnabled(False)
        self.dewCurrentA.setGeometry(QtCore.QRect(270, 20, 56, 21))
        self.dewCurrentA.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dewCurrentA.setObjectName("dewCurrentA")
        self.label_276 = QtWidgets.QLabel(self.groupDewA)
        self.label_276.setGeometry(QtCore.QRect(150, 20, 16, 21))
        self.label_276.setObjectName("label_276")
        self.label_268 = QtWidgets.QLabel(self.groupDewA)
        self.label_268.setGeometry(QtCore.QRect(215, 20, 51, 21))
        self.label_268.setObjectName("label_268")
        self.label_279 = QtWidgets.QLabel(self.groupDewA)
        self.label_279.setGeometry(QtCore.QRect(330, 20, 16, 21))
        self.label_279.setObjectName("label_279")
        self.label_266 = QtWidgets.QLabel(self.groupDewA)
        self.label_266.setGeometry(QtCore.QRect(11, 20, 79, 21))
        self.label_266.setObjectName("label_266")
        self.dewA = QtWidgets.QLineEdit(self.groupDewA)
        self.dewA.setEnabled(False)
        self.dewA.setGeometry(QtCore.QRect(110, 20, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.dewA.setFont(font)
        self.dewA.setMouseTracking(False)
        self.dewA.setFocusPolicy(QtCore.Qt.NoFocus)
        self.dewA.setAcceptDrops(False)
        self.dewA.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.dewA.setInputMask("")
        self.dewA.setCursorPosition(1)
        self.dewA.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dewA.setReadOnly(True)
        self.dewA.setProperty("input", True)
        self.dewA.setObjectName("dewA")
        self.groupDewB = QtWidgets.QGroupBox(self.Power)
        self.groupDewB.setGeometry(QtCore.QRect(15, 315, 366, 51))
        self.groupDewB.setProperty("large", True)
        self.groupDewB.setObjectName("groupDewB")
        self.label_280 = QtWidgets.QLabel(self.groupDewB)
        self.label_280.setGeometry(QtCore.QRect(150, 20, 16, 21))
        self.label_280.setObjectName("label_280")
        self.label_274 = QtWidgets.QLabel(self.groupDewB)
        self.label_274.setGeometry(QtCore.QRect(215, 20, 51, 21))
        self.label_274.setObjectName("label_274")
        self.label_281 = QtWidgets.QLabel(self.groupDewB)
        self.label_281.setGeometry(QtCore.QRect(330, 20, 16, 21))
        self.label_281.setObjectName("label_281")
        self.label_275 = QtWidgets.QLabel(self.groupDewB)
        self.label_275.setGeometry(QtCore.QRect(11, 20, 79, 21))
        self.label_275.setObjectName("label_275")
        self.dewCurrentB = QtWidgets.QLineEdit(self.groupDewB)
        self.dewCurrentB.setEnabled(False)
        self.dewCurrentB.setGeometry(QtCore.QRect(270, 20, 56, 21))
        self.dewCurrentB.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dewCurrentB.setObjectName("dewCurrentB")
        self.dewB = QtWidgets.QLineEdit(self.groupDewB)
        self.dewB.setEnabled(False)
        self.dewB.setGeometry(QtCore.QRect(110, 20, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.dewB.setFont(font)
        self.dewB.setMouseTracking(False)
        self.dewB.setFocusPolicy(QtCore.Qt.NoFocus)
        self.dewB.setAcceptDrops(False)
        self.dewB.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.dewB.setInputMask("")
        self.dewB.setCursorPosition(1)
        self.dewB.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dewB.setReadOnly(True)
        self.dewB.setProperty("input", True)
        self.dewB.setObjectName("dewB")
        self.groupDewC = QtWidgets.QGroupBox(self.Power)
        self.groupDewC.setGeometry(QtCore.QRect(15, 375, 366, 51))
        self.groupDewC.setProperty("large", True)
        self.groupDewC.setObjectName("groupDewC")
        self.label_282 = QtWidgets.QLabel(self.groupDewC)
        self.label_282.setGeometry(QtCore.QRect(150, 20, 16, 21))
        self.label_282.setObjectName("label_282")
        self.label_277 = QtWidgets.QLabel(self.groupDewC)
        self.label_277.setGeometry(QtCore.QRect(215, 20, 51, 21))
        self.label_277.setObjectName("label_277")
        self.label_290 = QtWidgets.QLabel(self.groupDewC)
        self.label_290.setGeometry(QtCore.QRect(330, 20, 16, 21))
        self.label_290.setObjectName("label_290")
        self.label_278 = QtWidgets.QLabel(self.groupDewC)
        self.label_278.setGeometry(QtCore.QRect(11, 20, 79, 21))
        self.label_278.setObjectName("label_278")
        self.dewCurrentC = QtWidgets.QLineEdit(self.groupDewC)
        self.dewCurrentC.setEnabled(False)
        self.dewCurrentC.setGeometry(QtCore.QRect(270, 20, 56, 21))
        self.dewCurrentC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dewCurrentC.setObjectName("dewCurrentC")
        self.dewC = QtWidgets.QLineEdit(self.groupDewC)
        self.dewC.setEnabled(False)
        self.dewC.setGeometry(QtCore.QRect(110, 20, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.dewC.setFont(font)
        self.dewC.setMouseTracking(False)
        self.dewC.setFocusPolicy(QtCore.Qt.NoFocus)
        self.dewC.setAcceptDrops(False)
        self.dewC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.dewC.setInputMask("")
        self.dewC.setCursorPosition(1)
        self.dewC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dewC.setReadOnly(True)
        self.dewC.setProperty("input", True)
        self.dewC.setObjectName("dewC")
        self.groupAutoDew = QtWidgets.QGroupBox(self.Power)
        self.groupAutoDew.setEnabled(True)
        self.groupAutoDew.setGeometry(QtCore.QRect(15, 195, 366, 51))
        self.groupAutoDew.setProperty("large", True)
        self.groupAutoDew.setObjectName("groupAutoDew")
        self.autoDew = QtWidgets.QPushButton(self.groupAutoDew)
        self.autoDew.setEnabled(False)
        self.autoDew.setGeometry(QtCore.QRect(10, 20, 101, 21))
        self.autoDew.setObjectName("autoDew")
        self.groupPortUSB = QtWidgets.QGroupBox(self.Power)
        self.groupPortUSB.setGeometry(QtCore.QRect(410, 320, 366, 51))
        self.groupPortUSB.setProperty("large", True)
        self.groupPortUSB.setObjectName("groupPortUSB")
        self.portUSB1 = QtWidgets.QPushButton(self.groupPortUSB)
        self.portUSB1.setGeometry(QtCore.QRect(10, 20, 51, 21))
        self.portUSB1.setObjectName("portUSB1")
        self.portUSB2 = QtWidgets.QPushButton(self.groupPortUSB)
        self.portUSB2.setGeometry(QtCore.QRect(70, 20, 51, 21))
        self.portUSB2.setObjectName("portUSB2")
        self.portUSB3 = QtWidgets.QPushButton(self.groupPortUSB)
        self.portUSB3.setGeometry(QtCore.QRect(130, 20, 51, 21))
        self.portUSB3.setObjectName("portUSB3")
        self.portUSB4 = QtWidgets.QPushButton(self.groupPortUSB)
        self.portUSB4.setGeometry(QtCore.QRect(190, 20, 51, 21))
        self.portUSB4.setObjectName("portUSB4")
        self.portUSB5 = QtWidgets.QPushButton(self.groupPortUSB)
        self.portUSB5.setGeometry(QtCore.QRect(250, 20, 51, 21))
        self.portUSB5.setObjectName("portUSB5")
        self.portUSB6 = QtWidgets.QPushButton(self.groupPortUSB)
        self.portUSB6.setGeometry(QtCore.QRect(310, 20, 51, 21))
        self.portUSB6.setObjectName("portUSB6")
        self.groupAdjustableOutput = QtWidgets.QGroupBox(self.Power)
        self.groupAdjustableOutput.setGeometry(QtCore.QRect(410, 375, 366, 51))
        self.groupAdjustableOutput.setProperty("large", True)
        self.groupAdjustableOutput.setObjectName("groupAdjustableOutput")
        self.adjustableOutput = QtWidgets.QLineEdit(self.groupAdjustableOutput)
        self.adjustableOutput.setEnabled(False)
        self.adjustableOutput.setGeometry(QtCore.QRect(110, 20, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.adjustableOutput.setFont(font)
        self.adjustableOutput.setMouseTracking(False)
        self.adjustableOutput.setFocusPolicy(QtCore.Qt.NoFocus)
        self.adjustableOutput.setAcceptDrops(False)
        self.adjustableOutput.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.adjustableOutput.setInputMask("")
        self.adjustableOutput.setCursorPosition(1)
        self.adjustableOutput.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.adjustableOutput.setReadOnly(True)
        self.adjustableOutput.setProperty("input", True)
        self.adjustableOutput.setObjectName("adjustableOutput")
        self.label_291 = QtWidgets.QLabel(self.groupAdjustableOutput)
        self.label_291.setGeometry(QtCore.QRect(15, 20, 79, 21))
        self.label_291.setObjectName("label_291")
        self.label_293 = QtWidgets.QLabel(self.groupAdjustableOutput)
        self.label_293.setGeometry(QtCore.QRect(150, 20, 16, 21))
        self.label_293.setObjectName("label_293")
        self.mainTabWidget.addTab(self.Power, "")
        self.Tools = QtWidgets.QWidget()
        self.Tools.setObjectName("Tools")
        self.line_71 = QtWidgets.QFrame(self.Tools)
        self.line_71.setGeometry(QtCore.QRect(385, 10, 11, 416))
        self.line_71.setStyleSheet("")
        self.line_71.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_71.setLineWidth(2)
        self.line_71.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_71.setObjectName("line_71")
        self.groupBox_29 = QtWidgets.QGroupBox(self.Tools)
        self.groupBox_29.setGeometry(QtCore.QRect(405, 20, 366, 191))
        self.groupBox_29.setProperty("large", True)
        self.groupBox_29.setObjectName("groupBox_29")
        self.posButton0 = QtWidgets.QPushButton(self.groupBox_29)
        self.posButton0.setGeometry(QtCore.QRect(10, 22, 166, 25))
        self.posButton0.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posButton0.setFont(font)
        self.posButton0.setObjectName("posButton0")
        self.posButton8 = QtWidgets.QPushButton(self.groupBox_29)
        self.posButton8.setGeometry(QtCore.QRect(10, 140, 166, 25))
        self.posButton8.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posButton8.setFont(font)
        self.posButton8.setObjectName("posButton8")
        self.posButton1 = QtWidgets.QPushButton(self.groupBox_29)
        self.posButton1.setGeometry(QtCore.QRect(190, 22, 166, 25))
        self.posButton1.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posButton1.setFont(font)
        self.posButton1.setObjectName("posButton1")
        self.posButton6 = QtWidgets.QPushButton(self.groupBox_29)
        self.posButton6.setGeometry(QtCore.QRect(10, 110, 166, 25))
        self.posButton6.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posButton6.setFont(font)
        self.posButton6.setObjectName("posButton6")
        self.posButton9 = QtWidgets.QPushButton(self.groupBox_29)
        self.posButton9.setGeometry(QtCore.QRect(190, 140, 166, 25))
        self.posButton9.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posButton9.setFont(font)
        self.posButton9.setObjectName("posButton9")
        self.posButton3 = QtWidgets.QPushButton(self.groupBox_29)
        self.posButton3.setGeometry(QtCore.QRect(190, 50, 166, 25))
        self.posButton3.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posButton3.setFont(font)
        self.posButton3.setObjectName("posButton3")
        self.posButton4 = QtWidgets.QPushButton(self.groupBox_29)
        self.posButton4.setGeometry(QtCore.QRect(10, 80, 166, 25))
        self.posButton4.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posButton4.setFont(font)
        self.posButton4.setObjectName("posButton4")
        self.posButton5 = QtWidgets.QPushButton(self.groupBox_29)
        self.posButton5.setGeometry(QtCore.QRect(190, 80, 166, 25))
        self.posButton5.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posButton5.setFont(font)
        self.posButton5.setObjectName("posButton5")
        self.posButton7 = QtWidgets.QPushButton(self.groupBox_29)
        self.posButton7.setGeometry(QtCore.QRect(190, 110, 166, 25))
        self.posButton7.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posButton7.setFont(font)
        self.posButton7.setObjectName("posButton7")
        self.posButton2 = QtWidgets.QPushButton(self.groupBox_29)
        self.posButton2.setGeometry(QtCore.QRect(10, 50, 166, 25))
        self.posButton2.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posButton2.setFont(font)
        self.posButton2.setObjectName("posButton2")
        self.parkMountAfterSlew = QtWidgets.QCheckBox(self.groupBox_29)
        self.parkMountAfterSlew.setGeometry(QtCore.QRect(10, 165, 171, 25))
        self.parkMountAfterSlew.setMinimumSize(QtCore.QSize(0, 25))
        self.parkMountAfterSlew.setObjectName("parkMountAfterSlew")
        self.groupBox_46 = QtWidgets.QGroupBox(self.Tools)
        self.groupBox_46.setGeometry(QtCore.QRect(10, 20, 366, 406))
        self.groupBox_46.setProperty("large", True)
        self.groupBox_46.setObjectName("groupBox_46")
        self.rename1 = QtWidgets.QComboBox(self.groupBox_46)
        self.rename1.setGeometry(QtCore.QRect(130, 210, 226, 21))
        self.rename1.setMinimumSize(QtCore.QSize(0, 0))
        self.rename1.setObjectName("rename1")
        self.label_306 = QtWidgets.QLabel(self.groupBox_46)
        self.label_306.setGeometry(QtCore.QRect(10, 135, 56, 21))
        self.label_306.setMinimumSize(QtCore.QSize(0, 0))
        self.label_306.setObjectName("label_306")
        self.renameText = QtWidgets.QLineEdit(self.groupBox_46)
        self.renameText.setEnabled(True)
        self.renameText.setGeometry(QtCore.QRect(130, 135, 226, 21))
        self.renameText.setMinimumSize(QtCore.QSize(0, 0))
        self.renameText.setObjectName("renameText")
        self.rename2 = QtWidgets.QComboBox(self.groupBox_46)
        self.rename2.setGeometry(QtCore.QRect(130, 235, 226, 21))
        self.rename2.setMinimumSize(QtCore.QSize(0, 0))
        self.rename2.setObjectName("rename2")
        self.renameDir = QtWidgets.QLineEdit(self.groupBox_46)
        self.renameDir.setEnabled(False)
        self.renameDir.setGeometry(QtCore.QRect(10, 45, 346, 21))
        self.renameDir.setMinimumSize(QtCore.QSize(0, 0))
        self.renameDir.setObjectName("renameDir")
        self.label_305 = QtWidgets.QLabel(self.groupBox_46)
        self.label_305.setGeometry(QtCore.QRect(9, 20, 56, 21))
        self.label_305.setMinimumSize(QtCore.QSize(0, 0))
        self.label_305.setObjectName("label_305")
        self.renameInputSelect = QtWidgets.QPushButton(self.groupBox_46)
        self.renameInputSelect.setGeometry(QtCore.QRect(265, 70, 91, 25))
        self.renameInputSelect.setMinimumSize(QtCore.QSize(0, 0))
        self.renameInputSelect.setObjectName("renameInputSelect")
        self.label_8 = QtWidgets.QLabel(self.groupBox_46)
        self.label_8.setGeometry(QtCore.QRect(9, 210, 66, 21))
        self.label_8.setObjectName("label_8")
        self.label_60 = QtWidgets.QLabel(self.groupBox_46)
        self.label_60.setGeometry(QtCore.QRect(9, 185, 186, 16))
        self.label_60.setObjectName("label_60")
        self.label_9 = QtWidgets.QLabel(self.groupBox_46)
        self.label_9.setGeometry(QtCore.QRect(9, 235, 66, 21))
        self.label_9.setObjectName("label_9")
        self.checkIncludeSubdirs = QtWidgets.QCheckBox(self.groupBox_46)
        self.checkIncludeSubdirs.setGeometry(QtCore.QRect(10, 70, 171, 25))
        self.checkIncludeSubdirs.setMinimumSize(QtCore.QSize(0, 25))
        self.checkIncludeSubdirs.setObjectName("checkIncludeSubdirs")
        self.rename5 = QtWidgets.QComboBox(self.groupBox_46)
        self.rename5.setGeometry(QtCore.QRect(131, 310, 226, 21))
        self.rename5.setMinimumSize(QtCore.QSize(0, 0))
        self.rename5.setObjectName("rename5")
        self.rename4 = QtWidgets.QComboBox(self.groupBox_46)
        self.rename4.setGeometry(QtCore.QRect(131, 285, 226, 21))
        self.rename4.setMinimumSize(QtCore.QSize(0, 0))
        self.rename4.setObjectName("rename4")
        self.label_57 = QtWidgets.QLabel(self.groupBox_46)
        self.label_57.setGeometry(QtCore.QRect(10, 340, 66, 21))
        self.label_57.setObjectName("label_57")
        self.label_10 = QtWidgets.QLabel(self.groupBox_46)
        self.label_10.setGeometry(QtCore.QRect(10, 260, 66, 21))
        self.label_10.setObjectName("label_10")
        self.label_11 = QtWidgets.QLabel(self.groupBox_46)
        self.label_11.setGeometry(QtCore.QRect(10, 285, 66, 21))
        self.label_11.setObjectName("label_11")
        self.renameProgress = QtWidgets.QProgressBar(self.groupBox_46)
        self.renameProgress.setGeometry(QtCore.QRect(80, 340, 276, 21))
        self.renameProgress.setMinimumSize(QtCore.QSize(0, 0))
        self.renameProgress.setProperty("value", 0)
        self.renameProgress.setAlignment(QtCore.Qt.AlignCenter)
        self.renameProgress.setObjectName("renameProgress")
        self.renameStart = QtWidgets.QPushButton(self.groupBox_46)
        self.renameStart.setGeometry(QtCore.QRect(80, 370, 111, 26))
        self.renameStart.setMinimumSize(QtCore.QSize(80, 0))
        self.renameStart.setObjectName("renameStart")
        self.label_39 = QtWidgets.QLabel(self.groupBox_46)
        self.label_39.setGeometry(QtCore.QRect(10, 310, 66, 21))
        self.label_39.setObjectName("label_39")
        self.rename3 = QtWidgets.QComboBox(self.groupBox_46)
        self.rename3.setGeometry(QtCore.QRect(131, 260, 226, 21))
        self.rename3.setMinimumSize(QtCore.QSize(0, 0))
        self.rename3.setObjectName("rename3")
        self.label_308 = QtWidgets.QLabel(self.groupBox_46)
        self.label_308.setGeometry(QtCore.QRect(10, 110, 116, 21))
        self.label_308.setMinimumSize(QtCore.QSize(0, 0))
        self.label_308.setObjectName("label_308")
        self.newObjectName = QtWidgets.QLineEdit(self.groupBox_46)
        self.newObjectName.setEnabled(True)
        self.newObjectName.setGeometry(QtCore.QRect(130, 110, 226, 21))
        self.newObjectName.setMinimumSize(QtCore.QSize(0, 0))
        self.newObjectName.setObjectName("newObjectName")
        self.tabMoving = QtWidgets.QTabWidget(self.Tools)
        self.tabMoving.setGeometry(QtCore.QRect(405, 230, 366, 196))
        self.tabMoving.setObjectName("tabMoving")
        self.tab_4 = QtWidgets.QWidget()
        self.tab_4.setObjectName("tab_4")
        self.moveNorth = QtWidgets.QPushButton(self.tab_4)
        self.moveNorth.setGeometry(QtCore.QRect(95, 10, 66, 25))
        self.moveNorth.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveNorth.setFont(font)
        self.moveNorth.setObjectName("moveNorth")
        self.moveNorthWest = QtWidgets.QPushButton(self.tab_4)
        self.moveNorthWest.setGeometry(QtCore.QRect(10, 10, 66, 25))
        self.moveNorthWest.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveNorthWest.setFont(font)
        self.moveNorthWest.setObjectName("moveNorthWest")
        self.moveWest = QtWidgets.QPushButton(self.tab_4)
        self.moveWest.setGeometry(QtCore.QRect(10, 50, 66, 25))
        self.moveWest.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveWest.setFont(font)
        self.moveWest.setObjectName("moveWest")
        self.moveSouthEast = QtWidgets.QPushButton(self.tab_4)
        self.moveSouthEast.setGeometry(QtCore.QRect(180, 90, 66, 25))
        self.moveSouthEast.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveSouthEast.setFont(font)
        self.moveSouthEast.setObjectName("moveSouthEast")
        self.moveSouth = QtWidgets.QPushButton(self.tab_4)
        self.moveSouth.setGeometry(QtCore.QRect(95, 90, 66, 25))
        self.moveSouth.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveSouth.setFont(font)
        self.moveSouth.setObjectName("moveSouth")
        self.moveNorthEast = QtWidgets.QPushButton(self.tab_4)
        self.moveNorthEast.setGeometry(QtCore.QRect(180, 10, 66, 25))
        self.moveNorthEast.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveNorthEast.setFont(font)
        self.moveNorthEast.setObjectName("moveNorthEast")
        self.moveSouthWest = QtWidgets.QPushButton(self.tab_4)
        self.moveSouthWest.setGeometry(QtCore.QRect(10, 90, 66, 25))
        self.moveSouthWest.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveSouthWest.setFont(font)
        self.moveSouthWest.setObjectName("moveSouthWest")
        self.stopMoveAll = QtWidgets.QPushButton(self.tab_4)
        self.stopMoveAll.setGeometry(QtCore.QRect(95, 50, 66, 25))
        self.stopMoveAll.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.stopMoveAll.setFont(font)
        self.stopMoveAll.setObjectName("stopMoveAll")
        self.moveEast = QtWidgets.QPushButton(self.tab_4)
        self.moveEast.setGeometry(QtCore.QRect(180, 50, 66, 25))
        self.moveEast.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveEast.setFont(font)
        self.moveEast.setObjectName("moveEast")
        self.groupBox_20 = QtWidgets.QGroupBox(self.tab_4)
        self.groupBox_20.setGeometry(QtCore.QRect(270, 5, 81, 116))
        self.groupBox_20.setObjectName("groupBox_20")
        self.slewSpeedMax = QtWidgets.QRadioButton(self.groupBox_20)
        self.slewSpeedMax.setGeometry(QtCore.QRect(10, 15, 56, 25))
        self.slewSpeedMax.setMinimumSize(QtCore.QSize(0, 25))
        self.slewSpeedMax.setObjectName("slewSpeedMax")
        self.slewSpeedMed = QtWidgets.QRadioButton(self.groupBox_20)
        self.slewSpeedMed.setGeometry(QtCore.QRect(10, 65, 61, 25))
        self.slewSpeedMed.setMinimumSize(QtCore.QSize(0, 25))
        self.slewSpeedMed.setObjectName("slewSpeedMed")
        self.slewSpeedLow = QtWidgets.QRadioButton(self.groupBox_20)
        self.slewSpeedLow.setGeometry(QtCore.QRect(10, 90, 61, 25))
        self.slewSpeedLow.setMinimumSize(QtCore.QSize(0, 25))
        self.slewSpeedLow.setObjectName("slewSpeedLow")
        self.slewSpeedHigh = QtWidgets.QRadioButton(self.groupBox_20)
        self.slewSpeedHigh.setGeometry(QtCore.QRect(10, 40, 61, 25))
        self.slewSpeedHigh.setMinimumSize(QtCore.QSize(0, 25))
        self.slewSpeedHigh.setObjectName("slewSpeedHigh")
        self.moveDuration = QtWidgets.QComboBox(self.tab_4)
        self.moveDuration.setGeometry(QtCore.QRect(10, 130, 236, 25))
        self.moveDuration.setMinimumSize(QtCore.QSize(0, 25))
        self.moveDuration.setObjectName("moveDuration")
        self.moveDuration.addItem("")
        self.moveDuration.addItem("")
        self.moveDuration.addItem("")
        self.moveDuration.addItem("")
        self.moveDuration.addItem("")
        self.tabMoving.addTab(self.tab_4, "")
        self.tab_5 = QtWidgets.QWidget()
        self.tab_5.setObjectName("tab_5")
        self.moveWestAltAz = QtWidgets.QPushButton(self.tab_5)
        self.moveWestAltAz.setGeometry(QtCore.QRect(10, 50, 66, 25))
        self.moveWestAltAz.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveWestAltAz.setFont(font)
        self.moveWestAltAz.setObjectName("moveWestAltAz")
        self.moveEastAltAz = QtWidgets.QPushButton(self.tab_5)
        self.moveEastAltAz.setGeometry(QtCore.QRect(180, 50, 66, 25))
        self.moveEastAltAz.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveEastAltAz.setFont(font)
        self.moveEastAltAz.setObjectName("moveEastAltAz")
        self.moveSouthWestAltAz = QtWidgets.QPushButton(self.tab_5)
        self.moveSouthWestAltAz.setGeometry(QtCore.QRect(10, 90, 66, 25))
        self.moveSouthWestAltAz.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveSouthWestAltAz.setFont(font)
        self.moveSouthWestAltAz.setObjectName("moveSouthWestAltAz")
        self.moveSouthAltAz = QtWidgets.QPushButton(self.tab_5)
        self.moveSouthAltAz.setGeometry(QtCore.QRect(95, 90, 66, 25))
        self.moveSouthAltAz.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveSouthAltAz.setFont(font)
        self.moveSouthAltAz.setObjectName("moveSouthAltAz")
        self.moveNorthWestAltAz = QtWidgets.QPushButton(self.tab_5)
        self.moveNorthWestAltAz.setGeometry(QtCore.QRect(10, 10, 66, 25))
        self.moveNorthWestAltAz.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveNorthWestAltAz.setFont(font)
        self.moveNorthWestAltAz.setObjectName("moveNorthWestAltAz")
        self.moveNorthAltAz = QtWidgets.QPushButton(self.tab_5)
        self.moveNorthAltAz.setGeometry(QtCore.QRect(95, 10, 66, 25))
        self.moveNorthAltAz.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveNorthAltAz.setFont(font)
        self.moveNorthAltAz.setObjectName("moveNorthAltAz")
        self.moveNorthEastAltAz = QtWidgets.QPushButton(self.tab_5)
        self.moveNorthEastAltAz.setGeometry(QtCore.QRect(180, 10, 66, 25))
        self.moveNorthEastAltAz.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveNorthEastAltAz.setFont(font)
        self.moveNorthEastAltAz.setObjectName("moveNorthEastAltAz")
        self.moveSouthEastAltAz = QtWidgets.QPushButton(self.tab_5)
        self.moveSouthEastAltAz.setGeometry(QtCore.QRect(180, 90, 66, 25))
        self.moveSouthEastAltAz.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveSouthEastAltAz.setFont(font)
        self.moveSouthEastAltAz.setObjectName("moveSouthEastAltAz")
        self.moveStepSizeAltAz = QtWidgets.QComboBox(self.tab_5)
        self.moveStepSizeAltAz.setGeometry(QtCore.QRect(10, 130, 236, 25))
        self.moveStepSizeAltAz.setMinimumSize(QtCore.QSize(0, 25))
        self.moveStepSizeAltAz.setObjectName("moveStepSizeAltAz")
        self.moveStepSizeAltAz.addItem("")
        self.moveStepSizeAltAz.addItem("")
        self.moveStepSizeAltAz.addItem("")
        self.moveStepSizeAltAz.addItem("")
        self.moveStepSizeAltAz.addItem("")
        self.tabMoving.addTab(self.tab_5, "")
        self.tab_6 = QtWidgets.QWidget()
        self.tab_6.setObjectName("tab_6")
        self.filterMinorPlanet_5 = QtWidgets.QLineEdit(self.tab_6)
        self.filterMinorPlanet_5.setGeometry(QtCore.QRect(315, 350, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.filterMinorPlanet_5.setFont(font)
        self.filterMinorPlanet_5.setMouseTracking(False)
        self.filterMinorPlanet_5.setAcceptDrops(False)
        self.filterMinorPlanet_5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.filterMinorPlanet_5.setText("")
        self.filterMinorPlanet_5.setReadOnly(False)
        self.filterMinorPlanet_5.setObjectName("filterMinorPlanet_5")
        self.groupBox = QtWidgets.QGroupBox(self.tab_6)
        self.groupBox.setGeometry(QtCore.QRect(10, 5, 166, 151))
        self.groupBox.setObjectName("groupBox")
        self.moveCoordinateAz = QtWidgets.QLineEdit(self.groupBox)
        self.moveCoordinateAz.setGeometry(QtCore.QRect(50, 70, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveCoordinateAz.setFont(font)
        self.moveCoordinateAz.setMouseTracking(False)
        self.moveCoordinateAz.setAcceptDrops(False)
        self.moveCoordinateAz.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.moveCoordinateAz.setText("")
        self.moveCoordinateAz.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.moveCoordinateAz.setReadOnly(False)
        self.moveCoordinateAz.setObjectName("moveCoordinateAz")
        self.moveAltAzAbsolute = QtWidgets.QPushButton(self.groupBox)
        self.moveAltAzAbsolute.setGeometry(QtCore.QRect(10, 115, 146, 26))
        self.moveAltAzAbsolute.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveAltAzAbsolute.setFont(font)
        self.moveAltAzAbsolute.setObjectName("moveAltAzAbsolute")
        self.moveCoordinateAlt = QtWidgets.QLineEdit(self.groupBox)
        self.moveCoordinateAlt.setGeometry(QtCore.QRect(50, 25, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveCoordinateAlt.setFont(font)
        self.moveCoordinateAlt.setMouseTracking(False)
        self.moveCoordinateAlt.setAcceptDrops(False)
        self.moveCoordinateAlt.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.moveCoordinateAlt.setText("")
        self.moveCoordinateAlt.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.moveCoordinateAlt.setReadOnly(False)
        self.moveCoordinateAlt.setObjectName("moveCoordinateAlt")
        self.label_315 = QtWidgets.QLabel(self.groupBox)
        self.label_315.setGeometry(QtCore.QRect(15, 25, 36, 21))
        self.label_315.setMinimumSize(QtCore.QSize(0, 0))
        self.label_315.setObjectName("label_315")
        self.label_316 = QtWidgets.QLabel(self.groupBox)
        self.label_316.setGeometry(QtCore.QRect(15, 70, 36, 21))
        self.label_316.setMinimumSize(QtCore.QSize(0, 0))
        self.label_316.setObjectName("label_316")
        self.label_325 = QtWidgets.QLabel(self.groupBox)
        self.label_325.setGeometry(QtCore.QRect(50, 90, 106, 16))
        self.label_325.setMinimumSize(QtCore.QSize(0, 0))
        self.label_325.setObjectName("label_325")
        self.label_326 = QtWidgets.QLabel(self.groupBox)
        self.label_326.setGeometry(QtCore.QRect(50, 45, 106, 16))
        self.label_326.setMinimumSize(QtCore.QSize(0, 0))
        self.label_326.setObjectName("label_326")
        self.groupBox_23 = QtWidgets.QGroupBox(self.tab_6)
        self.groupBox_23.setGeometry(QtCore.QRect(185, 5, 166, 151))
        self.groupBox_23.setObjectName("groupBox_23")
        self.moveCoordinateDec = QtWidgets.QLineEdit(self.groupBox_23)
        self.moveCoordinateDec.setGeometry(QtCore.QRect(50, 70, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveCoordinateDec.setFont(font)
        self.moveCoordinateDec.setMouseTracking(False)
        self.moveCoordinateDec.setAcceptDrops(False)
        self.moveCoordinateDec.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.moveCoordinateDec.setText("")
        self.moveCoordinateDec.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.moveCoordinateDec.setReadOnly(False)
        self.moveCoordinateDec.setObjectName("moveCoordinateDec")
        self.moveCoordinateRa = QtWidgets.QLineEdit(self.groupBox_23)
        self.moveCoordinateRa.setGeometry(QtCore.QRect(50, 25, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveCoordinateRa.setFont(font)
        self.moveCoordinateRa.setMouseTracking(False)
        self.moveCoordinateRa.setAcceptDrops(False)
        self.moveCoordinateRa.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.moveCoordinateRa.setText("")
        self.moveCoordinateRa.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.moveCoordinateRa.setReadOnly(False)
        self.moveCoordinateRa.setObjectName("moveCoordinateRa")
        self.moveRaDecAbsolute = QtWidgets.QPushButton(self.groupBox_23)
        self.moveRaDecAbsolute.setEnabled(True)
        self.moveRaDecAbsolute.setGeometry(QtCore.QRect(10, 115, 146, 26))
        self.moveRaDecAbsolute.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveRaDecAbsolute.setFont(font)
        self.moveRaDecAbsolute.setObjectName("moveRaDecAbsolute")
        self.label_323 = QtWidgets.QLabel(self.groupBox_23)
        self.label_323.setGeometry(QtCore.QRect(15, 25, 36, 21))
        self.label_323.setMinimumSize(QtCore.QSize(0, 0))
        self.label_323.setObjectName("label_323")
        self.label_324 = QtWidgets.QLabel(self.groupBox_23)
        self.label_324.setGeometry(QtCore.QRect(15, 70, 36, 21))
        self.label_324.setMinimumSize(QtCore.QSize(0, 0))
        self.label_324.setObjectName("label_324")
        self.moveCoordinateRaFloat = QtWidgets.QLineEdit(self.groupBox_23)
        self.moveCoordinateRaFloat.setEnabled(False)
        self.moveCoordinateRaFloat.setGeometry(QtCore.QRect(50, 45, 106, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveCoordinateRaFloat.setFont(font)
        self.moveCoordinateRaFloat.setMouseTracking(False)
        self.moveCoordinateRaFloat.setAcceptDrops(False)
        self.moveCoordinateRaFloat.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.moveCoordinateRaFloat.setText("")
        self.moveCoordinateRaFloat.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.moveCoordinateRaFloat.setReadOnly(False)
        self.moveCoordinateRaFloat.setObjectName("moveCoordinateRaFloat")
        self.moveCoordinateDecFloat = QtWidgets.QLineEdit(self.groupBox_23)
        self.moveCoordinateDecFloat.setEnabled(False)
        self.moveCoordinateDecFloat.setGeometry(QtCore.QRect(50, 90, 106, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.moveCoordinateDecFloat.setFont(font)
        self.moveCoordinateDecFloat.setMouseTracking(False)
        self.moveCoordinateDecFloat.setAcceptDrops(False)
        self.moveCoordinateDecFloat.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.moveCoordinateDecFloat.setText("")
        self.moveCoordinateDecFloat.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.moveCoordinateDecFloat.setReadOnly(False)
        self.moveCoordinateDecFloat.setObjectName("moveCoordinateDecFloat")
        self.tabMoving.addTab(self.tab_6, "")
        self.mainTabWidget.addTab(self.Tools, "")
        self.Relay = QtWidgets.QWidget()
        self.Relay.setObjectName("Relay")
        self.horizontalLayoutWidget = QtWidgets.QWidget(self.Relay)
        self.horizontalLayoutWidget.setGeometry(QtCore.QRect(10, 5, 766, 421))
        self.horizontalLayoutWidget.setObjectName("horizontalLayoutWidget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.horizontalLayoutWidget)
        self.horizontalLayout_2.setContentsMargins(0, 15, 4, 4)
        self.horizontalLayout_2.setSpacing(4)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(6)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_96 = QtWidgets.QLabel(self.horizontalLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_96.setFont(font)
        self.label_96.setObjectName("label_96")
        self.verticalLayout_2.addWidget(self.label_96)
        self.line_31 = QtWidgets.QFrame(self.horizontalLayoutWidget)
        self.line_31.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_31.setMidLineWidth(1)
        self.line_31.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_31.setObjectName("line_31")
        self.verticalLayout_2.addWidget(self.line_31)
        self.relayButton0 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton0.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton0.setObjectName("relayButton0")
        self.verticalLayout_2.addWidget(self.relayButton0)
        self.relayButton1 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton1.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton1.setObjectName("relayButton1")
        self.verticalLayout_2.addWidget(self.relayButton1)
        self.relayButton2 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton2.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton2.setObjectName("relayButton2")
        self.verticalLayout_2.addWidget(self.relayButton2)
        self.relayButton3 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton3.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton3.setObjectName("relayButton3")
        self.verticalLayout_2.addWidget(self.relayButton3)
        self.relayButton4 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton4.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton4.setObjectName("relayButton4")
        self.verticalLayout_2.addWidget(self.relayButton4)
        self.relayButton5 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton5.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton5.setObjectName("relayButton5")
        self.verticalLayout_2.addWidget(self.relayButton5)
        self.relayButton6 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton6.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton6.setObjectName("relayButton6")
        self.verticalLayout_2.addWidget(self.relayButton6)
        self.relayButton7 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton7.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton7.setObjectName("relayButton7")
        self.verticalLayout_2.addWidget(self.relayButton7)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.verticalLayout_2.setStretch(9, 1)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.line_59 = QtWidgets.QFrame(self.horizontalLayoutWidget)
        self.line_59.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_59.setMidLineWidth(1)
        self.line_59.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_59.setObjectName("line_59")
        self.horizontalLayout_2.addWidget(self.line_59)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.line_73 = QtWidgets.QFrame(self.horizontalLayoutWidget)
        self.line_73.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_73.setMidLineWidth(1)
        self.line_73.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_73.setObjectName("line_73")
        self.horizontalLayout_2.addWidget(self.line_73)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.horizontalLayout_2.setStretch(0, 1)
        self.horizontalLayout_2.setStretch(1, 1)
        self.horizontalLayout_2.setStretch(2, 1)
        self.horizontalLayout_2.setStretch(3, 1)
        self.horizontalLayout_2.setStretch(4, 1)
        self.mainTabWidget.addTab(self.Relay, "")
        self.Settings = QtWidgets.QWidget()
        self.Settings.setObjectName("Settings")
        self.settingsTabWidget = QtWidgets.QTabWidget(self.Settings)
        self.settingsTabWidget.setEnabled(True)
        self.settingsTabWidget.setGeometry(QtCore.QRect(5, 0, 776, 426))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.settingsTabWidget.sizePolicy().hasHeightForWidth())
        self.settingsTabWidget.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.settingsTabWidget.setFont(font)
        self.settingsTabWidget.setToolTip("")
        self.settingsTabWidget.setAutoFillBackground(True)
        self.settingsTabWidget.setStyleSheet("")
        self.settingsTabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.settingsTabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.settingsTabWidget.setElideMode(QtCore.Qt.ElideLeft)
        self.settingsTabWidget.setUsesScrollButtons(True)
        self.settingsTabWidget.setObjectName("settingsTabWidget")
        self.Devices = QtWidgets.QWidget()
        self.Devices.setObjectName("Devices")
        self.line_108 = QtWidgets.QFrame(self.Devices)
        self.line_108.setGeometry(QtCore.QRect(380, 10, 11, 371))
        self.line_108.setStyleSheet("")
        self.line_108.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_108.setLineWidth(2)
        self.line_108.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_108.setObjectName("line_108")
        self.groupBox_40 = QtWidgets.QGroupBox(self.Devices)
        self.groupBox_40.setGeometry(QtCore.QRect(400, 10, 366, 171))
        self.groupBox_40.setProperty("large", True)
        self.groupBox_40.setObjectName("groupBox_40")
        self.powerDevice = QtWidgets.QComboBox(self.groupBox_40)
        self.powerDevice.setGeometry(QtCore.QRect(85, 110, 221, 21))
        self.powerDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.powerDevice.setObjectName("powerDevice")
        self.po = QtWidgets.QLabel(self.groupBox_40)
        self.po.setGeometry(QtCore.QRect(12, 110, 71, 21))
        self.po.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.po.setFont(font)
        self.po.setObjectName("po")
        self.label_46 = QtWidgets.QLabel(self.groupBox_40)
        self.label_46.setGeometry(QtCore.QRect(12, 80, 71, 21))
        self.label_46.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_46.setFont(font)
        self.label_46.setObjectName("label_46")
        self.measureDevice = QtWidgets.QComboBox(self.groupBox_40)
        self.measureDevice.setGeometry(QtCore.QRect(85, 20, 221, 21))
        self.measureDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.measureDevice.setObjectName("measureDevice")
        self.label_48 = QtWidgets.QLabel(self.groupBox_40)
        self.label_48.setGeometry(QtCore.QRect(12, 20, 71, 21))
        self.label_48.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_48.setFont(font)
        self.label_48.setObjectName("label_48")
        self.remoteDevice = QtWidgets.QComboBox(self.groupBox_40)
        self.remoteDevice.setGeometry(QtCore.QRect(85, 50, 221, 21))
        self.remoteDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.remoteDevice.setObjectName("remoteDevice")
        self.label_49 = QtWidgets.QLabel(self.groupBox_40)
        self.label_49.setGeometry(QtCore.QRect(12, 50, 71, 21))
        self.label_49.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_49.setFont(font)
        self.label_49.setObjectName("label_49")
        self.relayDevice = QtWidgets.QComboBox(self.groupBox_40)
        self.relayDevice.setGeometry(QtCore.QRect(85, 80, 221, 21))
        self.relayDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.relayDevice.setObjectName("relayDevice")
        self.label_53 = QtWidgets.QLabel(self.groupBox_40)
        self.label_53.setGeometry(QtCore.QRect(12, 140, 71, 21))
        self.label_53.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_53.setFont(font)
        self.label_53.setObjectName("label_53")
        self.coverDevice = QtWidgets.QComboBox(self.groupBox_40)
        self.coverDevice.setGeometry(QtCore.QRect(85, 140, 221, 21))
        self.coverDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.coverDevice.setObjectName("coverDevice")
        self.powerSetup = QtWidgets.QPushButton(self.groupBox_40)
        self.powerSetup.setGeometry(QtCore.QRect(310, 110, 51, 21))
        self.powerSetup.setObjectName("powerSetup")
        self.coverSetup = QtWidgets.QPushButton(self.groupBox_40)
        self.coverSetup.setGeometry(QtCore.QRect(310, 140, 51, 21))
        self.coverSetup.setObjectName("coverSetup")
        self.relaySetup = QtWidgets.QPushButton(self.groupBox_40)
        self.relaySetup.setEnabled(True)
        self.relaySetup.setGeometry(QtCore.QRect(310, 80, 51, 21))
        self.relaySetup.setObjectName("relaySetup")
        self.groupBox_41 = QtWidgets.QGroupBox(self.Devices)
        self.groupBox_41.setGeometry(QtCore.QRect(5, 10, 366, 201))
        self.groupBox_41.setProperty("large", True)
        self.groupBox_41.setObjectName("groupBox_41")
        self.dome = QtWidgets.QLabel(self.groupBox_41)
        self.dome.setGeometry(QtCore.QRect(11, 170, 76, 21))
        self.dome.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.dome.setFont(font)
        self.dome.setObjectName("dome")
        self.ccd = QtWidgets.QLabel(self.groupBox_41)
        self.ccd.setGeometry(QtCore.QRect(11, 20, 76, 21))
        self.ccd.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.ccd.setFont(font)
        self.ccd.setObjectName("ccd")
        self.domeDevice = QtWidgets.QComboBox(self.groupBox_41)
        self.domeDevice.setGeometry(QtCore.QRect(85, 170, 221, 21))
        self.domeDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.domeDevice.setObjectName("domeDevice")
        self.cameraDevice = QtWidgets.QComboBox(self.groupBox_41)
        self.cameraDevice.setGeometry(QtCore.QRect(85, 20, 221, 21))
        self.cameraDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.cameraDevice.setObjectName("cameraDevice")
        self.telescopeDevice = QtWidgets.QComboBox(self.groupBox_41)
        self.telescopeDevice.setGeometry(QtCore.QRect(85, 140, 221, 21))
        self.telescopeDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.telescopeDevice.setObjectName("telescopeDevice")
        self.dome_3 = QtWidgets.QLabel(self.groupBox_41)
        self.dome_3.setGeometry(QtCore.QRect(11, 140, 76, 21))
        self.dome_3.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.dome_3.setFont(font)
        self.dome_3.setObjectName("dome_3")
        self.ast = QtWidgets.QLabel(self.groupBox_41)
        self.ast.setGeometry(QtCore.QRect(11, 110, 76, 21))
        self.ast.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.ast.setFont(font)
        self.ast.setObjectName("ast")
        self.astrometryDevice = QtWidgets.QComboBox(self.groupBox_41)
        self.astrometryDevice.setGeometry(QtCore.QRect(85, 110, 221, 21))
        self.astrometryDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.astrometryDevice.setObjectName("astrometryDevice")
        self.ccd_2 = QtWidgets.QLabel(self.groupBox_41)
        self.ccd_2.setGeometry(QtCore.QRect(10, 50, 76, 21))
        self.ccd_2.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.ccd_2.setFont(font)
        self.ccd_2.setObjectName("ccd_2")
        self.filterDevice = QtWidgets.QComboBox(self.groupBox_41)
        self.filterDevice.setGeometry(QtCore.QRect(85, 50, 221, 21))
        self.filterDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.filterDevice.setObjectName("filterDevice")
        self.dome_4 = QtWidgets.QLabel(self.groupBox_41)
        self.dome_4.setGeometry(QtCore.QRect(11, 80, 76, 21))
        self.dome_4.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.dome_4.setFont(font)
        self.dome_4.setObjectName("dome_4")
        self.focuserDevice = QtWidgets.QComboBox(self.groupBox_41)
        self.focuserDevice.setGeometry(QtCore.QRect(85, 80, 221, 21))
        self.focuserDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.focuserDevice.setObjectName("focuserDevice")
        self.domeSetup = QtWidgets.QPushButton(self.groupBox_41)
        self.domeSetup.setGeometry(QtCore.QRect(310, 170, 51, 21))
        self.domeSetup.setObjectName("domeSetup")
        self.telescopeSetup = QtWidgets.QPushButton(self.groupBox_41)
        self.telescopeSetup.setGeometry(QtCore.QRect(310, 140, 51, 21))
        self.telescopeSetup.setObjectName("telescopeSetup")
        self.focuserSetup = QtWidgets.QPushButton(self.groupBox_41)
        self.focuserSetup.setGeometry(QtCore.QRect(310, 80, 51, 21))
        self.focuserSetup.setObjectName("focuserSetup")
        self.filterSetup = QtWidgets.QPushButton(self.groupBox_41)
        self.filterSetup.setGeometry(QtCore.QRect(310, 50, 51, 21))
        self.filterSetup.setObjectName("filterSetup")
        self.cameraSetup = QtWidgets.QPushButton(self.groupBox_41)
        self.cameraSetup.setGeometry(QtCore.QRect(310, 20, 51, 21))
        self.cameraSetup.setObjectName("cameraSetup")
        self.astrometrySetup = QtWidgets.QPushButton(self.groupBox_41)
        self.astrometrySetup.setGeometry(QtCore.QRect(310, 110, 51, 21))
        self.astrometrySetup.setObjectName("astrometrySetup")
        self.groupBox_42 = QtWidgets.QGroupBox(self.Devices)
        self.groupBox_42.setGeometry(QtCore.QRect(5, 215, 366, 171))
        self.groupBox_42.setProperty("large", True)
        self.groupBox_42.setObjectName("groupBox_42")
        self.sky = QtWidgets.QLabel(self.groupBox_42)
        self.sky.setGeometry(QtCore.QRect(11, 50, 76, 21))
        self.sky.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.sky.setFont(font)
        self.sky.setObjectName("sky")
        self.sensorWeatherDevice = QtWidgets.QComboBox(self.groupBox_42)
        self.sensorWeatherDevice.setGeometry(QtCore.QRect(86, 20, 221, 21))
        self.sensorWeatherDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.sensorWeatherDevice.setObjectName("sensorWeatherDevice")
        self.env = QtWidgets.QLabel(self.groupBox_42)
        self.env.setGeometry(QtCore.QRect(10, 20, 76, 21))
        self.env.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.env.setFont(font)
        self.env.setObjectName("env")
        self.skymeterDevice = QtWidgets.QComboBox(self.groupBox_42)
        self.skymeterDevice.setGeometry(QtCore.QRect(86, 50, 221, 21))
        self.skymeterDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.skymeterDevice.setObjectName("skymeterDevice")
        self.env_2 = QtWidgets.QLabel(self.groupBox_42)
        self.env_2.setGeometry(QtCore.QRect(10, 110, 76, 21))
        self.env_2.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.env_2.setFont(font)
        self.env_2.setObjectName("env_2")
        self.onlineWeatherDevice = QtWidgets.QComboBox(self.groupBox_42)
        self.onlineWeatherDevice.setGeometry(QtCore.QRect(86, 110, 221, 21))
        self.onlineWeatherDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.onlineWeatherDevice.setObjectName("onlineWeatherDevice")
        self.env_3 = QtWidgets.QLabel(self.groupBox_42)
        self.env_3.setGeometry(QtCore.QRect(10, 140, 76, 21))
        self.env_3.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.env_3.setFont(font)
        self.env_3.setObjectName("env_3")
        self.directWeatherDevice = QtWidgets.QComboBox(self.groupBox_42)
        self.directWeatherDevice.setGeometry(QtCore.QRect(86, 140, 221, 21))
        self.directWeatherDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.directWeatherDevice.setObjectName("directWeatherDevice")
        self.skymeterSetup = QtWidgets.QPushButton(self.groupBox_42)
        self.skymeterSetup.setGeometry(QtCore.QRect(310, 50, 51, 21))
        self.skymeterSetup.setObjectName("skymeterSetup")
        self.sensorWeatherSetup = QtWidgets.QPushButton(self.groupBox_42)
        self.sensorWeatherSetup.setGeometry(QtCore.QRect(310, 20, 51, 21))
        self.sensorWeatherSetup.setObjectName("sensorWeatherSetup")
        self.sky_2 = QtWidgets.QLabel(self.groupBox_42)
        self.sky_2.setEnabled(True)
        self.sky_2.setGeometry(QtCore.QRect(10, 80, 76, 21))
        self.sky_2.setMinimumSize(QtCore.QSize(0, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.sky_2.setFont(font)
        self.sky_2.setObjectName("sky_2")
        self.powerWeatherDevice = QtWidgets.QComboBox(self.groupBox_42)
        self.powerWeatherDevice.setEnabled(True)
        self.powerWeatherDevice.setGeometry(QtCore.QRect(85, 80, 221, 21))
        self.powerWeatherDevice.setMinimumSize(QtCore.QSize(0, 0))
        self.powerWeatherDevice.setObjectName("powerWeatherDevice")
        self.powerWeatherSetup = QtWidgets.QPushButton(self.groupBox_42)
        self.powerWeatherSetup.setEnabled(True)
        self.powerWeatherSetup.setGeometry(QtCore.QRect(310, 80, 51, 21))
        self.powerWeatherSetup.setObjectName("powerWeatherSetup")
        self.onlineWeatherSetup = QtWidgets.QPushButton(self.groupBox_42)
        self.onlineWeatherSetup.setEnabled(True)
        self.onlineWeatherSetup.setGeometry(QtCore.QRect(310, 110, 51, 21))
        self.onlineWeatherSetup.setObjectName("onlineWeatherSetup")
        self.groupBox_57 = QtWidgets.QGroupBox(self.Devices)
        self.groupBox_57.setEnabled(True)
        self.groupBox_57.setGeometry(QtCore.QRect(400, 315, 366, 71))
        self.groupBox_57.setProperty("large", True)
        self.groupBox_57.setObjectName("groupBox_57")
        self.ascomConnect = QtWidgets.QPushButton(self.groupBox_57)
        self.ascomConnect.setEnabled(True)
        self.ascomConnect.setGeometry(QtCore.QRect(10, 20, 101, 21))
        self.ascomConnect.setObjectName("ascomConnect")
        self.ascomDisconnect = QtWidgets.QPushButton(self.groupBox_57)
        self.ascomDisconnect.setEnabled(True)
        self.ascomDisconnect.setGeometry(QtCore.QRect(145, 20, 101, 21))
        self.ascomDisconnect.setObjectName("ascomDisconnect")
        self.checkASCOMAutoConnect = QtWidgets.QCheckBox(self.groupBox_57)
        self.checkASCOMAutoConnect.setEnabled(True)
        self.checkASCOMAutoConnect.setGeometry(QtCore.QRect(10, 45, 211, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkASCOMAutoConnect.setFont(font)
        self.checkASCOMAutoConnect.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkASCOMAutoConnect.setChecked(False)
        self.checkASCOMAutoConnect.setObjectName("checkASCOMAutoConnect")
        self.settingsTabWidget.addTab(self.Devices, "")
        self.Imaging = QtWidgets.QWidget()
        self.Imaging.setObjectName("Imaging")
        self.line_92 = QtWidgets.QFrame(self.Imaging)
        self.line_92.setGeometry(QtCore.QRect(380, 10, 10, 376))
        self.line_92.setStyleSheet("")
        self.line_92.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_92.setLineWidth(2)
        self.line_92.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_92.setObjectName("line_92")
        self.groupBox_25 = QtWidgets.QGroupBox(self.Imaging)
        self.groupBox_25.setGeometry(QtCore.QRect(5, 10, 366, 226))
        self.groupBox_25.setProperty("large", True)
        self.groupBox_25.setObjectName("groupBox_25")
        self.label_299 = QtWidgets.QLabel(self.groupBox_25)
        self.label_299.setGeometry(QtCore.QRect(10, 25, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_299.setFont(font)
        self.label_299.setObjectName("label_299")
        self.label_294 = QtWidgets.QLabel(self.groupBox_25)
        self.label_294.setGeometry(QtCore.QRect(185, 25, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_294.setFont(font)
        self.label_294.setObjectName("label_294")
        self.expTime = QtWidgets.QDoubleSpinBox(self.groupBox_25)
        self.expTime.setGeometry(QtCore.QRect(110, 25, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.expTime.setFont(font)
        self.expTime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.expTime.setDecimals(3)
        self.expTime.setMinimum(0.001)
        self.expTime.setMaximum(300.0)
        self.expTime.setSingleStep(1.0)
        self.expTime.setProperty("value", 1.0)
        self.expTime.setObjectName("expTime")
        self.label_41 = QtWidgets.QLabel(self.groupBox_25)
        self.label_41.setGeometry(QtCore.QRect(205, 25, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_41.setFont(font)
        self.label_41.setObjectName("label_41")
        self.binning = QtWidgets.QDoubleSpinBox(self.groupBox_25)
        self.binning.setGeometry(QtCore.QRect(280, 25, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.binning.setFont(font)
        self.binning.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.binning.setDecimals(0)
        self.binning.setMinimum(1.0)
        self.binning.setMaximum(9.0)
        self.binning.setSingleStep(1.0)
        self.binning.setProperty("value", 1.0)
        self.binning.setObjectName("binning")
        self.label_296 = QtWidgets.QLabel(self.groupBox_25)
        self.label_296.setGeometry(QtCore.QRect(185, 75, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_296.setFont(font)
        self.label_296.setObjectName("label_296")
        self.subFrame = QtWidgets.QDoubleSpinBox(self.groupBox_25)
        self.subFrame.setGeometry(QtCore.QRect(110, 75, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.subFrame.setFont(font)
        self.subFrame.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.subFrame.setDecimals(0)
        self.subFrame.setMinimum(10.0)
        self.subFrame.setMaximum(100.0)
        self.subFrame.setSingleStep(10.0)
        self.subFrame.setProperty("value", 100.0)
        self.subFrame.setObjectName("subFrame")
        self.label_307 = QtWidgets.QLabel(self.groupBox_25)
        self.label_307.setGeometry(QtCore.QRect(10, 75, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_307.setFont(font)
        self.label_307.setObjectName("label_307")
        self.checkFastDownload = QtWidgets.QCheckBox(self.groupBox_25)
        self.checkFastDownload.setGeometry(QtCore.QRect(10, 180, 341, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkFastDownload.setFont(font)
        self.checkFastDownload.setChecked(False)
        self.checkFastDownload.setObjectName("checkFastDownload")
        self.checkKeepImages = QtWidgets.QCheckBox(self.groupBox_25)
        self.checkKeepImages.setGeometry(QtCore.QRect(10, 200, 231, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkKeepImages.setFont(font)
        self.checkKeepImages.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkKeepImages.setChecked(False)
        self.checkKeepImages.setObjectName("checkKeepImages")
        self.coolerTemp = QtWidgets.QLineEdit(self.groupBox_25)
        self.coolerTemp.setEnabled(False)
        self.coolerTemp.setGeometry(QtCore.QRect(110, 100, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.coolerTemp.setFont(font)
        self.coolerTemp.setMouseTracking(False)
        self.coolerTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.coolerTemp.setAcceptDrops(False)
        self.coolerTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.coolerTemp.setInputMask("")
        self.coolerTemp.setCursorPosition(1)
        self.coolerTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.coolerTemp.setReadOnly(True)
        self.coolerTemp.setProperty("input", True)
        self.coolerTemp.setObjectName("coolerTemp")
        self.label_402 = QtWidgets.QLabel(self.groupBox_25)
        self.label_402.setGeometry(QtCore.QRect(185, 100, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_402.setFont(font)
        self.label_402.setObjectName("label_402")
        self.label_400 = QtWidgets.QLabel(self.groupBox_25)
        self.label_400.setGeometry(QtCore.QRect(10, 100, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_400.setFont(font)
        self.label_400.setObjectName("label_400")
        self.coolerPower = QtWidgets.QLineEdit(self.groupBox_25)
        self.coolerPower.setEnabled(False)
        self.coolerPower.setGeometry(QtCore.QRect(280, 100, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.coolerPower.setFont(font)
        self.coolerPower.setMouseTracking(False)
        self.coolerPower.setFocusPolicy(QtCore.Qt.NoFocus)
        self.coolerPower.setAcceptDrops(False)
        self.coolerPower.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.coolerPower.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.coolerPower.setReadOnly(True)
        self.coolerPower.setObjectName("coolerPower")
        self.label_401 = QtWidgets.QLabel(self.groupBox_25)
        self.label_401.setGeometry(QtCore.QRect(205, 100, 66, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_401.setFont(font)
        self.label_401.setObjectName("label_401")
        self.downloadSlow = QtWidgets.QPushButton(self.groupBox_25)
        self.downloadSlow.setGeometry(QtCore.QRect(280, 150, 76, 21))
        self.downloadSlow.setObjectName("downloadSlow")
        self.coolerOn = QtWidgets.QPushButton(self.groupBox_25)
        self.coolerOn.setGeometry(QtCore.QRect(110, 125, 71, 21))
        self.coolerOn.setObjectName("coolerOn")
        self.downloadFast = QtWidgets.QPushButton(self.groupBox_25)
        self.downloadFast.setGeometry(QtCore.QRect(110, 150, 71, 21))
        self.downloadFast.setObjectName("downloadFast")
        self.coolerOff = QtWidgets.QPushButton(self.groupBox_25)
        self.coolerOff.setGeometry(QtCore.QRect(280, 125, 76, 21))
        self.coolerOff.setObjectName("coolerOff")
        self.label_407 = QtWidgets.QLabel(self.groupBox_25)
        self.label_407.setGeometry(QtCore.QRect(10, 125, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_407.setFont(font)
        self.label_407.setObjectName("label_407")
        self.label_408 = QtWidgets.QLabel(self.groupBox_25)
        self.label_408.setGeometry(QtCore.QRect(10, 150, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_408.setFont(font)
        self.label_408.setObjectName("label_408")
        self.label_302 = QtWidgets.QLabel(self.groupBox_25)
        self.label_302.setGeometry(QtCore.QRect(10, 50, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_302.setFont(font)
        self.label_302.setObjectName("label_302")
        self.expTimeN = QtWidgets.QDoubleSpinBox(self.groupBox_25)
        self.expTimeN.setGeometry(QtCore.QRect(110, 50, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.expTimeN.setFont(font)
        self.expTimeN.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.expTimeN.setDecimals(3)
        self.expTimeN.setMinimum(0.001)
        self.expTimeN.setMaximum(300.0)
        self.expTimeN.setSingleStep(1.0)
        self.expTimeN.setProperty("value", 1.0)
        self.expTimeN.setObjectName("expTimeN")
        self.label_304 = QtWidgets.QLabel(self.groupBox_25)
        self.label_304.setGeometry(QtCore.QRect(185, 50, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_304.setFont(font)
        self.label_304.setObjectName("label_304")
        self.binningN = QtWidgets.QDoubleSpinBox(self.groupBox_25)
        self.binningN.setGeometry(QtCore.QRect(280, 50, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.binningN.setFont(font)
        self.binningN.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.binningN.setDecimals(0)
        self.binningN.setMinimum(1.0)
        self.binningN.setMaximum(9.0)
        self.binningN.setSingleStep(1.0)
        self.binningN.setProperty("value", 1.0)
        self.binningN.setObjectName("binningN")
        self.label_51 = QtWidgets.QLabel(self.groupBox_25)
        self.label_51.setGeometry(QtCore.QRect(205, 50, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_51.setFont(font)
        self.label_51.setObjectName("label_51")
        self.groupBox_26 = QtWidgets.QGroupBox(self.Imaging)
        self.groupBox_26.setGeometry(QtCore.QRect(400, 10, 366, 126))
        self.groupBox_26.setProperty("large", True)
        self.groupBox_26.setObjectName("groupBox_26")
        self.label_166 = QtWidgets.QLabel(self.groupBox_26)
        self.label_166.setGeometry(QtCore.QRect(10, 20, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_166.setFont(font)
        self.label_166.setObjectName("label_166")
        self.pixelSizeX = QtWidgets.QLineEdit(self.groupBox_26)
        self.pixelSizeX.setEnabled(False)
        self.pixelSizeX.setGeometry(QtCore.QRect(90, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.pixelSizeX.setFont(font)
        self.pixelSizeX.setMouseTracking(False)
        self.pixelSizeX.setFocusPolicy(QtCore.Qt.NoFocus)
        self.pixelSizeX.setAcceptDrops(False)
        self.pixelSizeX.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.pixelSizeX.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.pixelSizeX.setReadOnly(True)
        self.pixelSizeX.setObjectName("pixelSizeX")
        self.label_379 = QtWidgets.QLabel(self.groupBox_26)
        self.label_379.setGeometry(QtCore.QRect(10, 60, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_379.setFont(font)
        self.label_379.setObjectName("label_379")
        self.label_380 = QtWidgets.QLabel(self.groupBox_26)
        self.label_380.setGeometry(QtCore.QRect(220, 20, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_380.setFont(font)
        self.label_380.setObjectName("label_380")
        self.pixelSizeY = QtWidgets.QLineEdit(self.groupBox_26)
        self.pixelSizeY.setEnabled(False)
        self.pixelSizeY.setGeometry(QtCore.QRect(165, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.pixelSizeY.setFont(font)
        self.pixelSizeY.setMouseTracking(False)
        self.pixelSizeY.setFocusPolicy(QtCore.Qt.NoFocus)
        self.pixelSizeY.setAcceptDrops(False)
        self.pixelSizeY.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.pixelSizeY.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.pixelSizeY.setReadOnly(True)
        self.pixelSizeY.setObjectName("pixelSizeY")
        self.resolutionX = QtWidgets.QLineEdit(self.groupBox_26)
        self.resolutionX.setEnabled(False)
        self.resolutionX.setGeometry(QtCore.QRect(90, 60, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.resolutionX.setFont(font)
        self.resolutionX.setMouseTracking(False)
        self.resolutionX.setFocusPolicy(QtCore.Qt.NoFocus)
        self.resolutionX.setAcceptDrops(False)
        self.resolutionX.setToolTip("")
        self.resolutionX.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.resolutionX.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.resolutionX.setReadOnly(True)
        self.resolutionX.setObjectName("resolutionX")
        self.resolutionY = QtWidgets.QLineEdit(self.groupBox_26)
        self.resolutionY.setEnabled(False)
        self.resolutionY.setGeometry(QtCore.QRect(165, 60, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.resolutionY.setFont(font)
        self.resolutionY.setMouseTracking(False)
        self.resolutionY.setFocusPolicy(QtCore.Qt.NoFocus)
        self.resolutionY.setAcceptDrops(False)
        self.resolutionY.setToolTip("")
        self.resolutionY.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.resolutionY.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.resolutionY.setReadOnly(True)
        self.resolutionY.setObjectName("resolutionY")
        self.label_381 = QtWidgets.QLabel(self.groupBox_26)
        self.label_381.setGeometry(QtCore.QRect(150, 20, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_381.setFont(font)
        self.label_381.setObjectName("label_381")
        self.label_382 = QtWidgets.QLabel(self.groupBox_26)
        self.label_382.setGeometry(QtCore.QRect(150, 60, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_382.setFont(font)
        self.label_382.setObjectName("label_382")
        self.label_383 = QtWidgets.QLabel(self.groupBox_26)
        self.label_383.setGeometry(QtCore.QRect(220, 60, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_383.setFont(font)
        self.label_383.setObjectName("label_383")
        self.label_384 = QtWidgets.QLabel(self.groupBox_26)
        self.label_384.setGeometry(QtCore.QRect(10, 80, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_384.setFont(font)
        self.label_384.setObjectName("label_384")
        self.FOVX = QtWidgets.QLineEdit(self.groupBox_26)
        self.FOVX.setEnabled(False)
        self.FOVX.setGeometry(QtCore.QRect(90, 80, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.FOVX.setFont(font)
        self.FOVX.setMouseTracking(False)
        self.FOVX.setFocusPolicy(QtCore.Qt.NoFocus)
        self.FOVX.setAcceptDrops(False)
        self.FOVX.setToolTip("")
        self.FOVX.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.FOVX.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.FOVX.setReadOnly(True)
        self.FOVX.setObjectName("FOVX")
        self.FOVY = QtWidgets.QLineEdit(self.groupBox_26)
        self.FOVY.setEnabled(False)
        self.FOVY.setGeometry(QtCore.QRect(165, 80, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.FOVY.setFont(font)
        self.FOVY.setMouseTracking(False)
        self.FOVY.setFocusPolicy(QtCore.Qt.NoFocus)
        self.FOVY.setAcceptDrops(False)
        self.FOVY.setToolTip("")
        self.FOVY.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.FOVY.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.FOVY.setReadOnly(True)
        self.FOVY.setObjectName("FOVY")
        self.label_385 = QtWidgets.QLabel(self.groupBox_26)
        self.label_385.setGeometry(QtCore.QRect(150, 80, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_385.setFont(font)
        self.label_385.setObjectName("label_385")
        self.label_386 = QtWidgets.QLabel(self.groupBox_26)
        self.label_386.setGeometry(QtCore.QRect(220, 80, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_386.setFont(font)
        self.label_386.setObjectName("label_386")
        self.label_313 = QtWidgets.QLabel(self.groupBox_26)
        self.label_313.setGeometry(QtCore.QRect(10, 40, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_313.setFont(font)
        self.label_313.setObjectName("label_313")
        self.pixelY = QtWidgets.QLineEdit(self.groupBox_26)
        self.pixelY.setEnabled(False)
        self.pixelY.setGeometry(QtCore.QRect(165, 40, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.pixelY.setFont(font)
        self.pixelY.setMouseTracking(False)
        self.pixelY.setFocusPolicy(QtCore.Qt.NoFocus)
        self.pixelY.setAcceptDrops(False)
        self.pixelY.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.pixelY.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.pixelY.setReadOnly(True)
        self.pixelY.setObjectName("pixelY")
        self.pixelX = QtWidgets.QLineEdit(self.groupBox_26)
        self.pixelX.setEnabled(False)
        self.pixelX.setGeometry(QtCore.QRect(90, 40, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.pixelX.setFont(font)
        self.pixelX.setMouseTracking(False)
        self.pixelX.setFocusPolicy(QtCore.Qt.NoFocus)
        self.pixelX.setAcceptDrops(False)
        self.pixelX.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.pixelX.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.pixelX.setReadOnly(True)
        self.pixelX.setObjectName("pixelX")
        self.label_388 = QtWidgets.QLabel(self.groupBox_26)
        self.label_388.setGeometry(QtCore.QRect(150, 40, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_388.setFont(font)
        self.label_388.setObjectName("label_388")
        self.rotation = QtWidgets.QLineEdit(self.groupBox_26)
        self.rotation.setEnabled(False)
        self.rotation.setGeometry(QtCore.QRect(90, 100, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.rotation.setFont(font)
        self.rotation.setMouseTracking(False)
        self.rotation.setFocusPolicy(QtCore.Qt.NoFocus)
        self.rotation.setAcceptDrops(False)
        self.rotation.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.rotation.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.rotation.setReadOnly(True)
        self.rotation.setObjectName("rotation")
        self.label_391 = QtWidgets.QLabel(self.groupBox_26)
        self.label_391.setGeometry(QtCore.QRect(10, 100, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_391.setFont(font)
        self.label_391.setObjectName("label_391")
        self.label_405 = QtWidgets.QLabel(self.groupBox_26)
        self.label_405.setGeometry(QtCore.QRect(170, 100, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_405.setFont(font)
        self.label_405.setObjectName("label_405")
        self.speed = QtWidgets.QLineEdit(self.groupBox_26)
        self.speed.setEnabled(False)
        self.speed.setGeometry(QtCore.QRect(230, 100, 26, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.speed.setFont(font)
        self.speed.setMouseTracking(False)
        self.speed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.speed.setAcceptDrops(False)
        self.speed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.speed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.speed.setReadOnly(True)
        self.speed.setObjectName("speed")
        self.groupBox_32 = QtWidgets.QGroupBox(self.Imaging)
        self.groupBox_32.setGeometry(QtCore.QRect(400, 145, 366, 86))
        self.groupBox_32.setProperty("large", True)
        self.groupBox_32.setObjectName("groupBox_32")
        self.label_403 = QtWidgets.QLabel(self.groupBox_32)
        self.label_403.setGeometry(QtCore.QRect(10, 60, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_403.setFont(font)
        self.label_403.setObjectName("label_403")
        self.rayleigh = QtWidgets.QLineEdit(self.groupBox_32)
        self.rayleigh.setEnabled(False)
        self.rayleigh.setGeometry(QtCore.QRect(110, 60, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.rayleigh.setFont(font)
        self.rayleigh.setMouseTracking(False)
        self.rayleigh.setFocusPolicy(QtCore.Qt.NoFocus)
        self.rayleigh.setAcceptDrops(False)
        self.rayleigh.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.rayleigh.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.rayleigh.setReadOnly(True)
        self.rayleigh.setObjectName("rayleigh")
        self.label_404 = QtWidgets.QLabel(self.groupBox_32)
        self.label_404.setGeometry(QtCore.QRect(10, 20, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_404.setFont(font)
        self.label_404.setObjectName("label_404")
        self.label_397 = QtWidgets.QLabel(self.groupBox_32)
        self.label_397.setGeometry(QtCore.QRect(10, 40, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_397.setFont(font)
        self.label_397.setObjectName("label_397")
        self.magLimit = QtWidgets.QLineEdit(self.groupBox_32)
        self.magLimit.setEnabled(False)
        self.magLimit.setGeometry(QtCore.QRect(110, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.magLimit.setFont(font)
        self.magLimit.setMouseTracking(False)
        self.magLimit.setFocusPolicy(QtCore.Qt.NoFocus)
        self.magLimit.setAcceptDrops(False)
        self.magLimit.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.magLimit.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.magLimit.setReadOnly(True)
        self.magLimit.setObjectName("magLimit")
        self.dawes = QtWidgets.QLineEdit(self.groupBox_32)
        self.dawes.setEnabled(False)
        self.dawes.setGeometry(QtCore.QRect(110, 40, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.dawes.setFont(font)
        self.dawes.setMouseTracking(False)
        self.dawes.setFocusPolicy(QtCore.Qt.NoFocus)
        self.dawes.setAcceptDrops(False)
        self.dawes.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.dawes.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dawes.setReadOnly(True)
        self.dawes.setObjectName("dawes")
        self.label_406 = QtWidgets.QLabel(self.groupBox_32)
        self.label_406.setGeometry(QtCore.QRect(165, 40, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_406.setFont(font)
        self.label_406.setObjectName("label_406")
        self.label_398 = QtWidgets.QLabel(self.groupBox_32)
        self.label_398.setGeometry(QtCore.QRect(165, 60, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_398.setFont(font)
        self.label_398.setObjectName("label_398")
        self.groupBox_33 = QtWidgets.QGroupBox(self.Imaging)
        self.groupBox_33.setGeometry(QtCore.QRect(400, 245, 366, 76))
        self.groupBox_33.setProperty("large", True)
        self.groupBox_33.setObjectName("groupBox_33")
        self.label_396 = QtWidgets.QLabel(self.groupBox_33)
        self.label_396.setGeometry(QtCore.QRect(185, 20, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_396.setFont(font)
        self.label_396.setObjectName("label_396")
        self.label_394 = QtWidgets.QLabel(self.groupBox_33)
        self.label_394.setGeometry(QtCore.QRect(220, 20, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_394.setFont(font)
        self.label_394.setObjectName("label_394")
        self.label_395 = QtWidgets.QLabel(self.groupBox_33)
        self.label_395.setGeometry(QtCore.QRect(335, 20, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_395.setFont(font)
        self.label_395.setObjectName("label_395")
        self.copyFromTelescopeDriver = QtWidgets.QPushButton(self.groupBox_33)
        self.copyFromTelescopeDriver.setGeometry(QtCore.QRect(225, 50, 131, 21))
        self.copyFromTelescopeDriver.setObjectName("copyFromTelescopeDriver")
        self.label_387 = QtWidgets.QLabel(self.groupBox_33)
        self.label_387.setGeometry(QtCore.QRect(10, 20, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_387.setFont(font)
        self.label_387.setObjectName("label_387")
        self.focalLength = QtWidgets.QDoubleSpinBox(self.groupBox_33)
        self.focalLength.setGeometry(QtCore.QRect(110, 20, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.focalLength.setFont(font)
        self.focalLength.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.focalLength.setDecimals(0)
        self.focalLength.setMinimum(0.0)
        self.focalLength.setMaximum(10000.0)
        self.focalLength.setSingleStep(50.0)
        self.focalLength.setProperty("value", 500.0)
        self.focalLength.setObjectName("focalLength")
        self.aperture = QtWidgets.QDoubleSpinBox(self.groupBox_33)
        self.aperture.setGeometry(QtCore.QRect(275, 20, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.aperture.setFont(font)
        self.aperture.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.aperture.setDecimals(0)
        self.aperture.setMinimum(0.0)
        self.aperture.setMaximum(10000.0)
        self.aperture.setSingleStep(20.0)
        self.aperture.setProperty("value", 100.0)
        self.aperture.setObjectName("aperture")
        self.checkAutomaticTelescope = QtWidgets.QCheckBox(self.groupBox_33)
        self.checkAutomaticTelescope.setGeometry(QtCore.QRect(10, 50, 211, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkAutomaticTelescope.setFont(font)
        self.checkAutomaticTelescope.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkAutomaticTelescope.setChecked(False)
        self.checkAutomaticTelescope.setObjectName("checkAutomaticTelescope")
        self.groupBox_43 = QtWidgets.QGroupBox(self.Imaging)
        self.groupBox_43.setGeometry(QtCore.QRect(400, 330, 366, 56))
        self.groupBox_43.setProperty("large", True)
        self.groupBox_43.setObjectName("groupBox_43")
        self.coverStatusText = QtWidgets.QLineEdit(self.groupBox_43)
        self.coverStatusText.setEnabled(False)
        self.coverStatusText.setGeometry(QtCore.QRect(250, 20, 111, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.coverStatusText.setFont(font)
        self.coverStatusText.setMouseTracking(False)
        self.coverStatusText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.coverStatusText.setAcceptDrops(False)
        self.coverStatusText.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.coverStatusText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.coverStatusText.setReadOnly(True)
        self.coverStatusText.setObjectName("coverStatusText")
        self.coverPark = QtWidgets.QPushButton(self.groupBox_43)
        self.coverPark.setGeometry(QtCore.QRect(90, 20, 76, 21))
        self.coverPark.setObjectName("coverPark")
        self.coverUnpark = QtWidgets.QPushButton(self.groupBox_43)
        self.coverUnpark.setGeometry(QtCore.QRect(10, 20, 76, 21))
        self.coverUnpark.setObjectName("coverUnpark")
        self.coverHalt = QtWidgets.QPushButton(self.groupBox_43)
        self.coverHalt.setGeometry(QtCore.QRect(170, 20, 76, 21))
        self.coverHalt.setObjectName("coverHalt")
        self.groupBox_34 = QtWidgets.QGroupBox(self.Imaging)
        self.groupBox_34.setGeometry(QtCore.QRect(5, 250, 366, 51))
        self.groupBox_34.setProperty("large", True)
        self.groupBox_34.setObjectName("groupBox_34")
        self.label_389 = QtWidgets.QLabel(self.groupBox_34)
        self.label_389.setGeometry(QtCore.QRect(205, 20, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_389.setFont(font)
        self.label_389.setObjectName("label_389")
        self.filterNumber = QtWidgets.QLineEdit(self.groupBox_34)
        self.filterNumber.setEnabled(False)
        self.filterNumber.setGeometry(QtCore.QRect(110, 20, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.filterNumber.setFont(font)
        self.filterNumber.setMouseTracking(False)
        self.filterNumber.setFocusPolicy(QtCore.Qt.NoFocus)
        self.filterNumber.setAcceptDrops(False)
        self.filterNumber.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.filterNumber.setInputMask("")
        self.filterNumber.setCursorPosition(1)
        self.filterNumber.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.filterNumber.setReadOnly(True)
        self.filterNumber.setProperty("input", True)
        self.filterNumber.setObjectName("filterNumber")
        self.label_390 = QtWidgets.QLabel(self.groupBox_34)
        self.label_390.setGeometry(QtCore.QRect(10, 20, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_390.setFont(font)
        self.label_390.setObjectName("label_390")
        self.filterName = QtWidgets.QLineEdit(self.groupBox_34)
        self.filterName.setEnabled(False)
        self.filterName.setGeometry(QtCore.QRect(280, 20, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.filterName.setFont(font)
        self.filterName.setMouseTracking(False)
        self.filterName.setFocusPolicy(QtCore.Qt.NoFocus)
        self.filterName.setAcceptDrops(False)
        self.filterName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.filterName.setInputMask("")
        self.filterName.setCursorPosition(1)
        self.filterName.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.filterName.setReadOnly(True)
        self.filterName.setProperty("input", True)
        self.filterName.setObjectName("filterName")
        self.groupBox_35 = QtWidgets.QGroupBox(self.Imaging)
        self.groupBox_35.setGeometry(QtCore.QRect(5, 310, 366, 76))
        self.groupBox_35.setProperty("large", True)
        self.groupBox_35.setObjectName("groupBox_35")
        self.label_410 = QtWidgets.QLabel(self.groupBox_35)
        self.label_410.setGeometry(QtCore.QRect(10, 45, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_410.setFont(font)
        self.label_410.setObjectName("label_410")
        self.moveFocuserIn = QtWidgets.QPushButton(self.groupBox_35)
        self.moveFocuserIn.setEnabled(True)
        self.moveFocuserIn.setGeometry(QtCore.QRect(215, 20, 76, 21))
        self.moveFocuserIn.setObjectName("moveFocuserIn")
        self.label_409 = QtWidgets.QLabel(self.groupBox_35)
        self.label_409.setGeometry(QtCore.QRect(10, 20, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_409.setFont(font)
        self.label_409.setObjectName("label_409")
        self.focuserPosition = QtWidgets.QLineEdit(self.groupBox_35)
        self.focuserPosition.setEnabled(False)
        self.focuserPosition.setGeometry(QtCore.QRect(110, 20, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.focuserPosition.setFont(font)
        self.focuserPosition.setMouseTracking(False)
        self.focuserPosition.setFocusPolicy(QtCore.Qt.NoFocus)
        self.focuserPosition.setAcceptDrops(False)
        self.focuserPosition.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.focuserPosition.setInputMask("")
        self.focuserPosition.setCursorPosition(1)
        self.focuserPosition.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.focuserPosition.setReadOnly(True)
        self.focuserPosition.setProperty("input", True)
        self.focuserPosition.setObjectName("focuserPosition")
        self.focuserStepsize = QtWidgets.QDoubleSpinBox(self.groupBox_35)
        self.focuserStepsize.setGeometry(QtCore.QRect(110, 45, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.focuserStepsize.setFont(font)
        self.focuserStepsize.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.focuserStepsize.setDecimals(0)
        self.focuserStepsize.setMinimum(1.0)
        self.focuserStepsize.setMaximum(1000.0)
        self.focuserStepsize.setSingleStep(10.0)
        self.focuserStepsize.setProperty("value", 100.0)
        self.focuserStepsize.setObjectName("focuserStepsize")
        self.moveFocuserOut = QtWidgets.QPushButton(self.groupBox_35)
        self.moveFocuserOut.setEnabled(True)
        self.moveFocuserOut.setGeometry(QtCore.QRect(215, 45, 76, 21))
        self.moveFocuserOut.setObjectName("moveFocuserOut")
        self.haltFocuser = QtWidgets.QPushButton(self.groupBox_35)
        self.haltFocuser.setEnabled(True)
        self.haltFocuser.setGeometry(QtCore.QRect(295, 20, 61, 46))
        self.haltFocuser.setObjectName("haltFocuser")
        self.settingsTabWidget.addTab(self.Imaging, "")
        self.MountRemote = QtWidgets.QWidget()
        self.MountRemote.setObjectName("MountRemote")
        self.line_97 = QtWidgets.QFrame(self.MountRemote)
        self.line_97.setGeometry(QtCore.QRect(380, 10, 11, 371))
        self.line_97.setStyleSheet("")
        self.line_97.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_97.setLineWidth(2)
        self.line_97.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_97.setObjectName("line_97")
        self.groupBox_62 = QtWidgets.QGroupBox(self.MountRemote)
        self.groupBox_62.setGeometry(QtCore.QRect(5, 15, 366, 106))
        self.groupBox_62.setProperty("large", True)
        self.groupBox_62.setObjectName("groupBox_62")
        self.mountHost = QtWidgets.QLineEdit(self.groupBox_62)
        self.mountHost.setGeometry(QtCore.QRect(115, 25, 241, 21))
        self.mountHost.setMinimumSize(QtCore.QSize(25, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mountHost.setFont(font)
        self.mountHost.setMouseTracking(False)
        self.mountHost.setAcceptDrops(False)
        self.mountHost.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mountHost.setMaxLength(50)
        self.mountHost.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mountHost.setDragEnabled(True)
        self.mountHost.setReadOnly(False)
        self.mountHost.setObjectName("mountHost")
        self.label_mountIP = QtWidgets.QLabel(self.groupBox_62)
        self.label_mountIP.setGeometry(QtCore.QRect(10, 25, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_mountIP.setFont(font)
        self.label_mountIP.setObjectName("label_mountIP")
        self.label_mountMAC = QtWidgets.QLabel(self.groupBox_62)
        self.label_mountMAC.setGeometry(QtCore.QRect(10, 75, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_mountMAC.setFont(font)
        self.label_mountMAC.setObjectName("label_mountMAC")
        self.mountMAC = QtWidgets.QLineEdit(self.groupBox_62)
        self.mountMAC.setEnabled(False)
        self.mountMAC.setGeometry(QtCore.QRect(115, 75, 116, 21))
        self.mountMAC.setMinimumSize(QtCore.QSize(25, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mountMAC.setFont(font)
        self.mountMAC.setMouseTracking(False)
        self.mountMAC.setAcceptDrops(False)
        self.mountMAC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mountMAC.setMaxLength(18)
        self.mountMAC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mountMAC.setDragEnabled(True)
        self.mountMAC.setReadOnly(True)
        self.mountMAC.setObjectName("mountMAC")
        self.port3492 = QtWidgets.QRadioButton(self.groupBox_62)
        self.port3492.setGeometry(QtCore.QRect(180, 50, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.port3492.setFont(font)
        self.port3492.setChecked(True)
        self.port3492.setObjectName("port3492")
        self.port3490 = QtWidgets.QRadioButton(self.groupBox_62)
        self.port3490.setGeometry(QtCore.QRect(115, 50, 66, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.port3490.setFont(font)
        self.port3490.setChecked(False)
        self.port3490.setObjectName("port3490")
        self.label_mountMAC_3 = QtWidgets.QLabel(self.groupBox_62)
        self.label_mountMAC_3.setGeometry(QtCore.QRect(10, 50, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_mountMAC_3.setFont(font)
        self.label_mountMAC_3.setObjectName("label_mountMAC_3")
        self.groupBox_63 = QtWidgets.QGroupBox(self.MountRemote)
        self.groupBox_63.setGeometry(QtCore.QRect(400, 15, 366, 51))
        self.groupBox_63.setProperty("large", True)
        self.groupBox_63.setObjectName("groupBox_63")
        self.label_259 = QtWidgets.QLabel(self.groupBox_63)
        self.label_259.setGeometry(QtCore.QRect(10, 20, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_259.setFont(font)
        self.label_259.setObjectName("label_259")
        self.bootRackComp = QtWidgets.QPushButton(self.groupBox_63)
        self.bootRackComp.setGeometry(QtCore.QRect(245, 20, 116, 21))
        self.bootRackComp.setMinimumSize(QtCore.QSize(0, 21))
        self.bootRackComp.setObjectName("bootRackComp")
        self.rackCompMAC = QtWidgets.QLineEdit(self.groupBox_63)
        self.rackCompMAC.setGeometry(QtCore.QRect(115, 20, 116, 21))
        self.rackCompMAC.setMinimumSize(QtCore.QSize(25, 0))
        self.rackCompMAC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.rackCompMAC.setObjectName("rackCompMAC")
        self.groupBox_31 = QtWidgets.QGroupBox(self.MountRemote)
        self.groupBox_31.setGeometry(QtCore.QRect(5, 135, 366, 51))
        self.groupBox_31.setProperty("large", True)
        self.groupBox_31.setObjectName("groupBox_31")
        self.label_40 = QtWidgets.QLabel(self.groupBox_31)
        self.label_40.setGeometry(QtCore.QRect(10, 20, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_40.setFont(font)
        self.label_40.setObjectName("label_40")
        self.label_295 = QtWidgets.QLabel(self.groupBox_31)
        self.label_295.setGeometry(QtCore.QRect(160, 20, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_295.setFont(font)
        self.label_295.setObjectName("label_295")
        self.settleTimeMount = QtWidgets.QDoubleSpinBox(self.groupBox_31)
        self.settleTimeMount.setGeometry(QtCore.QRect(115, 20, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.settleTimeMount.setFont(font)
        self.settleTimeMount.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.settleTimeMount.setDecimals(0)
        self.settleTimeMount.setMinimum(0.0)
        self.settleTimeMount.setMaximum(100.0)
        self.settleTimeMount.setSingleStep(1.0)
        self.settleTimeMount.setProperty("value", 0.0)
        self.settleTimeMount.setObjectName("settleTimeMount")
        self.FirmwareGroup = QtWidgets.QGroupBox(self.MountRemote)
        self.FirmwareGroup.setGeometry(QtCore.QRect(5, 200, 366, 101))
        self.FirmwareGroup.setProperty("large", True)
        self.FirmwareGroup.setObjectName("FirmwareGroup")
        self.fwtime = QtWidgets.QLineEdit(self.FirmwareGroup)
        self.fwtime.setEnabled(False)
        self.fwtime.setGeometry(QtCore.QRect(260, 70, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.fwtime.setFont(font)
        self.fwtime.setMouseTracking(False)
        self.fwtime.setFocusPolicy(QtCore.Qt.NoFocus)
        self.fwtime.setAcceptDrops(False)
        self.fwtime.setToolTip("")
        self.fwtime.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.fwtime.setMaxLength(8)
        self.fwtime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.fwtime.setReadOnly(True)
        self.fwtime.setObjectName("fwtime")
        self.firmwareProductName_3 = QtWidgets.QLabel(self.FirmwareGroup)
        self.firmwareProductName_3.setGeometry(QtCore.QRect(10, 45, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.firmwareProductName_3.setFont(font)
        self.firmwareProductName_3.setObjectName("firmwareProductName_3")
        self.firmwareProductName_4 = QtWidgets.QLabel(self.FirmwareGroup)
        self.firmwareProductName_4.setGeometry(QtCore.QRect(10, 70, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.firmwareProductName_4.setFont(font)
        self.firmwareProductName_4.setObjectName("firmwareProductName_4")
        self.firmwareProductName_5 = QtWidgets.QLabel(self.FirmwareGroup)
        self.firmwareProductName_5.setGeometry(QtCore.QRect(225, 70, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.firmwareProductName_5.setFont(font)
        self.firmwareProductName_5.setObjectName("firmwareProductName_5")
        self.product = QtWidgets.QLineEdit(self.FirmwareGroup)
        self.product.setEnabled(False)
        self.product.setGeometry(QtCore.QRect(65, 20, 291, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.product.setFont(font)
        self.product.setMouseTracking(False)
        self.product.setFocusPolicy(QtCore.Qt.NoFocus)
        self.product.setAcceptDrops(False)
        self.product.setToolTip("")
        self.product.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.product.setMaxLength(25)
        self.product.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.product.setReadOnly(True)
        self.product.setObjectName("product")
        self.fwdate = QtWidgets.QLineEdit(self.FirmwareGroup)
        self.fwdate.setEnabled(False)
        self.fwdate.setGeometry(QtCore.QRect(65, 70, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.fwdate.setFont(font)
        self.fwdate.setMouseTracking(False)
        self.fwdate.setFocusPolicy(QtCore.Qt.NoFocus)
        self.fwdate.setAcceptDrops(False)
        self.fwdate.setToolTip("")
        self.fwdate.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.fwdate.setMaxLength(25)
        self.fwdate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.fwdate.setReadOnly(True)
        self.fwdate.setObjectName("fwdate")
        self.firmwareProductName_2 = QtWidgets.QLabel(self.FirmwareGroup)
        self.firmwareProductName_2.setGeometry(QtCore.QRect(10, 20, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.firmwareProductName_2.setFont(font)
        self.firmwareProductName_2.setObjectName("firmwareProductName_2")
        self.hardware = QtWidgets.QLineEdit(self.FirmwareGroup)
        self.hardware.setEnabled(False)
        self.hardware.setGeometry(QtCore.QRect(260, 45, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.hardware.setFont(font)
        self.hardware.setMouseTracking(False)
        self.hardware.setFocusPolicy(QtCore.Qt.NoFocus)
        self.hardware.setAcceptDrops(False)
        self.hardware.setToolTip("")
        self.hardware.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.hardware.setMaxLength(25)
        self.hardware.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.hardware.setReadOnly(True)
        self.hardware.setObjectName("hardware")
        self.Hardware = QtWidgets.QLabel(self.FirmwareGroup)
        self.Hardware.setGeometry(QtCore.QRect(225, 45, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.Hardware.setFont(font)
        self.Hardware.setObjectName("Hardware")
        self.vString = QtWidgets.QLineEdit(self.FirmwareGroup)
        self.vString.setEnabled(False)
        self.vString.setGeometry(QtCore.QRect(65, 45, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.vString.setFont(font)
        self.vString.setMouseTracking(False)
        self.vString.setFocusPolicy(QtCore.Qt.NoFocus)
        self.vString.setAcceptDrops(False)
        self.vString.setToolTip("")
        self.vString.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.vString.setMaxLength(8)
        self.vString.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.vString.setReadOnly(True)
        self.vString.setObjectName("vString")
        self.settingsTabWidget.addTab(self.MountRemote, "")
        self.Dome = QtWidgets.QWidget()
        self.Dome.setObjectName("Dome")
        self.line_86 = QtWidgets.QFrame(self.Dome)
        self.line_86.setGeometry(QtCore.QRect(380, 10, 11, 371))
        self.line_86.setStyleSheet("")
        self.line_86.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_86.setLineWidth(2)
        self.line_86.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_86.setObjectName("line_86")
        self.groupBox_44 = QtWidgets.QGroupBox(self.Dome)
        self.groupBox_44.setGeometry(QtCore.QRect(5, 10, 366, 271))
        self.groupBox_44.setProperty("large", True)
        self.groupBox_44.setObjectName("groupBox_44")
        self.label_99 = QtWidgets.QLabel(self.groupBox_44)
        self.label_99.setGeometry(QtCore.QRect(15, 20, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_99.setFont(font)
        self.label_99.setObjectName("label_99")
        self.label_134 = QtWidgets.QLabel(self.groupBox_44)
        self.label_134.setGeometry(QtCore.QRect(330, 145, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_134.setFont(font)
        self.label_134.setObjectName("label_134")
        self.label_122 = QtWidgets.QLabel(self.groupBox_44)
        self.label_122.setGeometry(QtCore.QRect(330, 70, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_122.setFont(font)
        self.label_122.setObjectName("label_122")
        self.label_108 = QtWidgets.QLabel(self.groupBox_44)
        self.label_108.setGeometry(QtCore.QRect(15, 70, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_108.setFont(font)
        self.label_108.setObjectName("label_108")
        self.label_110 = QtWidgets.QLabel(self.groupBox_44)
        self.label_110.setGeometry(QtCore.QRect(15, 95, 216, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_110.setFont(font)
        self.label_110.setObjectName("label_110")
        self.label_116 = QtWidgets.QLabel(self.groupBox_44)
        self.label_116.setGeometry(QtCore.QRect(330, 20, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_116.setFont(font)
        self.label_116.setObjectName("label_116")
        self.checkUseDomeGeometry = QtWidgets.QCheckBox(self.groupBox_44)
        self.checkUseDomeGeometry.setGeometry(QtCore.QRect(15, 215, 264, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkUseDomeGeometry.setFont(font)
        self.checkUseDomeGeometry.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkUseDomeGeometry.setChecked(False)
        self.checkUseDomeGeometry.setObjectName("checkUseDomeGeometry")
        self.label_118 = QtWidgets.QLabel(self.groupBox_44)
        self.label_118.setGeometry(QtCore.QRect(330, 45, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_118.setFont(font)
        self.label_118.setObjectName("label_118")
        self.label_123 = QtWidgets.QLabel(self.groupBox_44)
        self.label_123.setGeometry(QtCore.QRect(330, 95, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_123.setFont(font)
        self.label_123.setObjectName("label_123")
        self.label_107 = QtWidgets.QLabel(self.groupBox_44)
        self.label_107.setGeometry(QtCore.QRect(15, 45, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_107.setFont(font)
        self.label_107.setObjectName("label_107")
        self.label_135 = QtWidgets.QLabel(self.groupBox_44)
        self.label_135.setGeometry(QtCore.QRect(15, 145, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_135.setFont(font)
        self.label_135.setObjectName("label_135")
        self.domeRadius = QtWidgets.QDoubleSpinBox(self.groupBox_44)
        self.domeRadius.setGeometry(QtCore.QRect(250, 20, 76, 22))
        self.domeRadius.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.domeRadius.setMinimum(1.0)
        self.domeRadius.setMaximum(3.0)
        self.domeRadius.setSingleStep(0.1)
        self.domeRadius.setProperty("value", 1.25)
        self.domeRadius.setObjectName("domeRadius")
        self.domeNorthOffset = QtWidgets.QDoubleSpinBox(self.groupBox_44)
        self.domeNorthOffset.setGeometry(QtCore.QRect(250, 45, 76, 22))
        self.domeNorthOffset.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.domeNorthOffset.setMinimum(-1.0)
        self.domeNorthOffset.setMaximum(1.0)
        self.domeNorthOffset.setSingleStep(0.01)
        self.domeNorthOffset.setObjectName("domeNorthOffset")
        self.domeEastOffset = QtWidgets.QDoubleSpinBox(self.groupBox_44)
        self.domeEastOffset.setGeometry(QtCore.QRect(250, 70, 76, 22))
        self.domeEastOffset.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.domeEastOffset.setMinimum(-1.0)
        self.domeEastOffset.setMaximum(1.0)
        self.domeEastOffset.setSingleStep(0.01)
        self.domeEastOffset.setObjectName("domeEastOffset")
        self.domeZoffGEM = QtWidgets.QDoubleSpinBox(self.groupBox_44)
        self.domeZoffGEM.setGeometry(QtCore.QRect(250, 95, 76, 22))
        self.domeZoffGEM.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.domeZoffGEM.setMinimum(-0.5)
        self.domeZoffGEM.setMaximum(0.5)
        self.domeZoffGEM.setSingleStep(0.01)
        self.domeZoffGEM.setObjectName("domeZoffGEM")
        self.offGEM = QtWidgets.QDoubleSpinBox(self.groupBox_44)
        self.offGEM.setGeometry(QtCore.QRect(250, 145, 76, 22))
        self.offGEM.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.offGEM.setSingleStep(0.01)
        self.offGEM.setObjectName("offGEM")
        self.copyFromDomeDriver = QtWidgets.QPushButton(self.groupBox_44)
        self.copyFromDomeDriver.setGeometry(QtCore.QRect(230, 240, 131, 21))
        self.copyFromDomeDriver.setObjectName("copyFromDomeDriver")
        self.checkAutomaticDome = QtWidgets.QCheckBox(self.groupBox_44)
        self.checkAutomaticDome.setGeometry(QtCore.QRect(15, 240, 211, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.checkAutomaticDome.setFont(font)
        self.checkAutomaticDome.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkAutomaticDome.setChecked(False)
        self.checkAutomaticDome.setObjectName("checkAutomaticDome")
        self.offLAT = QtWidgets.QDoubleSpinBox(self.groupBox_44)
        self.offLAT.setGeometry(QtCore.QRect(250, 170, 76, 22))
        self.offLAT.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.offLAT.setMinimum(-2.0)
        self.offLAT.setMaximum(2.0)
        self.offLAT.setSingleStep(0.01)
        self.offLAT.setObjectName("offLAT")
        self.label_136 = QtWidgets.QLabel(self.groupBox_44)
        self.label_136.setGeometry(QtCore.QRect(330, 170, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_136.setFont(font)
        self.label_136.setObjectName("label_136")
        self.label_137 = QtWidgets.QLabel(self.groupBox_44)
        self.label_137.setGeometry(QtCore.QRect(15, 170, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_137.setFont(font)
        self.label_137.setObjectName("label_137")
        self.label_124 = QtWidgets.QLabel(self.groupBox_44)
        self.label_124.setGeometry(QtCore.QRect(330, 120, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_124.setFont(font)
        self.label_124.setObjectName("label_124")
        self.label_113 = QtWidgets.QLabel(self.groupBox_44)
        self.label_113.setGeometry(QtCore.QRect(15, 120, 216, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_113.setFont(font)
        self.label_113.setObjectName("label_113")
        self.domeZoff10micron = QtWidgets.QDoubleSpinBox(self.groupBox_44)
        self.domeZoff10micron.setGeometry(QtCore.QRect(250, 120, 76, 22))
        self.domeZoff10micron.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.domeZoff10micron.setMinimum(-0.5)
        self.domeZoff10micron.setMaximum(0.5)
        self.domeZoff10micron.setSingleStep(0.01)
        self.domeZoff10micron.setObjectName("domeZoff10micron")
        self.groupBox_66 = QtWidgets.QGroupBox(self.Dome)
        self.groupBox_66.setGeometry(QtCore.QRect(400, 330, 361, 51))
        self.groupBox_66.setProperty("large", True)
        self.groupBox_66.setObjectName("groupBox_66")
        self.settleTimeDome = QtWidgets.QDoubleSpinBox(self.groupBox_66)
        self.settleTimeDome.setGeometry(QtCore.QRect(125, 20, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.settleTimeDome.setFont(font)
        self.settleTimeDome.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.settleTimeDome.setDecimals(0)
        self.settleTimeDome.setMinimum(0.0)
        self.settleTimeDome.setMaximum(100.0)
        self.settleTimeDome.setSingleStep(1.0)
        self.settleTimeDome.setProperty("value", 0.0)
        self.settleTimeDome.setObjectName("settleTimeDome")
        self.label_100 = QtWidgets.QLabel(self.groupBox_66)
        self.label_100.setGeometry(QtCore.QRect(10, 20, 111, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_100.setFont(font)
        self.label_100.setObjectName("label_100")
        self.label_314 = QtWidgets.QLabel(self.groupBox_66)
        self.label_314.setGeometry(QtCore.QRect(170, 20, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_314.setFont(font)
        self.label_314.setObjectName("label_314")
        self.tabDomeExplain = QtWidgets.QTabWidget(self.Dome)
        self.tabDomeExplain.setGeometry(QtCore.QRect(400, 15, 361, 306))
        self.tabDomeExplain.setTabPosition(QtWidgets.QTabWidget.West)
        self.tabDomeExplain.setElideMode(QtCore.Qt.ElideNone)
        self.tabDomeExplain.setObjectName("tabDomeExplain")
        self.tab_9 = QtWidgets.QWidget()
        self.tab_9.setObjectName("tab_9")
        self.picDome1 = QtWidgets.QLabel(self.tab_9)
        self.picDome1.setGeometry(QtCore.QRect(15, 0, 300, 300))
        self.picDome1.setFrameShape(QtWidgets.QFrame.Box)
        self.picDome1.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picDome1.setLineWidth(0)
        self.picDome1.setMidLineWidth(0)
        self.picDome1.setProperty("iconpicture", True)
        self.picDome1.setObjectName("picDome1")
        self.tabDomeExplain.addTab(self.tab_9, "")
        self.tab_10 = QtWidgets.QWidget()
        self.tab_10.setObjectName("tab_10")
        self.picDome2 = QtWidgets.QLabel(self.tab_10)
        self.picDome2.setGeometry(QtCore.QRect(15, 0, 300, 300))
        self.picDome2.setFrameShape(QtWidgets.QFrame.Box)
        self.picDome2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picDome2.setLineWidth(0)
        self.picDome2.setMidLineWidth(0)
        self.picDome2.setProperty("iconpicture", True)
        self.picDome2.setObjectName("picDome2")
        self.tabDomeExplain.addTab(self.tab_10, "")
        self.tab_11 = QtWidgets.QWidget()
        self.tab_11.setObjectName("tab_11")
        self.picDome3 = QtWidgets.QLabel(self.tab_11)
        self.picDome3.setGeometry(QtCore.QRect(15, 0, 300, 300))
        self.picDome3.setFrameShape(QtWidgets.QFrame.Box)
        self.picDome3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picDome3.setLineWidth(0)
        self.picDome3.setMidLineWidth(0)
        self.picDome3.setProperty("iconpicture", True)
        self.picDome3.setObjectName("picDome3")
        self.tabDomeExplain.addTab(self.tab_11, "")
        self.tab_12 = QtWidgets.QWidget()
        self.tab_12.setObjectName("tab_12")
        self.picDome4 = QtWidgets.QLabel(self.tab_12)
        self.picDome4.setGeometry(QtCore.QRect(15, 0, 300, 300))
        self.picDome4.setFrameShape(QtWidgets.QFrame.Box)
        self.picDome4.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picDome4.setLineWidth(0)
        self.picDome4.setMidLineWidth(0)
        self.picDome4.setProperty("iconpicture", True)
        self.picDome4.setObjectName("picDome4")
        self.tabDomeExplain.addTab(self.tab_12, "")
        self.tab_13 = QtWidgets.QWidget()
        self.tab_13.setObjectName("tab_13")
        self.picDome5 = QtWidgets.QLabel(self.tab_13)
        self.picDome5.setGeometry(QtCore.QRect(15, 0, 300, 300))
        self.picDome5.setFrameShape(QtWidgets.QFrame.Box)
        self.picDome5.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picDome5.setLineWidth(0)
        self.picDome5.setMidLineWidth(0)
        self.picDome5.setProperty("iconpicture", True)
        self.picDome5.setObjectName("picDome5")
        self.tabDomeExplain.addTab(self.tab_13, "")
        self.tab_14 = QtWidgets.QWidget()
        self.tab_14.setObjectName("tab_14")
        self.picDome6 = QtWidgets.QLabel(self.tab_14)
        self.picDome6.setGeometry(QtCore.QRect(15, 0, 300, 300))
        self.picDome6.setFrameShape(QtWidgets.QFrame.Box)
        self.picDome6.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picDome6.setLineWidth(0)
        self.picDome6.setMidLineWidth(0)
        self.picDome6.setProperty("iconpicture", True)
        self.picDome6.setObjectName("picDome6")
        self.tabDomeExplain.addTab(self.tab_14, "")
        self.tab_15 = QtWidgets.QWidget()
        self.tab_15.setObjectName("tab_15")
        self.picDome7 = QtWidgets.QLabel(self.tab_15)
        self.picDome7.setGeometry(QtCore.QRect(15, 0, 300, 300))
        self.picDome7.setFrameShape(QtWidgets.QFrame.Box)
        self.picDome7.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picDome7.setLineWidth(0)
        self.picDome7.setMidLineWidth(0)
        self.picDome7.setProperty("iconpicture", True)
        self.picDome7.setObjectName("picDome7")
        self.tabDomeExplain.addTab(self.tab_15, "")
        self.tab_16 = QtWidgets.QWidget()
        self.tab_16.setObjectName("tab_16")
        self.picDome8 = QtWidgets.QLabel(self.tab_16)
        self.picDome8.setGeometry(QtCore.QRect(15, 0, 300, 300))
        self.picDome8.setFrameShape(QtWidgets.QFrame.Box)
        self.picDome8.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picDome8.setLineWidth(0)
        self.picDome8.setMidLineWidth(0)
        self.picDome8.setProperty("iconpicture", True)
        self.picDome8.setObjectName("picDome8")
        self.tabDomeExplain.addTab(self.tab_16, "")
        self.groupBox_53 = QtWidgets.QGroupBox(self.Dome)
        self.groupBox_53.setGeometry(QtCore.QRect(5, 300, 366, 81))
        self.groupBox_53.setProperty("large", True)
        self.groupBox_53.setObjectName("groupBox_53")
        self.domeOpenShutter = QtWidgets.QPushButton(self.groupBox_53)
        self.domeOpenShutter.setGeometry(QtCore.QRect(10, 20, 76, 21))
        self.domeOpenShutter.setObjectName("domeOpenShutter")
        self.domeCloseShutter = QtWidgets.QPushButton(self.groupBox_53)
        self.domeCloseShutter.setGeometry(QtCore.QRect(90, 20, 76, 21))
        self.domeCloseShutter.setObjectName("domeCloseShutter")
        self.label_139 = QtWidgets.QLabel(self.groupBox_53)
        self.label_139.setGeometry(QtCore.QRect(15, 50, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_139.setFont(font)
        self.label_139.setObjectName("label_139")
        self.domeShutterWidth = QtWidgets.QDoubleSpinBox(self.groupBox_53)
        self.domeShutterWidth.setGeometry(QtCore.QRect(250, 50, 76, 22))
        self.domeShutterWidth.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.domeShutterWidth.setMinimum(0.0)
        self.domeShutterWidth.setMaximum(1.0)
        self.domeShutterWidth.setSingleStep(0.01)
        self.domeShutterWidth.setProperty("value", 0.4)
        self.domeShutterWidth.setObjectName("domeShutterWidth")
        self.label_140 = QtWidgets.QLabel(self.groupBox_53)
        self.label_140.setGeometry(QtCore.QRect(330, 50, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_140.setFont(font)
        self.label_140.setObjectName("label_140")
        self.domeAbortSlew = QtWidgets.QPushButton(self.groupBox_53)
        self.domeAbortSlew.setGeometry(QtCore.QRect(170, 20, 76, 21))
        self.domeAbortSlew.setObjectName("domeAbortSlew")
        self.domeShutterStatusText = QtWidgets.QLineEdit(self.groupBox_53)
        self.domeShutterStatusText.setEnabled(False)
        self.domeShutterStatusText.setGeometry(QtCore.QRect(250, 20, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.domeShutterStatusText.setFont(font)
        self.domeShutterStatusText.setMouseTracking(False)
        self.domeShutterStatusText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.domeShutterStatusText.setContextMenuPolicy(QtCore.Qt.PreventContextMenu)
        self.domeShutterStatusText.setAcceptDrops(False)
        self.domeShutterStatusText.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.domeShutterStatusText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.domeShutterStatusText.setReadOnly(True)
        self.domeShutterStatusText.setObjectName("domeShutterStatusText")
        self.settingsTabWidget.addTab(self.Dome, "")
        self.ParkPos = QtWidgets.QWidget()
        self.ParkPos.setObjectName("ParkPos")
        self.line_83 = QtWidgets.QFrame(self.ParkPos)
        self.line_83.setGeometry(QtCore.QRect(380, 10, 11, 371))
        self.line_83.setStyleSheet("")
        self.line_83.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_83.setLineWidth(2)
        self.line_83.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_83.setObjectName("line_83")
        self.groupBox_45 = QtWidgets.QGroupBox(self.ParkPos)
        self.groupBox_45.setGeometry(QtCore.QRect(5, 15, 366, 361))
        self.groupBox_45.setProperty("large", True)
        self.groupBox_45.setObjectName("groupBox_45")
        self.label_243 = QtWidgets.QLabel(self.groupBox_45)
        self.label_243.setGeometry(QtCore.QRect(230, 200, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_243.setFont(font)
        self.label_243.setAlignment(QtCore.Qt.AlignCenter)
        self.label_243.setWordWrap(False)
        self.label_243.setObjectName("label_243")
        self.label_262 = QtWidgets.QLabel(self.groupBox_45)
        self.label_262.setGeometry(QtCore.QRect(290, 260, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_262.setFont(font)
        self.label_262.setAlignment(QtCore.Qt.AlignCenter)
        self.label_262.setWordWrap(False)
        self.label_262.setObjectName("label_262")
        self.label_250 = QtWidgets.QLabel(self.groupBox_45)
        self.label_250.setGeometry(QtCore.QRect(290, 80, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_250.setFont(font)
        self.label_250.setAlignment(QtCore.Qt.AlignCenter)
        self.label_250.setWordWrap(False)
        self.label_250.setObjectName("label_250")
        self.posAz6 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAz6.setGeometry(QtCore.QRect(250, 230, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz6.setFont(font)
        self.posAz6.setMouseTracking(False)
        self.posAz6.setAcceptDrops(False)
        self.posAz6.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz6.setText("")
        self.posAz6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz6.setReadOnly(False)
        self.posAz6.setObjectName("posAz6")
        self.label_244 = QtWidgets.QLabel(self.groupBox_45)
        self.label_244.setGeometry(QtCore.QRect(290, 50, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_244.setFont(font)
        self.label_244.setAlignment(QtCore.Qt.AlignCenter)
        self.label_244.setWordWrap(False)
        self.label_244.setObjectName("label_244")
        self.posText4 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posText4.setGeometry(QtCore.QRect(60, 170, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText4.setFont(font)
        self.posText4.setMouseTracking(False)
        self.posText4.setAcceptDrops(False)
        self.posText4.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText4.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText4.setReadOnly(False)
        self.posText4.setObjectName("posText4")
        self.posAz1 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAz1.setGeometry(QtCore.QRect(250, 80, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz1.setFont(font)
        self.posAz1.setMouseTracking(False)
        self.posAz1.setAcceptDrops(False)
        self.posAz1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz1.setText("")
        self.posAz1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz1.setReadOnly(False)
        self.posAz1.setObjectName("posAz1")
        self.posAz2 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAz2.setGeometry(QtCore.QRect(250, 110, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz2.setFont(font)
        self.posAz2.setMouseTracking(False)
        self.posAz2.setAcceptDrops(False)
        self.posAz2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz2.setText("")
        self.posAz2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz2.setReadOnly(False)
        self.posAz2.setObjectName("posAz2")
        self.label_101 = QtWidgets.QLabel(self.groupBox_45)
        self.label_101.setGeometry(QtCore.QRect(10, 230, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_101.setFont(font)
        self.label_101.setAlignment(QtCore.Qt.AlignCenter)
        self.label_101.setObjectName("label_101")
        self.posAlt1 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAlt1.setGeometry(QtCore.QRect(195, 80, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt1.setFont(font)
        self.posAlt1.setMouseTracking(False)
        self.posAlt1.setAcceptDrops(False)
        self.posAlt1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt1.setReadOnly(False)
        self.posAlt1.setObjectName("posAlt1")
        self.label_92 = QtWidgets.QLabel(self.groupBox_45)
        self.label_92.setGeometry(QtCore.QRect(10, 25, 56, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_92.setFont(font)
        self.label_92.setObjectName("label_92")
        self.posText2 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posText2.setGeometry(QtCore.QRect(60, 110, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText2.setFont(font)
        self.posText2.setMouseTracking(False)
        self.posText2.setAcceptDrops(False)
        self.posText2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText2.setReadOnly(False)
        self.posText2.setObjectName("posText2")
        self.label_93 = QtWidgets.QLabel(self.groupBox_45)
        self.label_93.setGeometry(QtCore.QRect(230, 50, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_93.setFont(font)
        self.label_93.setAlignment(QtCore.Qt.AlignCenter)
        self.label_93.setWordWrap(False)
        self.label_93.setObjectName("label_93")
        self.posText0 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posText0.setGeometry(QtCore.QRect(60, 50, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText0.setFont(font)
        self.posText0.setMouseTracking(False)
        self.posText0.setAcceptDrops(False)
        self.posText0.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText0.setReadOnly(False)
        self.posText0.setObjectName("posText0")
        self.posAlt3 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAlt3.setGeometry(QtCore.QRect(195, 140, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt3.setFont(font)
        self.posAlt3.setMouseTracking(False)
        self.posAlt3.setAcceptDrops(False)
        self.posAlt3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt3.setReadOnly(False)
        self.posAlt3.setObjectName("posAlt3")
        self.label_257 = QtWidgets.QLabel(self.groupBox_45)
        self.label_257.setGeometry(QtCore.QRect(230, 230, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_257.setFont(font)
        self.label_257.setAlignment(QtCore.Qt.AlignCenter)
        self.label_257.setWordWrap(False)
        self.label_257.setObjectName("label_257")
        self.label_261 = QtWidgets.QLabel(self.groupBox_45)
        self.label_261.setGeometry(QtCore.QRect(230, 260, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_261.setFont(font)
        self.label_261.setAlignment(QtCore.Qt.AlignCenter)
        self.label_261.setWordWrap(False)
        self.label_261.setObjectName("label_261")
        self.posText3 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posText3.setGeometry(QtCore.QRect(60, 140, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText3.setFont(font)
        self.posText3.setMouseTracking(False)
        self.posText3.setAcceptDrops(False)
        self.posText3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText3.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText3.setReadOnly(False)
        self.posText3.setObjectName("posText3")
        self.label_132 = QtWidgets.QLabel(self.groupBox_45)
        self.label_132.setGeometry(QtCore.QRect(250, 20, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_132.setFont(font)
        self.label_132.setObjectName("label_132")
        self.posText5 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posText5.setGeometry(QtCore.QRect(60, 200, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText5.setFont(font)
        self.posText5.setMouseTracking(False)
        self.posText5.setAcceptDrops(False)
        self.posText5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText5.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText5.setReadOnly(False)
        self.posText5.setObjectName("posText5")
        self.label_47 = QtWidgets.QLabel(self.groupBox_45)
        self.label_47.setGeometry(QtCore.QRect(10, 140, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_47.setFont(font)
        self.label_47.setAlignment(QtCore.Qt.AlignCenter)
        self.label_47.setObjectName("label_47")
        self.label_249 = QtWidgets.QLabel(self.groupBox_45)
        self.label_249.setGeometry(QtCore.QRect(290, 140, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_249.setFont(font)
        self.label_249.setAlignment(QtCore.Qt.AlignCenter)
        self.label_249.setWordWrap(False)
        self.label_249.setObjectName("label_249")
        self.label_103 = QtWidgets.QLabel(self.groupBox_45)
        self.label_103.setGeometry(QtCore.QRect(10, 260, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_103.setFont(font)
        self.label_103.setAlignment(QtCore.Qt.AlignCenter)
        self.label_103.setObjectName("label_103")
        self.posAlt0 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAlt0.setGeometry(QtCore.QRect(195, 50, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt0.setFont(font)
        self.posAlt0.setMouseTracking(False)
        self.posAlt0.setAcceptDrops(False)
        self.posAlt0.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt0.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt0.setReadOnly(False)
        self.posAlt0.setObjectName("posAlt0")
        self.posAlt5 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAlt5.setGeometry(QtCore.QRect(195, 200, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt5.setFont(font)
        self.posAlt5.setMouseTracking(False)
        self.posAlt5.setAcceptDrops(False)
        self.posAlt5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt5.setReadOnly(False)
        self.posAlt5.setObjectName("posAlt5")
        self.posAlt4 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAlt4.setGeometry(QtCore.QRect(195, 170, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt4.setFont(font)
        self.posAlt4.setMouseTracking(False)
        self.posAlt4.setAcceptDrops(False)
        self.posAlt4.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt4.setReadOnly(False)
        self.posAlt4.setObjectName("posAlt4")
        self.label_247 = QtWidgets.QLabel(self.groupBox_45)
        self.label_247.setGeometry(QtCore.QRect(290, 170, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_247.setFont(font)
        self.label_247.setAlignment(QtCore.Qt.AlignCenter)
        self.label_247.setWordWrap(False)
        self.label_247.setObjectName("label_247")
        self.posAlt7 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAlt7.setGeometry(QtCore.QRect(195, 260, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt7.setFont(font)
        self.posAlt7.setMouseTracking(False)
        self.posAlt7.setAcceptDrops(False)
        self.posAlt7.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt7.setReadOnly(False)
        self.posAlt7.setObjectName("posAlt7")
        self.label_131 = QtWidgets.QLabel(self.groupBox_45)
        self.label_131.setGeometry(QtCore.QRect(230, 170, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_131.setFont(font)
        self.label_131.setAlignment(QtCore.Qt.AlignCenter)
        self.label_131.setWordWrap(False)
        self.label_131.setObjectName("label_131")
        self.label_258 = QtWidgets.QLabel(self.groupBox_45)
        self.label_258.setGeometry(QtCore.QRect(290, 230, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_258.setFont(font)
        self.label_258.setAlignment(QtCore.Qt.AlignCenter)
        self.label_258.setWordWrap(False)
        self.label_258.setObjectName("label_258")
        self.label_50 = QtWidgets.QLabel(self.groupBox_45)
        self.label_50.setGeometry(QtCore.QRect(10, 170, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_50.setFont(font)
        self.label_50.setAlignment(QtCore.Qt.AlignCenter)
        self.label_50.setObjectName("label_50")
        self.label_209 = QtWidgets.QLabel(self.groupBox_45)
        self.label_209.setGeometry(QtCore.QRect(230, 140, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_209.setFont(font)
        self.label_209.setAlignment(QtCore.Qt.AlignCenter)
        self.label_209.setWordWrap(False)
        self.label_209.setObjectName("label_209")
        self.posText7 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posText7.setGeometry(QtCore.QRect(60, 260, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText7.setFont(font)
        self.posText7.setMouseTracking(False)
        self.posText7.setAcceptDrops(False)
        self.posText7.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText7.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText7.setReadOnly(False)
        self.posText7.setObjectName("posText7")
        self.label_97 = QtWidgets.QLabel(self.groupBox_45)
        self.label_97.setGeometry(QtCore.QRect(60, 25, 51, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_97.setFont(font)
        self.label_97.setObjectName("label_97")
        self.label_246 = QtWidgets.QLabel(self.groupBox_45)
        self.label_246.setGeometry(QtCore.QRect(230, 110, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_246.setFont(font)
        self.label_246.setAlignment(QtCore.Qt.AlignCenter)
        self.label_246.setWordWrap(False)
        self.label_246.setObjectName("label_246")
        self.label_251 = QtWidgets.QLabel(self.groupBox_45)
        self.label_251.setGeometry(QtCore.QRect(290, 200, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_251.setFont(font)
        self.label_251.setAlignment(QtCore.Qt.AlignCenter)
        self.label_251.setWordWrap(False)
        self.label_251.setObjectName("label_251")
        self.posAz7 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAz7.setGeometry(QtCore.QRect(250, 260, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz7.setFont(font)
        self.posAz7.setMouseTracking(False)
        self.posAz7.setAcceptDrops(False)
        self.posAz7.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz7.setText("")
        self.posAz7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz7.setReadOnly(False)
        self.posAz7.setObjectName("posAz7")
        self.posText1 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posText1.setGeometry(QtCore.QRect(60, 80, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText1.setFont(font)
        self.posText1.setMouseTracking(False)
        self.posText1.setAcceptDrops(False)
        self.posText1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText1.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText1.setReadOnly(False)
        self.posText1.setObjectName("posText1")
        self.posAz3 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAz3.setGeometry(QtCore.QRect(250, 140, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz3.setFont(font)
        self.posAz3.setMouseTracking(False)
        self.posAz3.setAcceptDrops(False)
        self.posAz3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz3.setText("")
        self.posAz3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz3.setReadOnly(False)
        self.posAz3.setObjectName("posAz3")
        self.label_2 = QtWidgets.QLabel(self.groupBox_45)
        self.label_2.setGeometry(QtCore.QRect(10, 80, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_2.setFont(font)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.label_245 = QtWidgets.QLabel(self.groupBox_45)
        self.label_245.setGeometry(QtCore.QRect(230, 80, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_245.setFont(font)
        self.label_245.setAlignment(QtCore.Qt.AlignCenter)
        self.label_245.setWordWrap(False)
        self.label_245.setObjectName("label_245")
        self.label_95 = QtWidgets.QLabel(self.groupBox_45)
        self.label_95.setGeometry(QtCore.QRect(10, 200, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_95.setFont(font)
        self.label_95.setAlignment(QtCore.Qt.AlignCenter)
        self.label_95.setObjectName("label_95")
        self.posAlt2 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAlt2.setGeometry(QtCore.QRect(195, 110, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt2.setFont(font)
        self.posAlt2.setMouseTracking(False)
        self.posAlt2.setAcceptDrops(False)
        self.posAlt2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt2.setReadOnly(False)
        self.posAlt2.setObjectName("posAlt2")
        self.label_44 = QtWidgets.QLabel(self.groupBox_45)
        self.label_44.setGeometry(QtCore.QRect(10, 110, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_44.setFont(font)
        self.label_44.setAlignment(QtCore.Qt.AlignCenter)
        self.label_44.setObjectName("label_44")
        self.label_94 = QtWidgets.QLabel(self.groupBox_45)
        self.label_94.setGeometry(QtCore.QRect(195, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_94.setFont(font)
        self.label_94.setObjectName("label_94")
        self.posAz4 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAz4.setGeometry(QtCore.QRect(250, 170, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz4.setFont(font)
        self.posAz4.setMouseTracking(False)
        self.posAz4.setAcceptDrops(False)
        self.posAz4.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz4.setText("")
        self.posAz4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz4.setReadOnly(False)
        self.posAz4.setObjectName("posAz4")
        self.label_248 = QtWidgets.QLabel(self.groupBox_45)
        self.label_248.setGeometry(QtCore.QRect(290, 110, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_248.setFont(font)
        self.label_248.setAlignment(QtCore.Qt.AlignCenter)
        self.label_248.setWordWrap(False)
        self.label_248.setObjectName("label_248")
        self.posText6 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posText6.setGeometry(QtCore.QRect(60, 230, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText6.setFont(font)
        self.posText6.setMouseTracking(False)
        self.posText6.setAcceptDrops(False)
        self.posText6.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText6.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText6.setReadOnly(False)
        self.posText6.setObjectName("posText6")
        self.posAz5 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAz5.setGeometry(QtCore.QRect(250, 200, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz5.setFont(font)
        self.posAz5.setMouseTracking(False)
        self.posAz5.setAcceptDrops(False)
        self.posAz5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz5.setText("")
        self.posAz5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz5.setReadOnly(False)
        self.posAz5.setObjectName("posAz5")
        self.posAlt6 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAlt6.setGeometry(QtCore.QRect(195, 230, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt6.setFont(font)
        self.posAlt6.setMouseTracking(False)
        self.posAlt6.setAcceptDrops(False)
        self.posAlt6.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt6.setReadOnly(False)
        self.posAlt6.setObjectName("posAlt6")
        self.posAz0 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAz0.setGeometry(QtCore.QRect(250, 50, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz0.setFont(font)
        self.posAz0.setMouseTracking(False)
        self.posAz0.setAcceptDrops(False)
        self.posAz0.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz0.setText("")
        self.posAz0.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz0.setReadOnly(False)
        self.posAz0.setObjectName("posAz0")
        self.label_22 = QtWidgets.QLabel(self.groupBox_45)
        self.label_22.setGeometry(QtCore.QRect(10, 50, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_22.setFont(font)
        self.label_22.setAlignment(QtCore.Qt.AlignCenter)
        self.label_22.setObjectName("label_22")
        self.posSave0 = QtWidgets.QPushButton(self.groupBox_45)
        self.posSave0.setGeometry(QtCore.QRect(310, 50, 46, 21))
        self.posSave0.setObjectName("posSave0")
        self.posSave1 = QtWidgets.QPushButton(self.groupBox_45)
        self.posSave1.setGeometry(QtCore.QRect(310, 80, 46, 21))
        self.posSave1.setObjectName("posSave1")
        self.posSave2 = QtWidgets.QPushButton(self.groupBox_45)
        self.posSave2.setGeometry(QtCore.QRect(310, 110, 46, 21))
        self.posSave2.setObjectName("posSave2")
        self.posSave3 = QtWidgets.QPushButton(self.groupBox_45)
        self.posSave3.setGeometry(QtCore.QRect(310, 140, 46, 21))
        self.posSave3.setObjectName("posSave3")
        self.posSave4 = QtWidgets.QPushButton(self.groupBox_45)
        self.posSave4.setGeometry(QtCore.QRect(310, 170, 46, 21))
        self.posSave4.setObjectName("posSave4")
        self.posSave5 = QtWidgets.QPushButton(self.groupBox_45)
        self.posSave5.setGeometry(QtCore.QRect(310, 200, 46, 21))
        self.posSave5.setObjectName("posSave5")
        self.posSave6 = QtWidgets.QPushButton(self.groupBox_45)
        self.posSave6.setGeometry(QtCore.QRect(310, 230, 46, 21))
        self.posSave6.setObjectName("posSave6")
        self.posSave7 = QtWidgets.QPushButton(self.groupBox_45)
        self.posSave7.setGeometry(QtCore.QRect(310, 260, 46, 21))
        self.posSave7.setObjectName("posSave7")
        self.label_127 = QtWidgets.QLabel(self.groupBox_45)
        self.label_127.setGeometry(QtCore.QRect(10, 290, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_127.setFont(font)
        self.label_127.setAlignment(QtCore.Qt.AlignCenter)
        self.label_127.setObjectName("label_127")
        self.label_128 = QtWidgets.QLabel(self.groupBox_45)
        self.label_128.setGeometry(QtCore.QRect(10, 320, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_128.setFont(font)
        self.label_128.setAlignment(QtCore.Qt.AlignCenter)
        self.label_128.setObjectName("label_128")
        self.posText8 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posText8.setGeometry(QtCore.QRect(60, 290, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText8.setFont(font)
        self.posText8.setMouseTracking(False)
        self.posText8.setAcceptDrops(False)
        self.posText8.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText8.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText8.setReadOnly(False)
        self.posText8.setObjectName("posText8")
        self.posText9 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posText9.setGeometry(QtCore.QRect(60, 320, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText9.setFont(font)
        self.posText9.setMouseTracking(False)
        self.posText9.setAcceptDrops(False)
        self.posText9.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText9.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText9.setReadOnly(False)
        self.posText9.setObjectName("posText9")
        self.label_263 = QtWidgets.QLabel(self.groupBox_45)
        self.label_263.setGeometry(QtCore.QRect(290, 290, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_263.setFont(font)
        self.label_263.setAlignment(QtCore.Qt.AlignCenter)
        self.label_263.setWordWrap(False)
        self.label_263.setObjectName("label_263")
        self.posAlt8 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAlt8.setGeometry(QtCore.QRect(195, 290, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt8.setFont(font)
        self.posAlt8.setMouseTracking(False)
        self.posAlt8.setAcceptDrops(False)
        self.posAlt8.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt8.setReadOnly(False)
        self.posAlt8.setObjectName("posAlt8")
        self.posAz8 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAz8.setGeometry(QtCore.QRect(250, 290, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz8.setFont(font)
        self.posAz8.setMouseTracking(False)
        self.posAz8.setAcceptDrops(False)
        self.posAz8.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz8.setText("")
        self.posAz8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz8.setReadOnly(False)
        self.posAz8.setObjectName("posAz8")
        self.label_264 = QtWidgets.QLabel(self.groupBox_45)
        self.label_264.setGeometry(QtCore.QRect(230, 290, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_264.setFont(font)
        self.label_264.setAlignment(QtCore.Qt.AlignCenter)
        self.label_264.setWordWrap(False)
        self.label_264.setObjectName("label_264")
        self.posSave8 = QtWidgets.QPushButton(self.groupBox_45)
        self.posSave8.setGeometry(QtCore.QRect(310, 290, 46, 21))
        self.posSave8.setObjectName("posSave8")
        self.label_265 = QtWidgets.QLabel(self.groupBox_45)
        self.label_265.setGeometry(QtCore.QRect(290, 320, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_265.setFont(font)
        self.label_265.setAlignment(QtCore.Qt.AlignCenter)
        self.label_265.setWordWrap(False)
        self.label_265.setObjectName("label_265")
        self.posAlt9 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAlt9.setGeometry(QtCore.QRect(195, 320, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt9.setFont(font)
        self.posAlt9.setMouseTracking(False)
        self.posAlt9.setAcceptDrops(False)
        self.posAlt9.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt9.setReadOnly(False)
        self.posAlt9.setObjectName("posAlt9")
        self.posAz9 = QtWidgets.QLineEdit(self.groupBox_45)
        self.posAz9.setGeometry(QtCore.QRect(250, 320, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz9.setFont(font)
        self.posAz9.setMouseTracking(False)
        self.posAz9.setAcceptDrops(False)
        self.posAz9.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz9.setText("")
        self.posAz9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz9.setReadOnly(False)
        self.posAz9.setObjectName("posAz9")
        self.label_267 = QtWidgets.QLabel(self.groupBox_45)
        self.label_267.setGeometry(QtCore.QRect(230, 320, 16, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_267.setFont(font)
        self.label_267.setAlignment(QtCore.Qt.AlignCenter)
        self.label_267.setWordWrap(False)
        self.label_267.setObjectName("label_267")
        self.posSave9 = QtWidgets.QPushButton(self.groupBox_45)
        self.posSave9.setGeometry(QtCore.QRect(310, 320, 46, 21))
        self.posSave9.setObjectName("posSave9")
        self.groupBox_96 = QtWidgets.QGroupBox(self.ParkPos)
        self.groupBox_96.setGeometry(QtCore.QRect(400, 15, 366, 281))
        self.groupBox_96.setProperty("large", True)
        self.groupBox_96.setObjectName("groupBox_96")
        self.label_149 = QtWidgets.QLabel(self.groupBox_96)
        self.label_149.setGeometry(QtCore.QRect(15, 160, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_149.setFont(font)
        self.label_149.setObjectName("label_149")
        self.relayButtonText2 = QtWidgets.QLineEdit(self.groupBox_96)
        self.relayButtonText2.setGeometry(QtCore.QRect(55, 100, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText2.setFont(font)
        self.relayButtonText2.setMouseTracking(False)
        self.relayButtonText2.setAcceptDrops(False)
        self.relayButtonText2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText2.setReadOnly(False)
        self.relayButtonText2.setObjectName("relayButtonText2")
        self.relayButtonText5 = QtWidgets.QLineEdit(self.groupBox_96)
        self.relayButtonText5.setGeometry(QtCore.QRect(55, 190, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText5.setFont(font)
        self.relayButtonText5.setMouseTracking(False)
        self.relayButtonText5.setAcceptDrops(False)
        self.relayButtonText5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText5.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText5.setReadOnly(False)
        self.relayButtonText5.setObjectName("relayButtonText5")
        self.relayButtonText7 = QtWidgets.QLineEdit(self.groupBox_96)
        self.relayButtonText7.setGeometry(QtCore.QRect(55, 250, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText7.setFont(font)
        self.relayButtonText7.setMouseTracking(False)
        self.relayButtonText7.setAcceptDrops(False)
        self.relayButtonText7.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText7.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText7.setReadOnly(False)
        self.relayButtonText7.setObjectName("relayButtonText7")
        self.relayFun6 = QtWidgets.QComboBox(self.groupBox_96)
        self.relayFun6.setGeometry(QtCore.QRect(195, 220, 161, 21))
        self.relayFun6.setObjectName("relayFun6")
        self.relayFun3 = QtWidgets.QComboBox(self.groupBox_96)
        self.relayFun3.setGeometry(QtCore.QRect(195, 130, 161, 21))
        self.relayFun3.setObjectName("relayFun3")
        self.relayFun5 = QtWidgets.QComboBox(self.groupBox_96)
        self.relayFun5.setGeometry(QtCore.QRect(195, 190, 161, 21))
        self.relayFun5.setObjectName("relayFun5")
        self.label_220 = QtWidgets.QLabel(self.groupBox_96)
        self.label_220.setGeometry(QtCore.QRect(15, 130, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_220.setFont(font)
        self.label_220.setObjectName("label_220")
        self.label_129 = QtWidgets.QLabel(self.groupBox_96)
        self.label_129.setGeometry(QtCore.QRect(15, 100, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_129.setFont(font)
        self.label_129.setObjectName("label_129")
        self.label_211 = QtWidgets.QLabel(self.groupBox_96)
        self.label_211.setGeometry(QtCore.QRect(15, 70, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_211.setFont(font)
        self.label_211.setObjectName("label_211")
        self.relayButtonText4 = QtWidgets.QLineEdit(self.groupBox_96)
        self.relayButtonText4.setGeometry(QtCore.QRect(55, 160, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText4.setFont(font)
        self.relayButtonText4.setMouseTracking(False)
        self.relayButtonText4.setAcceptDrops(False)
        self.relayButtonText4.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText4.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText4.setReadOnly(False)
        self.relayButtonText4.setObjectName("relayButtonText4")
        self.relayButtonText1 = QtWidgets.QLineEdit(self.groupBox_96)
        self.relayButtonText1.setGeometry(QtCore.QRect(55, 70, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText1.setFont(font)
        self.relayButtonText1.setMouseTracking(False)
        self.relayButtonText1.setAcceptDrops(False)
        self.relayButtonText1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText1.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText1.setReadOnly(False)
        self.relayButtonText1.setObjectName("relayButtonText1")
        self.relayButtonText6 = QtWidgets.QLineEdit(self.groupBox_96)
        self.relayButtonText6.setGeometry(QtCore.QRect(55, 220, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText6.setFont(font)
        self.relayButtonText6.setMouseTracking(False)
        self.relayButtonText6.setAcceptDrops(False)
        self.relayButtonText6.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText6.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText6.setReadOnly(False)
        self.relayButtonText6.setObjectName("relayButtonText6")
        self.label_255 = QtWidgets.QLabel(self.groupBox_96)
        self.label_255.setGeometry(QtCore.QRect(15, 250, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_255.setFont(font)
        self.label_255.setObjectName("label_255")
        self.label_56 = QtWidgets.QLabel(self.groupBox_96)
        self.label_56.setGeometry(QtCore.QRect(10, 20, 51, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_56.setFont(font)
        self.label_56.setObjectName("label_56")
        self.label_199 = QtWidgets.QLabel(self.groupBox_96)
        self.label_199.setGeometry(QtCore.QRect(15, 220, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_199.setFont(font)
        self.label_199.setObjectName("label_199")
        self.relayFun1 = QtWidgets.QComboBox(self.groupBox_96)
        self.relayFun1.setGeometry(QtCore.QRect(195, 70, 161, 21))
        self.relayFun1.setObjectName("relayFun1")
        self.relayFun4 = QtWidgets.QComboBox(self.groupBox_96)
        self.relayFun4.setGeometry(QtCore.QRect(195, 160, 161, 21))
        self.relayFun4.setObjectName("relayFun4")
        self.label_198 = QtWidgets.QLabel(self.groupBox_96)
        self.label_198.setGeometry(QtCore.QRect(195, 20, 51, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_198.setFont(font)
        self.label_198.setObjectName("label_198")
        self.relayFun2 = QtWidgets.QComboBox(self.groupBox_96)
        self.relayFun2.setGeometry(QtCore.QRect(195, 100, 161, 21))
        self.relayFun2.setObjectName("relayFun2")
        self.label_43 = QtWidgets.QLabel(self.groupBox_96)
        self.label_43.setGeometry(QtCore.QRect(15, 40, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_43.setFont(font)
        self.label_43.setObjectName("label_43")
        self.label_45 = QtWidgets.QLabel(self.groupBox_96)
        self.label_45.setGeometry(QtCore.QRect(55, 20, 51, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_45.setFont(font)
        self.label_45.setObjectName("label_45")
        self.relayButtonText3 = QtWidgets.QLineEdit(self.groupBox_96)
        self.relayButtonText3.setGeometry(QtCore.QRect(55, 130, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText3.setFont(font)
        self.relayButtonText3.setMouseTracking(False)
        self.relayButtonText3.setAcceptDrops(False)
        self.relayButtonText3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText3.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText3.setReadOnly(False)
        self.relayButtonText3.setObjectName("relayButtonText3")
        self.relayButtonText0 = QtWidgets.QLineEdit(self.groupBox_96)
        self.relayButtonText0.setGeometry(QtCore.QRect(55, 40, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText0.setFont(font)
        self.relayButtonText0.setMouseTracking(False)
        self.relayButtonText0.setAcceptDrops(False)
        self.relayButtonText0.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText0.setReadOnly(False)
        self.relayButtonText0.setObjectName("relayButtonText0")
        self.relayFun0 = QtWidgets.QComboBox(self.groupBox_96)
        self.relayFun0.setGeometry(QtCore.QRect(195, 40, 161, 21))
        self.relayFun0.setObjectName("relayFun0")
        self.label_256 = QtWidgets.QLabel(self.groupBox_96)
        self.label_256.setGeometry(QtCore.QRect(15, 190, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_256.setFont(font)
        self.label_256.setObjectName("label_256")
        self.relayFun7 = QtWidgets.QComboBox(self.groupBox_96)
        self.relayFun7.setGeometry(QtCore.QRect(195, 250, 161, 21))
        self.relayFun7.setObjectName("relayFun7")
        self.settingsTabWidget.addTab(self.ParkPos, "")
        self.AudioMisc = QtWidgets.QWidget()
        self.AudioMisc.setObjectName("AudioMisc")
        self.line_84 = QtWidgets.QFrame(self.AudioMisc)
        self.line_84.setGeometry(QtCore.QRect(380, 10, 11, 371))
        self.line_84.setStyleSheet("")
        self.line_84.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_84.setLineWidth(2)
        self.line_84.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_84.setObjectName("line_84")
        self.LoglevelGroup = QtWidgets.QGroupBox(self.AudioMisc)
        self.LoglevelGroup.setGeometry(QtCore.QRect(5, 335, 366, 46))
        self.LoglevelGroup.setObjectName("LoglevelGroup")
        self.loglevelDebug = QtWidgets.QRadioButton(self.LoglevelGroup)
        self.loglevelDebug.setGeometry(QtCore.QRect(105, 20, 66, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.loglevelDebug.setFont(font)
        self.loglevelDebug.setChecked(False)
        self.loglevelDebug.setObjectName("loglevelDebug")
        self.loglevelStandard = QtWidgets.QRadioButton(self.LoglevelGroup)
        self.loglevelStandard.setGeometry(QtCore.QRect(10, 20, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.loglevelStandard.setFont(font)
        self.loglevelStandard.setChecked(False)
        self.loglevelStandard.setObjectName("loglevelStandard")
        self.loglevelDebugTrace = QtWidgets.QRadioButton(self.LoglevelGroup)
        self.loglevelDebugTrace.setGeometry(QtCore.QRect(190, 20, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.loglevelDebugTrace.setFont(font)
        self.loglevelDebugTrace.setChecked(True)
        self.loglevelDebugTrace.setObjectName("loglevelDebugTrace")
        self.CopyrightGroup = QtWidgets.QGroupBox(self.AudioMisc)
        self.CopyrightGroup.setGeometry(QtCore.QRect(400, 285, 366, 96))
        self.CopyrightGroup.setObjectName("CopyrightGroup")
        self.textEdit = QtWidgets.QTextEdit(self.CopyrightGroup)
        self.textEdit.setEnabled(True)
        self.textEdit.setGeometry(QtCore.QRect(5, 20, 356, 71))
        self.textEdit.setObjectName("textEdit")
        self.AudioGroup = QtWidgets.QGroupBox(self.AudioMisc)
        self.AudioGroup.setGeometry(QtCore.QRect(400, 15, 366, 176))
        self.AudioGroup.setProperty("large", True)
        self.AudioGroup.setObjectName("AudioGroup")
        self.soundMountAlert = QtWidgets.QComboBox(self.AudioGroup)
        self.soundMountAlert.setGeometry(QtCore.QRect(175, 70, 181, 21))
        self.soundMountAlert.setObjectName("soundMountAlert")
        self.label_217 = QtWidgets.QLabel(self.AudioGroup)
        self.label_217.setGeometry(QtCore.QRect(10, 95, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_217.setFont(font)
        self.label_217.setObjectName("label_217")
        self.soundDomeSlewFinished = QtWidgets.QComboBox(self.AudioGroup)
        self.soundDomeSlewFinished.setGeometry(QtCore.QRect(175, 45, 181, 21))
        self.soundDomeSlewFinished.setObjectName("soundDomeSlewFinished")
        self.label_214 = QtWidgets.QLabel(self.AudioGroup)
        self.label_214.setGeometry(QtCore.QRect(10, 45, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_214.setFont(font)
        self.label_214.setObjectName("label_214")
        self.label_212 = QtWidgets.QLabel(self.AudioGroup)
        self.label_212.setGeometry(QtCore.QRect(10, 20, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_212.setFont(font)
        self.label_212.setObjectName("label_212")
        self.soundModelingFinished = QtWidgets.QComboBox(self.AudioGroup)
        self.soundModelingFinished.setGeometry(QtCore.QRect(175, 95, 181, 21))
        self.soundModelingFinished.setObjectName("soundModelingFinished")
        self.label_215 = QtWidgets.QLabel(self.AudioGroup)
        self.label_215.setGeometry(QtCore.QRect(10, 70, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_215.setFont(font)
        self.label_215.setObjectName("label_215")
        self.soundMountSlewFinished = QtWidgets.QComboBox(self.AudioGroup)
        self.soundMountSlewFinished.setGeometry(QtCore.QRect(175, 20, 181, 21))
        self.soundMountSlewFinished.setObjectName("soundMountSlewFinished")
        self.label_219 = QtWidgets.QLabel(self.AudioGroup)
        self.label_219.setGeometry(QtCore.QRect(10, 120, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_219.setFont(font)
        self.label_219.setObjectName("label_219")
        self.soundImageSaved = QtWidgets.QComboBox(self.AudioGroup)
        self.soundImageSaved.setGeometry(QtCore.QRect(175, 120, 181, 21))
        self.soundImageSaved.setObjectName("soundImageSaved")
        self.label_223 = QtWidgets.QLabel(self.AudioGroup)
        self.label_223.setGeometry(QtCore.QRect(10, 145, 136, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_223.setFont(font)
        self.label_223.setObjectName("label_223")
        self.soundImageSolved = QtWidgets.QComboBox(self.AudioGroup)
        self.soundImageSolved.setGeometry(QtCore.QRect(175, 145, 181, 21))
        self.soundImageSolved.setObjectName("soundImageSolved")
        self.InternetOnlineGroup = QtWidgets.QGroupBox(self.AudioMisc)
        self.InternetOnlineGroup.setGeometry(QtCore.QRect(5, 15, 366, 51))
        self.InternetOnlineGroup.setProperty("large", True)
        self.InternetOnlineGroup.setObjectName("InternetOnlineGroup")
        self.isOnline = QtWidgets.QCheckBox(self.InternetOnlineGroup)
        self.isOnline.setGeometry(QtCore.QRect(10, 20, 331, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.isOnline.setFont(font)
        self.isOnline.setChecked(False)
        self.isOnline.setObjectName("isOnline")
        self.UpdaterGroup = QtWidgets.QGroupBox(self.AudioMisc)
        self.UpdaterGroup.setGeometry(QtCore.QRect(5, 85, 366, 96))
        self.UpdaterGroup.setProperty("large", True)
        self.UpdaterGroup.setObjectName("UpdaterGroup")
        self.installVersion = QtWidgets.QPushButton(self.UpdaterGroup)
        self.installVersion.setEnabled(False)
        self.installVersion.setGeometry(QtCore.QRect(280, 20, 75, 46))
        self.installVersion.setObjectName("installVersion")
        self.label_23 = QtWidgets.QLabel(self.UpdaterGroup)
        self.label_23.setGeometry(QtCore.QRect(10, 20, 121, 21))
        self.label_23.setObjectName("label_23")
        self.label_38 = QtWidgets.QLabel(self.UpdaterGroup)
        self.label_38.setGeometry(QtCore.QRect(10, 45, 121, 21))
        self.label_38.setObjectName("label_38")
        self.versionBeta = QtWidgets.QRadioButton(self.UpdaterGroup)
        self.versionBeta.setGeometry(QtCore.QRect(140, 70, 141, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.versionBeta.setFont(font)
        self.versionBeta.setChecked(False)
        self.versionBeta.setObjectName("versionBeta")
        self.versionRelease = QtWidgets.QRadioButton(self.UpdaterGroup)
        self.versionRelease.setGeometry(QtCore.QRect(10, 70, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.versionRelease.setFont(font)
        self.versionRelease.setChecked(True)
        self.versionRelease.setObjectName("versionRelease")
        self.versionActual = QtWidgets.QLineEdit(self.UpdaterGroup)
        self.versionActual.setEnabled(False)
        self.versionActual.setGeometry(QtCore.QRect(135, 20, 116, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.versionActual.setFont(font)
        self.versionActual.setMouseTracking(False)
        self.versionActual.setFocusPolicy(QtCore.Qt.NoFocus)
        self.versionActual.setAcceptDrops(False)
        self.versionActual.setToolTip("")
        self.versionActual.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.versionActual.setMaxLength(20)
        self.versionActual.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.versionActual.setReadOnly(True)
        self.versionActual.setObjectName("versionActual")
        self.versionAvailable = QtWidgets.QLineEdit(self.UpdaterGroup)
        self.versionAvailable.setGeometry(QtCore.QRect(135, 45, 116, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.versionAvailable.setFont(font)
        self.versionAvailable.setMouseTracking(False)
        self.versionAvailable.setAcceptDrops(False)
        self.versionAvailable.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.versionAvailable.setInputMask("")
        self.versionAvailable.setText("")
        self.versionAvailable.setMaxLength(20)
        self.versionAvailable.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.versionAvailable.setReadOnly(False)
        self.versionAvailable.setObjectName("versionAvailable")
        self.settingsTabWidget.addTab(self.AudioMisc, "")
        self.mainTabWidget.addTab(self.Settings, "")
        self.Analyse = QtWidgets.QWidget()
        self.Analyse.setObjectName("Analyse")
        self.cancelAnalyse = QtWidgets.QPushButton(self.Analyse)
        self.cancelAnalyse.setEnabled(False)
        self.cancelAnalyse.setGeometry(QtCore.QRect(10, 390, 366, 31))
        self.cancelAnalyse.setObjectName("cancelAnalyse")
        self.line_52 = QtWidgets.QFrame(self.Analyse)
        self.line_52.setGeometry(QtCore.QRect(385, 10, 11, 416))
        self.line_52.setStyleSheet("")
        self.line_52.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_52.setLineWidth(2)
        self.line_52.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_52.setObjectName("line_52")
        self.groupBox_14 = QtWidgets.QGroupBox(self.Analyse)
        self.groupBox_14.setGeometry(QtCore.QRect(10, 300, 366, 76))
        self.groupBox_14.setProperty("large", True)
        self.groupBox_14.setObjectName("groupBox_14")
        self.le_analyseFileName = QtWidgets.QLineEdit(self.groupBox_14)
        self.le_analyseFileName.setEnabled(False)
        self.le_analyseFileName.setGeometry(QtCore.QRect(5, 20, 356, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.le_analyseFileName.setFont(font)
        self.le_analyseFileName.setMouseTracking(False)
        self.le_analyseFileName.setAcceptDrops(False)
        self.le_analyseFileName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_analyseFileName.setText("")
        self.le_analyseFileName.setReadOnly(True)
        self.le_analyseFileName.setObjectName("le_analyseFileName")
        self.loadAnalyseData = QtWidgets.QPushButton(self.groupBox_14)
        self.loadAnalyseData.setGeometry(QtCore.QRect(5, 45, 71, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.loadAnalyseData.setFont(font)
        self.loadAnalyseData.setStyleSheet("")
        self.loadAnalyseData.setObjectName("loadAnalyseData")
        self.groupBox_15 = QtWidgets.QGroupBox(self.Analyse)
        self.groupBox_15.setGeometry(QtCore.QRect(10, 25, 366, 101))
        self.groupBox_15.setProperty("large", True)
        self.groupBox_15.setObjectName("groupBox_15")
        self.label_177 = QtWidgets.QLabel(self.groupBox_15)
        self.label_177.setGeometry(QtCore.QRect(210, 45, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_177.setFont(font)
        self.label_177.setObjectName("label_177")
        self.delayTimeFlexure = QtWidgets.QDoubleSpinBox(self.groupBox_15)
        self.delayTimeFlexure.setGeometry(QtCore.QRect(280, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.delayTimeFlexure.setFont(font)
        self.delayTimeFlexure.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.delayTimeFlexure.setDecimals(0)
        self.delayTimeFlexure.setMinimum(1.0)
        self.delayTimeFlexure.setMaximum(300.0)
        self.delayTimeFlexure.setSingleStep(1.0)
        self.delayTimeFlexure.setProperty("value", 30.0)
        self.delayTimeFlexure.setObjectName("delayTimeFlexure")
        self.label_111 = QtWidgets.QLabel(self.groupBox_15)
        self.label_111.setGeometry(QtCore.QRect(330, 20, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_111.setFont(font)
        self.label_111.setAlignment(QtCore.Qt.AlignCenter)
        self.label_111.setWordWrap(False)
        self.label_111.setObjectName("label_111")
        self.label_174 = QtWidgets.QLabel(self.groupBox_15)
        self.label_174.setGeometry(QtCore.QRect(10, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_174.setFont(font)
        self.label_174.setObjectName("label_174")
        self.label_112 = QtWidgets.QLabel(self.groupBox_15)
        self.label_112.setGeometry(QtCore.QRect(165, 20, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_112.setFont(font)
        self.label_112.setAlignment(QtCore.Qt.AlignCenter)
        self.label_112.setWordWrap(False)
        self.label_112.setObjectName("label_112")
        self.numberRunsTimeChange = QtWidgets.QDoubleSpinBox(self.groupBox_15)
        self.numberRunsTimeChange.setGeometry(QtCore.QRect(115, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberRunsTimeChange.setFont(font)
        self.numberRunsTimeChange.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberRunsTimeChange.setDecimals(0)
        self.numberRunsTimeChange.setMinimum(10.0)
        self.numberRunsTimeChange.setMaximum(300.0)
        self.numberRunsTimeChange.setSingleStep(1.0)
        self.numberRunsTimeChange.setProperty("value", 30.0)
        self.numberRunsTimeChange.setObjectName("numberRunsTimeChange")
        self.altitudeTimeChange = QtWidgets.QDoubleSpinBox(self.groupBox_15)
        self.altitudeTimeChange.setGeometry(QtCore.QRect(115, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeTimeChange.setFont(font)
        self.altitudeTimeChange.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeTimeChange.setDecimals(0)
        self.altitudeTimeChange.setMinimum(0.0)
        self.altitudeTimeChange.setMaximum(90.0)
        self.altitudeTimeChange.setSingleStep(1.0)
        self.altitudeTimeChange.setProperty("value", 45.0)
        self.altitudeTimeChange.setObjectName("altitudeTimeChange")
        self.label_176 = QtWidgets.QLabel(self.groupBox_15)
        self.label_176.setGeometry(QtCore.QRect(10, 45, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_176.setFont(font)
        self.label_176.setObjectName("label_176")
        self.azimuthTimeChange = QtWidgets.QDoubleSpinBox(self.groupBox_15)
        self.azimuthTimeChange.setGeometry(QtCore.QRect(280, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthTimeChange.setFont(font)
        self.azimuthTimeChange.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthTimeChange.setDecimals(0)
        self.azimuthTimeChange.setMaximum(359.0)
        self.azimuthTimeChange.setSingleStep(1.0)
        self.azimuthTimeChange.setProperty("value", 0.0)
        self.azimuthTimeChange.setObjectName("azimuthTimeChange")
        self.label_115 = QtWidgets.QLabel(self.groupBox_15)
        self.label_115.setGeometry(QtCore.QRect(330, 45, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_115.setFont(font)
        self.label_115.setAlignment(QtCore.Qt.AlignCenter)
        self.label_115.setWordWrap(False)
        self.label_115.setObjectName("label_115")
        self.label_175 = QtWidgets.QLabel(self.groupBox_15)
        self.label_175.setGeometry(QtCore.QRect(210, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_175.setFont(font)
        self.label_175.setObjectName("label_175")
        self.runFlexure = QtWidgets.QPushButton(self.groupBox_15)
        self.runFlexure.setEnabled(True)
        self.runFlexure.setGeometry(QtCore.QRect(5, 70, 356, 26))
        self.runFlexure.setObjectName("runFlexure")
        self.groupBox_16 = QtWidgets.QGroupBox(self.Analyse)
        self.groupBox_16.setGeometry(QtCore.QRect(10, 150, 366, 126))
        self.groupBox_16.setProperty("large", True)
        self.groupBox_16.setObjectName("groupBox_16")
        self.label_182 = QtWidgets.QLabel(self.groupBox_16)
        self.label_182.setGeometry(QtCore.QRect(220, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_182.setFont(font)
        self.label_182.setObjectName("label_182")
        self.label_183 = QtWidgets.QLabel(self.groupBox_16)
        self.label_183.setGeometry(QtCore.QRect(10, 70, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_183.setFont(font)
        self.label_183.setObjectName("label_183")
        self.delayTimeHysterese = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.delayTimeHysterese.setGeometry(QtCore.QRect(290, 70, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.delayTimeHysterese.setFont(font)
        self.delayTimeHysterese.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.delayTimeHysterese.setDecimals(0)
        self.delayTimeHysterese.setMinimum(1.0)
        self.delayTimeHysterese.setMaximum(300.0)
        self.delayTimeHysterese.setSingleStep(1.0)
        self.delayTimeHysterese.setProperty("value", 30.0)
        self.delayTimeHysterese.setObjectName("delayTimeHysterese")
        self.altitudeHysterese2 = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.altitudeHysterese2.setGeometry(QtCore.QRect(130, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeHysterese2.setFont(font)
        self.altitudeHysterese2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeHysterese2.setDecimals(0)
        self.altitudeHysterese2.setMinimum(0.0)
        self.altitudeHysterese2.setMaximum(90.0)
        self.altitudeHysterese2.setSingleStep(1.0)
        self.altitudeHysterese2.setProperty("value", 45.0)
        self.altitudeHysterese2.setObjectName("altitudeHysterese2")
        self.label_119 = QtWidgets.QLabel(self.groupBox_16)
        self.label_119.setGeometry(QtCore.QRect(340, 45, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_119.setFont(font)
        self.label_119.setAlignment(QtCore.Qt.AlignCenter)
        self.label_119.setWordWrap(False)
        self.label_119.setObjectName("label_119")
        self.altitudeHysterese1 = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.altitudeHysterese1.setGeometry(QtCore.QRect(130, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeHysterese1.setFont(font)
        self.altitudeHysterese1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeHysterese1.setDecimals(0)
        self.altitudeHysterese1.setMinimum(0.0)
        self.altitudeHysterese1.setMaximum(90.0)
        self.altitudeHysterese1.setSingleStep(1.0)
        self.altitudeHysterese1.setProperty("value", 45.0)
        self.altitudeHysterese1.setObjectName("altitudeHysterese1")
        self.label_120 = QtWidgets.QLabel(self.groupBox_16)
        self.label_120.setGeometry(QtCore.QRect(180, 20, 21, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_120.setFont(font)
        self.label_120.setAlignment(QtCore.Qt.AlignCenter)
        self.label_120.setWordWrap(False)
        self.label_120.setObjectName("label_120")
        self.label_185 = QtWidgets.QLabel(self.groupBox_16)
        self.label_185.setGeometry(QtCore.QRect(10, 20, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_185.setFont(font)
        self.label_185.setObjectName("label_185")
        self.numberRunsHysterese = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.numberRunsHysterese.setGeometry(QtCore.QRect(130, 70, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberRunsHysterese.setFont(font)
        self.numberRunsHysterese.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberRunsHysterese.setDecimals(0)
        self.numberRunsHysterese.setMinimum(1.0)
        self.numberRunsHysterese.setMaximum(50.0)
        self.numberRunsHysterese.setSingleStep(1.0)
        self.numberRunsHysterese.setProperty("value", 5.0)
        self.numberRunsHysterese.setObjectName("numberRunsHysterese")
        self.label_187 = QtWidgets.QLabel(self.groupBox_16)
        self.label_187.setGeometry(QtCore.QRect(10, 45, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_187.setFont(font)
        self.label_187.setObjectName("label_187")
        self.label_184 = QtWidgets.QLabel(self.groupBox_16)
        self.label_184.setGeometry(QtCore.QRect(220, 70, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_184.setFont(font)
        self.label_184.setObjectName("label_184")
        self.azimuthHysterese2 = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.azimuthHysterese2.setGeometry(QtCore.QRect(290, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthHysterese2.setFont(font)
        self.azimuthHysterese2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthHysterese2.setDecimals(0)
        self.azimuthHysterese2.setMaximum(359.0)
        self.azimuthHysterese2.setSingleStep(1.0)
        self.azimuthHysterese2.setProperty("value", 0.0)
        self.azimuthHysterese2.setObjectName("azimuthHysterese2")
        self.label_114 = QtWidgets.QLabel(self.groupBox_16)
        self.label_114.setGeometry(QtCore.QRect(180, 55, 21, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_114.setFont(font)
        self.label_114.setAlignment(QtCore.Qt.AlignCenter)
        self.label_114.setWordWrap(False)
        self.label_114.setObjectName("label_114")
        self.label_117 = QtWidgets.QLabel(self.groupBox_16)
        self.label_117.setGeometry(QtCore.QRect(340, 70, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_117.setFont(font)
        self.label_117.setAlignment(QtCore.Qt.AlignCenter)
        self.label_117.setWordWrap(False)
        self.label_117.setObjectName("label_117")
        self.label_186 = QtWidgets.QLabel(self.groupBox_16)
        self.label_186.setGeometry(QtCore.QRect(220, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_186.setFont(font)
        self.label_186.setObjectName("label_186")
        self.azimuthHysterese1 = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.azimuthHysterese1.setGeometry(QtCore.QRect(290, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthHysterese1.setFont(font)
        self.azimuthHysterese1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthHysterese1.setDecimals(0)
        self.azimuthHysterese1.setMaximum(359.0)
        self.azimuthHysterese1.setSingleStep(1.0)
        self.azimuthHysterese1.setProperty("value", 0.0)
        self.azimuthHysterese1.setObjectName("azimuthHysterese1")
        self.label_121 = QtWidgets.QLabel(self.groupBox_16)
        self.label_121.setGeometry(QtCore.QRect(340, 20, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_121.setFont(font)
        self.label_121.setAlignment(QtCore.Qt.AlignCenter)
        self.label_121.setWordWrap(False)
        self.label_121.setObjectName("label_121")
        self.runHysteresis = QtWidgets.QPushButton(self.groupBox_16)
        self.runHysteresis.setEnabled(True)
        self.runHysteresis.setGeometry(QtCore.QRect(5, 95, 356, 26))
        self.runHysteresis.setObjectName("runHysteresis")
        self.line_52.raise_()
        self.cancelAnalyse.raise_()
        self.groupBox_14.raise_()
        self.groupBox_15.raise_()
        self.groupBox_16.raise_()
        self.mainTabWidget.addTab(self.Analyse, "")
        self.status = QtWidgets.QGroupBox(MainWindow)
        self.status.setGeometry(QtCore.QRect(5, 5, 445, 106))
        self.status.setMinimumSize(QtCore.QSize(445, 105))
        self.status.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.status.setObjectName("status")
        self.astrometryText = QtWidgets.QLineEdit(self.status)
        self.astrometryText.setEnabled(False)
        self.astrometryText.setGeometry(QtCore.QRect(355, 15, 86, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.astrometryText.sizePolicy().hasHeightForWidth())
        self.astrometryText.setSizePolicy(sizePolicy)
        self.astrometryText.setMinimumSize(QtCore.QSize(0, 25))
        self.astrometryText.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.astrometryText.setFont(font)
        self.astrometryText.setMouseTracking(False)
        self.astrometryText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.astrometryText.setAcceptDrops(False)
        self.astrometryText.setText("")
        self.astrometryText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.astrometryText.setReadOnly(True)
        self.astrometryText.setObjectName("astrometryText")
        self.statusText = QtWidgets.QLineEdit(self.status)
        self.statusText.setEnabled(False)
        self.statusText.setGeometry(QtCore.QRect(5, 15, 166, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.statusText.sizePolicy().hasHeightForWidth())
        self.statusText.setSizePolicy(sizePolicy)
        self.statusText.setMinimumSize(QtCore.QSize(0, 25))
        self.statusText.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.statusText.setFont(font)
        self.statusText.setMouseTracking(False)
        self.statusText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusText.setAcceptDrops(False)
        self.statusText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.statusText.setReadOnly(True)
        self.statusText.setObjectName("statusText")
        self.environConnected = QtWidgets.QPushButton(self.status)
        self.environConnected.setEnabled(False)
        self.environConnected.setGeometry(QtCore.QRect(90, 45, 81, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.environConnected.sizePolicy().hasHeightForWidth())
        self.environConnected.setSizePolicy(sizePolicy)
        self.environConnected.setMinimumSize(QtCore.QSize(0, 25))
        self.environConnected.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.environConnected.setFont(font)
        self.environConnected.setObjectName("environConnected")
        self.domeConnected = QtWidgets.QPushButton(self.status)
        self.domeConnected.setEnabled(False)
        self.domeConnected.setGeometry(QtCore.QRect(175, 45, 86, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.domeConnected.sizePolicy().hasHeightForWidth())
        self.domeConnected.setSizePolicy(sizePolicy)
        self.domeConnected.setMinimumSize(QtCore.QSize(0, 25))
        self.domeConnected.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.domeConnected.setFont(font)
        self.domeConnected.setObjectName("domeConnected")
        self.openImageW = QtWidgets.QPushButton(self.status)
        self.openImageW.setGeometry(QtCore.QRect(220, 75, 71, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.openImageW.sizePolicy().hasHeightForWidth())
        self.openImageW.setSizePolicy(sizePolicy)
        self.openImageW.setMinimumSize(QtCore.QSize(0, 25))
        self.openImageW.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.openImageW.setFont(font)
        self.openImageW.setStyleSheet("")
        self.openImageW.setObjectName("openImageW")
        self.cameraConnected = QtWidgets.QPushButton(self.status)
        self.cameraConnected.setEnabled(False)
        self.cameraConnected.setGeometry(QtCore.QRect(265, 45, 86, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cameraConnected.sizePolicy().hasHeightForWidth())
        self.cameraConnected.setSizePolicy(sizePolicy)
        self.cameraConnected.setMinimumSize(QtCore.QSize(0, 25))
        self.cameraConnected.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.cameraConnected.setFont(font)
        self.cameraConnected.setObjectName("cameraConnected")
        self.openSatelliteW = QtWidgets.QPushButton(self.status)
        self.openSatelliteW.setGeometry(QtCore.QRect(295, 75, 71, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.openSatelliteW.sizePolicy().hasHeightForWidth())
        self.openSatelliteW.setSizePolicy(sizePolicy)
        self.openSatelliteW.setMinimumSize(QtCore.QSize(0, 25))
        self.openSatelliteW.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.openSatelliteW.setFont(font)
        self.openSatelliteW.setStyleSheet("")
        self.openSatelliteW.setObjectName("openSatelliteW")
        self.openMeasureW = QtWidgets.QPushButton(self.status)
        self.openMeasureW.setGeometry(QtCore.QRect(75, 75, 66, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.openMeasureW.sizePolicy().hasHeightForWidth())
        self.openMeasureW.setSizePolicy(sizePolicy)
        self.openMeasureW.setMinimumSize(QtCore.QSize(0, 25))
        self.openMeasureW.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.openMeasureW.setFont(font)
        self.openMeasureW.setStyleSheet("")
        self.openMeasureW.setObjectName("openMeasureW")
        self.astrometryConnected = QtWidgets.QPushButton(self.status)
        self.astrometryConnected.setEnabled(False)
        self.astrometryConnected.setGeometry(QtCore.QRect(355, 45, 86, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.astrometryConnected.sizePolicy().hasHeightForWidth())
        self.astrometryConnected.setSizePolicy(sizePolicy)
        self.astrometryConnected.setMinimumSize(QtCore.QSize(0, 25))
        self.astrometryConnected.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astrometryConnected.setFont(font)
        self.astrometryConnected.setObjectName("astrometryConnected")
        self.cameraText = QtWidgets.QLineEdit(self.status)
        self.cameraText.setEnabled(False)
        self.cameraText.setGeometry(QtCore.QRect(265, 15, 86, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cameraText.sizePolicy().hasHeightForWidth())
        self.cameraText.setSizePolicy(sizePolicy)
        self.cameraText.setMinimumSize(QtCore.QSize(0, 25))
        self.cameraText.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.cameraText.setFont(font)
        self.cameraText.setMouseTracking(False)
        self.cameraText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.cameraText.setAcceptDrops(False)
        self.cameraText.setText("")
        self.cameraText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.cameraText.setReadOnly(True)
        self.cameraText.setObjectName("cameraText")
        self.mountConnected = QtWidgets.QPushButton(self.status)
        self.mountConnected.setEnabled(True)
        self.mountConnected.setGeometry(QtCore.QRect(5, 45, 81, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mountConnected.sizePolicy().hasHeightForWidth())
        self.mountConnected.setSizePolicy(sizePolicy)
        self.mountConnected.setMinimumSize(QtCore.QSize(0, 25))
        self.mountConnected.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mountConnected.setFont(font)
        self.mountConnected.setFlat(False)
        self.mountConnected.setObjectName("mountConnected")
        self.openHemisphereW = QtWidgets.QPushButton(self.status)
        self.openHemisphereW.setGeometry(QtCore.QRect(145, 75, 71, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.openHemisphereW.sizePolicy().hasHeightForWidth())
        self.openHemisphereW.setSizePolicy(sizePolicy)
        self.openHemisphereW.setMinimumSize(QtCore.QSize(0, 25))
        self.openHemisphereW.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.openHemisphereW.setFont(font)
        self.openHemisphereW.setStyleSheet("")
        self.openHemisphereW.setObjectName("openHemisphereW")
        self.openMessageW = QtWidgets.QPushButton(self.status)
        self.openMessageW.setGeometry(QtCore.QRect(5, 75, 66, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.openMessageW.sizePolicy().hasHeightForWidth())
        self.openMessageW.setSizePolicy(sizePolicy)
        self.openMessageW.setMinimumSize(QtCore.QSize(0, 25))
        self.openMessageW.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.openMessageW.setFont(font)
        self.openMessageW.setStyleSheet("")
        self.openMessageW.setObjectName("openMessageW")
        self.domeText = QtWidgets.QLineEdit(self.status)
        self.domeText.setEnabled(False)
        self.domeText.setGeometry(QtCore.QRect(175, 15, 86, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.domeText.sizePolicy().hasHeightForWidth())
        self.domeText.setSizePolicy(sizePolicy)
        self.domeText.setMinimumSize(QtCore.QSize(0, 25))
        self.domeText.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.domeText.setFont(font)
        self.domeText.setMouseTracking(False)
        self.domeText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.domeText.setAcceptDrops(False)
        self.domeText.setText("")
        self.domeText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.domeText.setReadOnly(True)
        self.domeText.setObjectName("domeText")
        self.openKeypadW = QtWidgets.QPushButton(self.status)
        self.openKeypadW.setGeometry(QtCore.QRect(370, 75, 71, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.openKeypadW.sizePolicy().hasHeightForWidth())
        self.openKeypadW.setSizePolicy(sizePolicy)
        self.openKeypadW.setMinimumSize(QtCore.QSize(0, 25))
        self.openKeypadW.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.openKeypadW.setFont(font)
        self.openKeypadW.setStyleSheet("")
        self.openKeypadW.setObjectName("openKeypadW")
        self.statusOnline = QtWidgets.QGroupBox(MainWindow)
        self.statusOnline.setGeometry(QtCore.QRect(455, 5, 140, 66))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.statusOnline.sizePolicy().hasHeightForWidth())
        self.statusOnline.setSizePolicy(sizePolicy)
        self.statusOnline.setMinimumSize(QtCore.QSize(140, 0))
        self.statusOnline.setMaximumSize(QtCore.QSize(140, 16777215))
        self.statusOnline.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.statusOnline.setObjectName("statusOnline")
        self.timeUTC = QtWidgets.QLineEdit(self.statusOnline)
        self.timeUTC.setEnabled(False)
        self.timeUTC.setGeometry(QtCore.QRect(5, 45, 131, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeUTC.sizePolicy().hasHeightForWidth())
        self.timeUTC.setSizePolicy(sizePolicy)
        self.timeUTC.setMinimumSize(QtCore.QSize(100, 20))
        self.timeUTC.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeUTC.setFont(font)
        self.timeUTC.setMouseTracking(False)
        self.timeUTC.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeUTC.setAcceptDrops(False)
        self.timeUTC.setAlignment(QtCore.Qt.AlignCenter)
        self.timeUTC.setReadOnly(True)
        self.timeUTC.setObjectName("timeUTC")
        self.timeComputer = QtWidgets.QLineEdit(self.statusOnline)
        self.timeComputer.setEnabled(False)
        self.timeComputer.setGeometry(QtCore.QRect(5, 15, 131, 31))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeComputer.sizePolicy().hasHeightForWidth())
        self.timeComputer.setSizePolicy(sizePolicy)
        self.timeComputer.setMinimumSize(QtCore.QSize(0, 0))
        self.timeComputer.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(28)
        font.setBold(False)
        font.setWeight(50)
        self.timeComputer.setFont(font)
        self.timeComputer.setMouseTracking(False)
        self.timeComputer.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeComputer.setAcceptDrops(False)
        self.timeComputer.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeComputer.setStyleSheet("")
        self.timeComputer.setAlignment(QtCore.Qt.AlignCenter)
        self.timeComputer.setReadOnly(True)
        self.timeComputer.setProperty("large", True)
        self.timeComputer.setObjectName("timeComputer")
        self.main = QtWidgets.QGroupBox(MainWindow)
        self.main.setGeometry(QtCore.QRect(600, 5, 195, 106))
        self.main.setMinimumSize(QtCore.QSize(195, 105))
        self.main.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.main.setObjectName("main")
        self.mountOn = QtWidgets.QPushButton(self.main)
        self.mountOn.setGeometry(QtCore.QRect(6, 75, 90, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mountOn.sizePolicy().hasHeightForWidth())
        self.mountOn.setSizePolicy(sizePolicy)
        self.mountOn.setMinimumSize(QtCore.QSize(90, 25))
        self.mountOn.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mountOn.setFont(font)
        self.mountOn.setObjectName("mountOn")
        self.saveConfig = QtWidgets.QPushButton(self.main)
        self.saveConfig.setGeometry(QtCore.QRect(6, 45, 90, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.saveConfig.sizePolicy().hasHeightForWidth())
        self.saveConfig.setSizePolicy(sizePolicy)
        self.saveConfig.setMinimumSize(QtCore.QSize(90, 25))
        self.saveConfig.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.saveConfig.setFont(font)
        self.saveConfig.setStyleSheet("")
        self.saveConfig.setObjectName("saveConfig")
        self.loadFrom = QtWidgets.QPushButton(self.main)
        self.loadFrom.setGeometry(QtCore.QRect(6, 15, 90, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loadFrom.sizePolicy().hasHeightForWidth())
        self.loadFrom.setSizePolicy(sizePolicy)
        self.loadFrom.setMinimumSize(QtCore.QSize(90, 25))
        self.loadFrom.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.loadFrom.setFont(font)
        self.loadFrom.setStyleSheet("")
        self.loadFrom.setObjectName("loadFrom")
        self.saveConfigQuit = QtWidgets.QPushButton(self.main)
        self.saveConfigQuit.setGeometry(QtCore.QRect(100, 15, 90, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.saveConfigQuit.sizePolicy().hasHeightForWidth())
        self.saveConfigQuit.setSizePolicy(sizePolicy)
        self.saveConfigQuit.setMinimumSize(QtCore.QSize(90, 25))
        self.saveConfigQuit.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.saveConfigQuit.setFont(font)
        self.saveConfigQuit.setStyleSheet("")
        self.saveConfigQuit.setObjectName("saveConfigQuit")
        self.saveConfigAs = QtWidgets.QPushButton(self.main)
        self.saveConfigAs.setGeometry(QtCore.QRect(100, 45, 90, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.saveConfigAs.sizePolicy().hasHeightForWidth())
        self.saveConfigAs.setSizePolicy(sizePolicy)
        self.saveConfigAs.setMinimumSize(QtCore.QSize(90, 25))
        self.saveConfigAs.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.saveConfigAs.setFont(font)
        self.saveConfigAs.setStyleSheet("")
        self.saveConfigAs.setObjectName("saveConfigAs")
        self.mountOff = QtWidgets.QPushButton(self.main)
        self.mountOff.setGeometry(QtCore.QRect(100, 75, 90, 25))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mountOff.sizePolicy().hasHeightForWidth())
        self.mountOff.setSizePolicy(sizePolicy)
        self.mountOff.setMinimumSize(QtCore.QSize(90, 25))
        self.mountOff.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mountOff.setFont(font)
        self.mountOff.setObjectName("mountOff")
        self.statusOnline_2 = QtWidgets.QGroupBox(MainWindow)
        self.statusOnline_2.setGeometry(QtCore.QRect(455, 75, 140, 36))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.statusOnline_2.sizePolicy().hasHeightForWidth())
        self.statusOnline_2.setSizePolicy(sizePolicy)
        self.statusOnline_2.setMinimumSize(QtCore.QSize(140, 0))
        self.statusOnline_2.setMaximumSize(QtCore.QSize(140, 16777215))
        self.statusOnline_2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.statusOnline_2.setObjectName("statusOnline_2")
        self.profile = QtWidgets.QLineEdit(self.statusOnline_2)
        self.profile.setEnabled(False)
        self.profile.setGeometry(QtCore.QRect(3, 13, 131, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.profile.sizePolicy().hasHeightForWidth())
        self.profile.setSizePolicy(sizePolicy)
        self.profile.setMinimumSize(QtCore.QSize(80, 21))
        self.profile.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.profile.setFont(font)
        self.profile.setMouseTracking(False)
        self.profile.setFocusPolicy(QtCore.Qt.NoFocus)
        self.profile.setAcceptDrops(False)
        self.profile.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.profile.setReadOnly(True)
        self.profile.setObjectName("profile")

        self.retranslateUi(MainWindow)
        self.mainTabWidget.setCurrentIndex(0)
        self.tabWidget.setCurrentIndex(0)
        self.tabMoving.setCurrentIndex(0)
        self.settingsTabWidget.setCurrentIndex(0)
        self.tabDomeExplain.setCurrentIndex(7)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.openMessageW, self.openMeasureW)
        MainWindow.setTabOrder(self.openMeasureW, self.openHemisphereW)
        MainWindow.setTabOrder(self.openHemisphereW, self.openImageW)
        MainWindow.setTabOrder(self.openImageW, self.openSatelliteW)
        MainWindow.setTabOrder(self.openSatelliteW, self.openKeypadW)
        MainWindow.setTabOrder(self.openKeypadW, self.mountConnected)
        MainWindow.setTabOrder(self.mountConnected, self.environConnected)
        MainWindow.setTabOrder(self.environConnected, self.domeConnected)
        MainWindow.setTabOrder(self.domeConnected, self.cameraConnected)
        MainWindow.setTabOrder(self.cameraConnected, self.astrometryConnected)
        MainWindow.setTabOrder(self.astrometryConnected, self.loadFrom)
        MainWindow.setTabOrder(self.loadFrom, self.saveConfig)
        MainWindow.setTabOrder(self.saveConfig, self.mountOn)
        MainWindow.setTabOrder(self.mountOn, self.saveConfigQuit)
        MainWindow.setTabOrder(self.saveConfigQuit, self.saveConfigAs)
        MainWindow.setTabOrder(self.saveConfigAs, self.mountOff)
        MainWindow.setTabOrder(self.mountOff, self.mainTabWidget)
        MainWindow.setTabOrder(self.mainTabWidget, self.checkJ2000)
        MainWindow.setTabOrder(self.checkJ2000, self.checkJNow)
        MainWindow.setTabOrder(self.checkJNow, self.tracking)
        MainWindow.setTabOrder(self.tracking, self.flipMount)
        MainWindow.setTabOrder(self.flipMount, self.setLunarTracking)
        MainWindow.setTabOrder(self.setLunarTracking, self.setSolarTracking)
        MainWindow.setTabOrder(self.setSolarTracking, self.setSiderealTracking)
        MainWindow.setTabOrder(self.setSiderealTracking, self.park)
        MainWindow.setTabOrder(self.park, self.stop)
        MainWindow.setTabOrder(self.stop, self.mountTypeConnection)
        MainWindow.setTabOrder(self.mountTypeConnection, self.onlineWeatherGroup)
        MainWindow.setTabOrder(self.onlineWeatherGroup, self.sensorWeatherGroup)
        MainWindow.setTabOrder(self.sensorWeatherGroup, self.directWeatherGroup)
        MainWindow.setTabOrder(self.directWeatherGroup, self.checkRefracNone)
        MainWindow.setTabOrder(self.checkRefracNone, self.posAlt7)
        MainWindow.setTabOrder(self.posAlt7, self.posAz7)
        MainWindow.setTabOrder(self.posAz7, self.checkRefracCont)
        MainWindow.setTabOrder(self.checkRefracCont, self.checkRefracNoTrack)
        MainWindow.setTabOrder(self.checkRefracNoTrack, self.setRefractionManual)
        MainWindow.setTabOrder(self.setRefractionManual, self.twilightEvents)
        MainWindow.setTabOrder(self.twilightEvents, self.buildPFileName)
        MainWindow.setTabOrder(self.buildPFileName, self.genBuildFile)
        MainWindow.setTabOrder(self.genBuildFile, self.loadBuildPoints)
        MainWindow.setTabOrder(self.loadBuildPoints, self.saveBuildPoints)
        MainWindow.setTabOrder(self.saveBuildPoints, self.saveBuildPointsAs)
        MainWindow.setTabOrder(self.saveBuildPointsAs, self.numberGridPointsRow)
        MainWindow.setTabOrder(self.numberGridPointsRow, self.numberGridPointsCol)
        MainWindow.setTabOrder(self.numberGridPointsCol, self.altitudeMin)
        MainWindow.setTabOrder(self.altitudeMin, self.altitudeMax)
        MainWindow.setTabOrder(self.altitudeMax, self.genBuildGrid)
        MainWindow.setTabOrder(self.genBuildGrid, self.genBuildAlign3)
        MainWindow.setTabOrder(self.genBuildAlign3, self.genBuildAlign6)
        MainWindow.setTabOrder(self.genBuildAlign6, self.genBuildAlign9)
        MainWindow.setTabOrder(self.genBuildAlign9, self.genBuildMax)
        MainWindow.setTabOrder(self.genBuildMax, self.genBuildMed)
        MainWindow.setTabOrder(self.genBuildMed, self.genBuildNorm)
        MainWindow.setTabOrder(self.genBuildNorm, self.genBuildMin)
        MainWindow.setTabOrder(self.genBuildMin, self.keepGeneratedPoints)
        MainWindow.setTabOrder(self.keepGeneratedPoints, self.clearBuildP)
        MainWindow.setTabOrder(self.clearBuildP, self.genBuildSpiralMax)
        MainWindow.setTabOrder(self.genBuildSpiralMax, self.genBuildSpiralMed)
        MainWindow.setTabOrder(self.genBuildSpiralMed, self.genBuildSpiralNorm)
        MainWindow.setTabOrder(self.genBuildSpiralNorm, self.genBuildSpiralMin)
        MainWindow.setTabOrder(self.genBuildSpiralMin, self.numberDSOPoints)
        MainWindow.setTabOrder(self.numberDSOPoints, self.durationDSO)
        MainWindow.setTabOrder(self.durationDSO, self.timeShiftDSO)
        MainWindow.setTabOrder(self.timeShiftDSO, self.genBuildDSO)
        MainWindow.setTabOrder(self.genBuildDSO, self.checkSortNothing)
        MainWindow.setTabOrder(self.checkSortNothing, self.checkSortEW)
        MainWindow.setTabOrder(self.checkSortEW, self.checkSortHL)
        MainWindow.setTabOrder(self.checkSortHL, self.checkAvoidFlip)
        MainWindow.setTabOrder(self.checkAvoidFlip, self.loadHorizonMask)
        MainWindow.setTabOrder(self.loadHorizonMask, self.saveHorizonMask)
        MainWindow.setTabOrder(self.saveHorizonMask, self.saveHorizonMaskAs)
        MainWindow.setTabOrder(self.saveHorizonMaskAs, self.clearHorizonMask)
        MainWindow.setTabOrder(self.clearHorizonMask, self.checkAutoDeleteHorizon)
        MainWindow.setTabOrder(self.checkAutoDeleteHorizon, self.checkSafetyMarginHorizon)
        MainWindow.setTabOrder(self.checkSafetyMarginHorizon, self.checkAutoDeleteMeridian)
        MainWindow.setTabOrder(self.checkAutoDeleteMeridian, self.safetyMarginHorizon)
        MainWindow.setTabOrder(self.safetyMarginHorizon, self.runModel)
        MainWindow.setTabOrder(self.runModel, self.pauseModel)
        MainWindow.setTabOrder(self.pauseModel, self.endModel)
        MainWindow.setTabOrder(self.endModel, self.cancelModel)
        MainWindow.setTabOrder(self.cancelModel, self.checkDisableDAT)
        MainWindow.setTabOrder(self.checkDisableDAT, self.parkMountAfterModel)
        MainWindow.setTabOrder(self.parkMountAfterModel, self.excludeDonePoints)
        MainWindow.setTabOrder(self.excludeDonePoints, self.numberBuildRetries)
        MainWindow.setTabOrder(self.numberBuildRetries, self.plateSolveSync)
        MainWindow.setTabOrder(self.plateSolveSync, self.batchModel)
        MainWindow.setTabOrder(self.batchModel, self.tabWidget)
        MainWindow.setTabOrder(self.tabWidget, self.showNoAnnotation)
        MainWindow.setTabOrder(self.showNoAnnotation, self.showNumbers)
        MainWindow.setTabOrder(self.showNumbers, self.showErrorValues)
        MainWindow.setTabOrder(self.showErrorValues, self.deleteWorstPoint)
        MainWindow.setTabOrder(self.deleteWorstPoint, self.clearModel)
        MainWindow.setTabOrder(self.clearModel, self.refreshModel)
        MainWindow.setTabOrder(self.refreshModel, self.runOptimize)
        MainWindow.setTabOrder(self.runOptimize, self.cancelOptimize)
        MainWindow.setTabOrder(self.cancelOptimize, self.targetRMS)
        MainWindow.setTabOrder(self.targetRMS, self.optimizeOverall)
        MainWindow.setTabOrder(self.optimizeOverall, self.optimizeSingle)
        MainWindow.setTabOrder(self.optimizeSingle, self.openAnalyseW)
        MainWindow.setTabOrder(self.openAnalyseW, self.showOriginalModelAnalyse)
        MainWindow.setTabOrder(self.showOriginalModelAnalyse, self.showActualModelAnalyse)
        MainWindow.setTabOrder(self.showActualModelAnalyse, self.autoUpdateActualAnalyse)
        MainWindow.setTabOrder(self.autoUpdateActualAnalyse, self.nameList)
        MainWindow.setTabOrder(self.nameList, self.loadName)
        MainWindow.setTabOrder(self.loadName, self.saveName)
        MainWindow.setTabOrder(self.saveName, self.deleteName)
        MainWindow.setTabOrder(self.deleteName, self.refreshName)
        MainWindow.setTabOrder(self.refreshName, self.satelliteSource)
        MainWindow.setTabOrder(self.satelliteSource, self.filterSatellite)
        MainWindow.setTabOrder(self.filterSatellite, self.listSatelliteNames)
        MainWindow.setTabOrder(self.listSatelliteNames, self.progSatellitesFull)
        MainWindow.setTabOrder(self.progSatellitesFull, self.progSatellitesFiltered)
        MainWindow.setTabOrder(self.progSatellitesFiltered, self.satelliteName)
        MainWindow.setTabOrder(self.satelliteName, self.satelliteEpoch)
        MainWindow.setTabOrder(self.satelliteEpoch, self.satAzimuth)
        MainWindow.setTabOrder(self.satAzimuth, self.satAltitude)
        MainWindow.setTabOrder(self.satAltitude, self.satLatitude)
        MainWindow.setTabOrder(self.satLatitude, self.satelliteNumber)
        MainWindow.setTabOrder(self.satelliteNumber, self.satelliteDataAge)
        MainWindow.setTabOrder(self.satelliteDataAge, self.satRa)
        MainWindow.setTabOrder(self.satRa, self.satDec)
        MainWindow.setTabOrder(self.satDec, self.satLongitude)
        MainWindow.setTabOrder(self.satLongitude, self.satTransitStartUTC_1)
        MainWindow.setTabOrder(self.satTransitStartUTC_1, self.satTransitEndUTC_1)
        MainWindow.setTabOrder(self.satTransitEndUTC_1, self.satTransitStartUTC_2)
        MainWindow.setTabOrder(self.satTransitStartUTC_2, self.satTransitEndUTC_2)
        MainWindow.setTabOrder(self.satTransitEndUTC_2, self.satTransitStartUTC_3)
        MainWindow.setTabOrder(self.satTransitStartUTC_3, self.satTransitEndUTC_3)
        MainWindow.setTabOrder(self.satTransitEndUTC_3, self.satTransitStartUTC)
        MainWindow.setTabOrder(self.satTransitStartUTC, self.satTransitEndUTC)
        MainWindow.setTabOrder(self.satTransitEndUTC, self.satNeedFlip)
        MainWindow.setTabOrder(self.satNeedFlip, self.startSatelliteTracking)
        MainWindow.setTabOrder(self.startSatelliteTracking, self.stopSatelliteTracking)
        MainWindow.setTabOrder(self.stopSatelliteTracking, self.minorPlanetSource)
        MainWindow.setTabOrder(self.minorPlanetSource, self.filterMinorPlanet)
        MainWindow.setTabOrder(self.filterMinorPlanet, self.listMinorPlanetNames)
        MainWindow.setTabOrder(self.listMinorPlanetNames, self.progMinorPlanetsFull)
        MainWindow.setTabOrder(self.progMinorPlanetsFull, self.progMinorPlanetsFiltered)
        MainWindow.setTabOrder(self.progMinorPlanetsFiltered, self.downloadIERS)
        MainWindow.setTabOrder(self.downloadIERS, self.progEarthRotationData)
        MainWindow.setTabOrder(self.progEarthRotationData, self.sensorVoltage)
        MainWindow.setTabOrder(self.sensorVoltage, self.sensorCurrent)
        MainWindow.setTabOrder(self.sensorCurrent, self.sensorPower)
        MainWindow.setTabOrder(self.sensorPower, self.consumptionAvgAmps)
        MainWindow.setTabOrder(self.consumptionAvgAmps, self.consumptionAmpHours)
        MainWindow.setTabOrder(self.consumptionAmpHours, self.consumptionWattHours)
        MainWindow.setTabOrder(self.consumptionWattHours, self.rebootUPB)
        MainWindow.setTabOrder(self.rebootUPB, self.autoDew)
        MainWindow.setTabOrder(self.autoDew, self.dewCurrentA)
        MainWindow.setTabOrder(self.dewCurrentA, self.dewCurrentB)
        MainWindow.setTabOrder(self.dewCurrentB, self.dewCurrentC)
        MainWindow.setTabOrder(self.dewCurrentC, self.powerCurrent1)
        MainWindow.setTabOrder(self.powerCurrent1, self.powerPort1)
        MainWindow.setTabOrder(self.powerPort1, self.powerBootPort1)
        MainWindow.setTabOrder(self.powerBootPort1, self.powerCurrent2)
        MainWindow.setTabOrder(self.powerCurrent2, self.powerPort2)
        MainWindow.setTabOrder(self.powerPort2, self.powerBootPort2)
        MainWindow.setTabOrder(self.powerBootPort2, self.powerCurrent3)
        MainWindow.setTabOrder(self.powerCurrent3, self.powerPort3)
        MainWindow.setTabOrder(self.powerPort3, self.powerBootPort3)
        MainWindow.setTabOrder(self.powerBootPort3, self.powerCurrent4)
        MainWindow.setTabOrder(self.powerCurrent4, self.powerPort4)
        MainWindow.setTabOrder(self.powerPort4, self.powerBootPort4)
        MainWindow.setTabOrder(self.powerBootPort4, self.portUSB1)
        MainWindow.setTabOrder(self.portUSB1, self.portUSB2)
        MainWindow.setTabOrder(self.portUSB2, self.portUSB3)
        MainWindow.setTabOrder(self.portUSB3, self.portUSB4)
        MainWindow.setTabOrder(self.portUSB4, self.portUSB5)
        MainWindow.setTabOrder(self.portUSB5, self.portUSB6)
        MainWindow.setTabOrder(self.portUSB6, self.hubUSB)
        MainWindow.setTabOrder(self.hubUSB, self.renameDir)
        MainWindow.setTabOrder(self.renameDir, self.checkIncludeSubdirs)
        MainWindow.setTabOrder(self.checkIncludeSubdirs, self.renameInputSelect)
        MainWindow.setTabOrder(self.renameInputSelect, self.newObjectName)
        MainWindow.setTabOrder(self.newObjectName, self.renameText)
        MainWindow.setTabOrder(self.renameText, self.rename1)
        MainWindow.setTabOrder(self.rename1, self.rename2)
        MainWindow.setTabOrder(self.rename2, self.rename3)
        MainWindow.setTabOrder(self.rename3, self.rename4)
        MainWindow.setTabOrder(self.rename4, self.rename5)
        MainWindow.setTabOrder(self.rename5, self.renameStart)
        MainWindow.setTabOrder(self.renameStart, self.posButton0)
        MainWindow.setTabOrder(self.posButton0, self.posButton1)
        MainWindow.setTabOrder(self.posButton1, self.posButton2)
        MainWindow.setTabOrder(self.posButton2, self.posButton3)
        MainWindow.setTabOrder(self.posButton3, self.posButton4)
        MainWindow.setTabOrder(self.posButton4, self.posButton5)
        MainWindow.setTabOrder(self.posButton5, self.posButton6)
        MainWindow.setTabOrder(self.posButton6, self.posButton7)
        MainWindow.setTabOrder(self.posButton7, self.posButton8)
        MainWindow.setTabOrder(self.posButton8, self.posButton9)
        MainWindow.setTabOrder(self.posButton9, self.parkMountAfterSlew)
        MainWindow.setTabOrder(self.parkMountAfterSlew, self.tabMoving)
        MainWindow.setTabOrder(self.tabMoving, self.moveNorthWest)
        MainWindow.setTabOrder(self.moveNorthWest, self.moveNorth)
        MainWindow.setTabOrder(self.moveNorth, self.moveNorthEast)
        MainWindow.setTabOrder(self.moveNorthEast, self.moveWest)
        MainWindow.setTabOrder(self.moveWest, self.stopMoveAll)
        MainWindow.setTabOrder(self.stopMoveAll, self.moveEast)
        MainWindow.setTabOrder(self.moveEast, self.moveSouthWest)
        MainWindow.setTabOrder(self.moveSouthWest, self.moveSouth)
        MainWindow.setTabOrder(self.moveSouth, self.moveSouthEast)
        MainWindow.setTabOrder(self.moveSouthEast, self.moveDuration)
        MainWindow.setTabOrder(self.moveDuration, self.slewSpeedMax)
        MainWindow.setTabOrder(self.slewSpeedMax, self.slewSpeedHigh)
        MainWindow.setTabOrder(self.slewSpeedHigh, self.slewSpeedMed)
        MainWindow.setTabOrder(self.slewSpeedMed, self.slewSpeedLow)
        MainWindow.setTabOrder(self.slewSpeedLow, self.moveNorthWestAltAz)
        MainWindow.setTabOrder(self.moveNorthWestAltAz, self.moveNorthAltAz)
        MainWindow.setTabOrder(self.moveNorthAltAz, self.moveNorthEastAltAz)
        MainWindow.setTabOrder(self.moveNorthEastAltAz, self.moveWestAltAz)
        MainWindow.setTabOrder(self.moveWestAltAz, self.moveEastAltAz)
        MainWindow.setTabOrder(self.moveEastAltAz, self.moveSouthWestAltAz)
        MainWindow.setTabOrder(self.moveSouthWestAltAz, self.moveSouthAltAz)
        MainWindow.setTabOrder(self.moveSouthAltAz, self.moveSouthEastAltAz)
        MainWindow.setTabOrder(self.moveSouthEastAltAz, self.moveStepSizeAltAz)
        MainWindow.setTabOrder(self.moveStepSizeAltAz, self.moveCoordinateAlt)
        MainWindow.setTabOrder(self.moveCoordinateAlt, self.moveCoordinateAz)
        MainWindow.setTabOrder(self.moveCoordinateAz, self.moveAltAzAbsolute)
        MainWindow.setTabOrder(self.moveAltAzAbsolute, self.posAz9)
        MainWindow.setTabOrder(self.posAz9, self.moveCoordinateRa)
        MainWindow.setTabOrder(self.moveCoordinateRa, self.moveCoordinateRaFloat)
        MainWindow.setTabOrder(self.moveCoordinateRaFloat, self.moveCoordinateDec)
        MainWindow.setTabOrder(self.moveCoordinateDec, self.moveCoordinateDecFloat)
        MainWindow.setTabOrder(self.moveCoordinateDecFloat, self.moveRaDecAbsolute)
        MainWindow.setTabOrder(self.moveRaDecAbsolute, self.relayButton0)
        MainWindow.setTabOrder(self.relayButton0, self.relayButton1)
        MainWindow.setTabOrder(self.relayButton1, self.relayButton2)
        MainWindow.setTabOrder(self.relayButton2, self.relayButton3)
        MainWindow.setTabOrder(self.relayButton3, self.relayButton4)
        MainWindow.setTabOrder(self.relayButton4, self.relayButton5)
        MainWindow.setTabOrder(self.relayButton5, self.relayButton6)
        MainWindow.setTabOrder(self.relayButton6, self.relayButton7)
        MainWindow.setTabOrder(self.relayButton7, self.cameraDevice)
        MainWindow.setTabOrder(self.cameraDevice, self.cameraSetup)
        MainWindow.setTabOrder(self.cameraSetup, self.filterDevice)
        MainWindow.setTabOrder(self.filterDevice, self.filterSetup)
        MainWindow.setTabOrder(self.filterSetup, self.focuserDevice)
        MainWindow.setTabOrder(self.focuserDevice, self.focuserSetup)
        MainWindow.setTabOrder(self.focuserSetup, self.astrometryDevice)
        MainWindow.setTabOrder(self.astrometryDevice, self.astrometrySetup)
        MainWindow.setTabOrder(self.astrometrySetup, self.telescopeDevice)
        MainWindow.setTabOrder(self.telescopeDevice, self.telescopeSetup)
        MainWindow.setTabOrder(self.telescopeSetup, self.domeDevice)
        MainWindow.setTabOrder(self.domeDevice, self.domeSetup)
        MainWindow.setTabOrder(self.domeSetup, self.sensorWeatherDevice)
        MainWindow.setTabOrder(self.sensorWeatherDevice, self.sensorWeatherSetup)
        MainWindow.setTabOrder(self.sensorWeatherSetup, self.skymeterDevice)
        MainWindow.setTabOrder(self.skymeterDevice, self.skymeterSetup)
        MainWindow.setTabOrder(self.skymeterSetup, self.powerWeatherDevice)
        MainWindow.setTabOrder(self.powerWeatherDevice, self.powerWeatherSetup)
        MainWindow.setTabOrder(self.powerWeatherSetup, self.onlineWeatherDevice)
        MainWindow.setTabOrder(self.onlineWeatherDevice, self.onlineWeatherSetup)
        MainWindow.setTabOrder(self.onlineWeatherSetup, self.directWeatherDevice)
        MainWindow.setTabOrder(self.directWeatherDevice, self.measureDevice)
        MainWindow.setTabOrder(self.measureDevice, self.remoteDevice)
        MainWindow.setTabOrder(self.remoteDevice, self.relayDevice)
        MainWindow.setTabOrder(self.relayDevice, self.relaySetup)
        MainWindow.setTabOrder(self.relaySetup, self.powerDevice)
        MainWindow.setTabOrder(self.powerDevice, self.powerSetup)
        MainWindow.setTabOrder(self.powerSetup, self.coverDevice)
        MainWindow.setTabOrder(self.coverDevice, self.coverSetup)
        MainWindow.setTabOrder(self.coverSetup, self.ascomConnect)
        MainWindow.setTabOrder(self.ascomConnect, self.ascomDisconnect)
        MainWindow.setTabOrder(self.ascomDisconnect, self.checkASCOMAutoConnect)
        MainWindow.setTabOrder(self.checkASCOMAutoConnect, self.expTime)
        MainWindow.setTabOrder(self.expTime, self.binning)
        MainWindow.setTabOrder(self.binning, self.expTimeN)
        MainWindow.setTabOrder(self.expTimeN, self.binningN)
        MainWindow.setTabOrder(self.binningN, self.subFrame)
        MainWindow.setTabOrder(self.subFrame, self.coolerOn)
        MainWindow.setTabOrder(self.coolerOn, self.coolerOff)
        MainWindow.setTabOrder(self.coolerOff, self.downloadFast)
        MainWindow.setTabOrder(self.downloadFast, self.downloadSlow)
        MainWindow.setTabOrder(self.downloadSlow, self.checkFastDownload)
        MainWindow.setTabOrder(self.checkFastDownload, self.checkKeepImages)
        MainWindow.setTabOrder(self.checkKeepImages, self.focuserStepsize)
        MainWindow.setTabOrder(self.focuserStepsize, self.moveFocuserIn)
        MainWindow.setTabOrder(self.moveFocuserIn, self.moveFocuserOut)
        MainWindow.setTabOrder(self.moveFocuserOut, self.haltFocuser)
        MainWindow.setTabOrder(self.haltFocuser, self.focalLength)
        MainWindow.setTabOrder(self.focalLength, self.aperture)
        MainWindow.setTabOrder(self.aperture, self.checkAutomaticTelescope)
        MainWindow.setTabOrder(self.checkAutomaticTelescope, self.copyFromTelescopeDriver)
        MainWindow.setTabOrder(self.copyFromTelescopeDriver, self.coverUnpark)
        MainWindow.setTabOrder(self.coverUnpark, self.coverPark)
        MainWindow.setTabOrder(self.coverPark, self.coverHalt)
        MainWindow.setTabOrder(self.coverHalt, self.mountHost)
        MainWindow.setTabOrder(self.mountHost, self.mountMAC)
        MainWindow.setTabOrder(self.mountMAC, self.settleTimeMount)
        MainWindow.setTabOrder(self.settleTimeMount, self.rackCompMAC)
        MainWindow.setTabOrder(self.rackCompMAC, self.bootRackComp)
        MainWindow.setTabOrder(self.bootRackComp, self.domeRadius)
        MainWindow.setTabOrder(self.domeRadius, self.domeNorthOffset)
        MainWindow.setTabOrder(self.domeNorthOffset, self.domeEastOffset)
        MainWindow.setTabOrder(self.domeEastOffset, self.domeZoffGEM)
        MainWindow.setTabOrder(self.domeZoffGEM, self.domeZoff10micron)
        MainWindow.setTabOrder(self.domeZoff10micron, self.offGEM)
        MainWindow.setTabOrder(self.offGEM, self.offLAT)
        MainWindow.setTabOrder(self.offLAT, self.checkUseDomeGeometry)
        MainWindow.setTabOrder(self.checkUseDomeGeometry, self.checkAutomaticDome)
        MainWindow.setTabOrder(self.checkAutomaticDome, self.copyFromDomeDriver)
        MainWindow.setTabOrder(self.copyFromDomeDriver, self.domeOpenShutter)
        MainWindow.setTabOrder(self.domeOpenShutter, self.domeCloseShutter)
        MainWindow.setTabOrder(self.domeCloseShutter, self.domeAbortSlew)
        MainWindow.setTabOrder(self.domeAbortSlew, self.domeShutterWidth)
        MainWindow.setTabOrder(self.domeShutterWidth, self.tabDomeExplain)
        MainWindow.setTabOrder(self.tabDomeExplain, self.settleTimeDome)
        MainWindow.setTabOrder(self.settleTimeDome, self.isOnline)
        MainWindow.setTabOrder(self.isOnline, self.versionAvailable)
        MainWindow.setTabOrder(self.versionAvailable, self.versionRelease)
        MainWindow.setTabOrder(self.versionRelease, self.versionBeta)
        MainWindow.setTabOrder(self.versionBeta, self.installVersion)
        MainWindow.setTabOrder(self.installVersion, self.loglevelStandard)
        MainWindow.setTabOrder(self.loglevelStandard, self.loglevelDebug)
        MainWindow.setTabOrder(self.loglevelDebug, self.loglevelDebugTrace)
        MainWindow.setTabOrder(self.loglevelDebugTrace, self.soundMountSlewFinished)
        MainWindow.setTabOrder(self.soundMountSlewFinished, self.soundDomeSlewFinished)
        MainWindow.setTabOrder(self.soundDomeSlewFinished, self.soundMountAlert)
        MainWindow.setTabOrder(self.soundMountAlert, self.soundModelingFinished)
        MainWindow.setTabOrder(self.soundModelingFinished, self.soundImageSaved)
        MainWindow.setTabOrder(self.soundImageSaved, self.soundImageSolved)
        MainWindow.setTabOrder(self.soundImageSolved, self.textEdit)
        MainWindow.setTabOrder(self.textEdit, self.posSave4)
        MainWindow.setTabOrder(self.posSave4, self.runFlexure)
        MainWindow.setTabOrder(self.runFlexure, self.posAlt4)
        MainWindow.setTabOrder(self.posAlt4, self.altitudeHysterese1)
        MainWindow.setTabOrder(self.altitudeHysterese1, self.posSave7)
        MainWindow.setTabOrder(self.posSave7, self.posAlt8)
        MainWindow.setTabOrder(self.posAlt8, self.cancelAnalyse)
        MainWindow.setTabOrder(self.cancelAnalyse, self.posAz8)
        MainWindow.setTabOrder(self.posAz8, self.settingsTabWidget)
        MainWindow.setTabOrder(self.settingsTabWidget, self.loadAnalyseData)
        MainWindow.setTabOrder(self.loadAnalyseData, self.posText8)
        MainWindow.setTabOrder(self.posText8, self.posSave6)
        MainWindow.setTabOrder(self.posSave6, self.posAz5)
        MainWindow.setTabOrder(self.posAz5, self.posAlt5)
        MainWindow.setTabOrder(self.posAlt5, self.posText5)
        MainWindow.setTabOrder(self.posText5, self.posSave8)
        MainWindow.setTabOrder(self.posSave8, self.posSave9)
        MainWindow.setTabOrder(self.posSave9, self.posSave0)
        MainWindow.setTabOrder(self.posSave0, self.altitudeTimeChange)
        MainWindow.setTabOrder(self.altitudeTimeChange, self.posText6)
        MainWindow.setTabOrder(self.posText6, self.numberRunsHysterese)
        MainWindow.setTabOrder(self.numberRunsHysterese, self.posAz1)
        MainWindow.setTabOrder(self.posAz1, self.posText1)
        MainWindow.setTabOrder(self.posText1, self.posText9)
        MainWindow.setTabOrder(self.posText9, self.posAlt9)
        MainWindow.setTabOrder(self.posAlt9, self.posAz0)
        MainWindow.setTabOrder(self.posAz0, self.posSave1)
        MainWindow.setTabOrder(self.posSave1, self.posAz4)
        MainWindow.setTabOrder(self.posAz4, self.posSave5)
        MainWindow.setTabOrder(self.posSave5, self.posText3)
        MainWindow.setTabOrder(self.posText3, self.posAz6)
        MainWindow.setTabOrder(self.posAz6, self.posText4)
        MainWindow.setTabOrder(self.posText4, self.delayTimeFlexure)
        MainWindow.setTabOrder(self.delayTimeFlexure, self.altitudeHysterese2)
        MainWindow.setTabOrder(self.altitudeHysterese2, self.posSave3)
        MainWindow.setTabOrder(self.posSave3, self.relayButtonText2)
        MainWindow.setTabOrder(self.relayButtonText2, self.relayButtonText5)
        MainWindow.setTabOrder(self.relayButtonText5, self.relayButtonText7)
        MainWindow.setTabOrder(self.relayButtonText7, self.relayFun6)
        MainWindow.setTabOrder(self.relayFun6, self.relayFun3)
        MainWindow.setTabOrder(self.relayFun3, self.relayFun5)
        MainWindow.setTabOrder(self.relayFun5, self.relayButtonText4)
        MainWindow.setTabOrder(self.relayButtonText4, self.relayButtonText1)
        MainWindow.setTabOrder(self.relayButtonText1, self.relayButtonText6)
        MainWindow.setTabOrder(self.relayButtonText6, self.relayFun1)
        MainWindow.setTabOrder(self.relayFun1, self.relayFun4)
        MainWindow.setTabOrder(self.relayFun4, self.relayFun2)
        MainWindow.setTabOrder(self.relayFun2, self.relayButtonText3)
        MainWindow.setTabOrder(self.relayButtonText3, self.relayButtonText0)
        MainWindow.setTabOrder(self.relayButtonText0, self.relayFun0)
        MainWindow.setTabOrder(self.relayFun0, self.relayFun7)
        MainWindow.setTabOrder(self.relayFun7, self.runHysteresis)
        MainWindow.setTabOrder(self.runHysteresis, self.filterMinorPlanet_5)
        MainWindow.setTabOrder(self.filterMinorPlanet_5, self.azimuthHysterese1)
        MainWindow.setTabOrder(self.azimuthHysterese1, self.posAlt6)
        MainWindow.setTabOrder(self.posAlt6, self.azimuthTimeChange)
        MainWindow.setTabOrder(self.azimuthTimeChange, self.delayTimeHysterese)
        MainWindow.setTabOrder(self.delayTimeHysterese, self.le_analyseFileName)
        MainWindow.setTabOrder(self.le_analyseFileName, self.posAlt0)
        MainWindow.setTabOrder(self.posAlt0, self.posText0)
        MainWindow.setTabOrder(self.posText0, self.posAz3)
        MainWindow.setTabOrder(self.posAz3, self.posAz2)
        MainWindow.setTabOrder(self.posAz2, self.posSave2)
        MainWindow.setTabOrder(self.posSave2, self.posAlt1)
        MainWindow.setTabOrder(self.posAlt1, self.posAlt3)
        MainWindow.setTabOrder(self.posAlt3, self.azimuthHysterese2)
        MainWindow.setTabOrder(self.azimuthHysterese2, self.posText2)
        MainWindow.setTabOrder(self.posText2, self.posAlt2)
        MainWindow.setTabOrder(self.posAlt2, self.posText7)
        MainWindow.setTabOrder(self.posText7, self.numberRunsTimeChange)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MountWizzard4 (C) MW 2018"))
        self.groupBox_47.setTitle(_translate("MainWindow", "Tracking / Flip"))
        self.setSolarTracking.setToolTip(_translate("MainWindow", "<html><head/><body><p>Set tracking speed to solar. Shows the status in blue if solar tracking is on.</p></body></html>"))
        self.setSolarTracking.setText(_translate("MainWindow", "Solar"))
        self.setSiderealTracking.setToolTip(_translate("MainWindow", "<html><head/><body><p>Set tracking speed to sideral. Shows the status in blue if sidereal tracking is on.</p></body></html>"))
        self.setSiderealTracking.setText(_translate("MainWindow", "Sidereal"))
        self.setLunarTracking.setToolTip(_translate("MainWindow", "<html><head/><body><p>Set tracking speed to lunar. Shows the status in blue if lunar tracking is on.</p></body></html>"))
        self.setLunarTracking.setText(_translate("MainWindow", "Lunar"))
        self.flipMount.setToolTip(_translate("MainWindow", "Flips the mount if possible"))
        self.flipMount.setText(_translate("MainWindow", "Flip"))
        self.tracking.setToolTip(_translate("MainWindow", "Switching tracking on and off. Blue button show tracking is on."))
        self.tracking.setText(_translate("MainWindow", "Tracking"))
        self.followSat.setToolTip(_translate("MainWindow", "<html><head/><body><p>Show the status of satellite tracking. Blue if sat tracking is running.</p></body></html>"))
        self.followSat.setText(_translate("MainWindow", "Follow"))
        self.groupBox_48.setTitle(_translate("MainWindow", "Parking / Emergency stop"))
        self.stop.setToolTip(_translate("MainWindow", "Stops all mount movements immediatly"))
        self.stop.setText(_translate("MainWindow", "Mount Stop"))
        self.park.setToolTip(_translate("MainWindow", "Switches the parking state on and off. Blue button means mount parked. It is the position, which is in the mount computer defined as park position. If the mount is not parked, it slews to the park position. If parked, the button press just unparks and starts tracking."))
        self.park.setText(_translate("MainWindow", "Mount Park"))
        self.groupBox_49.setTitle(_translate("MainWindow", "Telescope pointing"))
        self.groupBoxCoordinateSystems.setTitle(_translate("MainWindow", "Epoch"))
        self.checkJNow.setToolTip(_translate("MainWindow", "Sets the epoch for showing the pointing coordinates to JNow"))
        self.checkJNow.setText(_translate("MainWindow", "JNow"))
        self.checkJ2000.setToolTip(_translate("MainWindow", "Sets the epoch for showing the pointing coordinates to J2000"))
        self.checkJ2000.setText(_translate("MainWindow", "J2000"))
        self.label_77.setText(_translate("MainWindow", "RA"))
        self.RAfloat.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:20pt;\">Display of right ascention in decimals</span></p></body></html>"))
        self.RAfloat.setText(_translate("MainWindow", "-"))
        self.ALT.setToolTip(_translate("MainWindow", "<html><head/><body><p>Display of altitude in degrees</p></body></html>"))
        self.ALT.setText(_translate("MainWindow", "ALT"))
        self.DECfloat.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:20pt;\">Display of declination in decimal</span></p></body></html>"))
        self.DECfloat.setText(_translate("MainWindow", "-"))
        self.label_79.setText(_translate("MainWindow", "AZ"))
        self.HAfloat.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:20pt;\">Display of hour angle in decimal</span></p></body></html>"))
        self.HAfloat.setText(_translate("MainWindow", "-"))
        self.label_102.setText(_translate("MainWindow", "DEC"))
        self.label_58.setText(_translate("MainWindow", "HA"))
        self.RA.setToolTip(_translate("MainWindow", "Display of right ascention in HH:MM:SS"))
        self.RA.setText(_translate("MainWindow", "RA"))
        self.HA.setToolTip(_translate("MainWindow", "<html><head/><body><p>Display of hour angle in HH:MM:SS</p></body></html>"))
        self.HA.setText(_translate("MainWindow", "HA"))
        self.AZ.setToolTip(_translate("MainWindow", "<html><head/><body><p>Display of azimuth in degrees</p></body></html>"))
        self.AZ.setText(_translate("MainWindow", "AZ"))
        self.DEC.setToolTip(_translate("MainWindow", "Display of declination in DD:MM:SS"))
        self.DEC.setText(_translate("MainWindow", "DEC"))
        self.label_78.setText(_translate("MainWindow", "ALT"))
        self.groupBox_50.setTitle(_translate("MainWindow", "Status and settings from mount computer"))
        self.label_148.setText(_translate("MainWindow", "GPS time synced"))
        self.label_12.setText(_translate("MainWindow", "Site Elevation"))
        self.label_34.setText(_translate("MainWindow", "Flip Guide Tolerance"))
        self.refractionPress.setToolTip(_translate("MainWindow", "Shows the refraction pressure used in mount"))
        self.refractionPress.setText(_translate("MainWindow", "-"))
        self.label_152.setText(_translate("MainWindow", "DeltaT expires"))
        self.statusRefraction.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the status if the mount takes refraction parameters into account. When clicking the field, you could change the value.</p></body></html>"))
        self.statusRefraction.setText(_translate("MainWindow", "-"))
        self.label_74.setText(_translate("MainWindow", "Dual Axis Tracking"))
        self.label_65.setText(_translate("MainWindow", "Flip Slew Tolerance"))
        self.slewRate.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the actual set slew rate for the mount.</p><p>When clicking the field, you could change the value.</p></body></html>"))
        self.slewRate.setText(_translate("MainWindow", "-"))
        self.pierside.setToolTip(_translate("MainWindow", "Shows the pierside reported from the mout. Eeast or West. Definitions are given in the manual of the mount computer."))
        self.pierside.setText(_translate("MainWindow", "-"))
        self.refractionTemp.setToolTip(_translate("MainWindow", "Shows the refraction temperature used in mount"))
        self.refractionTemp.setText(_translate("MainWindow", "-"))
        self.siteLongitude.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the longitude of the site.</p><p>When clicking the field, you could change the value.</p></body></html>"))
        self.siteLongitude.setText(_translate("MainWindow", "-"))
        self.label_55.setText(_translate("MainWindow", "Unattended Flip"))
        self.label_61.setText(_translate("MainWindow", "min"))
        self.label_86.setText(_translate("MainWindow", "Refraction correction"))
        self.UTCExpire.setToolTip(_translate("MainWindow", "Shows the expiration date of UTC timetable in the mount computer. If it\'s happening within 1 month, the frame turns to yellow. If expired, the frame will turn to red. "))
        self.UTCExpire.setText(_translate("MainWindow", "-"))
        self.siteLatitude.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the latitude of the site.</p><p>When clicking the field, you could change the value.</p></body></html>"))
        self.siteLatitude.setText(_translate("MainWindow", "-"))
        self.timeToFlip.setToolTip(_translate("MainWindow", "Shows the time until tracking ends because of flip / horizon limit"))
        self.timeToFlip.setText(_translate("MainWindow", "-"))
        self.label_4.setText(_translate("MainWindow", "Horizon Lim low"))
        self.statusUnattendedFlip.setToolTip(_translate("MainWindow", "Status of unattended Flip.When clicking the field, you could change the value"))
        self.statusUnattendedFlip.setText(_translate("MainWindow", "-"))
        self.label_82.setText(_translate("MainWindow", "m"))
        self.label_64.setText(_translate("MainWindow", "°C"))
        self.label_109.setText(_translate("MainWindow", "°"))
        self.timeSidereal.setToolTip(_translate("MainWindow", "Shows the local sidereal time from the mount computer"))
        self.timeSidereal.setText(_translate("MainWindow", "-"))
        self.label_130.setText(_translate("MainWindow", "HMS"))
        self.label_232.setText(_translate("MainWindow", "Refraction temperature"))
        self.horizonLimitHigh.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual horizon limit for slewing for high values (upper boundary).</p><p>When clicking the field, you could change the value.</p></body></html>"))
        self.horizonLimitHigh.setText(_translate("MainWindow", "-"))
        self.siteElevation.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the elevation of the site.</p><p>When clicking the field, you could change the value.</p></body></html>"))
        self.siteElevation.setText(_translate("MainWindow", "-"))
        self.label_91.setText(_translate("MainWindow", "min"))
        self.label_6.setText(_translate("MainWindow", "Site Latitude"))
        self.meridianLimitSlew.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the amount of degrees when the mount will flip after the meridian is passed when slewing. Positive values show after meridian, negative values before meridian.</p><p>When clicking the field, you could change the value.</p><p>Please refer to the mount manual for details</p></body></html>"))
        self.meridianLimitSlew.setText(_translate("MainWindow", "-"))
        self.label_242.setText(_translate("MainWindow", "Refraction pressure"))
        self.statusDualAxisTracking.setToolTip(_translate("MainWindow", "Status of dual tracking. When clicking the field, you could change the value.\n"
"If there is a yellow frame, the status is changed by MW4 due to functional reasons."))
        self.statusDualAxisTracking.setText(_translate("MainWindow", "-"))
        self.label_52.setText(_translate("MainWindow", "Time to Flip"))
        self.statusGPSSynced.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows if the mount is synced to an attached GPS receiver.</p></body></html>"))
        self.statusGPSSynced.setText(_translate("MainWindow", "-"))
        self.label_151.setText(_translate("MainWindow", "WakeOnLan"))
        self.label_105.setText(_translate("MainWindow", "DMS"))
        self.label_133.setText(_translate("MainWindow", "°/s"))
        self.meridianLimitTrack.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the amount of degrees when the mount will flip after the meridian is passed when tracking. Positive values show after meridian, negative values before meridian.</p><p>When clicking the field, you could change the value.</p><p>Please refer to the mount manual for details</p></body></html>"))
        self.meridianLimitTrack.setText(_translate("MainWindow", "-"))
        self.label_20.setText(_translate("MainWindow", "Horizon Lim high"))
        self.label_18.setText(_translate("MainWindow", "Pierside is"))
        self.label_88.setText(_translate("MainWindow", "°"))
        self.label_28.setText(_translate("MainWindow", "Slew Rate"))
        self.label_81.setText(_translate("MainWindow", "hPa"))
        self.label_7.setText(_translate("MainWindow", "Local Sidereal Time"))
        self.label_104.setText(_translate("MainWindow", "°"))
        self.label_106.setText(_translate("MainWindow", "DMS"))
        self.label_5.setText(_translate("MainWindow", "Site Longitude"))
        self.label_222.setText(_translate("MainWindow", "YMD"))
        self.label_90.setText(_translate("MainWindow", "Time to Meridian"))
        self.label_89.setText(_translate("MainWindow", "°"))
        self.timeToMeridian.setToolTip(_translate("MainWindow", "Shows the time until meridian is reached"))
        self.timeToMeridian.setText(_translate("MainWindow", "-"))
        self.horizonLimitLow.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual horizon limit for slewing for low limit (lower boundary).</p><p>When clicking the field, you could change the value.</p></body></html>"))
        self.horizonLimitLow.setText(_translate("MainWindow", "-"))
        self.statusWOL.setToolTip(_translate("MainWindow", "Status of Wake On Lan WOL feature. When clicking the field, you could change the value."))
        self.statusWOL.setText(_translate("MainWindow", "-"))
        self.label_mountMAC_2.setText(_translate("MainWindow", "Connection is:"))
        self.mountTypeConnection.setToolTip(_translate("MainWindow", "Type of the connection to the mount."))
        self.mountTypeConnection.setText(_translate("MainWindow", "-"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Mount), _translate("MainWindow", "Mount"))
        self.groupBox_51.setTitle(_translate("MainWindow", "Environment data from source"))
        self.label_25.setText(_translate("MainWindow", "DewPoint"))
        self.label_27.setText(_translate("MainWindow", "Pressure"))
        self.label_73.setText(_translate("MainWindow", "[hPa]"))
        self.label_142.setText(_translate("MainWindow", "SkyQuality"))
        self.mountGroup.setToolTip(_translate("MainWindow", "Actual used refraction parameters from the mount, if refraction is enabled."))
        self.mountGroup.setTitle(_translate("MainWindow", "Mount"))
        self.refractionTemp1.setToolTip(_translate("MainWindow", "shows the refraction temperature used in mount"))
        self.refractionTemp1.setText(_translate("MainWindow", "-"))
        self.refractionPress1.setToolTip(_translate("MainWindow", "shows the refraction pressure used in mount"))
        self.refractionPress1.setText(_translate("MainWindow", "-"))
        self.label_26.setText(_translate("MainWindow", "Humidity"))
        self.powerGroup.setToolTip(_translate("MainWindow", "iIf UPB power is present, show its values"))
        self.powerGroup.setTitle(_translate("MainWindow", "Power"))
        self.powerTemp.setText(_translate("MainWindow", "-"))
        self.powerDewPoint.setText(_translate("MainWindow", "-"))
        self.powerHumidity.setText(_translate("MainWindow", "-"))
        self.powerPress.setText(_translate("MainWindow", "-"))
        self.powerRainVol.setText(_translate("MainWindow", "-"))
        self.powerWindDir.setText(_translate("MainWindow", "-"))
        self.powerWindSpeed.setText(_translate("MainWindow", "-"))
        self.powerCloudCover.setText(_translate("MainWindow", "-"))
        self.powerSQR.setText(_translate("MainWindow", "-"))
        self.label_87.setText(_translate("MainWindow", "[mpas]"))
        self.label_69.setText(_translate("MainWindow", "[°]"))
        self.label_30.setText(_translate("MainWindow", "CloudCover"))
        self.label_67.setText(_translate("MainWindow", "[m/s]"))
        self.label_31.setText(_translate("MainWindow", "Rain vol."))
        self.label_24.setText(_translate("MainWindow", "Airtemp."))
        self.label_71.setText(_translate("MainWindow", "[°C]"))
        self.label_72.setText(_translate("MainWindow", "[%]"))
        self.label_70.setText(_translate("MainWindow", "[°C]"))
        self.refractionGroup.setToolTip(_translate("MainWindow", "Setting refraction update types from MountWizzard to the mount"))
        self.refractionGroup.setTitle(_translate("MainWindow", "Refraction update"))
        self.checkRefracCont.setToolTip(_translate("MainWindow", "Update the refraction parameters to the mount continiously with moving average filter over 1 minute"))
        self.checkRefracCont.setText(_translate("MainWindow", "continously"))
        self.checkRefracNoTrack.setToolTip(_translate("MainWindow", "Update the refraction parameters to the mount when the mount is not tracking"))
        self.checkRefracNoTrack.setText(_translate("MainWindow", "no tracking"))
        self.checkRefracNone.setToolTip(_translate("MainWindow", "No refraction update to the mount"))
        self.checkRefracNone.setText(_translate("MainWindow", "no update"))
        self.label_68.setText(_translate("MainWindow", "[mm]"))
        self.sensorWeatherGroup.setToolTip(_translate("MainWindow", "Select sensor device as source for refraction updates."))
        self.sensorWeatherGroup.setTitle(_translate("MainWindow", "Sensor"))
        self.sensorWeatherTemp.setText(_translate("MainWindow", "-"))
        self.sensorWeatherPress.setText(_translate("MainWindow", "-"))
        self.sensorWeatherDewPoint.setText(_translate("MainWindow", "-"))
        self.sensorWeatherHumidity.setText(_translate("MainWindow", "-"))
        self.sensorWeatherRainVol.setText(_translate("MainWindow", "-"))
        self.sensorWeatherWindDir.setText(_translate("MainWindow", "-"))
        self.sensorWeatherWindSpeed.setText(_translate("MainWindow", "-"))
        self.sensorWeatherCloudCover.setText(_translate("MainWindow", "-"))
        self.sensorWeatherSQR.setText(_translate("MainWindow", "-"))
        self.onlineWeatherGroup.setToolTip(_translate("MainWindow", "Select weather device as source for refraction updates."))
        self.onlineWeatherGroup.setTitle(_translate("MainWindow", "Online"))
        self.onlineWeatherRainVol.setText(_translate("MainWindow", "-"))
        self.onlineWeatherWindSpeed.setText(_translate("MainWindow", "-"))
        self.onlineWeatherCloudCover.setText(_translate("MainWindow", "-"))
        self.onlineWeatherTemp.setText(_translate("MainWindow", "-"))
        self.onlineWeatherPress.setText(_translate("MainWindow", "-"))
        self.onlineWeatherHumidity.setText(_translate("MainWindow", "-"))
        self.onlineWeatherWindDir.setText(_translate("MainWindow", "-"))
        self.onlineWeatherDewPoint.setText(_translate("MainWindow", "-"))
        self.onlineWeatherSQR.setText(_translate("MainWindow", "-"))
        self.label_32.setText(_translate("MainWindow", "WindSpeed"))
        self.label_33.setText(_translate("MainWindow", "Wind Dir"))
        self.label_143.setText(_translate("MainWindow", "[%]"))
        self.skymeterGroup.setToolTip(_translate("MainWindow", "iIf skymeter is present, show its values"))
        self.skymeterGroup.setTitle(_translate("MainWindow", "SkyMeter"))
        self.skymeterSQR.setText(_translate("MainWindow", "-"))
        self.skymeterTemp.setText(_translate("MainWindow", "-"))
        self.skymeterPress.setText(_translate("MainWindow", "-"))
        self.skymeterRainVol.setText(_translate("MainWindow", "-"))
        self.skymeterWindDir.setText(_translate("MainWindow", "-"))
        self.skymeterDewPoint.setText(_translate("MainWindow", "-"))
        self.skymeterWindSpeed.setText(_translate("MainWindow", "-"))
        self.skymeterHumidity.setText(_translate("MainWindow", "-"))
        self.skymeterCloudCover.setText(_translate("MainWindow", "-"))
        self.setRefractionManual.setToolTip(_translate("MainWindow", "Pushes the refraction temperature and pressure to mount from the selected source"))
        self.setRefractionManual.setText(_translate("MainWindow", "Push \n"
" manually \n"
"to mount"))
        self.directWeatherGroup.setToolTip(_translate("MainWindow", "Select internal sensor device which is directly linked to mount (e.g. MBox) as source for refraction updates."))
        self.directWeatherGroup.setTitle(_translate("MainWindow", "Direct"))
        self.directWeatherTemp.setText(_translate("MainWindow", "-"))
        self.directWeatherPress.setText(_translate("MainWindow", "-"))
        self.directWeatherDewPoint.setText(_translate("MainWindow", "-"))
        self.directWeatherHumidity.setText(_translate("MainWindow", "-"))
        self.refractionGroup_2.setTitle(_translate("MainWindow", "Clear Outside"))
        self.picClearOutside.setToolTip(_translate("MainWindow", "Data from clear outside for the location which is displayed"))
        self.picClearOutside.setText(_translate("MainWindow", "Clear Outside panel when connected to internet"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Environ), _translate("MainWindow", "Environ"))
        self.refractionGroup_4.setTitle(_translate("MainWindow", "Annual twilight phases"))
        self.twilight.setToolTip(_translate("MainWindow", "Annual twilight show the different phases for your location and your local time.\n"
"As the calulation is time consuming, it will be done only once per start of the app.\n"
"y-axis is local day time and x-axis is date.\n"
"MW4 will show the actual day always in the middel of the chart."))
        self.refractionGroup_5.setTitle(_translate("MainWindow", "Twilight phases upcoming night"))
        self.twilightEvents.setToolTip(_translate("MainWindow", "Displaying the upcoming twilight events which really happen at your site. Unseen events are not in the list"))
        self.refractionGroup_3.setTitle(_translate("MainWindow", "Moon phases"))
        self.moonPhaseIllumination.setToolTip(_translate("MainWindow", "Showing the actual illumination (light part) of the overall moon surface"))
        self.moonPhaseIllumination.setText(_translate("MainWindow", "-"))
        self.moonPhaseText.setToolTip(_translate("MainWindow", "Moon phase description"))
        self.moonPhaseText.setText(_translate("MainWindow", "-"))
        self.label_146.setText(_translate("MainWindow", "% Illumination"))
        self.moonPic.setToolTip(_translate("MainWindow", "Show the moon phase as picture"))
        self.moonPic.setText(_translate("MainWindow", "TextLabel"))
        self.moonPhaseDegree.setText(_translate("MainWindow", "-"))
        self.label_155.setText(_translate("MainWindow", "Degrees"))
        self.moonPhasePercent.setToolTip(_translate("MainWindow", "Showing the moon phase in percent"))
        self.moonPhasePercent.setText(_translate("MainWindow", "-"))
        self.label_161.setText(_translate("MainWindow", "% of phase"))
        self.refractionGroup_6.setTitle(_translate("MainWindow", "Lunar nodes"))
        self.lunarNodes.setToolTip(_translate("MainWindow", "Shows the actual lunar node for your site."))
        self.lunarNodes.setText(_translate("MainWindow", "-"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Almanac), _translate("MainWindow", "Almanac"))
        self.groupBox_2.setTitle(_translate("MainWindow", "Load model points from file"))
        self.buildPFileName.setToolTip(_translate("MainWindow", "Name of Model Points file, where the data is stored. "))
        self.saveBuildPointsAs.setToolTip(_translate("MainWindow", "Saves the model points in a new file"))
        self.saveBuildPointsAs.setText(_translate("MainWindow", "Save as"))
        self.saveBuildPoints.setToolTip(_translate("MainWindow", "Saves the model points in file."))
        self.saveBuildPoints.setText(_translate("MainWindow", "Save"))
        self.genBuildFile.setToolTip(_translate("MainWindow", "Show the model points from given file"))
        self.genBuildFile.setText(_translate("MainWindow", "Show"))
        self.loadBuildPoints.setToolTip(_translate("MainWindow", "Loads the model points from file"))
        self.loadBuildPoints.setText(_translate("MainWindow", "Load"))
        self.groupBox_3.setTitle(_translate("MainWindow", "Generate model points as grid"))
        self.label_178.setText(_translate("MainWindow", "Rows"))
        self.numberGridPointsRow.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of rows in the grid build</p></body></html>"))
        self.label_179.setText(_translate("MainWindow", "Cols"))
        self.label_190.setText(_translate("MainWindow", "AltMin"))
        self.label_203.setText(_translate("MainWindow", "°"))
        self.altitudeMax.setToolTip(_translate("MainWindow", "Model point stop at maximum altitude."))
        self.label_144.setText(_translate("MainWindow", "°"))
        self.numberGridPointsCol.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of coloms in the grid build</p></body></html>"))
        self.label_180.setText(_translate("MainWindow", "AltMax"))
        self.genBuildGrid.setToolTip(_translate("MainWindow", "Generates a regular pattern with the given parameters."))
        self.genBuildGrid.setText(_translate("MainWindow", "Generate Grid"))
        self.altitudeMin.setToolTip(_translate("MainWindow", "Model point start at minimum altitude."))
        self.groupBox_4.setTitle(_translate("MainWindow", "Generate model points as parallels of declination"))
        self.genBuildMax.setToolTip(_translate("MainWindow", "Generates a pointcloud with greater circles: maximal number of stars."))
        self.genBuildMax.setText(_translate("MainWindow", "Maximal"))
        self.genBuildMed.setToolTip(_translate("MainWindow", "Generates a pointcloud with greater circles: medium number of stars."))
        self.genBuildMed.setText(_translate("MainWindow", "Medium"))
        self.genBuildMin.setToolTip(_translate("MainWindow", "Generates a pointcloud with greater circles: low number of stars."))
        self.genBuildMin.setText(_translate("MainWindow", "Minimal"))
        self.genBuildNorm.setToolTip(_translate("MainWindow", "Generates a pointcloud with greater circles: medium number of stars.\n"
""))
        self.genBuildNorm.setText(_translate("MainWindow", "Norm"))
        self.groupBox_5.setTitle(_translate("MainWindow", "Generate model points from tracks"))
        self.label_188.setText(_translate("MainWindow", "Points"))
        self.numberDSOPoints.setToolTip(_translate("MainWindow", "Number of points."))
        self.durationDSO.setToolTip(_translate("MainWindow", "<html><head/><body><p>Length of the DSO track in hours.</p></body></html>"))
        self.timeShiftDSO.setToolTip(_translate("MainWindow", "Shift the DSO track for given number of hours."))
        self.label_204.setText(_translate("MainWindow", "h"))
        self.label_189.setText(_translate("MainWindow", "TimeShift"))
        self.label_303.setText(_translate("MainWindow", "h"))
        self.label_181.setText(_translate("MainWindow", "Duration"))
        self.genBuildDSO.setToolTip(_translate("MainWindow", "Generates model points along DSO track for a give time and other parameters."))
        self.genBuildDSO.setText(_translate("MainWindow", "Generate DSO Track"))
        self.groupBox_6.setTitle(_translate("MainWindow", "Generate model points equally distributed (Golden Spiral)"))
        self.genBuildSpiralMin.setToolTip(_translate("MainWindow", "Generates model points as golden spiral, means the points are in equal distances from each other on a sphere."))
        self.genBuildSpiralMin.setText(_translate("MainWindow", "Minimal"))
        self.genBuildSpiralNorm.setToolTip(_translate("MainWindow", "Generates model points as golden spiral, means the points are in equal distances from each other on a sphere."))
        self.genBuildSpiralNorm.setText(_translate("MainWindow", "Norm"))
        self.genBuildSpiralMed.setToolTip(_translate("MainWindow", "Generates model points as golden spiral, means the points are in equal distances from each other on a sphere."))
        self.genBuildSpiralMed.setText(_translate("MainWindow", "Medium"))
        self.genBuildSpiralMax.setToolTip(_translate("MainWindow", "Generates model points as golden spiral, means the points are in equal distances from each other on a sphere."))
        self.genBuildSpiralMax.setText(_translate("MainWindow", "Maximal"))
        self.groupBoxCoordinateSystems_3.setToolTip(_translate("MainWindow", "Different types of sorting model points"))
        self.groupBoxCoordinateSystems_3.setTitle(_translate("MainWindow", "Auto sort points"))
        self.checkSortHL.setToolTip(_translate("MainWindow", "Sorts the model points low to high altitude."))
        self.checkSortHL.setText(_translate("MainWindow", "High / Low"))
        self.checkSortEW.setToolTip(_translate("MainWindow", "Sorts the model points EAST / WEST side of pier."))
        self.checkSortEW.setText(_translate("MainWindow", "East / West"))
        self.checkSortNothing.setToolTip(_translate("MainWindow", "No sorting of the model points "))
        self.checkSortNothing.setText(_translate("MainWindow", "Default"))
        self.checkAvoidFlip.setToolTip(_translate("MainWindow", "<html><head/><body><p>If checked, MW4 tries to start build model point from the pierside where the mount is actually.</p></body></html>"))
        self.checkAvoidFlip.setText(_translate("MainWindow", "Avoid meridian flip when starting"))
        self.groupBox_10.setTitle(_translate("MainWindow", "Horizon mask file"))
        self.saveHorizonMask.setToolTip(_translate("MainWindow", "Saves the horizon mask in the given file."))
        self.saveHorizonMask.setText(_translate("MainWindow", "Save"))
        self.saveHorizonMaskAs.setToolTip(_translate("MainWindow", "Saves the horizon mask in a new file."))
        self.saveHorizonMaskAs.setText(_translate("MainWindow", "Save as"))
        self.loadHorizonMask.setToolTip(_translate("MainWindow", "Loads the horizon mask from given file."))
        self.loadHorizonMask.setText(_translate("MainWindow", "Load"))
        self.horizonFileName.setToolTip(_translate("MainWindow", "Name of Horizon Points file, where the data is stored. "))
        self.checkAutoDeleteHorizon.setToolTip(_translate("MainWindow", "If checked, all model points are deleted, which are below the horizon mask."))
        self.checkAutoDeleteHorizon.setText(_translate("MainWindow", "Auto delete points below horizon"))
        self.checkAutoDeleteMeridian.setToolTip(_translate("MainWindow", "If checked, all model points are deleted, which are within the meridian limits."))
        self.checkAutoDeleteMeridian.setText(_translate("MainWindow", "Auto delete points within meridian limits"))
        self.clearHorizonMask.setToolTip(_translate("MainWindow", "Saves the horizon mask in a new file."))
        self.clearHorizonMask.setText(_translate("MainWindow", "Clear"))
        self.safetyMarginHorizon.setToolTip(_translate("MainWindow", "Number of points."))
        self.label_162.setText(_translate("MainWindow", "°"))
        self.checkSafetyMarginHorizon.setToolTip(_translate("MainWindow", "If checked, all model points are deleted, which are below the horizon mask."))
        self.checkSafetyMarginHorizon.setText(_translate("MainWindow", "Add safety distance to horizon line"))
        self.clearBuildP.setToolTip(_translate("MainWindow", "Delete all model points on coordinate window and for usage."))
        self.clearBuildP.setText(_translate("MainWindow", "Clear Points"))
        self.groupBox_54.setTitle(_translate("MainWindow", "Generate model points align"))
        self.genBuildAlign3.setToolTip(_translate("MainWindow", "Generates three stars."))
        self.genBuildAlign3.setText(_translate("MainWindow", "3 points"))
        self.genBuildAlign6.setToolTip(_translate("MainWindow", "Generates six stars."))
        self.genBuildAlign6.setText(_translate("MainWindow", "6 points"))
        self.genBuildAlign9.setToolTip(_translate("MainWindow", "Generates nine stars."))
        self.genBuildAlign9.setText(_translate("MainWindow", "9 points"))
        self.keepGeneratedPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>If checked, MW4 tries to start build model point from the pierside where the mount is actually.</p></body></html>"))
        self.keepGeneratedPoints.setText(_translate("MainWindow", "Keep points from existing generation"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Modelpoints), _translate("MainWindow", "Model Points"))
        self.groupBox_11.setTitle(_translate("MainWindow", "Alignment info from mount computer"))
        self.label_169.setText(_translate("MainWindow", "arcsec"))
        self.label_218.setText(_translate("MainWindow", "°"))
        self.label_154.setText(_translate("MainWindow", "Terms"))
        self.label_14.setText(_translate("MainWindow", "RMS"))
        self.positionAngle.setToolTip(_translate("MainWindow", "Actual position angle of the Error vector in deg."))
        self.positionAngle.setText(_translate("MainWindow", "-"))
        self.label_158.setText(_translate("MainWindow", "Azimuth Pointing"))
        self.orthoError.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual Ortho Error</p></body></html>"))
        self.orthoError.setText(_translate("MainWindow", "-"))
        self.numberStars.setToolTip(_translate("MainWindow", "Number of alignment stars in actual used pointing model"))
        self.numberStars.setText(_translate("MainWindow", "-"))
        self.label_15.setText(_translate("MainWindow", "Polar Align Error "))
        self.label_167.setText(_translate("MainWindow", "°"))
        self.azimuthError.setToolTip(_translate("MainWindow", "Actual position angle of the Error vector in azimuth deg."))
        self.azimuthError.setText(_translate("MainWindow", "-"))
        self.label.setToolTip(_translate("MainWindow", "Number of stars"))
        self.label.setText(_translate("MainWindow", "Stars"))
        self.label_153.setText(_translate("MainWindow", "Ortho Align Error"))
        self.label_159.setText(_translate("MainWindow", "Altitude Pointing"))
        self.label_160.setText(_translate("MainWindow", "PosAngle"))
        self.altitudeError.setToolTip(_translate("MainWindow", "Actual position angle of the Error vector in altitude deg."))
        self.altitudeError.setText(_translate("MainWindow", "-"))
        self.errorRMS.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual RMS Error in arc sec</p><p><br/></p></body></html>"))
        self.errorRMS.setText(_translate("MainWindow", "-"))
        self.label_201.setText(_translate("MainWindow", "arcsec"))
        self.terms.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of Terms in actual used pointing model</p></body></html>"))
        self.terms.setText(_translate("MainWindow", "-"))
        self.polarError.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual Polar Error</p></body></html>"))
        self.polarError.setText(_translate("MainWindow", "-"))
        self.label_168.setText(_translate("MainWindow", "°"))
        self.groupBox_9.setTitle(_translate("MainWindow", "Fast align"))
        self.plateSolveSync.setToolTip(_translate("MainWindow", "Image, Platesolve and Sync the alignment model of the mount to the given image coordinates. There is no model point added nor the model refined. It is just a matching of the model to that point. Attention: this might lead in worse model quality elsewhere."))
        self.plateSolveSync.setText(_translate("MainWindow", "Plate solve image and sync model"))
        self.groupBox_7.setTitle(_translate("MainWindow", "Using model datafiles"))
        self.batchModel.setToolTip(_translate("MainWindow", "Select on or more existing model data files and program them into the mount as a new mount model. Only existing models are used. It is ideal for combining multiple model runs, like adding a DSO path to an overall model."))
        self.batchModel.setText(_translate("MainWindow", "Choose model from file and program"))
        self.groupBox_8.setTitle(_translate("MainWindow", "Model build"))
        self.cancelModel.setToolTip(_translate("MainWindow", "Cancels a model build."))
        self.cancelModel.setText(_translate("MainWindow", "Cancel"))
        self.mSlew.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.mSlew.setText(_translate("MainWindow", "-"))
        self.label_35.setText(_translate("MainWindow", "Estimated time:"))
        self.runModel.setToolTip(_translate("MainWindow", "Run a model build."))
        self.runModel.setText(_translate("MainWindow", "Run"))
        self.mPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.mPoints.setText(_translate("MainWindow", "-"))
        self.label_29.setText(_translate("MainWindow", "Elapsed time:"))
        self.mSolve.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.mSolve.setText(_translate("MainWindow", "-"))
        self.label_310.setText(_translate("MainWindow", "Slew:"))
        self.label_311.setText(_translate("MainWindow", "Solve:"))
        self.mImage.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.mImage.setText(_translate("MainWindow", "-"))
        self.timeElapsed.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.timeElapsed.setText(_translate("MainWindow", "00:00:00"))
        self.label_216.setText(_translate("MainWindow", "Points:"))
        self.timeFinished.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.timeFinished.setText(_translate("MainWindow", "00:00:00"))
        self.label_312.setText(_translate("MainWindow", "Image:"))
        self.timeEstimated.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.timeEstimated.setText(_translate("MainWindow", "00:00:00"))
        self.label_42.setText(_translate("MainWindow", "Finishing Time:"))
        self.pauseModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Pause a model build after actual point.</p></body></html>"))
        self.pauseModel.setText(_translate("MainWindow", "Pause"))
        self.checkDisableDAT.setToolTip(_translate("MainWindow", "Disables dual axis tracking and restores the status after model build. In case of automatic disabling a yellow frame shown around DAT setting."))
        self.checkDisableDAT.setText(_translate("MainWindow", "Disable dual axis tracking during model build"))
        self.parkMountAfterModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>When enabled MW4 issues a park command after a model run.</p></body></html>"))
        self.parkMountAfterModel.setText(_translate("MainWindow", "Park mount after model run"))
        self.excludeDonePoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>During the model run leave out already successful used build points</p></body></html>"))
        self.excludeDonePoints.setText(_translate("MainWindow", "Exclude successful points for next model build run"))
        self.endModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Stops the model model build but programs the actual work to the mount.</p></body></html>"))
        self.endModel.setText(_translate("MainWindow", "End"))
        self.label_36.setText(_translate("MainWindow", "retry run(s) to recover"))
        self.label_59.setText(_translate("MainWindow", "If model points fail use"))
        self.groupBox_28.setTitle(_translate("MainWindow", "Adjust polar alignment"))
        self.label_157.setText(_translate("MainWindow", "Turn Knob ALT"))
        self.picALT.setText(_translate("MainWindow", "TextLabel"))
        self.altitudeTurns.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of Turns altitude ajustment knobs for correction (down if the polar axis has to be moved downwards) </p></body></html>"))
        self.altitudeTurns.setText(_translate("MainWindow", "--- left/right"))
        self.label_156.setText(_translate("MainWindow", "Turn Knob AZ"))
        self.azimuthTurns.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of Turns azimuth ajustment knobs for correction (left if polar axis has to be moved to the left) </p></body></html>"))
        self.azimuthTurns.setText(_translate("MainWindow", "--- left/right"))
        self.picAZ.setText(_translate("MainWindow", "TextLabel"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Modelbuild), _translate("MainWindow", "Model Build"))
        self.groupBox_55.setTitle(_translate("MainWindow", "Optimize actual model"))
        self.clearModel.setToolTip(_translate("MainWindow", "Clear the alignment model in mount. Can\'t be revised."))
        self.clearModel.setText(_translate("MainWindow", "Clear"))
        self.label_75.setText(_translate("MainWindow", "Model RMS"))
        self.cancelOptimize.setToolTip(_translate("MainWindow", "Cancels the optimise run."))
        self.cancelOptimize.setText(_translate("MainWindow", "Cancel"))
        self.runOptimize.setToolTip(_translate("MainWindow", "Run the model optimization. Please take care about the chosen method."))
        self.runOptimize.setText(_translate("MainWindow", "Run"))
        self.refreshModel.setToolTip(_translate("MainWindow", "Reloads the model from mount. Needed if you change things manually with handcontroller or other software."))
        self.refreshModel.setText(_translate("MainWindow", "Refresh"))
        self.errorRMS1.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual RMS Error in arc sec</p><p><br/></p></body></html>"))
        self.errorRMS1.setText(_translate("MainWindow", "0.0"))
        self.label_37.setText(_translate("MainWindow", "Target (RMS)"))
        self.deleteWorstPoint.setToolTip(_translate("MainWindow", "Deletes worst point of actual alignment model in the mount. It could happen, that the overall RMS rises."))
        self.deleteWorstPoint.setText(_translate("MainWindow", "Delete"))
        self.optimizeOverall.setToolTip(_translate("MainWindow", "Delete as many point until an overall target RMS of the model is reached."))
        self.optimizeOverall.setText(_translate("MainWindow", "Optimize overall Target RMS"))
        self.optimizeSingle.setToolTip(_translate("MainWindow", "Delete model points where the value of the single point is above a certain value."))
        self.optimizeSingle.setText(_translate("MainWindow", "Delete points >Target RMS"))
        self.groupBox_56.setTitle(_translate("MainWindow", "Models stored in mount computer"))
        self.loadName.setToolTip(_translate("MainWindow", "Loads the actual selected alignment model."))
        self.loadName.setText(_translate("MainWindow", "Load"))
        self.refreshName.setToolTip(_translate("MainWindow", "Refreshes the list of in the mount stored models "))
        self.refreshName.setText(_translate("MainWindow", "Refresh"))
        self.deleteName.setToolTip(_translate("MainWindow", "Deletes the actual selected alignment model."))
        self.deleteName.setText(_translate("MainWindow", "Delete"))
        self.saveName.setToolTip(_translate("MainWindow", "Saves actual alignment model under selected name or edited name."))
        self.saveName.setText(_translate("MainWindow", "Save"))
        self.showErrorValues.setToolTip(_translate("MainWindow", "<html><head/><body><p>Show model point error in arcsec close to the model point.</p></body></html>"))
        self.showErrorValues.setText(_translate("MainWindow", "Error values"))
        self.showNumbers.setToolTip(_translate("MainWindow", "<html><head/><body><p>Show the number close to the model point.</p></body></html>"))
        self.showNumbers.setText(_translate("MainWindow", "Numbers"))
        self.showNoAnnotation.setToolTip(_translate("MainWindow", "<html><head/><body><p>No annotations are shown close to the model point. </p></body></html>"))
        self.showNoAnnotation.setText(_translate("MainWindow", "No Annotation"))
        self.numberStars1.setToolTip(_translate("MainWindow", "Number of alignment stars in actual used pointing model"))
        self.numberStars1.setText(_translate("MainWindow", "00"))
        self.label_76.setText(_translate("MainWindow", "# of Stars"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("MainWindow", "Model Positions"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_3), _translate("MainWindow", "Error Distribution"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("MainWindow", "Error Ascending"))
        self.groupBox_22.setTitle(_translate("MainWindow", "Analyse"))
        self.originalModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the actual discovered model file which was used for building the model.</p></body></html>"))
        self.originalModel.setText(_translate("MainWindow", "-"))
        self.openAnalyseW.setToolTip(_translate("MainWindow", "Opens an analyse window showing multiple informations about the model context."))
        self.openAnalyseW.setText(_translate("MainWindow", "Analyse"))
        self.showOriginalModelAnalyse.setToolTip(_translate("MainWindow", "<html><head/><body><p>Pushed the discovered model file name to the analyse window</p></body></html>"))
        self.showOriginalModelAnalyse.setText(_translate("MainWindow", "Original"))
        self.autoUpdateActualAnalyse.setToolTip(_translate("MainWindow", "Delete model points where the value of the single point is above a certain value."))
        self.autoUpdateActualAnalyse.setText(_translate("MainWindow", "Auto Act."))
        self.showActualModelAnalyse.setToolTip(_translate("MainWindow", "<html><head/><body><p>Pushed the discovered model file name to the analyse window</p></body></html>"))
        self.showActualModelAnalyse.setText(_translate("MainWindow", "Actual"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.ManageModel), _translate("MainWindow", "Manage Model"))
        self.groupBox_12.setTitle(_translate("MainWindow", "TLE satellite data calculation"))
        self.label_227.setText(_translate("MainWindow", "No."))
        self.label_234.setText(_translate("MainWindow", "Azimuth"))
        self.satLongitude.setToolTip(_translate("MainWindow", "Actual longitude where the satellite could be seen in zenith."))
        self.satLongitude.setText(_translate("MainWindow", "-"))
        self.satDec.setToolTip(_translate("MainWindow", "Actual declination of the satelite from observers position."))
        self.satDec.setText(_translate("MainWindow", "-"))
        self.label_225.setText(_translate("MainWindow", "Name"))
        self.label_236.setText(_translate("MainWindow", "RA"))
        self.satRa.setToolTip(_translate("MainWindow", "Actual right ascention of the satelite from observers position."))
        self.satRa.setText(_translate("MainWindow", "-"))
        self.label_239.setText(_translate("MainWindow", "Lat"))
        self.label_235.setText(_translate("MainWindow", "Altitude"))
        self.satAzimuth.setToolTip(_translate("MainWindow", "Actual azimuth of the satelite from observers position."))
        self.satAzimuth.setText(_translate("MainWindow", "-"))
        self.label_233.setText(_translate("MainWindow", "days"))
        self.label_207.setText(_translate("MainWindow", "°"))
        self.satelliteEpoch.setToolTip(_translate("MainWindow", "Epoch of ephermeris"))
        self.satelliteEpoch.setText(_translate("MainWindow", "-"))
        self.label_221.setText(_translate("MainWindow", "hours"))
        self.label_231.setText(_translate("MainWindow", "Age"))
        self.label_237.setText(_translate("MainWindow", "DEC"))
        self.label_230.setText(_translate("MainWindow", "Epoch"))
        self.satelliteDataAge.setToolTip(_translate("MainWindow", "Age of the TLE data"))
        self.satelliteDataAge.setText(_translate("MainWindow", "-"))
        self.label_252.setText(_translate("MainWindow", "°"))
        self.label_202.setText(_translate("MainWindow", "°"))
        self.satAltitude.setToolTip(_translate("MainWindow", "Actual altitude of the satelite from observers position."))
        self.satAltitude.setText(_translate("MainWindow", "-"))
        self.satelliteNumber.setToolTip(_translate("MainWindow", "Number of the satellite."))
        self.satelliteNumber.setText(_translate("MainWindow", "-"))
        self.satLatitude.setToolTip(_translate("MainWindow", "Actual latitude where the satellite could be seen in zenith."))
        self.satLatitude.setText(_translate("MainWindow", "-"))
        self.label_253.setText(_translate("MainWindow", "°"))
        self.satelliteName.setToolTip(_translate("MainWindow", "Name of the satellite."))
        self.satelliteName.setText(_translate("MainWindow", "-"))
        self.label_226.setText(_translate("MainWindow", "°"))
        self.label_240.setText(_translate("MainWindow", "Lon"))
        self.satTransitStartUTC_1.setToolTip(_translate("MainWindow", "This is a prognosis:\n"
"1st point in time when satellite rises over horizon."))
        self.satTransitStartUTC_1.setText(_translate("MainWindow", "-"))
        self.satTransitStartUTC_1.setProperty("color", _translate("MainWindow", "red"))
        self.satTransitEndUTC_1.setToolTip(_translate("MainWindow", "This is a prognosis:\n"
"1st point in time when satellite settles."))
        self.satTransitEndUTC_1.setText(_translate("MainWindow", "-"))
        self.satTransitEndUTC_1.setProperty("color", _translate("MainWindow", "red"))
        self.satTransitEndUTC_2.setToolTip(_translate("MainWindow", "This is a prognosis:\n"
"2nd point in time when satellite settles."))
        self.satTransitEndUTC_2.setText(_translate("MainWindow", "-"))
        self.satTransitEndUTC_2.setProperty("color", _translate("MainWindow", "yellow"))
        self.satTransitStartUTC_2.setToolTip(_translate("MainWindow", "This is a prognosis:\n"
"2nd point in time when satellite rises over horizon."))
        self.satTransitStartUTC_2.setText(_translate("MainWindow", "-"))
        self.satTransitStartUTC_2.setProperty("color", _translate("MainWindow", "yellow"))
        self.satTransitEndUTC_3.setToolTip(_translate("MainWindow", "This is a prognosis:\n"
"3rd point in time when satellite settles."))
        self.satTransitEndUTC_3.setText(_translate("MainWindow", "-"))
        self.satTransitEndUTC_3.setProperty("color", _translate("MainWindow", "green"))
        self.satTransitStartUTC_3.setToolTip(_translate("MainWindow", "This is a prognosis:\n"
"3rd point in time when satellite rises over horizon."))
        self.satTransitStartUTC_3.setText(_translate("MainWindow", "-"))
        self.satTransitStartUTC_3.setProperty("color", _translate("MainWindow", "green"))
        self.label_298.setText(_translate("MainWindow", "Prognosis rise (UTC)"))
        self.label_309.setText(_translate("MainWindow", "Prognosis settle (UTC)"))
        self.label_241.setText(_translate("MainWindow", "1st pass"))
        self.label_254.setText(_translate("MainWindow", "2nd pass"))
        self.label_260.setText(_translate("MainWindow", "3rd pass"))
        self.groupBox_19.setTitle(_translate("MainWindow", "Mount satellite tracking"))
        self.stopSatelliteTracking.setToolTip(_translate("MainWindow", "Stops the satellite tracking mode. In some cases this does not work. Please start / stop tracking under mount."))
        self.stopSatelliteTracking.setText(_translate("MainWindow", "Stop satellite tracking"))
        self.startSatelliteTracking.setToolTip(_translate("MainWindow", "Starts the satellite tracking mode."))
        self.startSatelliteTracking.setText(_translate("MainWindow", "Start satellite tracking"))
        self.label_271.setText(_translate("MainWindow", "Need Flip"))
        self.satTransitStartUTC.setToolTip(_translate("MainWindow", "This is calculated by the mount computer !\n"
"Next point in time when satellite rises over horizon."))
        self.satTransitStartUTC.setText(_translate("MainWindow", "-"))
        self.satTransitEndUTC.setToolTip(_translate("MainWindow", "This is calculated by the mount computer !\n"
"Next point in time when satellite settles."))
        self.satTransitEndUTC.setText(_translate("MainWindow", "-"))
        self.satNeedFlip.setToolTip(_translate("MainWindow", "This is calculated by the mount computer !\n"
"Hint if mount will need a flip when following satellite."))
        self.satNeedFlip.setText(_translate("MainWindow", "-"))
        self.label_300.setText(_translate("MainWindow", "Trackable rise (UTC)"))
        self.label_301.setText(_translate("MainWindow", "Trackable settle (UTC)"))
        self.groupBox_27.setTitle(_translate("MainWindow", "Satellite data file"))
        self.satelliteSource.setToolTip(_translate("MainWindow", "Selects the TLE database from where the satellite list should be loaded."))
        self.filterSatellite.setToolTip(_translate("MainWindow", "Filter for selecting satellite names. The filter is not case sensitive."))
        self.label_269.setText(_translate("MainWindow", "search string"))
        self.groupBox_52.setTitle(_translate("MainWindow", "List of available satellites"))
        self.listSatelliteNames.setToolTip(_translate("MainWindow", "List of satellites."))
        self.progSatellitesFiltered.setToolTip(_translate("MainWindow", "<html><head/><body><p>Programs the actual selected but filtered (with text string) satellite database to the mount computer.</p></body></html>"))
        self.progSatellitesFiltered.setText(_translate("MainWindow", "Program filtered database"))
        self.progSatellitesFull.setToolTip(_translate("MainWindow", "<html><head/><body><p>Programs the actual selected satellite database to the mount computer.</p></body></html>"))
        self.progSatellitesFull.setText(_translate("MainWindow", "Program complete database"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Satellite), _translate("MainWindow", "Satellite"))
        self.groupBox_30.setTitle(_translate("MainWindow", "Minor planets data file"))
        self.minorPlanetSource.setToolTip(_translate("MainWindow", "<html><head/><body><p>Selects the minor planets database from where the minor planet list should be loaded.</p></body></html>"))
        self.filterMinorPlanet.setToolTip(_translate("MainWindow", "<html><head/><body><p>Filter for selecting minor planet names. The filter is not case sensitive.</p></body></html>"))
        self.label_270.setText(_translate("MainWindow", "search string"))
        self.groupBox_58.setTitle(_translate("MainWindow", "List of available minor planets (comets, asteroids)"))
        self.listMinorPlanetNames.setToolTip(_translate("MainWindow", "<html><head/><body><p>List of minor planets. This includes comets, asteroids and others.</p></body></html>"))
        self.progMinorPlanetsFull.setToolTip(_translate("MainWindow", "<html><head/><body><p>Programs the actual selected minor planet database (comets or asteroids) to the mount computer.</p></body></html>"))
        self.progMinorPlanetsFull.setText(_translate("MainWindow", "Program complete database"))
        self.progMinorPlanetsFiltered.setToolTip(_translate("MainWindow", "<html><head/><body><p>Programs the actual selected but filtered (with text string) minor planet database (comets or asteroids) to the mount computer.</p></body></html>"))
        self.progMinorPlanetsFiltered.setText(_translate("MainWindow", "Program filtered database"))
        self.groupBox_36.setTitle(_translate("MainWindow", "Earth rotation data"))
        self.downloadIERS.setToolTip(_translate("MainWindow", "<html><head/><body><p>Downloads new databases from IERS service finals2000a.all to update the buildin ones.</p></body></html>"))
        self.downloadIERS.setText(_translate("MainWindow", "Update IERS finals2000A.all "))
        self.progEarthRotationData.setToolTip(_translate("MainWindow", "<html><head/><body><p>Programs the actual in /data existing earth rotation data to the mount computer. Therefore MW4 uses the built in databases from IERS service finals2000a.all and the actual tai-utc.dat.</p></body></html>"))
        self.progEarthRotationData.setText(_translate("MainWindow", "Program earth rotation data"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.MPC), _translate("MainWindow", "MPC/IERS"))
        self.groupBox_13.setTitle(_translate("MainWindow", "Power Port 4"))
        self.label_283.setText(_translate("MainWindow", "A"))
        self.label_292.setText(_translate("MainWindow", "Current"))
        self.powerBootPort4.setText(_translate("MainWindow", "Power on boot"))
        self.powerLabel4.setText(_translate("MainWindow", "---"))
        self.powerPort4.setToolTip(_translate("MainWindow", "Switches the power output on and off."))
        self.powerPort4.setText(_translate("MainWindow", "Power"))
        self.groupBox_17.setTitle(_translate("MainWindow", "Power Port 3"))
        self.label_317.setText(_translate("MainWindow", "A"))
        self.label_318.setText(_translate("MainWindow", "Current"))
        self.powerLabel3.setText(_translate("MainWindow", "---"))
        self.powerBootPort3.setText(_translate("MainWindow", "Power on boot"))
        self.powerPort3.setToolTip(_translate("MainWindow", "Switches the power output on and off."))
        self.powerPort3.setText(_translate("MainWindow", "Power"))
        self.groupBox_18.setTitle(_translate("MainWindow", "Power Port 2"))
        self.label_319.setText(_translate("MainWindow", "A"))
        self.label_320.setText(_translate("MainWindow", "Current"))
        self.powerLabel2.setText(_translate("MainWindow", "---"))
        self.powerBootPort2.setText(_translate("MainWindow", "Power on boot"))
        self.powerPort2.setToolTip(_translate("MainWindow", "Switches the power output on and off."))
        self.powerPort2.setText(_translate("MainWindow", "Power"))
        self.box.setTitle(_translate("MainWindow", "Power Port 1"))
        self.label_321.setText(_translate("MainWindow", "A"))
        self.label_322.setText(_translate("MainWindow", "Current"))
        self.powerLabel1.setText(_translate("MainWindow", "---"))
        self.powerBootPort1.setText(_translate("MainWindow", "Power on boot"))
        self.powerPort1.setToolTip(_translate("MainWindow", "Switches the power output on and off."))
        self.powerPort1.setText(_translate("MainWindow", "Power"))
        self.groupHubUSB.setTitle(_translate("MainWindow", "USB Hub"))
        self.hubUSB.setToolTip(_translate("MainWindow", "Switches the USB Hub on and off"))
        self.hubUSB.setText(_translate("MainWindow", "Power"))
        self.groupBox_21.setTitle(_translate("MainWindow", "Consumption"))
        self.label_192.setText(_translate("MainWindow", "A"))
        self.label_284.setText(_translate("MainWindow", "Wh"))
        self.label_193.setText(_translate("MainWindow", "Ah"))
        self.label_191.setText(_translate("MainWindow", "W"))
        self.label_286.setText(_translate("MainWindow", "AmpHours"))
        self.label_287.setText(_translate("MainWindow", "Aver. Current"))
        self.label_165.setText(_translate("MainWindow", "Voltage"))
        self.label_285.setText(_translate("MainWindow", "WattHours"))
        self.label_289.setText(_translate("MainWindow", "Current"))
        self.label_288.setText(_translate("MainWindow", "Power"))
        self.label_173.setText(_translate("MainWindow", "V"))
        self.label_172.setText(_translate("MainWindow", "A"))
        self.rebootUPB.setToolTip(_translate("MainWindow", "Toggles the auto dew setting."))
        self.rebootUPB.setText(_translate("MainWindow", "Reboot"))
        self.groupDewA.setTitle(_translate("MainWindow", "Dew heater A"))
        self.label_276.setText(_translate("MainWindow", "%"))
        self.label_268.setText(_translate("MainWindow", "Current"))
        self.label_279.setText(_translate("MainWindow", "A"))
        self.label_266.setText(_translate("MainWindow", "DutyCycle"))
        self.dewA.setToolTip(_translate("MainWindow", "Sets the dew heater power from 0 to 100 %"))
        self.dewA.setText(_translate("MainWindow", "-"))
        self.groupDewB.setTitle(_translate("MainWindow", "Dew heater B"))
        self.label_280.setText(_translate("MainWindow", "%"))
        self.label_274.setText(_translate("MainWindow", "Current"))
        self.label_281.setText(_translate("MainWindow", "A"))
        self.label_275.setText(_translate("MainWindow", "DutyCycle"))
        self.dewB.setToolTip(_translate("MainWindow", "Sets the dew heater power from 0 to 100 %"))
        self.dewB.setText(_translate("MainWindow", "-"))
        self.groupDewC.setTitle(_translate("MainWindow", "Dew heater C"))
        self.label_282.setText(_translate("MainWindow", "%"))
        self.label_277.setText(_translate("MainWindow", "Current"))
        self.label_290.setText(_translate("MainWindow", "A"))
        self.label_278.setText(_translate("MainWindow", "DutyCycle"))
        self.dewC.setToolTip(_translate("MainWindow", "Sets the dew heater power from 0 to 100 %"))
        self.dewC.setText(_translate("MainWindow", "-"))
        self.groupAutoDew.setTitle(_translate("MainWindow", "Automatic dew"))
        self.autoDew.setToolTip(_translate("MainWindow", "Toggles the auto dew setting."))
        self.autoDew.setText(_translate("MainWindow", "Auto Dew"))
        self.groupPortUSB.setTitle(_translate("MainWindow", "USB Port"))
        self.portUSB1.setToolTip(_translate("MainWindow", "Switches the USB Hub on and off"))
        self.portUSB1.setText(_translate("MainWindow", "USB 1"))
        self.portUSB2.setToolTip(_translate("MainWindow", "Switches the USB Hub on and off"))
        self.portUSB2.setText(_translate("MainWindow", "USB 2"))
        self.portUSB3.setToolTip(_translate("MainWindow", "Switches the USB Hub on and off"))
        self.portUSB3.setText(_translate("MainWindow", "USB 3"))
        self.portUSB4.setToolTip(_translate("MainWindow", "Switches the USB Hub on and off"))
        self.portUSB4.setText(_translate("MainWindow", "USB 4"))
        self.portUSB5.setToolTip(_translate("MainWindow", "Switches the USB Hub on and off"))
        self.portUSB5.setText(_translate("MainWindow", "USB 5"))
        self.portUSB6.setToolTip(_translate("MainWindow", "Switches the USB Hub on and off"))
        self.portUSB6.setText(_translate("MainWindow", "USB 6"))
        self.groupAdjustableOutput.setTitle(_translate("MainWindow", "Adjustable output"))
        self.adjustableOutput.setToolTip(_translate("MainWindow", "Sets the dew heater power from 0 to 100 %"))
        self.adjustableOutput.setText(_translate("MainWindow", "-"))
        self.label_291.setText(_translate("MainWindow", "Voltage"))
        self.label_293.setText(_translate("MainWindow", "V"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Power), _translate("MainWindow", "Power"))
        self.groupBox_29.setToolTip(_translate("MainWindow", "Slews and parks the mount to a preprogrammed position."))
        self.groupBox_29.setTitle(_translate("MainWindow", "Defined positions (slew & park)"))
        self.posButton0.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton0.setText(_translate("MainWindow", "Button 0"))
        self.posButton8.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton8.setText(_translate("MainWindow", "Button 8"))
        self.posButton1.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton1.setText(_translate("MainWindow", "Button 1"))
        self.posButton6.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton6.setText(_translate("MainWindow", "Button 6"))
        self.posButton9.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton9.setText(_translate("MainWindow", "Button 9"))
        self.posButton3.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton3.setText(_translate("MainWindow", "Button 3"))
        self.posButton4.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton4.setText(_translate("MainWindow", "Button 4"))
        self.posButton5.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton5.setText(_translate("MainWindow", "Button 5"))
        self.posButton7.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton7.setText(_translate("MainWindow", "Button 7"))
        self.posButton2.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton2.setText(_translate("MainWindow", "Button 2"))
        self.parkMountAfterSlew.setToolTip(_translate("MainWindow", "<html><head/><body><p>If checked, the mount will be parked after slew and settling time.</p></body></html>"))
        self.parkMountAfterSlew.setText(_translate("MainWindow", "Park mount after slew"))
        self.groupBox_46.setTitle(_translate("MainWindow", "Batch renaming fits files"))
        self.rename1.setToolTip(_translate("MainWindow", "Chooses the FITS header Entry for renaming."))
        self.label_306.setText(_translate("MainWindow", "Textstring"))
        self.rename2.setToolTip(_translate("MainWindow", "Chooses the FITS header Entry for renaming."))
        self.label_305.setText(_translate("MainWindow", "Directory"))
        self.renameInputSelect.setToolTip(_translate("MainWindow", "Select the directory for file manipulations."))
        self.renameInputSelect.setText(_translate("MainWindow", "Select"))
        self.label_8.setText(_translate("MainWindow", "Position 1"))
        self.label_60.setText(_translate("MainWindow", "Choice of FITS header entries"))
        self.label_9.setText(_translate("MainWindow", "Position 2"))
        self.checkIncludeSubdirs.setToolTip(_translate("MainWindow", "If checked, the renaming process will run through all subdirectories."))
        self.checkIncludeSubdirs.setText(_translate("MainWindow", "Include subdirectories"))
        self.rename5.setToolTip(_translate("MainWindow", "Chooses the FITS header Entry for renaming."))
        self.rename4.setToolTip(_translate("MainWindow", "Chooses the FITS header Entry for renaming."))
        self.label_57.setText(_translate("MainWindow", "Progress"))
        self.label_10.setText(_translate("MainWindow", "Position 3"))
        self.label_11.setText(_translate("MainWindow", "Position 4"))
        self.renameStart.setToolTip(_translate("MainWindow", "Starts the renaming process. There will be no backup files made. Please take care about your raw files !"))
        self.renameStart.setText(_translate("MainWindow", "Start"))
        self.label_39.setText(_translate("MainWindow", "Position 5"))
        self.rename3.setToolTip(_translate("MainWindow", "Chooses the FITS header Entry for renaming."))
        self.label_308.setText(_translate("MainWindow", "New Object Name"))
        self.moveNorth.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveNorth.setText(_translate("MainWindow", "N"))
        self.moveNorthWest.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveNorthWest.setText(_translate("MainWindow", "NW"))
        self.moveWest.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveWest.setText(_translate("MainWindow", "W"))
        self.moveSouthEast.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveSouthEast.setText(_translate("MainWindow", "SE"))
        self.moveSouth.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveSouth.setText(_translate("MainWindow", "S"))
        self.moveNorthEast.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveNorthEast.setText(_translate("MainWindow", "NE"))
        self.moveSouthWest.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveSouthWest.setText(_translate("MainWindow", "SW"))
        self.stopMoveAll.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.stopMoveAll.setText(_translate("MainWindow", "Stop"))
        self.moveEast.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveEast.setText(_translate("MainWindow", "E"))
        self.groupBox_20.setTitle(_translate("MainWindow", "Speed"))
        self.slewSpeedMax.setToolTip(_translate("MainWindow", "<html><head/><body><p>Mount will move on max speed.</p></body></html>"))
        self.slewSpeedMax.setText(_translate("MainWindow", "Max"))
        self.slewSpeedMed.setToolTip(_translate("MainWindow", "<html><head/><body><p>Mount will move on med speed.</p></body></html>"))
        self.slewSpeedMed.setText(_translate("MainWindow", "Med"))
        self.slewSpeedLow.setToolTip(_translate("MainWindow", "<html><head/><body><p>Mount will move on low speed.</p></body></html>"))
        self.slewSpeedLow.setText(_translate("MainWindow", "Low"))
        self.slewSpeedHigh.setToolTip(_translate("MainWindow", "<html><head/><body><p>Mount will move on high speed.</p></body></html>"))
        self.slewSpeedHigh.setText(_translate("MainWindow", "High"))
        self.moveDuration.setToolTip(_translate("MainWindow", "<html><head/><body><p>Selects the time window for movements in RaDec direction</p></body></html>"))
        self.moveDuration.setItemText(0, _translate("MainWindow", "Duration continous"))
        self.moveDuration.setItemText(1, _translate("MainWindow", "Duration 10 s"))
        self.moveDuration.setItemText(2, _translate("MainWindow", "Duration 5 s"))
        self.moveDuration.setItemText(3, _translate("MainWindow", "Duration 2 s"))
        self.moveDuration.setItemText(4, _translate("MainWindow", "Duration 1 s"))
        self.tabMoving.setTabText(self.tabMoving.indexOf(self.tab_4), _translate("MainWindow", "Move Classic"))
        self.moveWestAltAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveWestAltAz.setText(_translate("MainWindow", "Left"))
        self.moveEastAltAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveEastAltAz.setText(_translate("MainWindow", "Right"))
        self.moveSouthWestAltAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveSouthWestAltAz.setText(_translate("MainWindow", "DL"))
        self.moveSouthAltAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveSouthAltAz.setText(_translate("MainWindow", "Down"))
        self.moveNorthWestAltAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveNorthWestAltAz.setText(_translate("MainWindow", "UL"))
        self.moveNorthAltAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveNorthAltAz.setText(_translate("MainWindow", "Up"))
        self.moveNorthEastAltAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveNorthEastAltAz.setText(_translate("MainWindow", "UR"))
        self.moveSouthEastAltAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined direction.</p></body></html>"))
        self.moveSouthEastAltAz.setText(_translate("MainWindow", "DR"))
        self.moveStepSizeAltAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Selects the stepsize for movements in Alt / Az direction</p></body></html>"))
        self.moveStepSizeAltAz.setItemText(0, _translate("MainWindow", "Stepsize 0.5 °"))
        self.moveStepSizeAltAz.setItemText(1, _translate("MainWindow", "Stepsize 1.0 °"))
        self.moveStepSizeAltAz.setItemText(2, _translate("MainWindow", "Stepsize 2.0 °"))
        self.moveStepSizeAltAz.setItemText(3, _translate("MainWindow", "Stepsize 5.0 °"))
        self.moveStepSizeAltAz.setItemText(4, _translate("MainWindow", "Stepsize 10.0 °"))
        self.tabMoving.setTabText(self.tabMoving.indexOf(self.tab_5), _translate("MainWindow", "Move Alt/Az"))
        self.filterMinorPlanet_5.setToolTip(_translate("MainWindow", "<html><head/><body><p>Filter for selecting minor planet names. The filter is not case sensitive.</p></body></html>"))
        self.groupBox.setTitle(_translate("MainWindow", "Alt/Az"))
        self.moveCoordinateAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Input value for azimuth direct slew setting. There is a range and consitence check in place.</p><p>Format is: &lt;[+-]d.d&gt; in degrees</p></body></html>"))
        self.moveAltAzAbsolute.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the given coordinate Alt &amp; Az</p></body></html>"))
        self.moveAltAzAbsolute.setText(_translate("MainWindow", "Slew to ALT / AZ"))
        self.moveCoordinateAlt.setToolTip(_translate("MainWindow", "<html><head/><body><p>Input value for alitude direct slew setting. There is a range and consitence check in place.</p><p>Format is: &lt;[+-]d.d&gt; in degrees</p></body></html>"))
        self.label_315.setText(_translate("MainWindow", "ALT"))
        self.label_316.setText(_translate("MainWindow", "AZ"))
        self.label_325.setText(_translate("MainWindow", "[+ -] d.d degrees"))
        self.label_326.setText(_translate("MainWindow", "[+ -] d.d degrees"))
        self.groupBox_23.setTitle(_translate("MainWindow", "RA/DEC"))
        self.moveCoordinateDec.setToolTip(_translate("MainWindow", "<html><head/><body><p>Input value for declination direct slew setting. There is a range and consitence check in place.</p></body></html>"))
        self.moveCoordinateRa.setToolTip(_translate("MainWindow", "<html><head/><body><p>Input value for right ascension direct slew setting. There is a range and consitence check in place.</p></body></html>"))
        self.moveRaDecAbsolute.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the given coordinate Ra &amp; Dec</p></body></html>"))
        self.moveRaDecAbsolute.setText(_translate("MainWindow", "Slew to RA / DEC"))
        self.label_323.setText(_translate("MainWindow", "RA"))
        self.label_324.setText(_translate("MainWindow", "DEC"))
        self.moveCoordinateRaFloat.setToolTip(_translate("MainWindow", "<html><head/><body><p>Input value for right ascension direct slew setting. There is a range and consitence check in place.</p></body></html>"))
        self.moveCoordinateDecFloat.setToolTip(_translate("MainWindow", "<html><head/><body><p>Input value for declination direct slew setting. There is a range and consitence check in place.</p></body></html>"))
        self.tabMoving.setTabText(self.tabMoving.indexOf(self.tab_6), _translate("MainWindow", "Move Absolute"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Tools), _translate("MainWindow", "Tools"))
        self.label_96.setText(_translate("MainWindow", "Relays"))
        self.relayButton0.setToolTip(_translate("MainWindow", "Starts for Relay the defined function (Switch or Pulse)"))
        self.relayButton0.setText(_translate("MainWindow", "Relay1"))
        self.relayButton1.setToolTip(_translate("MainWindow", "Starts for Relay the defined function (Switch or Pulse)"))
        self.relayButton1.setText(_translate("MainWindow", "Relay2"))
        self.relayButton2.setToolTip(_translate("MainWindow", "Starts for Relay the defined function (Switch or Pulse)"))
        self.relayButton2.setText(_translate("MainWindow", "Relay3"))
        self.relayButton3.setToolTip(_translate("MainWindow", "Starts for Relay the defined function (Switch or Pulse)"))
        self.relayButton3.setText(_translate("MainWindow", "Relay4"))
        self.relayButton4.setToolTip(_translate("MainWindow", "Starts for Relay the defined function (Switch or Pulse)"))
        self.relayButton4.setText(_translate("MainWindow", "Relay5"))
        self.relayButton5.setToolTip(_translate("MainWindow", "Starts for Relay the defined function (Switch or Pulse)"))
        self.relayButton5.setText(_translate("MainWindow", "Relay6"))
        self.relayButton6.setToolTip(_translate("MainWindow", "Starts for Relay the defined function (Switch or Pulse)"))
        self.relayButton6.setText(_translate("MainWindow", "Relay7"))
        self.relayButton7.setToolTip(_translate("MainWindow", "Starts for Relay the defined function (Switch or Pulse)"))
        self.relayButton7.setText(_translate("MainWindow", "Relay8"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Relay), _translate("MainWindow", "Relay"))
        self.groupBox_40.setTitle(_translate("MainWindow", "Accessories"))
        self.po.setText(_translate("MainWindow", "Power"))
        self.label_46.setText(_translate("MainWindow", "Relay"))
        self.label_48.setText(_translate("MainWindow", "Measure"))
        self.label_49.setText(_translate("MainWindow", "Remote"))
        self.label_53.setText(_translate("MainWindow", "Cover"))
        self.powerSetup.setToolTip(_translate("MainWindow", "Configure the pegasus UPB device. A popup will open and ask for the parameters."))
        self.powerSetup.setText(_translate("MainWindow", "Set"))
        self.coverSetup.setToolTip(_translate("MainWindow", "Configure the cover device. A popup will open and ask for the parameters."))
        self.coverSetup.setText(_translate("MainWindow", "Set"))
        self.relaySetup.setToolTip(_translate("MainWindow", "<html><head/><body><p>Configure the observing conditions part of the relay device. A popup will open and ask for the parameters.</p></body></html>"))
        self.relaySetup.setText(_translate("MainWindow", "Set"))
        self.groupBox_41.setTitle(_translate("MainWindow", "Core devices"))
        self.dome.setText(_translate("MainWindow", "Dome"))
        self.ccd.setText(_translate("MainWindow", "Camera"))
        self.dome_3.setText(_translate("MainWindow", "Telescope"))
        self.ast.setText(_translate("MainWindow", "Astrometry"))
        self.ccd_2.setText(_translate("MainWindow", "Filter"))
        self.dome_4.setText(_translate("MainWindow", "Focuser"))
        self.domeSetup.setToolTip(_translate("MainWindow", "Configure the dome device. A popup will open and ask for the parameters."))
        self.domeSetup.setText(_translate("MainWindow", "Set"))
        self.telescopeSetup.setToolTip(_translate("MainWindow", "Configure the telescope device. A popup will open and ask for the parameters."))
        self.telescopeSetup.setText(_translate("MainWindow", "Set"))
        self.focuserSetup.setToolTip(_translate("MainWindow", "Configure the focuser device. A popup will open and ask for the parameters."))
        self.focuserSetup.setText(_translate("MainWindow", "Set"))
        self.filterSetup.setToolTip(_translate("MainWindow", "Configure the filter device. A popup will open and ask for the parameters."))
        self.filterSetup.setText(_translate("MainWindow", "Set"))
        self.cameraSetup.setToolTip(_translate("MainWindow", "Configure the camera device. A popup will open and ask for the parameters."))
        self.cameraSetup.setText(_translate("MainWindow", "Set"))
        self.astrometrySetup.setToolTip(_translate("MainWindow", "Configure the filter device. A popup will open and ask for the parameters."))
        self.astrometrySetup.setText(_translate("MainWindow", "Set"))
        self.groupBox_42.setTitle(_translate("MainWindow", "Environment conditions"))
        self.sky.setText(_translate("MainWindow", "Skymeter"))
        self.env.setText(_translate("MainWindow", "Sensor"))
        self.env_2.setText(_translate("MainWindow", "Online"))
        self.env_3.setText(_translate("MainWindow", "Mount dir."))
        self.skymeterSetup.setToolTip(_translate("MainWindow", "Configure the skymeter device. A popup will open and ask for the parameters."))
        self.skymeterSetup.setText(_translate("MainWindow", "Set"))
        self.sensorWeatherSetup.setToolTip(_translate("MainWindow", "Configure the external sensor device. A popup will open and ask for the parameters."))
        self.sensorWeatherSetup.setText(_translate("MainWindow", "Set"))
        self.sky_2.setText(_translate("MainWindow", "Power"))
        self.powerWeatherSetup.setToolTip(_translate("MainWindow", "<html><head/><body><p>Configure the observing conditions part of the power device. A popup will open and ask for the parameters.</p></body></html>"))
        self.powerWeatherSetup.setText(_translate("MainWindow", "Set"))
        self.onlineWeatherSetup.setToolTip(_translate("MainWindow", "<html><head/><body><p>Configure the observing conditions part of the online weather device. A popup will open and ask for the parameters.</p></body></html>"))
        self.onlineWeatherSetup.setText(_translate("MainWindow", "Set"))
        self.groupBox_57.setTitle(_translate("MainWindow", "ASCOM connect devices"))
        self.ascomConnect.setToolTip(_translate("MainWindow", "<html><head/><body><p>For ASCOM! Connect all devices.</p></body></html>"))
        self.ascomConnect.setText(_translate("MainWindow", "Connect"))
        self.ascomDisconnect.setToolTip(_translate("MainWindow", "<html><head/><body><p>For ASCOM! Disconnect all devices.</p></body></html>"))
        self.ascomDisconnect.setText(_translate("MainWindow", "Disconnect"))
        self.checkASCOMAutoConnect.setToolTip(_translate("MainWindow", "<html><head/><body><p>If enabled, MW4 will connect all adcom devices automatically.</p></body></html>"))
        self.checkASCOMAutoConnect.setText(_translate("MainWindow", "Automatic connect devices"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.Devices), _translate("MainWindow", "Devices"))
        self.groupBox_25.setTitle(_translate("MainWindow", "Imaging settings"))
        self.label_299.setText(_translate("MainWindow", "Exposure"))
        self.label_294.setText(_translate("MainWindow", "s"))
        self.expTime.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the exposure time</p></body></html>"))
        self.label_41.setText(_translate("MainWindow", "Binning"))
        self.binning.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the binning for the camera</p></body></html>"))
        self.label_296.setText(_translate("MainWindow", "%"))
        self.subFrame.setToolTip(_translate("MainWindow", "<html><head/><body><p>sets the percentage of frame size for subframe </p></body></html>"))
        self.label_307.setText(_translate("MainWindow", "Subframe"))
        self.checkFastDownload.setToolTip(_translate("MainWindow", "Use Highspeed download of CCD camera if availabe"))
        self.checkFastDownload.setText(_translate("MainWindow", "Use fast image download for modeling (if present)"))
        self.checkKeepImages.setToolTip(_translate("MainWindow", "Checked if you would like to keep the images of modeling session"))
        self.checkKeepImages.setText(_translate("MainWindow", "Keep images after modeling"))
        self.coolerTemp.setToolTip(_translate("MainWindow", "Shows the actual cooler temp for the camera. When clicking you could change the value."))
        self.coolerTemp.setText(_translate("MainWindow", "-"))
        self.label_402.setText(_translate("MainWindow", "°"))
        self.label_400.setText(_translate("MainWindow", "Cooler temp"))
        self.coolerPower.setToolTip(_translate("MainWindow", "Shows the actual coller power."))
        self.coolerPower.setText(_translate("MainWindow", "-"))
        self.label_401.setText(_translate("MainWindow", "Power [%]"))
        self.downloadSlow.setToolTip(_translate("MainWindow", "Set readout speed for the camera to high quality if available."))
        self.downloadSlow.setText(_translate("MainWindow", "High Qual"))
        self.coolerOn.setToolTip(_translate("MainWindow", "Switches the camera cooler on."))
        self.coolerOn.setText(_translate("MainWindow", "Cooler On"))
        self.downloadFast.setToolTip(_translate("MainWindow", "Set readout speed for the camera to fast if available."))
        self.downloadFast.setText(_translate("MainWindow", "Fast"))
        self.coolerOff.setToolTip(_translate("MainWindow", "Switches the camera cooler off."))
        self.coolerOff.setText(_translate("MainWindow", "Cooler Off"))
        self.label_407.setText(_translate("MainWindow", "Cooler status"))
        self.label_408.setText(_translate("MainWindow", "Readout speed"))
        self.label_302.setText(_translate("MainWindow", "Exposure N"))
        self.expTimeN.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the exposure time</p></body></html>"))
        self.label_304.setText(_translate("MainWindow", "s"))
        self.binningN.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the binning for the camera</p></body></html>"))
        self.label_51.setText(_translate("MainWindow", "Binning"))
        self.groupBox_26.setTitle(_translate("MainWindow", "Imaging information readout and calculations"))
        self.label_166.setText(_translate("MainWindow", "Pixel size"))
        self.pixelSizeX.setToolTip(_translate("MainWindow", "Shows the pixel size set in the camera driver."))
        self.pixelSizeX.setText(_translate("MainWindow", "-"))
        self.label_379.setText(_translate("MainWindow", "Resolution"))
        self.label_380.setText(_translate("MainWindow", "um"))
        self.pixelSizeY.setToolTip(_translate("MainWindow", "Shows the pixel size set in the camera driver."))
        self.pixelSizeY.setText(_translate("MainWindow", "-"))
        self.resolutionX.setText(_translate("MainWindow", "-"))
        self.resolutionY.setText(_translate("MainWindow", "-"))
        self.label_381.setText(_translate("MainWindow", "x"))
        self.label_382.setText(_translate("MainWindow", "x"))
        self.label_383.setText(_translate("MainWindow", "arcsec per pixel"))
        self.label_384.setText(_translate("MainWindow", "FOV"))
        self.FOVX.setText(_translate("MainWindow", "-"))
        self.FOVY.setText(_translate("MainWindow", "-"))
        self.label_385.setText(_translate("MainWindow", "x"))
        self.label_386.setText(_translate("MainWindow", "degrees"))
        self.label_313.setText(_translate("MainWindow", "Pixels"))
        self.pixelY.setToolTip(_translate("MainWindow", "Shows the pixel number set in the camera driver."))
        self.pixelY.setText(_translate("MainWindow", "-"))
        self.pixelX.setToolTip(_translate("MainWindow", "Shows the pixel number set in the camera driver."))
        self.pixelX.setText(_translate("MainWindow", "-"))
        self.label_388.setText(_translate("MainWindow", "x"))
        self.rotation.setToolTip(_translate("MainWindow", "Shows the rotation of the camera image set in driver."))
        self.rotation.setText(_translate("MainWindow", "-"))
        self.label_391.setText(_translate("MainWindow", "Rotation"))
        self.label_405.setText(_translate("MainWindow", "Speed  f /"))
        self.speed.setToolTip(_translate("MainWindow", "Shows the speed of your imaging setup."))
        self.speed.setText(_translate("MainWindow", "-"))
        self.groupBox_32.setTitle(_translate("MainWindow", "Imaging quality calculations"))
        self.label_403.setText(_translate("MainWindow", "Rayleigh limit"))
        self.rayleigh.setToolTip(_translate("MainWindow", "Shows the actual Rayleigh limit for your current imaging setup and parameters"))
        self.rayleigh.setText(_translate("MainWindow", "-"))
        self.label_404.setText(_translate("MainWindow", "Magitude limit"))
        self.label_397.setText(_translate("MainWindow", "Dawes\' limit"))
        self.magLimit.setToolTip(_translate("MainWindow", "Shows the actual magnitude limit for your current imaging setup and parameters."))
        self.magLimit.setText(_translate("MainWindow", "-"))
        self.dawes.setToolTip(_translate("MainWindow", "Shows the actual Dawes\' limit for your current imaging setup and parameters"))
        self.dawes.setText(_translate("MainWindow", "-"))
        self.label_406.setText(_translate("MainWindow", "arcsec"))
        self.label_398.setText(_translate("MainWindow", "arcsec"))
        self.groupBox_33.setTitle(_translate("MainWindow", "Telescope information readout"))
        self.label_396.setText(_translate("MainWindow", "mm"))
        self.label_394.setText(_translate("MainWindow", "Aperture"))
        self.label_395.setText(_translate("MainWindow", "mm"))
        self.copyFromTelescopeDriver.setText(_translate("MainWindow", "Data from Driver"))
        self.label_387.setText(_translate("MainWindow", "Focal length"))
        self.focalLength.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the exposure time</p></body></html>"))
        self.aperture.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the exposure time</p></body></html>"))
        self.checkAutomaticTelescope.setToolTip(_translate("MainWindow", "If enabled, MW4 will calculate the dome shutter displacement based on the real geometry and azimuth of dome will be different to azimuth of mount."))
        self.checkAutomaticTelescope.setText(_translate("MainWindow", "Automatic takeover from driver"))
        self.groupBox_43.setTitle(_translate("MainWindow", "Flip Flat"))
        self.coverStatusText.setToolTip(_translate("MainWindow", "Shows the status of the cover."))
        self.coverStatusText.setText(_translate("MainWindow", "-"))
        self.coverPark.setText(_translate("MainWindow", "Close"))
        self.coverUnpark.setText(_translate("MainWindow", "Open"))
        self.coverHalt.setText(_translate("MainWindow", "Stop"))
        self.groupBox_34.setTitle(_translate("MainWindow", "Filterwheel"))
        self.label_389.setText(_translate("MainWindow", "Filter name"))
        self.filterNumber.setToolTip(_translate("MainWindow", "Shows the actual filter number.\n"
"When clicking the button, you could change the value."))
        self.filterNumber.setText(_translate("MainWindow", "-"))
        self.label_390.setText(_translate("MainWindow", "Filter number"))
        self.filterName.setToolTip(_translate("MainWindow", "Shows the actual filter name text.\n"
"When clicking the button, you could change the value."))
        self.filterName.setText(_translate("MainWindow", "-"))
        self.groupBox_35.setTitle(_translate("MainWindow", "Focuser"))
        self.label_410.setText(_translate("MainWindow", "Steps"))
        self.moveFocuserIn.setText(_translate("MainWindow", "In"))
        self.label_409.setText(_translate("MainWindow", "Position"))
        self.focuserPosition.setToolTip(_translate("MainWindow", "Shows the actual focuser position."))
        self.focuserPosition.setText(_translate("MainWindow", "-"))
        self.focuserStepsize.setToolTip(_translate("MainWindow", "<html><head/><body><p>sets the percentage of frame size for subframe </p></body></html>"))
        self.moveFocuserOut.setText(_translate("MainWindow", "Out"))
        self.haltFocuser.setText(_translate("MainWindow", "Stop"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.Imaging), _translate("MainWindow", "Imaging"))
        self.groupBox_62.setTitle(_translate("MainWindow", "Mount connectivity"))
        self.mountHost.setToolTip(_translate("MainWindow", "<html><head/><body><p>IP address of mount, ip address and host names!</p></body></html>"))
        self.mountHost.setText(_translate("MainWindow", "127.0.0.1"))
        self.label_mountIP.setText(_translate("MainWindow", "IP Address"))
        self.label_mountMAC.setText(_translate("MainWindow", "MAC Address"))
        self.mountMAC.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">MAC address of the mount</span></p></body></html>"))
        self.mountMAC.setText(_translate("MainWindow", "00:00:00:00:00:00"))
        self.port3492.setToolTip(_translate("MainWindow", "<html><head/><body><p>Changing to an alternative port. 10micron only allows 10 connections at the same time on one port. If you have other equipment, it might be helpful changing the port.</p></body></html>"))
        self.port3492.setText(_translate("MainWindow", "3492"))
        self.port3490.setToolTip(_translate("MainWindow", "<html><head/><body><p>Changing to an alternative port. 10micron only allows 10 connections at the same time on one port. If you have other equipment, it might be helpful changing the port.</p></body></html>"))
        self.port3490.setText(_translate("MainWindow", "3490"))
        self.label_mountMAC_3.setText(_translate("MainWindow", "Port"))
        self.groupBox_63.setTitle(_translate("MainWindow", "Computer on rig"))
        self.label_259.setText(_translate("MainWindow", "MAC Address"))
        self.bootRackComp.setText(_translate("MainWindow", "Boot / WOL"))
        self.rackCompMAC.setText(_translate("MainWindow", "00:00:00:00:00:00"))
        self.groupBox_31.setTitle(_translate("MainWindow", "Settle Times"))
        self.label_40.setText(_translate("MainWindow", "Mount settle time"))
        self.label_295.setText(_translate("MainWindow", "s"))
        self.settleTimeMount.setToolTip(_translate("MainWindow", "Sets the time to wait after mount stop before imaging"))
        self.FirmwareGroup.setTitle(_translate("MainWindow", "Firmware data"))
        self.fwtime.setText(_translate("MainWindow", "-"))
        self.firmwareProductName_3.setText(_translate("MainWindow", "FW:"))
        self.firmwareProductName_4.setText(_translate("MainWindow", "Date:"))
        self.firmwareProductName_5.setText(_translate("MainWindow", "Time:"))
        self.product.setText(_translate("MainWindow", "-"))
        self.fwdate.setText(_translate("MainWindow", "-"))
        self.firmwareProductName_2.setText(_translate("MainWindow", "Product:"))
        self.hardware.setText(_translate("MainWindow", "-"))
        self.Hardware.setText(_translate("MainWindow", "HW:"))
        self.vString.setText(_translate("MainWindow", "-"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.MountRemote), _translate("MainWindow", "Mount"))
        self.groupBox_44.setTitle(_translate("MainWindow", "Dome geometry"))
        self.label_99.setText(_translate("MainWindow", "Radius"))
        self.label_134.setText(_translate("MainWindow", "m"))
        self.label_122.setText(_translate("MainWindow", "m"))
        self.label_108.setText(_translate("MainWindow", "East offset"))
        self.label_110.setText(_translate("MainWindow", "Vertical offset (Dome center to GEM)"))
        self.label_116.setText(_translate("MainWindow", "m"))
        self.checkUseDomeGeometry.setToolTip(_translate("MainWindow", "If enabled, MW4 will calculate the dome shutter displacement based on the real geometry and azimuth of dome will be different to azimuth of mount."))
        self.checkUseDomeGeometry.setText(_translate("MainWindow", "Enable dome geometry offset calculations"))
        self.label_118.setText(_translate("MainWindow", "m"))
        self.label_123.setText(_translate("MainWindow", "m"))
        self.label_107.setText(_translate("MainWindow", "North offset"))
        self.label_135.setText(_translate("MainWindow", "GEM offset"))
        self.copyFromDomeDriver.setText(_translate("MainWindow", "Data from Driver"))
        self.checkAutomaticDome.setToolTip(_translate("MainWindow", "If enabled, MW4 will calculate the dome shutter displacement based on the real geometry and azimuth of dome will be different to azimuth of mount."))
        self.checkAutomaticDome.setText(_translate("MainWindow", "Automatic takeover from driver"))
        self.label_136.setText(_translate("MainWindow", "m"))
        self.label_137.setText(_translate("MainWindow", "Lateral offset"))
        self.label_124.setText(_translate("MainWindow", "m"))
        self.label_113.setText(_translate("MainWindow", "Vertical offset defined in 10micron"))
        self.groupBox_66.setTitle(_translate("MainWindow", "Settle Times"))
        self.settleTimeDome.setToolTip(_translate("MainWindow", "Sets the time to wait after mount stop before imaging"))
        self.label_100.setText(_translate("MainWindow", "Dome settle time"))
        self.label_314.setText(_translate("MainWindow", "s"))
        self.picDome1.setText(_translate("MainWindow", "TextLabel"))
        self.tabDomeExplain.setTabText(self.tabDomeExplain.indexOf(self.tab_9), _translate("MainWindow", "1"))
        self.picDome2.setText(_translate("MainWindow", "TextLabel"))
        self.tabDomeExplain.setTabText(self.tabDomeExplain.indexOf(self.tab_10), _translate("MainWindow", "2"))
        self.picDome3.setText(_translate("MainWindow", "TextLabel"))
        self.tabDomeExplain.setTabText(self.tabDomeExplain.indexOf(self.tab_11), _translate("MainWindow", "3"))
        self.picDome4.setText(_translate("MainWindow", "TextLabel"))
        self.tabDomeExplain.setTabText(self.tabDomeExplain.indexOf(self.tab_12), _translate("MainWindow", "4"))
        self.picDome5.setText(_translate("MainWindow", "TextLabel"))
        self.tabDomeExplain.setTabText(self.tabDomeExplain.indexOf(self.tab_13), _translate("MainWindow", "5"))
        self.picDome6.setText(_translate("MainWindow", "TextLabel"))
        self.tabDomeExplain.setTabText(self.tabDomeExplain.indexOf(self.tab_14), _translate("MainWindow", "6"))
        self.picDome7.setText(_translate("MainWindow", "TextLabel"))
        self.tabDomeExplain.setTabText(self.tabDomeExplain.indexOf(self.tab_15), _translate("MainWindow", "7"))
        self.picDome8.setText(_translate("MainWindow", "TextLabel"))
        self.tabDomeExplain.setTabText(self.tabDomeExplain.indexOf(self.tab_16), _translate("MainWindow", "8"))
        self.groupBox_53.setTitle(_translate("MainWindow", "Dome shutter"))
        self.domeOpenShutter.setToolTip(_translate("MainWindow", "<html><head/><body><p>Closing the dome shutter</p><p><br/></p></body></html>"))
        self.domeOpenShutter.setText(_translate("MainWindow", "Open"))
        self.domeCloseShutter.setToolTip(_translate("MainWindow", "<html><head/><body><p>Opening the dome shutter</p></body></html>"))
        self.domeCloseShutter.setText(_translate("MainWindow", "Close"))
        self.label_139.setText(_translate("MainWindow", "Dome shutter width"))
        self.label_140.setText(_translate("MainWindow", "m"))
        self.domeAbortSlew.setToolTip(_translate("MainWindow", "<html><head/><body><p>Opening the dome shutter</p></body></html>"))
        self.domeAbortSlew.setText(_translate("MainWindow", "Stop"))
        self.domeShutterStatusText.setToolTip(_translate("MainWindow", "Shows the status of the cover."))
        self.domeShutterStatusText.setText(_translate("MainWindow", "-"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.Dome), _translate("MainWindow", "Dome"))
        self.groupBox_45.setTitle(_translate("MainWindow", "Configuration park positions"))
        self.label_243.setText(_translate("MainWindow", "°"))
        self.label_262.setText(_translate("MainWindow", "°"))
        self.label_250.setText(_translate("MainWindow", "°"))
        self.posAz6.setInputMask(_translate("MainWindow", "999"))
        self.label_244.setText(_translate("MainWindow", "°"))
        self.posText4.setToolTip(_translate("MainWindow", "Button text park menu 4"))
        self.posText4.setText(_translate("MainWindow", "Position 4"))
        self.posAz1.setInputMask(_translate("MainWindow", "999"))
        self.posAz2.setInputMask(_translate("MainWindow", "999"))
        self.label_101.setText(_translate("MainWindow", "6"))
        self.posAlt1.setInputMask(_translate("MainWindow", "#99"))
        self.posAlt1.setText(_translate("MainWindow", "-"))
        self.label_92.setText(_translate("MainWindow", "Button"))
        self.posText2.setToolTip(_translate("MainWindow", "Button text park menu 2"))
        self.posText2.setText(_translate("MainWindow", "Position 2"))
        self.label_93.setText(_translate("MainWindow", "°"))
        self.posText0.setToolTip(_translate("MainWindow", "Button text park menu 0"))
        self.posText0.setText(_translate("MainWindow", "Position 0"))
        self.posAlt3.setInputMask(_translate("MainWindow", "#99"))
        self.posAlt3.setText(_translate("MainWindow", "-"))
        self.label_257.setText(_translate("MainWindow", "°"))
        self.label_261.setText(_translate("MainWindow", "°"))
        self.posText3.setToolTip(_translate("MainWindow", "Button text park menu 3"))
        self.posText3.setText(_translate("MainWindow", "Position 3"))
        self.label_132.setText(_translate("MainWindow", "Azimuth"))
        self.posText5.setToolTip(_translate("MainWindow", "Button text park menu 5"))
        self.posText5.setText(_translate("MainWindow", "Position 5"))
        self.label_47.setText(_translate("MainWindow", "3"))
        self.label_249.setText(_translate("MainWindow", "°"))
        self.label_103.setText(_translate("MainWindow", "7"))
        self.posAlt0.setInputMask(_translate("MainWindow", "#99"))
        self.posAlt0.setText(_translate("MainWindow", "-"))
        self.posAlt5.setInputMask(_translate("MainWindow", "#99"))
        self.posAlt5.setText(_translate("MainWindow", "-"))
        self.posAlt4.setInputMask(_translate("MainWindow", "#99"))
        self.posAlt4.setText(_translate("MainWindow", "-"))
        self.label_247.setText(_translate("MainWindow", "°"))
        self.posAlt7.setInputMask(_translate("MainWindow", "#99"))
        self.posAlt7.setText(_translate("MainWindow", "-"))
        self.label_131.setText(_translate("MainWindow", "°"))
        self.label_258.setText(_translate("MainWindow", "°"))
        self.label_50.setText(_translate("MainWindow", "4"))
        self.label_209.setText(_translate("MainWindow", "°"))
        self.posText7.setToolTip(_translate("MainWindow", "Button text park menu 7"))
        self.posText7.setText(_translate("MainWindow", "Position 7"))
        self.label_97.setText(_translate("MainWindow", "Text"))
        self.label_246.setText(_translate("MainWindow", "°"))
        self.label_251.setText(_translate("MainWindow", "°"))
        self.posAz7.setInputMask(_translate("MainWindow", "999"))
        self.posText1.setToolTip(_translate("MainWindow", "Button text park menu 1"))
        self.posText1.setText(_translate("MainWindow", "Position 1"))
        self.posAz3.setInputMask(_translate("MainWindow", "999"))
        self.label_2.setText(_translate("MainWindow", "1"))
        self.label_245.setText(_translate("MainWindow", "°"))
        self.label_95.setText(_translate("MainWindow", "5"))
        self.posAlt2.setInputMask(_translate("MainWindow", "#99"))
        self.posAlt2.setText(_translate("MainWindow", "-"))
        self.label_44.setText(_translate("MainWindow", "2"))
        self.label_94.setText(_translate("MainWindow", "Altitude"))
        self.posAz4.setInputMask(_translate("MainWindow", "999"))
        self.label_248.setText(_translate("MainWindow", "°"))
        self.posText6.setToolTip(_translate("MainWindow", "Button text park menu 6"))
        self.posText6.setText(_translate("MainWindow", "Position 6"))
        self.posAz5.setInputMask(_translate("MainWindow", "999"))
        self.posAlt6.setInputMask(_translate("MainWindow", "#99"))
        self.posAlt6.setText(_translate("MainWindow", "-"))
        self.posAz0.setInputMask(_translate("MainWindow", "999"))
        self.label_22.setText(_translate("MainWindow", "0"))
        self.posSave0.setToolTip(_translate("MainWindow", "Fill in the actual values for altitude and azimuth in the value fields."))
        self.posSave0.setText(_translate("MainWindow", "Fill"))
        self.posSave1.setToolTip(_translate("MainWindow", "Fill in the actual values for altitude and azimuth in the value fields."))
        self.posSave1.setText(_translate("MainWindow", "Fill"))
        self.posSave2.setToolTip(_translate("MainWindow", "Fill in the actual values for altitude and azimuth in the value fields."))
        self.posSave2.setText(_translate("MainWindow", "Fill"))
        self.posSave3.setToolTip(_translate("MainWindow", "Fill in the actual values for altitude and azimuth in the value fields."))
        self.posSave3.setText(_translate("MainWindow", "Fill"))
        self.posSave4.setToolTip(_translate("MainWindow", "Fill in the actual values for altitude and azimuth in the value fields."))
        self.posSave4.setText(_translate("MainWindow", "Fill"))
        self.posSave5.setToolTip(_translate("MainWindow", "Fill in the actual values for altitude and azimuth in the value fields."))
        self.posSave5.setText(_translate("MainWindow", "Fill"))
        self.posSave6.setToolTip(_translate("MainWindow", "Fill in the actual values for altitude and azimuth in the value fields."))
        self.posSave6.setText(_translate("MainWindow", "Fill"))
        self.posSave7.setToolTip(_translate("MainWindow", "Fill in the actual values for altitude and azimuth in the value fields."))
        self.posSave7.setText(_translate("MainWindow", "Fill"))
        self.label_127.setText(_translate("MainWindow", "8"))
        self.label_128.setText(_translate("MainWindow", "9"))
        self.posText8.setToolTip(_translate("MainWindow", "Button text park menu 8"))
        self.posText8.setText(_translate("MainWindow", "Position 8"))
        self.posText9.setToolTip(_translate("MainWindow", "Button text park menu 9"))
        self.posText9.setText(_translate("MainWindow", "Position 9"))
        self.label_263.setText(_translate("MainWindow", "°"))
        self.posAlt8.setInputMask(_translate("MainWindow", "#99"))
        self.posAlt8.setText(_translate("MainWindow", "-"))
        self.posAz8.setInputMask(_translate("MainWindow", "999"))
        self.label_264.setText(_translate("MainWindow", "°"))
        self.posSave8.setToolTip(_translate("MainWindow", "Fill in the actual values for altitude and azimuth in the value fields."))
        self.posSave8.setText(_translate("MainWindow", "Fill"))
        self.label_265.setText(_translate("MainWindow", "°"))
        self.posAlt9.setInputMask(_translate("MainWindow", "#99"))
        self.posAlt9.setText(_translate("MainWindow", "-"))
        self.posAz9.setInputMask(_translate("MainWindow", "999"))
        self.label_267.setText(_translate("MainWindow", "°"))
        self.posSave9.setToolTip(_translate("MainWindow", "Fill in the actual values for altitude and azimuth in the value fields."))
        self.posSave9.setText(_translate("MainWindow", "Fill"))
        self.groupBox_96.setTitle(_translate("MainWindow", "Relay button setup"))
        self.label_149.setText(_translate("MainWindow", "5"))
        self.relayButtonText2.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText2.setText(_translate("MainWindow", "relais 3"))
        self.relayButtonText5.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText5.setText(_translate("MainWindow", "relais 6"))
        self.relayButtonText7.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText7.setText(_translate("MainWindow", "relais 8"))
        self.label_220.setText(_translate("MainWindow", "4"))
        self.label_129.setText(_translate("MainWindow", "3"))
        self.label_211.setText(_translate("MainWindow", "2"))
        self.relayButtonText4.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText4.setText(_translate("MainWindow", "relais 5"))
        self.relayButtonText1.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText1.setText(_translate("MainWindow", "relais 2"))
        self.relayButtonText6.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText6.setText(_translate("MainWindow", "relais 7"))
        self.label_255.setText(_translate("MainWindow", "8"))
        self.label_56.setText(_translate("MainWindow", "Relay"))
        self.label_199.setText(_translate("MainWindow", "7"))
        self.label_198.setText(_translate("MainWindow", "Function"))
        self.label_43.setText(_translate("MainWindow", "1"))
        self.label_45.setText(_translate("MainWindow", "Text"))
        self.relayButtonText3.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText3.setText(_translate("MainWindow", "relais 4"))
        self.relayButtonText0.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText0.setText(_translate("MainWindow", "relais 1"))
        self.label_256.setText(_translate("MainWindow", "6"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.ParkPos), _translate("MainWindow", "ParkPositions / Relay"))
        self.LoglevelGroup.setTitle(_translate("MainWindow", "Logging"))
        self.loglevelDebug.setToolTip(_translate("MainWindow", "Debug logging will be stored in log file."))
        self.loglevelDebug.setText(_translate("MainWindow", "Debug"))
        self.loglevelStandard.setToolTip(_translate("MainWindow", "Minimum logging will be stored in log file."))
        self.loglevelStandard.setText(_translate("MainWindow", "Standard"))
        self.loglevelDebugTrace.setToolTip(_translate("MainWindow", "Debug logging and communication will be stored in log file."))
        self.loglevelDebugTrace.setText(_translate("MainWindow", "Debug & Trace"))
        self.CopyrightGroup.setTitle(_translate("MainWindow", "MountWizzard4 (C) 2019-2021 MWORION"))
        self.textEdit.setHtml(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Arial\',\'Arial\',\'Arial\',\'Arial\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Arial\';\">pyqt5, matplotlib, wakeonlan, numpy, requests, astropy, skyfield, qimage2ndarray, importlib_metadata, scipy, sep, colour_demosaicing, deepdiff, pywin32, pywinauto</span></p></body></html>"))
        self.AudioGroup.setTitle(_translate("MainWindow", "Play audio signals"))
        self.soundMountAlert.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the audio signal to be played when mount stopped in emergency case</p></body></html>"))
        self.label_217.setText(_translate("MainWindow", "Modeling finished"))
        self.soundDomeSlewFinished.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the audio signal to be played when dome finished slewing</p></body></html>"))
        self.label_214.setText(_translate("MainWindow", "Dome slew finished"))
        self.label_212.setText(_translate("MainWindow", "Mount slew finished"))
        self.soundModelingFinished.setToolTip(_translate("MainWindow", "Sets the audio signal to be played when model build is finished"))
        self.label_215.setText(_translate("MainWindow", "Mount alert for STOP"))
        self.soundMountSlewFinished.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the audio signal to be played when mount finished slewing</p></body></html>"))
        self.label_219.setText(_translate("MainWindow", "Image saved"))
        self.soundImageSaved.setToolTip(_translate("MainWindow", "Sets the audio signal to be played when image is saved"))
        self.label_223.setText(_translate("MainWindow", "Image solved"))
        self.soundImageSolved.setToolTip(_translate("MainWindow", "Sets the audio signal to be played when image is plate solved"))
        self.InternetOnlineGroup.setTitle(_translate("MainWindow", "Internet online connection"))
        self.isOnline.setToolTip(_translate("MainWindow", "When selected data fiels for weather, satellites and programm updates will be loaded over internet."))
        self.isOnline.setText(_translate("MainWindow", "online and update data files if necessary"))
        self.UpdaterGroup.setTitle(_translate("MainWindow", "Updater"))
        self.installVersion.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starting process for updating MountWizzard4 to selected version. The button remains blue as long as the installations is running.</p></body></html>"))
        self.installVersion.setText(_translate("MainWindow", "Install"))
        self.label_23.setText(_translate("MainWindow", "Actual version"))
        self.label_38.setText(_translate("MainWindow", "Available version"))
        self.versionBeta.setToolTip(_translate("MainWindow", "Show only beta versions of MountWizzard4"))
        self.versionBeta.setText(_translate("MainWindow", "Show beta releases"))
        self.versionRelease.setToolTip(_translate("MainWindow", "Show only release versions of MountWizzard4"))
        self.versionRelease.setText(_translate("MainWindow", "Show releases"))
        self.versionActual.setText(_translate("MainWindow", "-"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.AudioMisc), _translate("MainWindow", "Misc / Audio"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Settings), _translate("MainWindow", "Settings"))
        self.cancelAnalyse.setToolTip(_translate("MainWindow", "<html><head/><body><p>Cancels the analyse run.</p></body></html>"))
        self.cancelAnalyse.setText(_translate("MainWindow", "Cancel Analyse Run"))
        self.groupBox_14.setTitle(_translate("MainWindow", "Load analyse file"))
        self.le_analyseFileName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Analysis file, where the data ist stored.</p><p>Please Double Click to select.</p></body></html>"))
        self.loadAnalyseData.setToolTip(_translate("MainWindow", "<html><head/><body><p>Loads the modeling data from file.</p></body></html>"))
        self.loadAnalyseData.setText(_translate("MainWindow", "Load"))
        self.groupBox_15.setTitle(_translate("MainWindow", "Analyse flexure"))
        self.label_177.setText(_translate("MainWindow", "Delay"))
        self.label_111.setText(_translate("MainWindow", "°"))
        self.label_174.setText(_translate("MainWindow", "Altitude"))
        self.label_112.setText(_translate("MainWindow", "°"))
        self.label_176.setText(_translate("MainWindow", "Number Cycles"))
        self.label_115.setText(_translate("MainWindow", "s"))
        self.label_175.setText(_translate("MainWindow", "Azimuth"))
        self.runFlexure.setToolTip(_translate("MainWindow", "<html><head/><body><p>Runs a analyse cycle by constant pointing to given alt/az and measuring the error each delay seconds.</p></body></html>"))
        self.runFlexure.setText(_translate("MainWindow", "Run Flexure"))
        self.groupBox_16.setTitle(_translate("MainWindow", "Analyse hysterese"))
        self.label_182.setText(_translate("MainWindow", "Azimuth"))
        self.label_183.setText(_translate("MainWindow", "Number Cycles"))
        self.label_119.setText(_translate("MainWindow", "°"))
        self.label_120.setText(_translate("MainWindow", "°"))
        self.label_185.setText(_translate("MainWindow", "Point 1    Altitude"))
        self.label_187.setText(_translate("MainWindow", "Point 2    Altitude"))
        self.label_184.setText(_translate("MainWindow", "Delay"))
        self.label_114.setText(_translate("MainWindow", "°"))
        self.label_117.setText(_translate("MainWindow", "s"))
        self.label_186.setText(_translate("MainWindow", "Azimuth"))
        self.label_121.setText(_translate("MainWindow", "°"))
        self.runHysteresis.setToolTip(_translate("MainWindow", "<html><head/><body><p>Runs a analyse cycle by swapping two alt/az positions and measuring the error each delay seconds</p></body></html>"))
        self.runHysteresis.setText(_translate("MainWindow", "Run Hysteresis"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Analyse), _translate("MainWindow", "Analyse"))
        self.status.setTitle(_translate("MainWindow", "Status"))
        self.astrometryText.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status feedback from solver source</p></body></html>"))
        self.statusText.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.statusText.setText(_translate("MainWindow", "-"))
        self.environConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of environment: </p><p>Red: Error</p><p>Gray: No Configured</p><p>Green: Connected</p></body></html>"))
        self.environConnected.setText(_translate("MainWindow", "Environ"))
        self.domeConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of dome: </p><p>Red: Error</p><p>Gray: No Configured</p><p>Green: Connected</p></body></html>"))
        self.domeConnected.setText(_translate("MainWindow", "Dome"))
        self.openImageW.setToolTip(_translate("MainWindow", "Opens and closes the image window. If blue, the windows should be visible. Position and size will be stored in case of closing."))
        self.openImageW.setText(_translate("MainWindow", "Image"))
        self.cameraConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of camera:</p><p>Red: Error</p><p>Yellow: Connecting</p><p>Green: Connected</p><p>Gray: No Configured</p></body></html>"))
        self.cameraConnected.setText(_translate("MainWindow", "Camera"))
        self.openSatelliteW.setToolTip(_translate("MainWindow", "Opens and closes the satellite window. If blue, the windows should be visible. Position and size will be stored in case of closing."))
        self.openSatelliteW.setText(_translate("MainWindow", "Satellite"))
        self.openMeasureW.setToolTip(_translate("MainWindow", "Opens and closes the measure window. If blue, the windows should be visible. Position and size will be stored in case of closing."))
        self.openMeasureW.setText(_translate("MainWindow", "Measure"))
        self.astrometryConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of plate solver:</p><p>Red: Error</p><p>Yellow: Connecting</p><p>Green: Connected</p><p>Gray: No Configured</p></body></html>"))
        self.astrometryConnected.setText(_translate("MainWindow", "Solver"))
        self.cameraText.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status feedback from imaging source</p></body></html>"))
        self.mountConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of mount</p><p>Red: Error</p><p>Yellow: Connecting</p><p>Green: Connected</p></body></html>"))
        self.mountConnected.setText(_translate("MainWindow", "Mount"))
        self.mountConnected.setProperty("color", _translate("MainWindow", "gray"))
        self.openHemisphereW.setToolTip(_translate("MainWindow", "Opens and closes the hemishpere window. If blue, the windows should be visible. Position and size will be stored in case of closing."))
        self.openHemisphereW.setText(_translate("MainWindow", "Hemis"))
        self.openMessageW.setToolTip(_translate("MainWindow", "Opens and closes the message window. If blue, the windows should be visible. Position and size will be stored in case of closing."))
        self.openMessageW.setText(_translate("MainWindow", "Message"))
        self.domeText.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status feedback from dome if connected</p></body></html>"))
        self.openKeypadW.setToolTip(_translate("MainWindow", "Opens and closes the keypad window. If blue, the windows should be visible. Position and size will be stored in case of closing.\n"
"The keypad is a webbrowser showing the content from 10microns webservices directy."))
        self.openKeypadW.setText(_translate("MainWindow", "Keypad"))
        self.statusOnline.setTitle(_translate("MainWindow", "Time"))
        self.timeUTC.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the actual utc of site</p></body></html>"))
        self.timeUTC.setText(_translate("MainWindow", "-"))
        self.timeComputer.setToolTip(_translate("MainWindow", "Shows the actual time of the computer"))
        self.timeComputer.setText(_translate("MainWindow", "time"))
        self.main.setTitle(_translate("MainWindow", "Main"))
        self.mountOn.setToolTip(_translate("MainWindow", "Boots the mount if Wake on Lan is anabled, a MAC address is set in the configuration and the mount computer is connected to the same subnet as your computer. A wired connection is needed. "))
        self.mountOn.setText(_translate("MainWindow", "Mount On"))
        self.saveConfig.setToolTip(_translate("MainWindow", "Saves the actual profile to disk under the same name without closing the application."))
        self.saveConfig.setText(_translate("MainWindow", "Save"))
        self.loadFrom.setToolTip(_translate("MainWindow", "Loads a profile from disk"))
        self.loadFrom.setText(_translate("MainWindow", "Load"))
        self.saveConfigQuit.setToolTip(_translate("MainWindow", "Saves the actual profile and closes the application"))
        self.saveConfigQuit.setText(_translate("MainWindow", "Save/Quit"))
        self.saveConfigAs.setToolTip(_translate("MainWindow", "Saves the actual profile under a selectable new name on the disk without closing the appliation."))
        self.saveConfigAs.setText(_translate("MainWindow", "Save as"))
        self.mountOff.setToolTip(_translate("MainWindow", "Shuts the mount down."))
        self.mountOff.setText(_translate("MainWindow", "Mount Off"))
        self.statusOnline_2.setTitle(_translate("MainWindow", "Profile"))
        self.profile.setToolTip(_translate("MainWindow", "Shows the actual selected profile name."))
        self.profile.setText(_translate("MainWindow", "-"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QWidget()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())
